#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para las opciones de configuracion de la cuenta de los usuarios registrados
class AccountController < ApplicationController
  before_filter :login_required, :except => [:activate, :activity]
  before_filter :maybe_redirect_to_user_settings, :only => [:edit]
  before_filter :maybe_redirect_to_user_sadmin_account, :only => [:show]  
  before_filter :login_required_for_activity, :only => :activity
  before_filter :outside_user_forbidden, :only => [:pwd_edit, :pwd_update]
  before_filter :get_current_user_and_followings, :only => [:show, :questions, :proposals, :activity, :followings]
  
  # La cuenta del usuario loggeado
  def show
    get_activity
    @items = (@my_precontent + @precontent).flatten.uniq.sort {|a,b| b.published_at.to_i <=> a.published_at.to_i}[0..19]
    # HQ
    # @question = Question.new
  end
  
  # Modificar los datos personales del usuario loggeado
  def edit
    @user = current_user    
    @title = t('account.your_profile')
    @breadcrumbs_info = [[t('account.your_profile'), edit_account_path]]
  end
  
  # Actualizar los datos personales del usuario loggeado
  def update
    @user = User.find(current_user.id)
    
    if @user.update_attributes(params[:user])
      flash[:notice] = t('account.datos_guardados')
      redirect_to account_path
    else
      render :action => 'edit'
    end
  end
  
  # Modificar la contraseña del usuario loggeado
  def pwd_edit
    @user = User.find(current_user.id)
    @title = t('account.cambiando_contrasena')
    @breadcrumbs_info = [[t('account.mi_cuenta'), account_path], [t('account.cambiando_contrasena'), pwd_edit_account_path]]
    render :action => "pwd_edit"
  end
  
  # Actualizar la contraseña del usuario loggeado
  def pwd_update
    @user = User.find(current_user.id)
    if params[:old_password] != @user.plain_password
      flash[:error] = t('account.contrasena_erronea')
      redirect_to :action => "pwd_edit"
    else
      if params[:user][:password].empty?
        @user.errors.add(:password, ActiveRecord::Errors.default_error_messages[:empty])
        render :action => "pwd_edit" and return
      end
      if @user.update_attributes(:password => params[:user][:password], :password_confirmation => params[:user][:password_confirmation])
        flash[:notice] = t('account.contrasena_modificada')
        redirect_to(params[:return_to].blank? ? account_path : params[:return_to])
      else
        render :action => "pwd_edit"
      end
    end
  end
  
  # Confirmación de la baja del sistema del usuario loggeado
  def confirm_delete
    @title = t('account.confirm_delete_account_title')
    render :action => "confirm_delete"
  end
  
  # Desactivar la cuenta del usuario loggeado
  def destroy
    if params[:cancel]
      flash[:notice] = t('account.cuenta_no_eliminada')
      redirect_to account_path
    else
      @user = User.find(current_user.id)
      if @user.deactivate_account
        self.current_user.forget_me if logged_in?
        cookies.delete :auth_token
        reset_session
        flash[:notice] = t('account.cuenta_eliminada')
        redirect_to root_path
      else
        flash[:error] = t('account.cuenta_no_eliminada_razon')
        redirect_to account_path
      end
    end
  end
  
  
  def image
    uploader = ImageUploader.cache_from_io!(request.body, params.delete(:qqfile))
    @image = {
      :success => true,
      :image_cache_name => uploader.cache_name
    }
    render :text => @image.to_json and return
  end
  
  def photo
    uploader = PhotoUploader.cache_from_io!(request.body, params.delete(:qqfile))
    @photo = {
      :success => true,
      :photo_cache_name => uploader.cache_name
    }
    render :text => @photo.to_json and return
  end
  
  # Activación de la cuenta de un usuario registrado
  def activate
    @title = "Activar cuenta"
    @user = Person.find(:first, :conditions => ["id= ? AND crypted_password = ? AND status IN ('pendiente', 'aprobado')", params[:u], params[:p]])
    if @user.present?
      @user.update_attribute(:status, "aprobado")
      self.current_user = @user
      @success = true
    else
      success = false
    end
  end
  
  def questions
    # Tus preguntas
    @answered_questions_count = current_user.questions.approved.published.answered.count
    @user = current_user
    # HQ
    # @question = Question.new
    # get_questions

    per_page = request.xhr? ? 5 : 20
    order = params[:more_polemic] ? "comments_count DESC" : "published_at DESC"

    if params[:answered]
      @questions = @user.questions.published.answered.paginate :order => order,
        :per_page => per_page, :page => params[:page]
    else
      @questions = @user.questions.published.paginate :order => order,
        :per_page => per_page, :page => params[:page]
    end

    respond_to do |format|
      format.html do
        if request.xhr?
          render :partial => '/questions/questions_or_none', :layout => false
        else
          render
        end
      end
      format.floki do
        render :template => "/mob_app/questions.json", :layout => false, :content_type => "application/json"
      end
    end
  end
  
  def proposals
    @user = current_user
    
    per_page = request.xhr? ? 5 : 20
    order = params[:more_polemic] ? "comments_count DESC" : "contributions.published_at DESC"
    
    finder_params = {:order => order, :per_page => per_page, :page => params[:page]}
    
    @proposals = current_user.proposals.paginate finder_params
    
    @approved_by_majority = Proposal.find_by_sql(["SELECT contributions.id, sum(value) as sum 
      FROM contributions INNER JOIN votes ON (votes.votable_id = contributions.id AND votes.votable_type = 'Contribution')
      WHERE (((((contributions.published_at <= ?)) AND (contributions.status='aprobado')) 
        AND (contributions.user_id = #{current_user.id}))) AND ( (contributions.type = 'Proposal' ) ) 
      group by contributions.id 
      having sum(value)>0", Time.zone.now]).length
    
    respond_to do |format|
      format.html do
        if request.xhr?
          render :partial => '/shared/list_items', :locals => {:items => @proposals, :type => 'proposals'}, :layout => false
        else
          render
        end
      end
      format.floki do
        render :template => "/mob_app/proposals.json", :layout => false, :content_type => "application/json"
      end
    end
  end
  
  def activity
    get_activity
    respond_to do |format|
      format.html do
        @content = (@my_precontent + @precontent).flatten.uniq.sort {|a,b| b.published_at.to_i <=> a.published_at.to_i}[0..19]
        render
      end
      format.floki do
        @my_content = @my_precontent.flatten.sort {|a,b| b.published_at.to_i <=> a.published_at.to_i}
        @content = (@precontent.flatten - @my_content).sort {|a,b| b.published_at.to_i <=> a.published_at.to_i}[0..19]
        render :action => 'activity.json', :layout => false
      end
    end
  end
  
  def followings
  end
  
  def notifications
    @notifications = current_user.notifications.pending.find(:all, :order => "created_at")
    @title = Notification.human_name(:count => 2).capitalize
  end
  
  private  
  def outside_user_forbidden
    if current_user.is_outside_user?
      flash[:notice] = I18n.t('no_tienes_permiso')
      redirect_to account_path
    end
  end
  
  def login_required_for_activity
    if !logged_in?
      respond_to do |format|
        format.html {access_denied}
        format.floki {render :action => 'login_required.floki', :layout => false, :content_type => "application/json"}
      end
      # self.current_user = User.authenticate('gab-lehendak@ej-gv.es', '8080-1')
    end
  end
  
  def get_activity
    @my_precontent = []
    
    if current_user.is_a?(Politician)
      # HQ
      # # Preguntas para mi por responder
      # @my_precontent = Question.approved.published.unanswered.for_whom(current_user.id)
      # # Mis respuestas y las respuestas de preguntas para mi
      # @my_precontent << Answer.from_politician(current_user).published.find(:all, 
      #                   :select => "answers.id, body, answers.question_id, answered_at, answered_by, answers.created_at",
      #                   :joins => :question,
      #                   :order => "answered_at DESC")
      
    else
      # reacciones a mis propuestas y propuestas en los últimos 3 días

      days_ago = 3.days.ago
      # Comentarios en mis propuestas y preguntas
      @my_precontent << Comment.approved.find(:all, 
                      :select => "comments.id, commentable_id, commentable_type, body, comments.created_at, 
                                  comments.status, comments.user_id, comments.name, comments.email", 
                      :joins => "INNER JOIN contributions on (contributions.id=comments.commentable_id and commentable_type='Contribution')", 
                      :conditions => ["contributions.user_id=#{current_user.id} AND comments.created_at >=?", days_ago], 
                      :order => "comments.created_at DESC")
                      
      # Votos en mis propuestas
      @my_precontent << Vote.find(:all, 
                      :select => "votes.id, value, votable_id, votable_type, votes.created_at, votes.user_id, contributions.title_#{I18n.locale} AS proposal_title",
                      :joins => "INNER JOIN contributions on (contributions.id=votes.votable_id and votable_type='Contribution')",
                      :conditions => ["contributions.user_id=#{current_user.id} AND votes.created_at >=?", days_ago],
                      :order => "votes.created_at DESC")
      
      # Argumentos en mis propuestas
      @my_precontent << Argument.published.find(:all, 
                      :select => "arguments.id, reason, arguments.published_at, arguments.created_at, argumentable_id, argumentable_type, arguments.user_id, arguments.value",
                      :joins => "INNER JOIN contributions on (contributions.id=arguments.argumentable_id and argumentable_type='Contribution')",                      
                      :conditions => ["contributions.user_id=#{current_user.id} AND arguments.created_at >=?", days_ago],
                      :order => "arguments.published_at DESC")

      # HQ
      # # Respuestas a mis preguntas
      # @my_precontent << Answer.published.find(:all, 
      #                 :select => "answers.id, body, question_id, answered_at, answered_by, answers.created_at",
      #                 :joins => :question,
      #                 :conditions => ["contributions.user_id=#{current_user.id} AND answers.created_at >=?", days_ago],
      #                 :order => "answered_at DESC")
      # 
      # @my_precontent << AnswerRequest.find(:all, 
      #                 :select => "answer_requests.id, answer_requests.created_at, question_data_id, answer_requests.user_id",
      #                 :joins => {:question_data => :question},
      #                 :conditions => ["contributions.user_id=#{current_user.id} AND answer_requests.created_at >=?", days_ago],
      #                 :order => "answer_requests.created_at DESC")
      # 
    end
    
    
    # Noticias de las áreas a las que sigue
    @precontent = []
    if current_user.following_areas.length > 0
      @precontent << News.published.translated.tagged_with(current_user.following_areas.collect(&:area_tag)).find(:all, 
        :select => "documents.id, title_#{I18n.locale}, published_at, has_comments, multimedia_path, cover_photo_file_name", 
        :order => 'published_at DESC', :limit => 20)
        
      @precontent << Event.published.translated.tagged_with(current_user.following_areas.collect(&:area_tag)).find(:all, 
        :select => "documents.id, title_es, title_eu, title_en, 
                    published_at, starts_at, ends_at, place, city, location_for_gmaps", 
        :order => 'published_at DESC', :limit => 20)
        
      @precontent << Proposal.approved.published.translated.tagged_with(current_user.following_areas.collect(&:area_tag)).find(:all, 
        :select => "contributions.id, title_es, title_eu, title_en, 
                    substring(body_es from 0 for 500) as body_es,
                    substring(body_es from 0 for 500) as body_eu,
                    substring(body_es from 0 for 500) as body_en, 
                    published_at, has_comments, comments_closed, status,
                    user_id", 
        :order => 'published_at DESC', :limit => 20)

      # TODO: should albums and videos be included?
      # @precontent << Video.published.translated.tagged_with(current_user.following_areas.collect(&:area_tag)).find(:all, 
      #   :select => "videos.id, title_#{I18n.locale}, published_at, video_path, subtitles_es_file_name, subtitles_eu_file_name, subtitles_en_file_name", 
      #   :order => 'published_at DESC', :limit => 20)

      # @precontent << Album.published.tagged_with(current_user.following_areas.collect(&:area_tag)).find(:all, 
      #   :select => "albums.id, title_#{I18n.locale}, albums.created_at", 
      #   :order => 'created_at DESC', :limit => 20)


      # HQ
      # @precontent << Question.approved.published.translated.tagged_with(current_user.following_areas.collect(&:area_tag)).find(:all, 
      #   :select => "contributions.id, 
      #               title_es, title_eu, title_en, 
      #               substring(body_es from 0 for 500) as body_es,
      #               substring(body_es from 0 for 500) as body_eu,
      #               substring(body_es from 0 for 500) as body_en,
      #               published_at, has_comments, comments_closed, status,
      #               user_id", 
      #   :order => 'published_at DESC', :limit => 20)
      #   
      # @precontent << Answer.find(:all, 
      #   :select => "answers.id, answered_at, question_datas.question_id, has_comments, comments_closed, body, answered_by, answers.created_at",
      #   :joins => {:question => [:tags, :question_data]}, 
      #   :conditions => "sanitized_name_es IN (#{current_user.following_areas.collect {|t| "'#{t.area_tag}'"}.join(',')})", 
      #   :order => 'answered_at DESC', :limit => 20)
        
    end

    # HQ
    # if current_user.following_politicians.length > 0
    #   politician_ids = current_user.following_politicians.collect(&:id).join(',')
    #   @precontent << Question.approved.published.translated.find(:all, :joins => :question_data, 
    #     :select => "contributions.id, title_#{I18n.locale}, body_#{I18n.locale}, published_at, has_comments, comments_closed, 
    #                 status, user_id", 
    #     :conditions => "for_whom_id in (#{politician_ids})", 
    #     :order => 'published_at DESC', :limit => 20)
    #   @precontent << Answer.find(:all, :joins => {:question => :question_data}, 
    #     :select => "answers.id, answered_at, question_datas.question_id, has_comments, comments_closed, body, answered_by, answers.created_at",
    #     :conditions => "for_whom_id in (#{politician_ids})", 
    #     :order => 'answered_at DESC', :limit => 20)
    # end
    
    
  end

  def maybe_redirect_to_user_settings
    if current_user.has_admin_access?
      redirect_to admin_url 
    else
      return true
    end
  end
  
  def maybe_redirect_to_user_sadmin_account
    if current_user.is_a?(Politician)
      redirect_to politician_url(current_user)
    elsif current_user.has_admin_access? && !current_user.is_admin?
      redirect_to sadmin_account_url
    else
      return true
    end
  end
  
  def get_current_user_and_followings
    @user = current_user
    @following_politicians = current_user.following_politicians
    @following_areas = current_user.following_areas    
  end
  
  def make_breadcrumbs
    @breadcrumbs_info = []
    if params[:action].eql?('proposals')
      @breadcrumbs_info << [t("account.your_proposals"), proposals_account_path]  
    # elsif params[:action].eql?('questions')
    #     @breadcrumbs_info << [t("account.your_questions"), questions_account_path]
    elsif params[:action].eql?('show')
      @breadcrumbs_info << [t("account.your_profile"), account_path]  
    elsif params[:action].eql?('edit')
      @breadcrumbs_info << [t("account.edit_profile"), edit_account_path]  
    elsif params[:action].eql?('followings')
      @breadcrumbs_info << [t("users.following"), followings_account_path]
    elsif params[:action].eql?('notifications')
      @breadcrumbs_info << [@title, notifications_account_path]
    end  
  end

end
