#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controller de administracion para los banners de la pagina de inicio
class Admin::BannersController < Admin::BaseController
  cache_sweeper :footer_sweeper, :only => [ :create, :update, :destroy, :sort]
  
  # Listado de todos los banners con la imagen y el alt del idioma y el enlace
  def index
    # @banners=Banner.paginate :page => params[:page], :per_page => 10, :order => "position DESC"
    @banners=Banner.all(:order => "active DESC, position DESC")
  end  
  
  # No se si sera necesario mostrar un banner en administracion
  def show
  end  
  
  # Crear un nuevo banner
  def new
    @banner=Banner.new
  end
  
  def create
    @banner=Banner.new(params[:banner])
    if @banner.save
      flash[:notice] = "El nuevo banner se ha guardado correctamente"
      redirect_to admin_banners_path
    else
      render :action => :new
    end
  end
  
  # Actualizar los datos de un banner ya existente
  def edit
    @banner=Banner.find(params[:id])
  end
  
  def update
    @banner=Banner.find(params[:id])
    if @banner.update_attributes(params[:banner])
      flash[:notice] = "El banner se ha guardado correctamente"
      redirect_to admin_banners_path
    else
      render :action => :edit
    end    
  end
  
  # Eliminar un banner
  def destroy
    banner=Banner.find(params[:id])
    if banner.destroy
      respond_to do |format|
        format.html {
          flash[:notice]='El banner se ha eliminado'
          redirect_to admin_banners_path
        }
        format.js {
          render :update do |page| 
            page.visual_effect :fade, "banner_#{banner.id}"
          end
        }
      end
    else
      respond_to do |format|
        format.html {
          flash[:error]='El banner NO se ha eliminado correctamente'
        }
        format.js {
          render :update do |page|
            page.alert "El banner NO se ha eliminado correctamente"
          end  
        }
      end
    end
  end
  
  # Sera necesario poder ordenarlos ¿quizas drag &drop?
  def sort
    @banners=Banner.find(:all)
    @banners.each do |banner|
      new_position = params["banners-list"].reverse.index(banner.id.to_s)+1
      banner.update_attributes(:position => new_position)
    end 

    render :nothing => true
  end
  
  private
  
  def set_current_tab    
    @current_tab = :banners
  end
  
end  
