#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Métodos comunes para todos los controladores de la administración,
# aquella a la que acceden los Admin.
# 
# Debajo del namespace "admin" estaba la administración completa sólo para administradores generales.
# Paulatinamente los permisos se fueron complicando y ahora otros roles tienen
# permiso para acceder a algunas partes de "admin"
class Admin::BaseController < ApplicationController
  layout 'admin'

  # Comprobamos que el usuario está aprobado
  before_filter :approved_user_required

  # Comprobamos que el usuario tiene permiso de administrador
  before_filter :admin_required

  before_filter :set_current_tab

  # Determina la pestaña activa en el menú de la administración.
  # Cada controller determina el suyo
  def set_current_tab
    @current_tab = :categories
  end
  
  # Configuración de las opciones para el editor TinyMCE
  TINYMCE_OPTIONS = {
                  :theme => 'advanced',
                  :language => 'es',
                  :theme_advanced_toolbar_location => 'top',
                  # :theme_advanced_statusbar_location => 'top',
                  # :theme_advanced_resizing => true,
                  # :theme_advanced_resize_horizontal => false,
                  # :theme_advanced_blockformats => 'p,div,h1,h2,h3,h4,h5,h6,blockquote,dt,dd,code,samp',
                  :content_css => "/stylesheets/tinymce.css",
                  # :theme_advanced_styles => "Foto izquierda=photo_left;Foto derecha=photo_right",
                  # :theme_advanced_blockformats => "p,div,h1,h2,h3,h4,h5,h6,blockquote,dt,dd,code,samp",
                  :theme_advanced_styles => "Subtitular=r01Subtitular;Entradilla=r01Entradilla",
                  :content_css => "/stylesheets/admin/admin.css",
                  :plugins => %w{ table fullscreen paste},
                  # :theme_advanced_buttons1_add => 'removeformat',
                  # :theme_advanced_buttons1 => "bold,italic,strikethrough,|,justifyleft,justifycenter,justifyright,justifyfull,|,formatselect,styleselect,|,removeformat,|,pastetext",
                  :theme_advanced_buttons1 => "bold,italic,strikethrough|,justifyleft,justifycenter,justifyright,justifyfull,|,styleselect,|,removeformat",
                  :theme_advanced_buttons2 => "bullist,numlist,|,outdent,indent,|,undo,redo,|,link,unlink,anchor,image,cleanup,code,|,tablecontrols",
                  :theme_advanced_buttons3 => '',
                  :paste_auto_cleanup_on_paste => true,
                  :paste_remove_styles => true,
                  # No queremos acutes:
                  :entities => '',
                  :relative_urls => false,
                  :extended_valid_elements => "iframe[align<bottom?left?middle?right?top|class|frameborder|height|id|longdesc|marginheight|marginwidth|name|scrolling<auto?no?yes|src|style|title|width],"
                }


  # Filtro que determina si el usuario tiene permiso para modificar los permisos
  def super_user_required
    unless can?('administer', 'permissions')
      flash[:notice] = t('no_tienes_permiso')
      access_denied
    end
  end
                
end
