#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para la gestión de categorías de los menús
class Admin::CategoriesController < Admin::BaseController
  
  before_filter :get_tree

  uses_tiny_mce :only => [:edit, :update], :options => TINYMCE_OPTIONS
  
  # cache_sweeper :category_sweeper, :only => [:create, :update, :destroy, :sort]
  
  # Listado de categorías
  def index
  end

  # Formulario de creación de categoría
  def new
    @category = @tree.categories.new
    @page_title = "Nueva categoría"
    respond_to do |format|
      format.html # show.rhtml
      format.js {
        render :update do |page| 
          page.replace_html "new_container#{params[:parent_id]}", :partial => 'new'
          page[:name_es].focus
        end
      }
    end
  end

  # Creación de categoría
  def create
    @page_title = "Nueva categoría"
    @category = @tree.categories.new(params[:category])
    if @category.parent
      @category.position = @category.parent.position + 1
    end
    
    respond_to do |format|
      if @category.save
        format.html { 
          flash[:notice] = 'category was successfully created.'
          redirect_to admin_tree_category_url(@tree, @category) 
        }
        format.js { 
          render :update do |page|
            if @category.parent
              page.replace_html "cat_#{@category.parent.id}", :partial => "category", :locals => {:category => @category.parent, :tree => @category.tree}
              page.replace_html "new_container#{@category.parent.id}", ""
            else
              page.insert_html :bottom, "categories", :partial => "category", :locals => {:category => @category, :tree => @category.tree}
              page.replace_html "new_container", ""
            end
            
            page.visual_effect :highlight, "cat_#{@category.id}"
            
          end
        }
      else
        format.html { render :action => "new" }
        format.js {
          render :update do |page|
            page.replace_html "new_container#{@category.parent_id}", :partial => "new", :object => @category
            page[:name_es].focus
          end
        }
      end
    end
  end
  
  # Modificación de categoría
  def edit
    @category = @tree.categories.find(params[:id])
    respond_to do |format|
      format.html {
        
      }
      format.js {
        render :update do |page| 
          page.replace_html "cat_#{params[:id].to_i}", :partial => 'edit'
          page[:name_es].focus
        end
      }
    end
  end
  
  # Actualización de categoría
  def update
    @category = @tree.categories.find(params[:id])
    if @category.update_attributes(params[:category])
      respond_to do |format|
        format.html {
          flash[:notice] = "La sección se ha actualizado correctamente"
          redirect_to admin_tree_path(@tree)
        }
        format.js {
          render :update do |page| 
            if params[:category][:tag_list]
              page.replace_html "tags_#{@category.id}", :partial => 'tags', :locals => { :category => @category}
              page.visual_effect :highlight, "tags_#{@category.id}"
            else
              page.replace "cat_#{@category.id}", :partial => 'category', :object => @category
              page.visual_effect :highlight, "cat_#{@category.id}"
            end
          end
        }
      end
    else
      respond_to do |format|
        format.html {
          render :action => "edit"
        }
        format.js {
          render :update do |page| 
            page.insert_html :top, "cat_#{@category.id}", "<div class='flash_error'>la categoria no se ha actualizado</div>"
          end
        }
      end
    end
  end  

  # Eliminación de categoría
  def destroy
    @page_title = "Eliminar categoría"
    @category = Category.find(params[:id])
    if @category.destroy
      respond_to do |format|
        format.html { 
          flash[:notice] = 'La categoría se ha eliminado correctamente'
          redirect_to admin_tree_url(@tree) 
        }
        format.js {
          render :update do |page|
            page.visual_effect :fade, "cat_#{@category.id}"
          end
        }
      end
    else
      respond_to do |format|
        format.html { 
          flash[:error] = 'La categoría NO se ha eliminado correctamente'
          redirect_to admin_tree_url(@tree) 
        }
        format.js {
          render :update do |page|
            page.alert 'La categoría NO se ha eliminado correctamente'
          end
        }
      end
    end
  end
  
  
  # Reordenación de categorías
  def sort
    @page_title = "Reordenar categorías"
    @categories = Category.find(:all, :conditions => {:tree_id => params[:tree_id], :parent_id => params[:parent_id]})
    # The parameter containing the items to be ordered has different name depending on the sublist
    # we are ordering. Its name always starts with "categories"
    order_param = params.keys.select {|k| k =~ /categories/}[0]
    Category.transaction do 
      @categories.each do |cat|
        cat.position = params[order_param].index(cat.id.to_s) + 1
        cat.save
      end
    end
    render :nothing => true
  end

  # Modificación de tags de una categoría
  def edit_tags
    @category= @tree.categories.find(params[:id])
    render :update do |page|
      page.replace_html "tags_#{@category.id}", :partial => "tags_form", :locals => { :category => @category}
      page[:category_tag_list].focus
    end
  end  

  private
  # Las categorías pertenecen siempre a un árbol, en este caso el de menú de Irekia, 
  # Canales e WebTV
  def get_tree
    begin
      @tree = Tree.find(params[:tree_id])
    rescue ActiveRecord::RecordNotFound
      flash[:notice] = "La sección no es correcta"
      redirect_to :back
    end
  end
  
  
end
