#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para la administración de comentarios
class Admin::CommentsController < Sadmin::BaseController
  before_filter :access_to_comments_required
  before_filter :admin_required, :only => [:edit, :update]
  # before_filter :set_conditions
  before_filter :maybe_restrict_to_user_department
  before_filter :status_and_oficiales_conditions, :only => [:index, :comments_on_item]
  before_filter :get_comment, :only => [:show, :edit, :update, :reject, :do_reject, :update_status, :mark_as_spam, :mark_as_ham, :destroy, :zforward_form, :zforward] 
  
  # Listado de comentarios
  def index    
    @title ||= 'Todos los comentarios'

    if @department_tag_id.nil?
      @comments = Comment.paginate :per_page => 20, 
        :conditions => @additional_conditions,
        :joins => "INNER JOIN users ON (users.id = comments.user_id) ",
        :page => params[:page], :order => "created_at DESC"
    else
      @comments = Comment.paginate_by_sql(query_for_department(@department_tag_id, @additional_conditions), 
        :page => params[:page], :per_page => 20, :order => "created_at DESC")
    end    
  end
  
  def pending
    @comments = Comment.pending

    if params[:c_id]
      @comment = @comments.select {|c| c.id == params[:c_id].to_i}.first
    end
    
    @comment = @comments.first unless @comment
    if @comments.length > 0
      @previous = @comments[@comments.index(@comment) - 1] 
      @next     = @comments[@comments.index(@comment) + 1] || @comments.first
    end
    @page_title = "1 de #{@comments.length}"
    render :action => 'pending.iphone', :layout => false
  end
  
  
  # Modificación de comentarios
  def edit
  end

  # Pantalla de confirmación de rechazo un comentario
  def reject
  end
  
  # Rechacar un comentario
  def do_reject
    if @comment.update_attributes(:status => "rechazado")
      if params[:reject_and_mail]
        email = Notifier.create_rejected_comment(@comment)

        begin
          logger.info("Mandando email sobre comentario rechazado a #{@comment.user.email}")
          Notifier.deliver(email)
          flash[:notice] = "El email ha sido enviado"
        rescue Net::SMTPServerBusy, Net::SMTPSyntaxError => err_type
          logger.info("Error al mandar el email: " + err_type)
          flash[:error] = t('session.Error_servidor_correo')
        end
      end
      redirect_to(params[:return_to].present? ? params[:return_to] :  admin_comments_path)
    else
      render :action => :reject
    end    
  end
  
  # Actualización de un comentario
  def update
    if @comment.update_attributes(params[:comment])
      redirect_to(params[:return_to].present? ? params[:return_to] :  admin_comments_path)
    else
      render :action => :edit
    end
  end
  
  # Cambiar el estado de un comentario (aprobado, spam, etc...)
  def update_status
    @comment.update_attributes(params[:comment])
    render_updated_comment
  end
  
  # Marcar un comentario como spam
  def mark_as_spam
    @comment.mark_as_spam!
    render_updated_comment
  end
  
  # Marcar un comentario como no-spam
  def mark_as_ham
    @comment.mark_as_ham!
    render_updated_comment
  end
  
  # Eliminación de un comentario
  def destroy
    if @comment.destroy
      respond_to do |format|
        format.html {
          flash[:notice] = 'El comentario ha sido eliminado'
          redirect_to admin_comments_path
        }
        format.js { 
          render :update do |page|
            page.visual_effect :fade, "comment_#{@comment.id}"
          end
        }
      end
    else
      respond_to do |format|
        format.html {
          flash[:error] = 'El comentario no ha podido ser eliminado'
          redirect_to admin_comments_path
        }
        format.js { 
          render :update do |page|
            page.alert "El comentario no se ha borrado"
          end
        }
      end
      
    end
  end
  
  # Muestra los comentarios de un contenido concreto
  def comments_on_item    
    @item = params[:type].constantize.find(params[:id])
    if @department_tag_id && !@item.tag_ids.include?(@department_tag_id)
      flash[:notice] = t('no_tienes_permiso')
      redirect_to admin_comments_path
    end

    @additional_conditions =  nil if @additional_conditions[0].blank?
    
    if @department_tag_id.nil?
      @comments = @item.comments.paginate :per_page => 20, :page => params[:page],
        :conditions => @additional_conditions ,
        :joins => "INNER JOIN users ON (users.id = comments.user_id) ",
        :order => "created_at DESC"
    else
      @additional_conditions[0] << " AND commentable_id=#{@item.id}"
      @comments = @item.comments.paginate_by_sql(query_for_department(@department_tag_id, @additional_conditions), 
        :page => params[:page], :per_page => 20, :order => "created_at DESC")
    end
        
    @title = "Comentarios en #{@item.class.human_name} \"#{@item.title_es}\""
    rewrite_current_tab
  end
  
  # Actualiza el campo de si un contenido admite comentarios o no
  def update_comments_status
    @item = params[:type].constantize.find(params[:item_id], :readonly => false)
    if @department_tag_id && !@item.tag_ids.include?(@department_tag_id)
      render :update do |page|
        flash[:notice] = t('no_tienes_permiso')
        page.redirect_to comments_on_item_admin_comment_path(@item, :type => params[:type])
      end
    else
      if @item.update_attribute(:comments_closed, params[:comments_closed])
        render :update do |page|
          page.replace :comments_status, :partial => 'comments_status'
        end
      end
    end
  end

  # Formulario para reenviar un comentario a Zuzenean, el servicio de atención ciudadana de euskadi.net
  def zforward_form
    render :update do |page|
      page.replace_html "comment_id_#{@comment.id}", :partial => "zforward_form", :locals => {:comment => @comment}
    end
  end 
  
  require 'net/http'
  # Reenvía un comentario a Zuzenean
  def zforward

    body_for_z = "#{@comment.body}
    ---------
    Esta consulta corresponde al comentario nº #{@comment.id} de Irekia."

    res = Net::HTTP.post_form(URI.parse('http://www.ej-gv.net/o53/visualizador/o53regbuzonEuskadi.jsp'),
                                {'por'=>params[:por], 'lenguaje'=>'1', 'sugerencia' => body_for_z, 
                                 'name' => @comment.user.public_name, 'usuario' => @comment.user.email})

    # res = Net::HTTP.post_form(URI.parse('http://www.google.es/'),
    #                             {'por'=> params[:por], 'lenguaje'=>'1', 'sugerencia' => body_for_z, 
    #                              'name' => @comment.user.public_name, 'usuario' => @comment.user.email})

    case res
    when Net::HTTPSuccess #, Net::HTTPRedirection
      flash[:notice] = "El comentario se ha redigido correctamente"
      # La respuesta esta en res.read_body
      @comment.update_attributes(:zforwarded_at => Time.zone.now, :zforwarded_to => params[:por])
      redirect_to :back
    when Net::HTTPRedirection
      flash[:error] = "Peticion redirigida a #{res['location']}"
      redirect_to :back
      # response = Net::HTTP.get(URI.parse(res['location']))
    else
      flash[:error] = "La redirección del comentario no ha funcionado: #{res.body}"
      redirect_to :back
      # res.error!
    end

  end

  private
  # Determina el tab de la administración que estará activo
  def set_current_tab
    @current_tab = :comments
  end
  
  # Sobreescribe el tab activo de la administración
  def rewrite_current_tab
    @t = @item ? @item.type.to_s.downcase.pluralize : (params[:t].present? ? params[:t] : 'news')
    
    @current_tab = @t.to_sym
    @current_tab
  end
  
  # Respuesta común para todas las acciones que actualizan un comentario desde el listado de comentarios
  def render_updated_comment
    respond_to do |format|
      format.iphone {redirect_to pending_admin_comments_path}
      format.js do
        render :update do |page|
          if params[:item_id] && params[:type]
            @item = params[:type].constantize.find(params[:item_id])
            page.replace "comment_#{@comment.id}", :partial => "comment_on_item", :locals => {:comment_on_item => @comment}
          else
            page.replace "comment_#{@comment.id}", :partial => "comment", :locals => {:comment => @comment}
          end
          page["comment_#{@comment.id}"].className = @comment.status
        end    
      end
    end
  end

  # Determina las condiciones que restringen el listado de comentarios que se mostrará,
  # en función del perfil del usuario y de si se ha restringido la selección a un departamento concreto
  # def set_conditions
  #   @joins = ""
  #   @conditions = ["1=1", {}]
  #   organization_ids = []
  #   department_tag_id = nil
  #   
  #   if [DepartmentEditor, StaffChief].include?(current_user.class)
  #     department_tag_id = Tag.find_by_name_es(Department.find(current_user.department_id).tag_name).id
  #     organization_ids = [current_user.department_id] + current_user.department.organization_ids
  #     @title = "Comentarios del departamento #{current_user.department.name}"
  #   elsif params[:dep_id] && params[:dep_id].to_i != 0
  #     dep = Department.find(params[:dep_id])
  #     department_tag_id = Tag.find_by_name_es(dep.tag_name).id
  #     @title = "Comentarios del departamento #{dep.name}"
  #     organization_ids = dep.organization_ids + [params[:dep_id]]
  #   end
  #   
  #   if !department_tag_id.nil?
  #     logger.info "AAAAAAAAAAAAAAA #{department_tag_id}"
  #     # @joins = "LEFT OUTER JOIN documents ON (documents.id=comments.commentable_id AND commentable_type='Document')
  #     #           LEFT OUTER JOIN proposals ON (proposals.id=comments.commentable_id AND commentable_type='Proposal')
  #     #           LEFT OUTER JOIN videos ON (comments.id=comments.commentable_id AND commentable_type='Video')"
  #     @joins = "LEFT OUTER JOIN documents ON (documents.id=comments.commentable_id AND commentable_type='Document')
  #               LEFT OUTER JOIN videos ON (videos.id=comments.commentable_id AND commentable_type='Video')"      
  #     @conditions = [" (CASE WHEN commentable_type = 'Document' THEN documents.organization_id IN (#{organization_ids.join(',')}) ELSE true END)", {}]
  #   end
  # end
  
  def query_for_department(department_tag_id, additional_conditions="")
    query = "SELECT comments.id, coalesce(document_id, video_id) AS commentable_id, commentable_type, 
        comments.status, body, comments.name, user_id, comments.email, comments.created_at 
      FROM (SELECT comments.*, documents.id AS document_id, videos.id as video_id FROM comments  
        LEFT OUTER JOIN documents ON (documents.id=comments.commentable_id AND commentable_type='Document') 
        LEFT OUTER JOIN videos ON (videos.id=comments.commentable_id AND commentable_type='Video')) as comments, taggings, users
        WHERE users.id = comments.user_id
          AND coalesce(document_id, video_id)= taggings.taggable_id AND comments.commentable_type=taggings.taggable_type 
          AND taggings.tag_id=#{department_tag_id} AND #{additional_conditions[0]}
        ORDER BY comments.created_at DESC"
    [query, additional_conditions[1]]
  end
  
  # Filtro que determina si el usuario puede entrar en la administración de comentarios
  def access_to_comments_required
    unless (logged_in? && can_edit?("comments"))
      flash[:notice] = t('no_tienes_permiso')
      access_denied
    end
  end
  
  def get_comment
    @comment = Comment.find(params[:id])
    if @department_tag_id && !@comment.commentable.tag_ids.include?(@department_tag_id)
      respond_to do |format|
        format.html {
          flash[:notice] = t('no_tienes_permiso')
          access_denied      
        }
        format.js { 
          render :update do |page|
            flash[:notice] = t('no_tienes_permiso')
            page.redirect_to admin_comments_path
          end
        }
      end
      
    end
  end
  
  def maybe_restrict_to_user_department
    if [DepartmentEditor, StaffChief].include?(current_user.class)
      @department_tag_id = Tag.find_by_name_es(Department.find(current_user.department_id).tag_name).id
      @title = "Comentarios del departamento #{current_user.department.name}"
    elsif params[:dep_id] && params[:dep_id].to_i != 0
      dep = Department.find(params[:dep_id])
      @department_tag_id = Tag.find_by_name_es(dep.tag_name).id
      @title = "Comentarios del departamento #{dep.name}"
    end
  end
  
  def status_and_oficiales_conditions
    @additional_conditions = ["1=1", {}]
    @status = params[:status]    
    if @status 
      @additional_conditions[0] << " AND comments.status=:status"
      @additional_conditions[1][:status] = @status
    end

    # Esti: Esto no es muy eficiente, habría que usar el método Department#official_commenters o
    # algo parecido, pero como esta acción no se usa mucho lo dejo así
    @oficiales = (params[:oficiales] && params[:oficiales].to_i == 1) || false
    if @oficiales
      official_commenters = User.all.select {|u| u.is_official_commenter?}.collect(&:id)
      @additional_conditions[0] << " AND users.id IN (#{official_commenters.join(', ')})"
    end
  end
  
end
