#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador par la administración de propuestas del gobierno (debates).
class Admin::DebatesController < Admin::BaseController
  skip_before_filter :verify_authenticity_token, :only => [:auto_complete_for_debate_tag_list_es, :auto_complete_for_debate_entity_organization_name]
    
  before_filter :set_form_lang, :only => [:new, :edit, :create, :update]
  before_filter :get_debate, :except => [:index, :arguments, :new, :create, :auto_complete_for_debate_tag_list_es, :auto_complete_for_debate_entity_organization_name]
  
  uses_tiny_mce :only => [:new, :create, :edit, :edit_common, :update],
                :options => TINYMCE_OPTIONS
  
  def index
    @debates = Debate.paginate :page => params[:page], :per_page => 20, :order => 'updated_at DESC'
    @title = "Propuestas del Gobierno"
  end

  def arguments
    @title = "Propuestas del Gobierno"
    @arguments = Argument.for_debates.paginate :per_page => 20, 
       :joins => "INNER JOIN users ON (users.id = arguments.user_id) ",
       :page => params[:page], 
       :order => "published_at DESC, created_at DESC"
  end

  def new
    @title = "Crear propuesta gubernamental"
    get_departments()
    @debate = Debate.new
    @debate.init_stages
  end

  def edit
    @title = "Modificar propuesta gubernamental"
    if params[:w].eql?("traducciones")
      @show_only_translatable = true
    else
      get_departments()
    end
  end

  def create
    @title = "Crear propuesta gubernamental"
    @debate = Debate.new(params[:debate])
    if @debate.save
      flash[:notice] = 'La propuesta se ha guardado correctamente'
      redirect_to admin_debate_path(@debate)
    else
      get_departments()
      render :action => 'new'
    end    
  end

  def update
    @debate.attributes = params[:debate]

    @title = "Actualizar propuesta gubernamental"
    
    if @debate.save
      flash[:notice] = 'Los cambios se han guardado correctamente'
      redirect_to params[:redirect_to] || admin_debate_path(@debate)
    else
      get_departments()
      render :action => params[:return_to] || 'edit'
    end
  end

  def show
  end

  def destroy
    @title = 'Eliminar propuesta'
    
    if @debate.destroy
      flash[:notice] = 'La propuesta se ha eliminado correctamente'
      redirect_to admin_debates_path
    else
      flash[:error] = 'La propuesta no se ha podido eliminar'
      redirect_to admin_debate_path(@debate)
    end
  end
  
  def common
  end

  def edit_common
  end

  def translations
    @t = "debate"
  end

  def auto_complete_for_debate_tag_list_es
    auto_complete_for_tag_list(params[:debate][:tag_list_es])
    if @tags.length > 0
      render :inline => "<%= content_tag(:ul, @tags.map {|t| content_tag(:li, t.nombre)}) %>"
    else
      render :nothing => true
    end  
  end

  def auto_complete_for_debate_entity_organization_name
    q = params[:debate_entity][:organization_name].strip.tildes.downcase
  
    @organizations = OutsideOrganization.find(:all, :conditions => ["(lower(tildes(name_es || coalesce(name_eu, '') || coalesce(name_en, ''))) like ?)", "%#{q}%"])  
    if @organizations.length > 0
      render :inline => '<%= content_tag(:ul, @organizations.map {|o| content_tag(:li, "#{o.name}")}) %>'
    else
      render :nothing => true
    end
  end

  def publish
    @debate.update_attributes(:published_at => Time.zone.now)
    redirect_to :back
  end

  private
  
  def set_form_lang
    @lang = (params[:lang] || "es").to_sym
  end

  def set_current_tab
    @current_tab = :debates
  end

  def get_debate  
    @debate = Debate.find(params[:id])    
    @title = @debate.title    
  end

  # Lista de departamentos para el select:
  # para debates nuevos, salen sólo los departamentos activos
  # para debates con departamento que ya no es activo, salen todos los departamentos.
  def get_departments
    @departments = if @debate && @debate.organization && !@debate.organization.active?
      Department.find(:all, :order => "position")
    else
      Department.active.find(:all, :order => "position")
    end
  end  
  
end
