#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para la administración de departamentos y organismos
class Admin::OrganizationsController < Admin::BaseController
  
  before_filter :department_or_organization, :only => [:new, :create]

  # Listado de departamentos
  def index
    @organizations = Organization.find(:all, :conditions => "parent_id IS NULL", :order => "position")

    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :xml => @admin_organizations }
    end
  end

  # Vista de un departamento
  def show
    @organization = Organization.find(params[:id])

    respond_to do |format|
      format.html # show.html.erb
      format.xml  { render :xml => @organization }
    end
  end

  # Formulario de nuevo departamento
  def new
    respond_to do |format|
      format.html # new.html.erb
      format.xml  { render :xml => @organization }
    end
  end

  # Modificar departamento
  def edit
    @organization = Organization.find(params[:id])
  end


  # Crear departamento
  def create
    @organization.attributes = params[:organization]

    respond_to do |format|
      if @organization.save
        flash[:notice] = "El #{@organization.class.human_name} se ha creado correctamente."
        format.html { redirect_to(admin_organization_path(@organization)) }
        format.xml  { render :xml => @organization, :status => :created, :location => @organization }
      else
        format.html { render :action => "new" }
        format.xml  { render :xml => @organization.errors, :status => :unprocessable_entity }
      end
    end
  end

  # Actualizar departamento
  def update
    @organization = Organization.find(params[:id])

    respond_to do |format|
      if @organization.update_attributes(params[:organization])
        flash[:notice] = "El #{@organization.class.human_name} se ha actualizado correctamente."
        format.html { redirect_to(admin_organization_path(@organization)) }
        format.xml  { head :ok }
      else
        format.html { render :action => "edit" }
        format.xml  { render :xml => @organization.errors, :status => :unprocessable_entity }
      end
    end
  end

  # # Eliminar departamento
  # def destroy
  #   @organization = Organization.find(params[:id])
  #   @organization.destroy
  # 
  #   respond_to do |format|
  #     format.html { redirect_to(admin_organizations_url) }
  #     format.xml  { head :ok }
  #   end
  # end
  
  private
  def set_current_tab
    @current_subtab = :organization
    @current_tab = :depts
  end
  
  def department_or_organization
    @organization = params[:d] && params[:d].to_i == 1 ? Department.new : Organization.new
  end
end
