#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para los álbums de la Fototeca
class AlbumsController < ApplicationController
  before_filter :get_context, :only => [:index]
  before_filter :get_criterio, :only => [:show]
  after_filter :track_clickthrough, :only => [:show]
  
  def index    
    prepare_albums(@context, request.xhr?)
    
    respond_to do |format|
      format.html do
        if request.xhr?
          render :partial => '/shared/list_items', :locals => {:items => @albums, :type => 'album'}, :layout => false
        else
          render
        end
      end
    end
  end
  
  
  # Vista de un album de la fototeca
  def show
    begin
      @album = Album.published.find(params[:id])
    rescue ActiveRecord::RecordNotFound
      if can_edit?("videos")
        @album = Album.find(params[:id])
      else
        raise ActiveRecord::RecordNotFound
      end
    end
    
    @photos = @album.photos.ordered_by_title.paginate :page => params[:page], :per_page => 6, 
      :conditions => ["cover_photo=?", false]

    if params[:photo_id] && @photo = @album.photos.find(params[:photo_id])
      @photo_id = params[:photo_id]
      unless @photos.collect(&:id).include?(@photo_id.to_i)
        @photos.pop
        @photos << @photo
      end
    end
    
    @related_albums = Album.published.tagged_with(@album.area_tags).paginate :page => params[:p], :per_page => 8,
        :conditions => "album_photos_count>0", 
        :order => "featured DESC, created_at DESC"
        
    respond_to do |format|
      format.html
      format.js {
        render :update do |page|
          page.replace 'thumbnails', :partial => '/albums/thumbnails', :locals => {:photos => @photos}
        end
      }
    end
  
  end
  
  def cat
    @categories = Album.categories
    @category = Category.find(params[:id])
    @title = @category.name
    @albums = @category.albums.with_photos.published.paginate(:per_page => 12, :order => "created_at DESC, matching_tag_counter DESC", :page  => params[:page] || 1)
  end
  
  private
  # Construye los breadcrumbs de cada acción de la fototeca
  def make_breadcrumbs
    if @context
      @breadcrumbs_info << [t('albums.title'), send("#{@context.class.to_s.downcase}_albums_path", @context)]
    else
      @breadcrumbs_info = [[t('photos.fototeca'), albums_path]]
      if @album
        @breadcrumbs_info << [@album.title, album_path(@album)]
      end
    end
    @breadcrumbs_info
  end
  
  def prepare_albums(context, is_xhr)
    @title = t('albums.fototeca')
    if context
      @title << " #{t('shared.from_context', :name => context.public_name)}"
      if params[:page].eql?(1) || params[:page].nil?
        @albums = @context.albums.published.paginate(:page => params[:page], :per_page => 17, :order => 'created_at DESC')      
        @featured_album = @albums.shift
      else
        @albums = @context.albums.published.paginate(:page => params[:page], :per_page => 16, :order => 'created_at DESC')      
      end
    else
      @featured_album = Album.published.with_photos.featured.first || Album.published.with_photos.first
    end
  end
  
end
