#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Filters added to this controller apply to all controllers in the application.
# Likewise, all the methods added will be available for all controllers.
require 'efaber'
# require 'acts_as_taggable_on_steroids_will_paginate'
require 'form_submit_tag_helper'
require 'csv'
require 'in_place_with_auto_complete'
require 'couchrest'
require 'open-uri'
# require 'carrierwave_minimagick' 
# require 'cache_extension_fix'
class ApplicationController < ActionController::Base
  helper :all # include all helpers, all the time
  protect_from_forgery # See ActionController::RequestForgeryProtection for details
  
  include AuthenticatedSystem
  
  include ExceptionNotifiable
  
  include GeoKit::Geocoders
  
  # See ActionController::RequestForgeryProtection for details
  # Uncomment the :secret if you're not using the cookie session store
  # protect_from_forgery # :secret => 'd96cbc5f8680e007f4cbdac7ce00de99'
  
  # See ActionController::Base for details 
  # Uncomment this to filter the contents of submitted sensitive data parameters
  # from your application log (in this case, all fields with names like "password"). 
  # filter_parameter_logging :password
    
  # Idiomas disponibles a lo largo del site
  def locales 
    AvailableLocales::AVAILABLE_LANGUAGES
  end
  helper_method :locales

  before_filter :set_locale
  before_filter :set_current_user
  before_filter :adjust_format_for_iphone
  before_filter :get_areas
 
  private

  # HQ
  # def build_question
  #   @question = Question.new
  #   @question.area_id = @area.id if @area
  #   @question_data = @question.build_question_data
  #   @question_data.for_whom_id = @politician.id if @politician
  # end

  # Determina el idioma preferido del visitante de la web. Se determina, por este orden:
  # * Idioma especificado en la URL
  # * Idioma guardado en la cookie, elegido en la pantalla "splash" de elección de idioma
  # * Idioma por defecto de la aplicación: castellano
  # 
  # Se llama antes de cada petición a la web con un <tt>before_filter</tt>
  def set_locale
    @locale = params[:locale] || cookies['locale'] || I18n.default_locale.to_s
    unless AvailableLocales::AVAILABLE_LANGUAGES.keys.include?(@locale.to_sym)
      @locale = I18n.default_locale
    end
    cookies['locale'] = { :value => @locale , :expires => 1.year.from_now } if @locale != cookies['locale']
    I18n.locale = @locale.to_sym
  end
  
  # def default_url_options(options={})
  #     {:locale => @locale}
  # end
  
  # Da valor al usuario actual, para poder rellenar los campos <tt>updated_by</tt> y <tt>created_by</tt>
  # de todos los modelos observados por #UserActionObserver.
  # 
  # Se llama desde <tt>before_filter</tt>
  def set_current_user
    UserActionObserver.current_user = current_user.id if current_user.is_a?(User)
    cookies[:irekia_uuid] = {:value => UUIDTools::UUID.timestamp_create.to_s, :expires => 20.years.from_now } unless cookies[:irekia_uuid].present?
  end
  
  def base_url
    "#{request.protocol}#{request.host_with_port}"
  end
  helper_method :base_url
  
  # Determina el formato a devolver si la petición se hace desde un iphone
  def adjust_format_for_iphone
    if iphone_request?
      request.format = :iphone 
    elsif android_request?
      request.format = :android
    elsif floki_user_agent?
      request.format = :floki unless params[:format]
    end
  end
  
  # Indica si la petición se hace desde un iphone
  def iphone_request?
    return (request.subdomains.first == "i" || params[:format] == "iphone")
  end
  
  # Indica si la petición se hace desde un iphone
  def android_request?
    return (request.subdomains.first == "a" || params[:format] == "android")
  end

  def iphone_user_agent?
    request.env["HTTP_USER_AGENT"] && request.env["HTTP_USER_AGENT"][/iPhone/i]
  end
  helper_method :iphone_user_agent?

  def android_user_agent?
    request.env["HTTP_USER_AGENT"] && request.env["HTTP_USER_AGENT"][/Android/i]
  end
  helper_method :android_user_agent?

  def floki_user_agent?
    request.env["HTTP_USER_AGENT"] && request.env["HTTP_USER_AGENT"][/(Irekia|floki)/i]
  end  
  helper_method :floki_user_agent?

  def iphone_app_request?
    if cookies["DINFO"].present?
      width = cookies["DINFO"].split(/,/)[1].to_i
      floki_user_agent? && (width < 500)
    else  
      floki_user_agent? && !params[:d].present?
    end  
  end
  helper_method :iphone_app_request?

  def ipad_app_request?
    if cookies["DINFO"].present?
      width = cookies["DINFO"].split(/,/)[1].to_i
      floki_user_agent? && (width > 1000)
    else  
      floki_user_agent? && params[:d].present? && params[:d].to_i == 1
    end
  end
  helper_method :ipad_app_request?

  def iphone4_app_request?
    if cookies["DINFO"].present?
      dpi = cookies["DINFO"].split(/,/)[2].to_i
      floki_user_agent? && (dpi == 326)
    else  
      floki_user_agent? && params[:d].present? && params[:d].to_i == 2
    end
  end
  helper_method :iphone4_app_request?
  
  rescue_from ActiveRecord::RecordNotFound, :with => :bad_record
  # Devuelve la página de "página no encontrada" cuando no se encuentra alguna petición en
  # la base de datos. Se llama desde <tt>rescue_from</tt>
  def bad_record
    @notfound = true
    render :template => '/site/notfound.html', :layout => 'application.html', :status => 404
  end
  
  rescue_from ActionController::InvalidAuthenticityToken, :with => :invalid_token
  # Cuando un usuario hace logout en una pestaña y luego intenta rellenar un formulario en otra pestaña
  # le da error InvalidAuthenticityToken. Mejor decirle que su sesión ha caducado
  def invalid_token
    flash[:notice] = t('session.sesion_caducada')
    redirect_to new_session_path(:locale => I18n.locale, :return_to => request.env["HTTP_REFERER"])
  end
  
  rescue_from WillPaginate::InvalidPage, :with => :bad_record
  
  # HQ
  rescue_from ActiveRecord::SubclassNotFound, :with => :questions_disabled
  def questions_disabled
    unless exception.match(/The single-table inheritance mechanism failed to locate the subclass: 'Question'/)
      raise ActiveRecord::SubclassNotFound
    end
  end
    
  # Devuelve la lista de tags para un <tt>autocomplete</tt> de tags.
  def auto_complete_for_tag_list(search_string, beginning_of_string=true)
    exclusion_list = []
    @tags = []
    
    # "_" parece ser un simbolo reservado en los LIKE clauses, así que si buscamos tags ocultos, hay que escaparlo
    raw_param = search_string.tildes
    parsed_param = raw_param.match(/^_/) ? raw_param.sub(/^_/, '\\_'): raw_param

    find_pattern = parsed_param + '%'
    find_pattern = '%' + find_pattern unless beginning_of_string
    
    # Los tags hay que introducirlos en castellano y luego se traducen a otros idiomas
    conditions = ["tildes(name_es) ILIKE :tag_list", {:tag_list => find_pattern}]
    if exclusion_list.length>0
      conditions[0] << " AND id NOT IN (#{exclusion_list.join(', ')})" 
    end
    
    Tag.find(:all, :conditions => conditions).each do |t|
      t[:nombre] = t.name_es
      exclusion_list << t.id
      @tags << t
    end
    
    return @tags
  end
  
  # URL por defecto para cada usuario: corresponde a
  # - el enlace "Tú irekia" en la home para políticos, ciudadanos y periodistas, 
  # - el enlace "Administración" para los usuarios que tienen acceso a la admin,
  # - el redirect después de login 
  # - el link con el nombre del usuario en la barra de navegación (2DO)
  def default_url_for_user
    if logged_in?
      if current_user.is_admin?
        admin_path
      elsif current_user.is_a?(DepartmentMember) || current_user.is_a?(DepartmentEditor) || current_user.is_a?(StaffChief) || current_user.is_a?(StreamingOperator) || current_user.is_a?(Colaborator)
        admin_path
      elsif current_user.is_a?(StreamingOperator)  
        admin_stream_flows_path
      elsif current_user.is_a?(RoomManager)
        sadmin_account_path
      elsif current_user.is_a?(Politician)
        if current_user.has_admin_access?
          admin_path
        else
          politician_path(current_user)
        end
      else
        account_path  
      end
    else
      root_path
    end  
  end
  helper_method :default_url_for_user
    
  # Determina si el usuario actual tiene permiso para ver la administración de los 
  # contenidos de tipo <tt>doc_type</tt>.
  # <tt>doc_type</tt> puede tener los valores: news, events, videos, photos, pages, links, stream_flows

  def can_access?(doc_type)
    logged_in? && (current_user.can_access?(doc_type) || can?('access', doc_type))
  end
  helper_method :can_access?
  
  # Determina si el usuario actual tiene permiso para modificar los
  # contenidos de tipo <tt>doc_type</tt>. 
  def can_edit?(doc_type)
    logged_in? && (current_user.can_edit?(doc_type) || can?('edit', doc_type))
  end
  helper_method :can_edit?

  # Determina si el usuario actual tiene permiso para crear los
  # contenidos de tipo <tt>doc_type</tt>.   
  def can_create?(doc_type)
    logged_in? && (current_user.can_create?(doc_type) || can?('create', doc_type))
  end
  helper_method :can_create?
  
  # Determina si el usuario actual tiene permiso de tipo <tt>perm_type</tt> en los
  # contenidos de tipo <tt>doc_type</tt>. Ver Permission#can?
  def can?(perm_type, doc_type)
    logged_in? && current_user.can?(perm_type, doc_type)
  end
  helper_method :can?

  # Determina si el usuario actual tiene permiso para modificar el evento <tt>event</tt>. 
  def can_edit_event?(event)
    if current_user.is_a?(DepartmentMember) || current_user.is_a?(Politician)
      event.visible_in.collect {|subsite| current_user.can?("create_#{subsite}", "events")}.uniq != [false]
    else
      can_edit?("events")
    end
  end
  helper_method :can_edit_event?
  
  # Determina si el usuario puede ver las estadísticas de consultas y descragas de una noticia o un evento.
  def can_see_stats?
    logged_in? && (is_admin? || current_user.is_a?(DepartmentEditor))
  end
  helper_method :can_see_stats?
    
  # Devuelve los contenidos taggeados con las condiciones especificadas en <tt>conditions</tt>.
  # <tt>options</tt> son las opciones de paginación que se le pasa a WillPaginate 
  def paginated_collection_of_tags_for(conditions, options = {}, model="all")
    options.reverse_merge!({:page => 1, :per_page => 10})
    page, per_page, total = options[:page], options[:per_page]         
    pager = WillPaginate::Collection.new(page, per_page, total)
    options.merge!(:offset => pager.offset, :limit => per_page)

    # 
    # No puedo usar "paginate" directamente porque si uso "distinct on" para que no me 
    # aparezca la misma noticia varias veces, no puedo ordenar por fecha, asi que primero cojo todos, 
    # los ordeno por fecha, y luego lo pagino. 
    
    #
    if model.eql?("all")
      logger.info "AAAAAAAAAA #{conditions}"
      result = Tagging.find :all, 
        :select => "distinct ON (taggable_id) taggings.*, coalesce(documents.published_at, videos.published_at, 
                    contributions.published_at, photos.created_at, '2009-03-01') AS pub_at",
        :joins => "LEFT OUTER JOIN documents ON (documents.id=taggings.taggable_id AND taggable_type='Document' AND published_at IS NOT NULL)
                   LEFT OUTER JOIN contributions ON (contributions.id=taggings.taggable_id AND taggable_type='Contribution')
                   LEFT OUTER JOIN photos ON (photos.id=taggings.taggable_id AND taggable_type='Photo')
                   LEFT OUTER JOIN videos ON (videos.id=taggings.taggable_id AND taggable_type='Video')", 
        :conditions => conditions + " AND (CASE WHEN taggable_type IN ('Document', 'Video', 'Contribution') THEN COALESCE(documents.published_at, videos.published_at, contributions.published_at) IS NOT NULL
                                           ELSE true END)
          AND (CASE WHEN taggable_type IN ('Document', 'Contribution', 'Video') 
               THEN COALESCE(documents.published_at, contributions.published_at, videos.published_at) <= '#{Time.now.to_s(:db)}' 
               ELSE true END)
          AND (CASE WHEN taggable_type = 'Photo' THEN exists (select 1 from album_photos where album_photos.photo_id=photos.id) ELSE true END)"
        # :order => "pub_at desc"
    else
      publication_conditions = " AND published_at IS NOT NULL AND published_at <= '#{Time.now.to_s(:db)}'"
      case 
      when document_subclasses.include?(model.constantize)
        result = Tagging.find :all,
          :select => "distinct ON (taggable_id) taggings.*, coalesce(documents.published_at, '2009-03-01') AS pub_at",
          :joins => "INNER JOIN documents ON (documents.id=taggings.taggable_id AND taggable_type='Document' AND published_at IS NOT NULL)",
          :conditions => conditions + publication_conditions + " AND documents.type='#{model}'"
      when model.eql?("Contribution")
        result = Tagging.find :all,
          :select => "distinct ON (taggable_id) taggings.*, coalesce(contributions.published_at, '2009-03-01') AS pub_at",
          :joins => "INNER JOIN contributions ON (contributions.id=taggings.taggable_id AND taggable_type='Contribution')",
          :conditions => conditions + publication_conditions 
      when model.eql?("Video")
        result = Tagging.find :all,
          :select => "distinct ON (taggable_id) taggings.*, coalesce(videos.published_at, '2009-03-01') AS pub_at",
          :joins => "INNER JOIN videos ON (videos.id=taggings.taggable_id AND taggable_type='Video')",
          :conditions => conditions + publication_conditions 
      when model.eql?("Photo")
        result = Tagging.find :all,
          :select => "distinct ON (taggable_id) taggings.*, coalesce(photos.created_at, '2009-03-01') AS pub_at",
          :joins => "INNER JOIN photos ON (photos.id=taggings.taggable_id AND taggable_type='Photo')",
          :conditions => conditions + " AND exists (select 1 from album_photos where album_photos.photo_id=photos.id)"
      end
    end

    
    res = result.sort {|a,b| Time.parse(b.pub_at) <=> Time.parse(a.pub_at)}    
    # Esti
    # returning WillPaginate::Collection.new(page, per_page, result.length) do |pager|
    WillPaginate::Collection.new(page, per_page, result.length).tap do |pager|
      start_index = (page.to_i - 1) * per_page.to_i
      end_index = (page.to_i * per_page.to_i)-1
      pager.replace res[start_index..end_index] unless start_index >= result.length
    end    
  end
  
  # Devuelve la ruta a la versión de tamaño <tt>size</tt> de la imagen <tt>path</tt>
  def photo_path(path, size="original")
    path = Pathname.new(path)
    return "#{path.dirname}/#{size}/#{path.basename}"
  end
  helper_method :photo_path  

  
  def taggable_modules
    [News, Event, Page, Video, Photo, Proposal, Headline]
  end
  helper_method :taggable_modules
  
  def document_subclasses
    [News, Event, Page]
  end
  helper_method :document_subclasses
  
  # Irekia3
  def get_areas
    @areas = Area.ordered
  end
  
  def get_followings
    item = @politician.present? ? @politician : @area
    if logged_in? && current_user.follows?(item)
      @following = Following.find(:first, :conditions => {:followed_id => item.id, :user_id => current_user.id, :followed_type => item.class.name})
    else
      @following = Following.new
    end
    @following
  end
  helper_method :get_followings

  # HQ
  # def get_questions
  #   per_page = request.xhr? ? 5 : 20
  #   order = params[:more_polemic] ? "comments_count DESC" : "published_at DESC"
  # 
  #   if @context
  #     if params[:answered]
  #       @questions = @context.approved_and_published_questions.answered.paginate :order => order,
  #         :per_page => per_page, :page => params[:page]
  #     else
  #       @questions = @context.approved_and_published_questions.paginate :order => order,
  #         :per_page => per_page, :page => params[:page]
  #     end
  #   else
  #     if params[:answered]
  #       @questions = Question.published.approved.answered.paginate :order => order,
  #         :per_page => per_page, :page => params[:page]
  #     else
  #       @questions = Question.published.approved.paginate :order => order,
  #         :per_page => per_page, :page => params[:page]
  #     end
  #   end
  # end
  
  
  def get_selected_date
    @selected_date = {}
    @today = Time.zone.now
    [:year, :month].each do |key|
      @selected_date[key] = (params[key].to_i > 0) ? params[key].to_i : @today.send(key.to_s)
    end
    # Si no está indicada la fecha, cogemos el día 1, para evitar problemas con los enlaces
    # mes siguiente - mes anterior.
    # Como efecto secundario, si llamamos a la acción list sin argumentos nos devolverá 
    # la lista de eventos del día 1 del mes actual.
    @selected_date[:day] = (params[:day].to_i > 0) ? params[:day].to_i : 1

    @today = Time.zone.now
    @year = @selected_date[:year]
    @month = @selected_date[:month]
    @day = @selected_date[:day]    
    
    @date = Date.new(@year, @month, @day)
  end
  
  def get_area
    @area = Area.find(params[:area_id]) if params[:area_id]
  end
  
  def get_politician
    @politician = Politician.find(params[:politician_id]) if params[:politician_id]
  end
  

  # Las acciones pueden pertenecer a un área, un político, un ciudadano 
  # o podemos coger las últimas acciones en general (para la home)
  def get_context
    if params[:area_id] && Area.exists?(params[:area_id]) 
      @context = Area.find(params[:area_id])
      @area = @context
      @breadcrumbs_info = [[t('organizations.title'), areas_path]]
      @breadcrumbs_info << [@area.name,  area_path(@area)]
    elsif params[:politician_id] && params[:politician_id].to_i != 0 && Politician.exists?(params[:politician_id]) 
      @context = Politician.find(params[:politician_id])
      # HQ
      # @question = Question.new # Cuando se muestra un político siempre sale el botón "Haz una pregunta"
      @politician = @context
      @breadcrumbs_info = [[Politician.human_name(:count => 2).capitalize, politicians_path]]
      @breadcrumbs_info << [@politician.public_name,  politician_path(@politician)]
    elsif params[:user_id] && User.exists?(params[:user_id]) 
      @context = User.find(params[:user_id])
      @user = @context
      @breadcrumbs_info = [[@user.public_name, user_path(@user)]]
    end
  end
  
  def context_type
    case @context.class.name.to_s.downcase
    when 'politician'
      @context.class.name.to_s.downcase
    when 'person'
      'user'
    when 'area'
      @context.class.name.to_s.downcase
    else
      'user'
    end
  end
  
  # Devuelve la lista de acciones dentro de un context: area, político o todas (ciudadano?)
  def get_context_actions(context=nil)
    if context.present?
      news_finder = context.news.listable
      events_finder = context.events
      proposals_finder = context.approved_and_published_proposals
      # HQ
      # questions_finder = context.approved_and_published_questions
      # answers_finder = context.answers.present? ? context.answers.published : nil
      comments_finder = context.approved_comments
      arguments_finder = context.approved_arguments   
    else
      news_finder = News.published.translated.listable
      events_finder = Event.published.translated
      proposals_finder = Proposal.approved.published
      # HQ
      # questions_finder = Question.approved.published
      # answers_finder = Answer.published
      comments_finder = Comment.approved
      arguments_finder = Argument.published.for_proposals
    end
    
    actions = []
    actions << news_finder.find(:all, 
      :select => "documents.id, title_es, title_eu, title_en, body_#{I18n.locale}, published_at, has_comments, 
                  comments_closed, multimedia_path, cover_photo_file_name", 
      :order => 'published_at DESC', :limit => 20) if news_finder
      
    actions << events_finder.find(:all, 
        :select => "documents.id, title_es, title_eu, title_en, body_#{I18n.locale}, published_at, starts_at, ends_at, has_comments, comments_closed", 
        :order => 'published_at DESC', :limit => 20) if events_finder
     
    actions << proposals_finder.find(:all, 
      :select => "contributions.id, title_es, title_eu, title_en, body_#{I18n.locale}, published_at, status, user_id, has_comments, comments_closed",
      :order => 'published_at DESC', :limit => 20)

    # HQ
    # actions << questions_finder.find(:all, 
    #   :select => "contributions.id, title_es, title_eu, title_en, body_#{I18n.locale}, published_at, status, user_id, has_comments, comments_closed", 
    #   :order => 'published_at DESC', :limit => 20)
    #   
    # actions << answers_finder.find(:all,
    #   :select => "answers.id, answered_at, answers.question_id, answered_by, body, answers.created_at, youtube_url, vimeo_url",
    #   :order => 'answered_at DESC', :limit => 20) if answers_finder
    
    actions << comments_finder.find(:all, 
      :select => "comments.id, commentable_id, commentable_type, user_id, body, created_at, status, name, email",
      :order => "created_at DESC", :limit => 20) if comments_finder
    
    actions << arguments_finder.find(:all, 
      :select => "argumentable_type, argumentable_id, value, reason, published_at, user_id",
      :order => "created_at DESC", :limit => 20) if arguments_finder  
      
    actions = actions.flatten.sort {|a,b| b.published_at <=> a.published_at}[0..19].map {|a| a.is_a?(Event) ? a.reload : a}
      
    actions
  end

  def prepare_news(context, is_xhr)
    # No quieren que se vean las noticias ocultas (acuerdos de consejo y tres compromisos)
    # Ahora /news y /transparency son lo mismo, así que las noticias ocultas sólo se ven a través de link de 
    # la crónica de acuerdos de consejo, siguiendo un tag o la búsqueda
    
    if context
      @news = @context.news.listable.paginate(:page => params[:page], :per_page => 15, :order => 'published_at DESC')
      @title = "#{t('news.title')} #{t('shared.from_context', :name => context.public_name)}"
      if is_xhr
      else
        @leading_news = @context.featured_news.find(:first, :order => "documents.published_at DESC")
        if @leading_news
          @other_news = @news - [@leading_news]
        else
          @leading_news, @other_news = @news[0], @news[1..-1]
        end
      end
    else
      @title = t('news.leading_news')
      @news = News.published.translated.listable.paginate(:page => params[:page], :per_page => 15, :order => 'published_at DESC')
      if is_xhr
      else
        get_month_events
        @headlines = Headline.published.translated.find(:all, :limit => 12)
        @streaming = Streaming.new
        if params[:page].present?
          @other_news = @news
        else
          @leading_news = News.featured_a
          @secondary_news = News.featured_4b
          @other_news = (@news - [@leading_news] - @secondary_news)[0..15]
        end
      end
    end
  end

  def get_news_videos_and_photos(news)
    videos = news.videos
    videos_list = videos[:list][I18n.locale.to_sym].present? ? videos[:list][I18n.locale.to_sym].sort : []
    @videos = ([news.featured_video] + videos_list).compact
    
    @videos_mpg = news.videos_mpg #videos[:mpg][I18n.locale.to_sym]

    # Mostramos todas las fotos del directorio multimedia de la noticia
    @photos = news.photos
    if news.cover_photo?
      @photos.delete_if {|ph| File.basename(ph).eql?(news.cover_photo_file_name)}
      @photos.insert(0, news.cover_photo)
    end
  end

  def get_month_events
    get_selected_date if @day.nil? && @month.nil? && @year.nil?
    @events = Event.published.translated.find(:all, :conditions =>  [ "(ends_at > :beginning_of_day)", {:beginning_of_day => Time.now.beginning_of_day}], :order => "starts_at", :limit => 15)
    if @context
      @month_events = @context.events.month_events_by_day4cal(@month, @year)
    else
      @month_events = Event.published.translated.month_events_by_day4cal(@month, @year)
    end
  end

  # Construye los <tt>breadcrumbs</tt> o "migas de pan" que indican la ruta para llegar hasta 
  # cada página de la web.
  # 
  # Se llama antes del <tt>render</tt> de cada página
  def make_breadcrumbs
    @breadcrumbs_info = [] unless @breadcrumbs_info
  end
  
  # Queremos saber en qué noticias relacionadas pinchan los usuarios
  def track_clickthrough
    if params[:t] && params[:t].to_i == 1 && request.env['HTTP_REFERER']
      begin 
        referer = request.env['HTTP_REFERER'].gsub(/^#{request.protocol}#{request.host_with_port}/, '')
        dummy, referer_path, dummy2, referer_querystring = get_params_for(referer)
        referer_params = ActionController::Routing::Routes.recognize_path(referer_path, :method => :get)
      
        dummy, current_page_path, dummy2, current_page_querystring = get_params_for(request.request_uri)
        current_page_params = ActionController::Routing::Routes.recognize_path(current_page_path, :method => :get)
        
        if ['search', 'orders'].include?(referer_params[:controller])
          source_type = referer_params[:controller].singularize.camelize
        else
          source_type = 'Document'
        end
        
        target_type=current_page_params[:controller].singularize.camelize
        
        source_subtype = referer_params[:controller].singularize.camelize
        target_subtype = current_page_params[:controller].singularize.camelize
        
        source_id = source_type.eql?('Order') ? referer_params[:no_orden] : referer_params[:id]
        target_id = target_type.eql?('Order') ? current_page_params[:no_orden] : current_page_params[:id]
        Clickthrough.create! :source_type => source_type, :source_id => source_id,
          :target_type => target_type, :target_id => target_id,
          :locale => I18n.locale, :user_id => (logged_in? ? current_user.id : nil), 
          :uuid => cookies[:irekia_uuid], :source_subtype => source_subtype, :target_subtype => target_subtype
      rescue => err
        logger.error "No he podido guardar el clickthrough de #{referer} a #{request.request_uri}: #{err.inspect}"
      end
    end
  end
  
  def resource_for(controller)
    document.is_a?(News) ? "documents" : document.class.to_s.downcase.pluralize
  end

  def get_params_for(url)
    dummy, path, dummy2, querystring = url.match(/([^\?]+)(\?(.+))*/).to_a
  end        
   
  # get criterio to highlight matching keywords in news, order, etc.
  #  used in news, order controllers before show action
  def get_criterio
    @criterio = Criterio.find(params[:criterio_id].to_i) if params[:criterio_id].present?    
  end 
  
  def set_query
    q=[]
    filters=[]         
    @criterio.title.split(' AND ').each do |member|
      if member[/keyword: /]    
        if @criterio.only_title
          q.push(member.gsub('keyword: ', '').to_s.tildes)
        else
          q.push(member.gsub('keyword: ', '').to_s.escape_for_elasticsearch.tildes)          
        end    
      elsif member[/tags: |organization: |areas: |politicians: |seccion: |rango: |organo: |materias: /]
        key=member[/tags: |organization: |areas: |politicians: |seccion: |rango: |organo: |materias: /]
        filters.push(["#{key.split(':').first}_an", member.split(key).last])
      elsif member[/type: /]  
        filters.push(['_type', member.split('type: ').last])
      elsif member[/date: |year: |month: |term: /]
        key=member[/date: |year: |month: |term: /]  
        filters.push([key.split(':').first, member.split(key).last])  
      else
        q.push(member)
      end  
    end                                 
    @q=q.join(' ')
    @q = '*' if @q.empty?
    @filters=filters
  end
  
  def do_elasticsearch_and_save_results(save_criterio = true, only_title = false)
    # Results          
    all_results=Elasticsearch::Base::do_elasticsearch_with_facets(@q, @filters, @from, @sort, only_title)
    return false if all_results.empty?  # elasticsearch not available

    @search_results=all_results['hits']  
    @total_hits=all_results['total_hits']
    
    # Facets
    @search_facets=Hash.new
    Elasticsearch::Base::FACETS.keys.each do |facet_type|
      exp = @criterio.title.match("(#{facet_type}: (.*))( AND|$)")
      if exp.present? && exp[2].present?                                                  
        @search_facets[facet_type] = all_results['facets'][facet_type].delete_if{|a,b| "#{facet_type}: #{exp[2]}".match(Regexp.escape(a))} 
      else  
        @search_facets[facet_type] = all_results['facets'][facet_type]
      end  
    end
    update_criterio if save_criterio    
    return true
  end
  
  def update_criterio
    @criterio.results_count=@total_hits
    if @criterio.save
      session[:criterio_id]=@criterio.id
      @criterio.reload
    else
      @criterio.destroy
    end
    @criterios=@criterio.ancestors.reverse + [@criterio]    
  end      
  
  def elasticsearch_not_available_redirect_url
    flash[:error]=t('search.servidor_no_disponible')
    if request.referer.present?
      request.referer 
    elsif !(params[:controller].eql?('search') && params[:action].eql?('new'))
      new_search_path
    else
      root_path
    end    
  end
  
  def set_sort_order
    if params[:sort].present? && params[:sort].eql?('date')
      @sort='date'        
    end
  end
  
  def set_page
    if params[:page]
      @page=params[:page]
      @from=(params[:page].to_i-1)*Elasticsearch::Base::ITEMS_PER_PAGE
    else  
      @page=1
      @from=0
    end
  end                  
  
  def category_name_and_url(category)
    if match = category.name.match(/\"(.+)\":(.+$)/)
      if match[2].match("http://")
       [match[1], match[2], {:rel => 'external'}]   
      else
        [match[1], match[2], {}]
      end
    else
      if category.name.match(/\+$/)
        [category.pretty_name, '#', {}]
      else
        [category.pretty_name, category_path(category), {}]
      end
    end
  end
  helper_method :category_name_and_url

  # HQ
  # def can_answer_question?(question)
  #   return false if question.answered?
  #   
  #   # Puede editar preguntas
  #   has_permission = can_edit?("questions")
  #   return true if has_permission
  #   
  #   # o va dirigida a él
  #   if question.for_whom 
  #     has_permission = has_permission || current_user == question.for_whom
  #   end
  #   return true if has_permission
  #   
  #   # o es comentador oficial del departamento al que va dirigida
  #   has_permission = has_permission || (logged_in? && current_user.has_department? && current_user.department.official_commenters.include?(current_user))
  #   return has_permission
  # end  
  # helper_method :can_answer_question?

  def string_to_time(val)
    val.is_a?(String) ? Time.parse(val) : val
  end
  helper_method :string_to_time

  # Filtro que comprueba que el usuario está logeado y está aprobado.
  # Se usa en los controllers de sadmin y admin para evitar el acceso de usuarios vetados 
  # que no han cerrado la sesión
  def approved_user_required
    unless (logged_in? && current_user.approved?)
      flash[:notice] = t('no_tienes_permiso')
      access_denied
    end
  end
  
  protected
    # Inspired by:
    # http://www.aldenta.com/2006/09/25/execute-rails-code-before-the-view-is-rendered/
    # Redefinición del método render para que se construyan antes los <tt>breadcrumbs</tt>
    def render(options = nil, extra_options = {}, &block) 
      # we want to build the breadcrums just before rendering
      make_breadcrumbs

      # call the ActionController::Base render to show the page
      super
    end
    
  
end
