#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# 
# Controlador para los eventos públicos
#
class EventsController < ApplicationController
  before_filter :get_context, :only => [:index]
  before_filter :check_politician_has_agenda, :only => [:index]
  before_filter :get_category
  before_filter :get_selected_date, :only => [:index, :summary]    
  before_filter :get_criterio, :only => [:show]  
  after_filter  :track_clickthrough, :only => [:show]

  def index
    prepare_events(@context, request.xhr?)

    respond_to do |format|
      format.html do
        if request.xhr?
          render :partial => '/shared/list_items', :locals => {:items => @actions, :type => 'events'}, :layout => false
        else
          render
        end
      end
      format.ics { render :layout => false }
    end
  end
  
  def summary
    get_month_events
    render :layout => !request.xhr?
  end
  
  def list 
    redirect_to params.merge(:action => 'index')
  end
  
  # Ver los datos públicos de un evento publicado.
  def show
    @title = t('events.agenda')
    begin
      @event = Event.published.find(params[:id])
    rescue ActiveRecord::RecordNotFound
      if can_edit?("events")
        @event = Event.find(params[:id])
      else
        #raise ActiveRecord::RecordNotFound
        bad_record
        return
      end
    end
    
    @comments = @event.comments.approved.paginate :page => params[:page], :per_page => 25
    
    if @category
      # Comprobar que la categoria y el documento estan relacionados
      if @event.tags.private.collect(&:name_es) & @category.tags.private.collect(&:name_es) == []
        logger.info "Categoria y documento no relacionados"
        raise ActiveRecord::RecordNotFound
      end
    end
  
    respond_to do |format|
      format.html { render }
      format.ics { render :layout => false}
      format.xml
      format.iphone 
      format.android
      format.floki
    end
  
  end
  
  # RSS de los eventos públicos
  def myfeed
    @title = t('events.agenda')
    cj = get_list_conditions()
    respond_to do |format|
      format.ics {
        @events = Event.published.translated.future(1.month.ago).find(:all, :conditions => cj[:conditions], :joins => cj[:joins])
        render :layout => false
      }
    end        
  end
  
  private 
  
  # Los eventos suelen ir debajo de alguna categoría del menú.
  # Aquí se coge la categoría para construir los breadcrumbs.
  def get_category
    @category = Category.find(params[:category_id]) if params[:category_id]
  end
  
  # Construye los breadcrumbs de cada acción de las noticias
  def make_breadcrumbs
    @date ||= Date.today
    if @category
      @category.ancestors.reverse.each {|a| @breadcrumbs_info << [a.name, category_path(a)]}
      @breadcrumbs_info = [[@category.name, category_path(@category)]]
      if @event && !@event.new_record?
        @breadcrumbs_info << [@event.title, category_event_path(@category, @event)]
      end
    elsif @context.present?
      @breadcrumbs_info << [t('events.agenda'), send("#{@context.class.name.to_s.downcase}_events_path", @context)]
    else
      @breadcrumbs_info = [[t('events.agenda'), events_path]]
      if @event && !@event.new_record?
        @breadcrumbs_info << [@event.title,  event_path(@event)]
      end
    end
    @breadcrumbs_info
  end
  
   def get_list_conditions
     @subscription_title = "Irekia::#{t('events.agenda')}"
     
     conditions = {}
     joins = nil
     
     @tag_label = params[:tag_label]
     if @tag_label && Event.show_in_opts.map {|o| "_irekia_#{o}"}.include?(@tag_label)
       if @itag = Tag.find_by_sanitized_name_es(@tag_label)
         conditions['taggings.tag_id'] = @itag.id
         joins = [:taggings]
         @subscription_title = "#{@itag.name}"
       end
     end
     
     @dept_label = params[:dept_label]  || 'all'
     if params[:dept_label]
       tag_name = "_#{params[:dept_label]}".gsub(/[^\w_]/,'')
       if Department.tag_names.include?(tag_name)
         @dept = Department.find_by_tag_name(tag_name)
         conditions["documents.organization_id"] = [@dept.id] + @dept.organization_ids
         @title << ": #{@dept.name}"
         @subscription_title << ": #{@dept.name}"         
       end
     end
       
     {:conditions => conditions, :joins => joins}     
   end


   def prepare_events(context, is_xhr)
     if @context.present?
       events_finder = @context.events
       if params[:year].present? 
         @title = "#{Event.human_name(:count => 2).capitalize} #{@context.public_name} #{l(@date, :format => :long)}"
       else
         @title = @context.is_a?(Politician) ? t('politicians.agenda', :name => @context.public_name) : t('areas.agenda', :name => @context.name)      
       end        
     else
       events_finder = Event.published.translated
       if params[:year].present?
         @title = "#{Event.human_name(:count => 2).capitalize} #{l(@date, :format => :long)}"
       else
         @title = t('events.comming_events')
       end
     end

     if params[:year]
       if request.xhr?
         @actions = events_finder.day_events(@day, @month, @year)
       else
         @actions = events_finder.day_events(@day, @month, @year).paginate
         get_month_events
       end
     else
       # Los eventos de hoy más los próximos futuros, hasta 15.
       @actions = events_finder.find(:all, :conditions =>  [ "(ends_at > :beginning_of_day)", {:beginning_of_day => Time.now.beginning_of_day}], :order => "starts_at", :limit => 15).paginate
       if @actions.blank?
         @actions = events_finder.find(:all, :order => "starts_at DESC", :limit => 15).paginate
       end
       get_month_events
     end
   end
   
   def check_politician_has_agenda
    if @context.is_a?(Politician) && !@context.politician_has_agenda?
      flash[:notice] = t('politicians.no_tiene_agenda_publica')
      redirect_to request.env["HTTP_REFERER"] ? :back : politician_path(:id => @context, :anchor => 'top') and return
    end
   end
end
