#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para las propuestas ciudadanas
class ProposalsController < ApplicationController
  before_filter :login_required, :only => [:new, :create]
  before_filter :get_context, :only => [:index]
  before_filter :get_criterio, :only => [:show]
  after_filter :track_clickthrough, :only => [:show]
  
  before_filter :transform_floki_params, :only => :create
  
  # Listado de propuestas
  def index
    get_proposals
    
    respond_to do |format|
      format.html do
        if request.xhr?
          render :partial => '/shared/list_items', :locals => {:items => @proposals, :type => 'proposals'}, :layout => false
        else
          render
        end
      end
      format.floki {render :template => "mob_app/proposals.json", :layout => false, :content_type => "application/json"}
      format.rss do
        @feed_title = t('proposals.feed_title', :name => @context ? @context.name : t('site_name'))
        render :layout => false        
      end
    end
  end

  def summary
    @proposals = Proposal.approved.published.find(:all, :order => "published_at DESC", :limit => 2)
    @debates = Debate.published.translated.find(:all, :order => "published_at DESC", :limit => 2)
    render :layout => !request.xhr?
  end

  # Vista de una propuesta
  def show
    begin
      @proposal = Proposal.approved.published.find(params[:id])
    rescue ActiveRecord::RecordNotFound
      @proposal = Proposal.find_by_id(params[:id].to_i)
      if @proposal.present? && !(logged_in? && (is_admin? || @proposal.user_id == current_user.id))
        raise ActiveRecord::RecordNotFound and return
      elsif @proposal.nil?
        @debate = Debate.published.find(params[:id])
        redirect_to debate_path(@debate) and return
      end
    end
    
    @parent = @proposal
    
    @comments = @proposal.comments.approved.paginate :page => params[:page], :per_page => 25
    @arguments = @proposal.arguments.published.paginate :page => params[:page], :per_page => 50
    
    if params[:n_id].present?
      notification = current_user.notifications.find(params[:n_id])
      notification.read_at = Time.zone.now if notification.read_at.nil?
      notification.save
    end

    respond_to do |format|
      format.html { render }
      format.floki { render }
    end
  end
  
  # Formulario de nueva propuesta ciudadana
  def new
    @proposal = Proposal.new(:area_id => params[:area_id])
    respond_to do |format|
      format.html
      format.floki { render :action => 'new.floki', :content_type => 'application/json', :layout => false}
    end
  end

  # Creación de nueva propuesta ciudadana
  def create
    @proposal = current_user.proposals.new(params[:proposal])
    # @proposal.organization_id = Organization.find_by_tag_name("_gobierno_vasco") if @proposal.organization_id.blank?
    if @proposal.save
      email = Notifier.create_new_proposal(@proposal)
      Notifier.deliver(email)
      
      respond_to do |format|
        format.html do
          if request.xhr?
            render :action => "create", :layout => false
          else
            if @proposal.approved?
              flash[:notice] = t('proposals.gracias')
            else
              flash[:notice] = t('proposals.gracias_revisaremos')
            end
            redirect_to proposals_path(:s => "f")
          end
        end
        format.json do
          render :json => {:sequence_finished => true, 
                           :success_message => @proposal.approved? ? t('proposals.gracias') : t('proposals.gracias_revisaremos'), 
                           :refresh_controller => true}.to_json
          
        end
      end
    else
      respond_to do |format|
        format.html {render :action => "new"}
        format.json {render :json => {:error_message => @proposal.errors.inject('') {|messages, err| messages += err[1] + ". "},
          :needs_auth => false}.to_json}
      end
    end
  end
  
  def image
    uploader = ImageUploader.cache_from_io!(request.body, params.delete(:qqfile))
    @image = {
      :success => true,
      :image_cache_name => uploader.cache_name
    }
    render :text => @image.to_json and return
  end

  def department
    @department = Department.find(params[:id])
    @feed_title = t('proposals.feed_title', :name => @department.name)
    organization_ids = [@department.id] + @department.organization_ids
    @proposals = Proposals.approved_and_published_proposals.find :all, 
      :conditions => "organization_id in (#{organization_ids.join(',')})",
      :order => 'published_at DESC', :limit => 10
    render :action => "index.rss", :layout => false
  end

  private
  def transform_floki_params
    if params[:format].eql?('json')
      params[:proposal] = JSON.parse(params['data'])
    end
  end
  
  def make_breadcrumbs
    if @context.present?
      # context_type = @context.class.name.to_s.downcase.eql?('person') ? 'user' : @context.class.name.to_s.downcase
      @breadcrumbs_info << [t('proposals.title'),  send("#{context_type}_proposals_path", @context)]
    else
      @breadcrumbs_info = [[t('proposals.title'), proposals_path]]
      if @proposal.present? && @proposal.new_record?
        @breadcrumbs_info << [t('proposals.nueva_propuesta'), new_proposal_path]
      elsif @proposal.present? && !@proposal.new_record?
        @breadcrumbs_info << [@proposal.title, proposal_path(:id => @proposal)]
      end
      
    end    
  end

  def get_proposals    
    order = params[:more_polemic] ? "comments_count DESC" : "contributions.published_at DESC"
    finder_params = {:order => order, :per_page => 20, :page => params[:page]}
    
    if @context
      @proposals = @context.approved_and_published_proposals.paginate finder_params # Propuestas que ha hecho
      @title = @context.is_a?(Area) ? t('proposals.propuestas_a', :name => @area.name) : t("proposals.politician_proposals.#{am_I?(@politician) ? 'yours' : 'others'}")
    else
      @title = t('proposals.title')
      @proposals = Proposal.approved.published.paginate finder_params
    end
  end
  
end
