#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

#
# Controlador para el acceso a las agendas desde la parte de administración de la web.
#
# Sólo los usuarios que tienen permiso para ver las agendas pueden acceder a este controlador.
#
class Sadmin::EventsController < Sadmin::BaseController
  before_filter :access_to_events_required, :except => [:myfeed]
  before_filter :access_to_schedules_required, :only => [:select_schedule, :change_schedule]

  before_filter :get_show_agenda, :only => [:index, :calendar, :list, :week]
  before_filter :get_title, :only => [:index, :calendar, :list, :week]
  
  before_filter :get_calendar_events, :only => [:index, :calendar]

  before_filter :get_event, :only => [:show, :edit, :update, :select_schedule, :change_schedule, :toggle_l_attends, :unrelate, :set_event_related_news_title]   
  before_filter :edit_permission_required, :except => [:index, :calendar, :list, :week, :myfeed, :show]
  before_filter :get_selected_date, :only => [:list, :new, :week, :week2]

  cache_sweeper :event_sweeper, :only => [ :update, :delete, :change_schedule_to]
  
  auto_complete_for :event, :place
  auto_complete_for :event, :related_news_title
  # in_place_edit_for :event, :related_news_title
  
  skip_before_filter :verify_authenticity_token, :only => [:auto_complete_for_event_place, :auto_complete_for_event_related_news_title, :auto_complete_for_event_politicians_tag_list]
  
  uses_tiny_mce :only => [:new, :create, :edit, :update], :options => Admin::BaseController::TINYMCE_OPTIONS
  
  # Calendario de los eventos del mes actual.
  def index
    respond_to do |format|
      format.html {
        render
      }
      format.ics {
        @events = Event.find(:all, :conditions => ["starts_at >= ?", 3.months.ago])
        render :layout => false
      }
    end
  end
  
  # RSS de todos los eventos
  def myfeed
    if params[:u] && params[:p]
      user = User.authenticate_from_url(params[:u], params[:p])
    end
    
    if user && user.can_access?("events")
      respond_to do |format|
        format.ics {
          # Eventos públicos o privados que ven todos los usuarios con acceso a la agenda.
          @events = Event.find(:all, :conditions => ["starts_at >= ?", 3.months.ago])
          # Eventos de las agendas privadas de los departamentos a los que el usuario tiene acceso.
          user.schedules.each do |s|
            @events += s.events.find(:all, :conditions => ["starts_at >= ?", 3.months.ago])
          end
          render :layout => false
        }
      end        
    else
      flash[:notice] = t('no_tienes_permiso')
      redirect_to "index"
    end    
  end

  # Lista de los eventos de un día.
  def list
    for_date =  Date.new(@selected_date[:year], @selected_date[:month], @selected_date[:day])
    
    conditions = ["(starts_at <= :end_hour) and (ends_at >= :beginning)", {:beginning => for_date.beginning_of_day, :end_hour => for_date.end_of_day}]
    
    if session[:l_attends] && current_user.can_see_l_schedule?
      conditions[0] += " AND l_attends='t'"
    end
    
    if @show_agenda.include?('agenda')
      @documents = event_finder.find(:all, :conditions => conditions, :order => "starts_at")
    else
      @documents = []
    end
    @documents += get_schedule_events(conditions)
    @documents.sort! {|a,b| a.sort_position <=> b.sort_position}
    
    
    # @title = t("documents.#{@t.titleize}")
    @subtitle = I18n.localize(for_date, :format => :long)
    @prev_day = for_date - 1.day
    @next_day = for_date + 1.day    
  
    # If there are no events, redirect to the create event form.  
    redirect_to new_sadmin_event_path(@selected_date) if @documents.empty?  && !session[:l_attends] && can_create?("events")
  end
  
  # Lista de los eventos en una semana.
  def week
    for_date =  Date.new(@selected_date[:year], @selected_date[:month], @selected_date[:day])
    
    @first_day_of_week = for_date.beginning_of_week
    
    conditions = ["(starts_at <= :end_hour) and (ends_at >= :beginning)", {:beginning => for_date.beginning_of_week.to_time, :end_hour => for_date.end_of_week.to_time.end_of_day}]
    
    if session[:l_attends] && current_user.can_see_l_schedule?
      conditions[0] += " AND l_attends='t'"
    end
    
    if @show_agenda.include?('agenda')
      @events = event_finder.find(:all, :conditions => conditions, :order => "starts_at")
    else
      @events = []
    end
    @events += get_schedule_events(conditions)
    
    #@title = t("documents.#{@t.titleize}")
    @subtitle = "#{I18n.localize(for_date.beginning_of_week, :format => :long)} - #{I18n.localize(for_date.end_of_week, :format => :long)}"
  end
  
  # Vista alternativa para los eventos de una semana.
  def week2
    week
  end

  # Formulario para crear un nuevo evento.
  def new
    @title = t('sadmin.create_what', :what => Event.human_name)
    
    begin
      default_date = Date.parse("#{@year}-#{@month}-#{@day}") 
    rescue ArgumentError
      default_date = Time.now
    end
    
    event_params =  { :starts_at => default_date, :ends_at => default_date, 
                      :organization_id => current_user.department_id,
                      :has_journalists => false, :has_photographers => false}
                        
    @event = can_create?('events') ? Event.new(event_params) : ScheduleEvent.new(event_params.merge(:schedule_id => current_user.editable_schedules.first.id))
  end
    
  # Crear un evento nuevo. 
  #
  # Si los datos son correctos el evento se crea y se muestra la página del evento nuevo.
  # Si algún dato no es válido, se muestra de nuevo el formulario con explicación 
  # sobre los datos que no son válidos.
  def create
    @title = t('sadmin.create_what', :what => Event.human_name)
    
    schedule_id = params[:event][:schedule_id].to_i
    if params[:event][:schedule_id] && Schedule.exists?(schedule_id)
      @schedule = Schedule.find(schedule_id)
    end
    
    @event = @schedule.nil? ? Event.new(params[:event]) : @schedule.events.new(params[:event])
    if can_create?(@event.class.to_s.tableize)
      if @event.save
        flash[:notice] = t('sadmin.events.guardado_correctamente', :what => Event.human_name)
        redirect_to @schedule.nil? ? sadmin_event_path(@event.id, :fresh => 1) : sadmin_schedule_event_path(:id => @event.id, :schedule_id => @schedule.id, :fresh => 1)
      else
        render :action => 'new'
      end
    else
      flash[:notice] = t('no_tienes_permiso')
      access_denied
    end    
    
  end

  # Página de un evento. Se muestran todos los datos del evento.
  def show
    @just_created = params[:fresh].to_i.eql?(1)
    if @just_created
      @selected_date = {}
      [:year, :month, :day].each do |key|
        @selected_date[key] = @event.starts_at.send(key.to_s)
      end
    end    
    respond_to do |format|
      format.html {
        render
      }
      format.ics {
        render :layout => false
      }
    end
  end

  # Formulario para modificar los datos de un evento.
  def edit
    @title = t('sadmin.modificar_what', :what => Event.human_name)
  end

  # Modificar los datos de un evento. 
  #
  # Si los datos son válidos, estos se guardan en la base de datos y se muestra el evento. 
  # Si los datos no son válidos se muestra el formulario con los mensajes de error correspondientes.
  def update
    new_schedule = params[:event].delete(:schedule_id)
    #new_schedule = nil if new_schedule.to_i.eql?(0) || !Schedule.exists?(new_schedule)
    
    @event.attributes = params[:event]
    @title = t('sadmin.modificar_what', :what => Event.human_name)
    
    if event_type_changed?(new_schedule)
      if @event.is_a?(Event)
        if schedule = Schedule.find(new_schedule)
          change_schedule_to(schedule)
        else
          flash[:notice] = 'No puede modificar la agenda'
          render(:action => :edit)
        end   
      end
      if @event.is_a?(ScheduleEvent)
        change_schedule_to_agenda
      end
      return
    else
      if (new_schedule.to_i <= 0)
        # cambio entre público y privado
        @event.schedule_id = new_schedule if @event.is_a?(Event)
      else
        # cambio entre agendas privadas
        @event.schedule_id = new_schedule if @event.is_a?(ScheduleEvent) && current_user.can_change_schedule?(@event.schedule_id) && current_user.can_change_schedule?(new_schedule)
      end
      if @event.save
        flash[:notice] = t('sadmin.events.guardado_correctamente', :what => Event.human_name)
        redirect_to @event.is_a?(Event) ? sadmin_event_path(:id => @event.id) : sadmin_schedule_event_path(:id => @event.id, :schedule_id => @event.schedule_id)
      else
        render :action => params[:return_to] || 'edit'
      end
    end
  end

  # Calendario de los eventos en un mes.
  def calendar
    @t = "events"
    
    respond_to do |format|
      format.html do
        render :action => 'index'
      end
      format.js do
        render :update do |page|
          page.replace_html "events_calendar", :partial => "calendar"
        end
      end
      format.xml  { head :ok }
    end
  end
  
  # Marcar un evento como 'borrado'.
  def mark_for_deletion
    @event = Event.find(params[:id])
  end
  
  # Borrar un evento.
  def delete
    @event = Event.find(params[:id])
    
    unless params[:cancel]
      if @event.update_attributes(:deleted => true)
        flash[:notice] = t('sadmin.events.marcado_eliminado')
      else
        flash[:error] = t('sadmin.events.no_marcado_eliminado')
      end
    end
    redirect_to sadmin_event_path(@event)
  end
  
  # Elegir una nueva agenda para el evento.
  def select_schedule
  end
  
  # Cambiar la agenda de un evento.
  def change_schedule
    if s = Schedule.exists?(params[:new_schedule_id])
      s = Schedule.find(params[:new_schedule_id])
      change_schedule_to(s)
    else
      flash[:warning] = t('sadmin.events.no_ha_indicado_la_agenda')
      redirect_to sadmin_event_path(@event)
    end  
  end

  # Cambiar el valor del campo <tt>l_attends</tt>.
  def toggle_l_attends
    if current_user.can_see_l_schedule?
      @event.toggle!(:l_attends)
      session[:l_attends] = nil
      respond_to do |format|
        format.html do
          render :action => 'show'
        end
        format.js do
          render :update do |page|
            page.replace "l_attends_info_div", :partial => '/sadmin/events/l_attends'
          end
        end
        format.xml  { head :ok }
      end
    else
      flash[:warning] = 'No tiene permiso para cambiar estos datos.'
      redirect_to @schedule.nil? ? sadmin_event_path(@event.id) : sadmin_schedule_event_path(:id => @event.id, :schedule_id => @schedule.id)
    end
    
  end
    
  # Asignar un valor nuevo a <tt>session[:l_attends]</tt>. 
  #
  # El  valor de <tt>session[:l_attends]</tt> indica si en la vista de los eventos 
  # salen sólo los eventos a los que asiste el Lehendakari o todos los eventos.
  def set_l_attends
    if session[:l_attends]
      session[:l_attends] = nil
    else
      if current_user.can_see_l_schedule?
        session[:l_attends] = true
      else
        session[:l_attends] = nil
      end
    end    
    redirect_to :back
  end

  def unrelate
    if rel = @event.related_items.find_by_eventable_id(params[:related_item_id])
      rel.destroy
    end
    redirect_to sadmin_event_path(:id => @event.id)
  end

  # Inplace editor call
  def set_event_related_news_title
    unless [:post, :put].include?(request.method) then
      return render(:text => 'Method not allowed', :status => 405)
    end
    @event.update_attribute(:related_news_title, params[:value])
    
    render :update do |page|
      page.replace 'related_news_info', :partial => 'related_news', :locals => {:document => @event}
    end
    
  end

  #
  # Autocomplete para el nombre del sitio
  #
  def auto_complete_for_event_place
    places = EventLocation.find(:all, :conditions => ["tildes(place) ilike ?", "%"+params[:event][:place].tildes+"%"])
    render :partial => 'event_locations', :object => places
  end
  
  #
  # Autocomplete para el título de la noticia relacionada
  #  
  def auto_complete_for_event_related_news_title
    @items = News.find(:all, :conditions => ["tildes(title_es) ILIKE ?", params[:value]+'%'])
    render :inline => "<%= content_tag(:ul, @items.map {|item| content_tag(:li, item.title)}) %>"
  end

  #
  # Autocomplete para la lista de políticos
  #
  def auto_complete_for_event_politicians_tag_list
    auto_complete_for_document_politicians_tag_list(params[:event][:politicians_tag_list])
  end  

  
  private
  
  def get_selected_date
    @selected_date = {}
    @today = Time.zone.now
    [:year, :month, :day].each do |key|
      @selected_date[key] = (params[key].to_i > 0) ? params[key].to_i : @today.send(key.to_s)
    end
    
    @today = Time.zone.now
    @year = @selected_date[:year]
    @month = @selected_date[:month]
    @day = @selected_date[:day]
  end
  
  def set_current_tab
    @t = "events"
    @pretty_type = t("documents.#{@t.titleize.singularize}").singularize
    
    @current_tab = @t.to_sym
    @current_tab
  end
  
  def get_calendar_events
     get_selected_date
     if @show_agenda.include?('agenda')
       @events = event_finder.month_events_by_day4cal(@month, @year)
     else
       @events = {}
       @events[@month] = {}
     end
     
     if current_user.has_schedule_permissions?
        get_show_agenda
        @show_agenda.to_a.each do |a_name|
          if s = Schedule.find_by_short_name(a_name)
            @schedule_events = (session[:l_attends] ?  s.events.l_attends : s.events).month_events_by_day4cal(@month, @year)
            unless @schedule_events.blank?
              @schedule_events.keys.each do |month|
                @events[month] = {} if  @events[month].nil?
                @schedule_events[month].keys.each do |day|
                  val = @schedule_events[month][day]
                  if @events[month][day]
                    @events[month][day] += val
                  else
                    @events[month][day] = val
                  end
                end
              end
            end
          end
        end
        
     end
     
     # borar del hash events los eventos que no necesitamos para el calendario del mes
     
     first_day_of_month = Date.new(@year, @month, 1).to_datetime
     first_week = first_day_of_month.at_beginning_of_week
     last_week = first_day_of_month.at_end_of_month.at_end_of_week
     cal_months = [first_day_of_month.month, first_week.month, last_week.month].uniq.sort
     @events.delete_if {|key, value| !cal_months.include?(key)} if @events
     @events[first_week.month].delete_if {|key, value| !(first_week.day..first_week.at_end_of_month.day).include?(key)} if @events[first_week.month]
     @events[last_week.month].delete_if {|key, value| !(last_week.at_beginning_of_month.day..last_week.day).include?(key)} if @events[last_week.month]
      
     sort_day_events
  end

  def sort_day_events
    # sort day events by hour
    @events.keys.each do |m|
      @events[m].each do |day, ev_list|
         ev_list.sort! {|a,b| a.sort_position <=> b.sort_position}
      end
    end
  end
  
  def get_event
    @event = Event.find(params[:id])    
  end
  
  def get_schedule_events(conditions)
    schedule_events = []    
    if current_user.has_schedule_permissions?
      @show_agenda.each do |a_name|
        if s = Schedule.find_by_short_name(a_name)
          schedule_events += s.events.find(:all, :conditions => conditions, :order => "starts_at")
        end
      end 
    end
    schedule_events.flatten
  end
  
  def get_show_agenda
    @show_agenda = ['agenda']
    if current_user.has_schedule_permissions?
      if params[:show_agenda]
        @show_agenda = params[:show_agenda].to_a
      else
        @show_agenda = session[:show_agenda] || ['agenda']+current_user.schedules.map {|s| s.short_name}
      end
    end
    session[:show_agenda] = @show_agenda
  end
  
  def get_title
    a_names = []
    if @show_agenda.include?('agenda')
     a_names << t('sadmin.events.agenda_compartida')
    end
    (@show_agenda - ['agenda']).each do |a|
      if s = Schedule.find_by_short_name(a)
        a_names << s.name
      end
    end
    
    @title = a_names.to_sentence
    if session[:l_attends] && current_user.can_see_l_schedule?
      @l_subtitle = t('sadmin.events.Asiste_el_lehendakari')
    end
  end
  
  def access_to_events_required
    permission_ok = false
    if (logged_in?)
      if can_access?("events") || can_access?("schedule_events")
        permission_ok = true
      end
    end
    
    unless permission_ok
      flash[:notice] = t('no_tienes_permiso')
      access_denied
    end
  end
  
  def access_to_schedules_required
    if current_user.schedules.blank?
      flash[:notice] = t('no_tienes_permiso')
      redirect_back_or_default sadmin_events_path
    end
  end

  def edit_permission_required
    permission_ok = false
    if (logged_in?)
      if @event
        permission_ok = can_edit_event?(@event)
      else
        if can_edit?("events") || can_edit?("schedule_events")
          permission_ok = true
        end
      end
    end
    
    unless permission_ok
      flash[:notice] = t('no_tienes_permiso')
      access_denied
    end
  end

  def restrict_events?
    current_user.is_a?(DepartmentMember)
  end  
  
  def event_finder
    if restrict_events?
      #finder = current_user.department.events
      finder = Event
    else
      finder = (session[:l_attends] ?  Event.l_attends : Event)
    end
    return finder    
  end

  def event_type_changed?(new_schedule)
    (@event.visibility_can_change?) && ((@event.is_a?(Event) && new_schedule.to_i > 0) || (@event.is_a?(ScheduleEvent) && new_schedule.to_i <= 0))
  end

  def change_schedule_to(schedule)
    
    if current_user.can_change_schedule?(schedule.id)
      if params[:event]
        @new_event = schedule.events.new(params[:event])
      else
        @new_event = schedule.events.new
        @new_event.copy_from(@event)
      end
    
      @event.update_attributes(:deleted => true)
      
      if @new_event.save
        @event.destroy if @event.is_destroyable?
        flash[:notice] = t('sadmin.events.la_agenda_ha_sido_cambiada')
        redirect_to sadmin_schedule_event_path(:id => @new_event.id, :schedule_id => schedule.id)
      else
        # The old event is not deleted and will be present in the events list with the notice "to be deleted".
        @event = @new_event
        @schedule = schedule
        render :template => '/sadmin/schedule_events/new'
      end
    else
      flash[:warning] = t('sadmin.events.no_tiene_permiso_para_cambiar_la_agenda')
      redirect_to sadmin_event_path(@event) and return
    end      
  end 
  
end
