#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para el acceso a las agendas privadas desde la parte de administración de la web.
#
# Sólo los usuarios que tienen permiso para ver alguna agenda privada pueden acceder a este controlador.
class Sadmin::ScheduleEventsController < Sadmin::EventsController
  skip_before_filter :edit_permission_required
  
  before_filter :get_schedule
  before_filter :check_schedule_permissions
  before_filter :check_editable_schedule, :only => [:new, :destroy]
  before_filter :check_changable_schedule, :only => [:change_schedule, :select_schedule]
  
  # Formulario para crear nuevo evento privado.
  def new
    @title = t('sadmin.modificar_what', :what => Event.human_name)
    
    begin
       default_date = Date.parse("#{@year}-#{@month}-#{@day}") 
     rescue ArgumentError
       default_date = Time.now
     end
       
    @event = @schedule.events.new(:starts_at => default_date, :ends_at => default_date)
    
  end
  
  # Eliminar evento. 
  #
  # Los eventos privados no están incluidos en las alertas por email así que se pueden borrar directamente.
  def destroy
    get_event
    month = @event.starts_at.month
    year = @event.starts_at.year
    
    if @event.destroy
      flash[:notice] = t('sadmin.eliminado_correctamente', :article => Event.human_name.gender_article, :what => Event.human_name)
      redirect_to calendar_sadmin_events_url(:year => year, :month => month)
    else
      flash[:error] = t('sadmin.no_eliminado_correctamente', :article => Event.human_name.gender_article, :what => Event.human_name)
      redirect_to :action => 'show'
    end
  end

  # Cambiar la agenda de un evento.
  def change_schedule
    if params[:new_schedule_id].eql?('agenda')
      change_schedule_to_agenda
    else
      if s = Schedule.exists?(params[:new_schedule_id])
        @schedule = Schedule.find(params[:new_schedule_id])
        @event.update_attributes(:schedule_id => @schedule.id)
        redirect_to sadmin_schedule_event_path(:id => @event.id, :schedule_id => @schedule.id)
      else
        flash[:warning] = t('sadmin.events.no_ha_indicado_la_agenda')
        redirect_to sadmin_event_path(@event)
      end  
    end
  end

  
  private
  
  def get_event
    @event = ScheduleEvent.find(params[:id])
  end
  
  def get_schedule
    @schedule = Schedule.find(params[:schedule_id])
  end
  
  def check_schedule_permissions
    unless (logged_in? && current_user.can_see_schedule?(@schedule.id))
      flash[:notice] = t('no_tienes_permiso')
      access_denied
    end
  end

  def check_editable_schedule
    unless current_user.can_edit_schedule?(@schedule.id)
      flash[:notice] = t('no_tienes_permiso')
      access_denied
    end
  end  

  def check_changable_schedule
    unless current_user.can_change_schedule?(@schedule.id)
      flash[:notice] = t('no_tienes_permiso')
      access_denied
    end
  end  

  def change_schedule_to_agenda
    if current_user.can_create?('events')
      if params[:event]
        @new_event = Event.new(params[:event])
      else
        @new_event = Event.new
        @new_event.copy_from(@event)
      end
    
      if @new_event.save
        @event.destroy if @event.is_destroyable?
        flash[:notice] = t('sadmin.events.la_agenda_ha_sido_cambiada')
        redirect_to sadmin_event_path(:id => @new_event.id)
      else
        # The old event is not deleted and will be present in the events list with the notice "to be deleted".
        @event = @new_event
        # logger.info "..................... #{@event.inspect}"
        render :template => '/sadmin/events/new'
      end
    else
      flash[:warning] = t('sadmin.events.no_tiene_permiso_para_cambiar_la_agenda')
      redirect_to sadmin_schedule_event_path(@event) and return
    end
  end  
end
