#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para los procesos de login/logout.
include ActionView::Helpers::UrlHelper
class SessionsController < ApplicationController
  skip_before_filter :http_authentication, :only => [:new, :create] # para las pruebas de Grzegorz
  
  # Formulario de inicio de sessión
  def new
    @title = t('session.login_in', :name => site_name)
    @breadcrumbs_info = [[@title, login_path]]
    if request.xhr?
      render :template => 'sessions/new_for_nav', :layout => false and return
    end
  end

  # Creación de una sessión, es decir, login.
  def create
    self.current_user = User.authenticate(params[:email], params[:password])
    process_login
  end
  
  # Logout del sistema
  def destroy
    # store_previous_location unless session[:return_to]
    if logged_in?
      @user_name = current_user.name
      SessionLog.create(:user_id => current_user.id, :action => "logout", :action_at => Time.zone.now, :user_ip => request.remote_ip)
      self.current_user.forget_me 
    end
    cookies.delete :auth_token
    reset_session
    session[:return_to]=request.referer if request.referer && !request.referer.match(/account/)
    respond_to do |format|
      format.html do
        flash[:notice] = t('session.Has_salido', :name => @user_name)
        redirect_back_or_default(root_path)
      end
      format.floki do
        render :json => ''
      end
    end
  end
  
  # Formulario de recordatorio de contraseña
  def password_reminder
    @breadcrumbs_info = [[I18n.t('session.Recordatorio'), password_reminder_path]]
    # render :template => "/sessions/password_reminder"
  end
  
  # Envio de la contraseña por correo electrónico
  def email_password
    if params[:email].blank?
      flash[:notice] = t('session.Por_favor_email')
      redirect_to :action => "password_reminder" and return
    end

    user = User.approved.find_by_email(params[:email])

    if user
      if user.is_twitter_user?
        flash[:notice] = t("session.usuario_twitter_sin_password")
      elsif user.is_facebook_user?
        flash[:notice] = t("session.usuario_facebook_sin_password")
      else
        email = Notifier.create_password_reminder(user)

        begin
          logger.info("Mandando login olvidado")
          Notifier.deliver(email)
          flash[:notice] = t('session.Password_enviado')
        rescue Net::SMTPServerBusy, Net::SMTPSyntaxError => err_type
          logger.info("Error al mandar mail de recordatorio de contraseña: " + err_type)
          flash[:error] = t('session.Error_servidor_correo')
        end
      end
      if !request.xhr?
        redirect_to :action => "new" and return
      else
        render :nothing => true
      end    
    else
      user = User.pending.find_by_email(params[:email])
      if user
        flash[:error] = t('session.pendiente_aprobacion')
        #redirect_to password_reminder_path and return
      else
        flash[:error] = t('session.No_hay_usuario')
        #redirect_to password_reminder_path and return
      end
      redirect_to password_reminder_path and return
    end
  end
  
  # Las nuevas cuentas quedan pendientes de activación por el usuario.
  # Esta acción activa la cuenta a partir del email que se le mandó al usuario al registrarse.
  def email_activation
    @user = User.find_by_email_and_status(params[:email], "pendiente")
    
    if @user
      email = Notifier.create_activate_person_account(@user)

      begin
        logger.info("Mandando activacion a #{@user.email}")
        Notifier.deliver(email)
      rescue Net::SMTPServerBusy, Net::SMTPSyntaxError => err_type
        logger.info("Error al mandar mail de activacion: " + err_type)
        flash[:error] = t('session.Error_servidor_correo')
      else
        flash[:notice] = t('session.activacion_enviada')
      end
    else
      flash[:error] = t('session.No_hay_usuario')
    end
    redirect_to login_path and return      
  end
  
  # def mobile
  #   render :action => 'mobile.html', :layout => false
  # end
  
  def auth_info
    render :action => 'auth_info.json', :layout => false
  end

  def nav_user_info
    render :partial => '/shared/nav_user_logged', :layout => false
  end
  
  private
  def site_name
    t('site_name')
  end

  def process_login
    if logged_in?
      SessionLog.create(:user_id => current_user.id, :action => "login", :action_at => Time.zone.now, :user_ip => request.remote_ip)
      
      respond_to do |format|
        format.html do
          if params[:remember_me] == "1"
            self.current_user.remember_me
            cookies[:auth_token] = { :value => self.current_user.remember_token , :expires => self.current_user.remember_token_expires_at }
          end
          # flash[:notice] = t('session.Has_entrado', :name => current_user.name) if !request.xhr? 
          if params[:home].present? || request.xhr?
            render :json => true, :status => :ok
          else  
            redirect_back_or_default(default_url_for_user)
          end  
        end
        format.floki do
          if params[:remember_me] == "1"
            self.current_user.remember_me
            remember_time = 20.year
          else  
            remember_time = 2.hour
          end
          self.current_user.remember_me_for(remember_time)
          cookies[:auth_token] = { :value => self.current_user.remember_token , :expires => (Time.now + remember_time)}
          flash[:error] = nil
          render :template => 'sessions/success.floki', :layout => false
        end
      end
    else
      if @user = User.find_by_email_and_status(params[:email], "pendiente")
        respond_to do |format|
          format.html {
            if request.xhr?
              render :json => t('session.No_has_activado_cuenta', :link => link_to(t('session.pincha_aqui'), email_activation_session_path(:email => params[:email]))).to_json, :status => 500
            else  
              render :template => "/sessions/waiting_for_approval"
            end  
          }  
          format.floki {
            render :action => 'waiting_for_approval.floki', :layout => "application.floki.erb"
          }
        end
      else
        respond_to do |format|
          format.html do
            if request.xhr?
              render :json => [['email'], ['password']].to_json, :status => 500
            else
              flash[:error] = t('session.Email_incorrecto')
              render :action => 'new'  
            end
          end
          format.floki do
            flash[:error] = t('session.Email_incorrecto')
            render :action => 'new.floki', :layout => false
          end
        end
      end
    end
  end
    
end
