#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para paginas del site en general
class SiteController < ApplicationController
  skip_before_filter :set_locale, :only => [:splash, :change_locale, :show]
  skip_before_filter :http_authentication, :only => :show # para las pruebas de Grzegorz

  def show
    respond_to do |format|
      format.html do
        if params[:locale].blank? && cookies['locale'].blank?
          redirect_to lang_path
        else
          set_locale
          @carousel_news = [News.featured_a] + News.featured_4b
          @streaming = Streaming.new
          @debates = (Debate.published.featured + Debate.published.find(:all, :order => "published_at DESC", :limit => 4))[0..3]
          @albums = Album.with_photos.featured.find(:all, :limit => 4)
          @videos = Video.published.translated.find(:all, :limit => 4, :order => "featured DESC, published_at DESC") 
          render
        end
      end
      format.iphone do
        get_iphone_info
        render
      end
      format.android do
        get_iphone_info
        render
      end      
    end
  end
  
  # Redirect to new search controller
  def search
    redirect_to get_create_search_url(:key => 'keyword', :value => params[:q], :new => true)
  end

  # Cambia de idioma, y redirige a la página en la que se encontraba.
  def change_locale
    old_url = request.env["HTTP_REFERER"] ? request.env["HTTP_REFERER"].gsub(/^#{request.protocol}#{request.host_with_port}/, '') : "/"
    new_url = old_url.dup
    if locales.keys.include?(params[:l].to_sym)
      cookies['locale'] = { :value => params[:l] , :expires => 1.year.from_now }
      I18n.locale = params[:l]

      # Separa la parte de delante y detrás de la ?
      dummy, referer_path, dummy2, referer_querystring = get_params_for(old_url)
      if !referer_querystring.nil?
        # convierte "a=b&c=d" en {"a" => "b", "c" => "d"}
        additional_params = referer_querystring.split('&').inject(Hash.new) do |h, pair| 
          k, v = pair.split('=')
          h[k] = v
          h
        end
      end

      referer_params = ActionController::Routing::Routes.recognize_path(referer_path, :method => :get)
      referer_params = referer_params.merge(additional_params) if additional_params
      
      new_url = url_for(referer_params.merge(:locale => params[:l]))
      
      if (locales.keys.collect {|l| "/#{l}"} + ["/"]).include?(old_url)
        new_url = "/#{params[:l]}"        
      elsif should_translate_url_slug(referer_params)
        # En las URL que tienen el titulo "sluggeado" para buscadores, hay que traducir la parte de detras
        # del id. Eso solo se puede hacer, cogiendo de nuevo el objeto para saber cual es su titulo en otro idioma
        obj = referer_params[:controller].singularize.camelize.constantize.find(referer_params[:id])
        new_url = url_for(referer_params.merge(:locale => params[:l], :id => obj))
      end
    end
    redirect_to new_url
  end
  
  # Página para errores 404 de página no encontrada
  def notfound
    render(:status => "404 Not Found" )
  end
  
  def contact
    @title = t('site.contactar')
    @breadcrumbs_info = [[@title, contact_site_path]]
  end
  
  # Envío de sugerencia al administrador
  def send_contact
    @title = t('site.contactar')
    if params[:name].blank? || params[:email].blank? || params[:message].blank?
      @form_errors = [['name', t('activerecord.errors.messages.blank')], ['message', t('activerecord.errors.messages.blank')], ['email', t('activerecord.errors.messages.blank')]]
      # flash[:error] = t('share.todos_campos')
      render :action => "contact" and return
    elsif !params[:email].match(/^([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})$/i)
      # flash[:error] = t('share.email_incorrecto')
      @form_errors = [['email', t('share.email_incorrecto')]]
      render :action => "contact" and return
    end
    
    email = Notifier.create_contact(params[:name], params[:email], params[:message])

    begin
      logger.info("Mandando email de contacto")
      Notifier.deliver(email)
      @title =  t('site.contacto_enviado')
      @message = t('site.body_contacto_enviado')
    rescue Net::SMTPServerBusy, Net::SMTPSyntaxError => err_type
      logger.info("Error al mandar mail de pagina: " + err_type)
      flash[:error] = t('session.Error_servidor_correo')
      @message = t('site.body_contacto_no_enviado')
    end    
  end
  
  # Página para que un robot monitorice y compruebe que el site no se ha caido.
  def stat
    @app_stat = "OK"
    begin
      ActiveRecord::Base.connection.execute("SELECT 1 FROM documents")
      @db_stat = "OK"
    rescue
      @db_stat = "KO"
    end
    render :layout => false
  end
  
  # Página splash para la elección del idioma
  def splash
    respond_to do |format|
      format.html {
        if cookies['locale'].blank? || request.request_uri.eql?(lang_path)
          render :layout => false
        else
          redirect_to "/#{cookies['locale']}"
        end        
      }
      format.iphone {
        redirect_to iphone_path
      }
    end    
  end
  
  # Página de redes sociales
  def snetworking
    @page_title = t('snetworks.title')
    @departments=Department.without_gv.find(:all, :order => 'position').select{|a| a.sorganizations.present?}
    @breadcrumbs_info = [[@page_title, snetworking_site_path]]
  end
  
  # Páginas englobadas en "sobre irekia"
  def page
    @page = Page.send(params[:label])
    @breadcrumbs_info = [[Page::ABOUT.detect {|a| a[:label] == params[:label]}[:title], page_site_path(:label => params[:label])]]
    
    respond_to do |format|
      format.html {
        render :template => "/pages/show"
      }
      format.floki {
        render :template => "/pages/show.floki"
      }
    end
    
  end
  
  # Listado de todos los Feeds RSS
  def feeds
    @departments = Department.active.find(:all, :order => "position")
    @title = t('documents.feeds_rss')
    @breadcrumbs_info = [[@title, feeds_site_path]]
  end
  
  def email_item
    if !params[:t].blank? && %(News Event Page Proposal Album Photo Video).include?(params[:t])
      @document = params[:t].constantize.find(params[:id])
    else
      raise ActiveRecord::RecordNotFound
    end
  end
  
  # Envia la noticia a un amigo
  def send_email
    if %(News Event Page Proposal Album Photo Video).include?(params[:t])
      @document = params[:t].constantize.find(params[:id])
    else
      raise ActiveRecord::RecordNotFound
    end
    
    if params[:sender_name].blank? || params[:recipient_name].blank? || params[:recipient_email].blank?
      flash[:error] = t('share.todos_campos')
      render :action => "email_item" and return
    elsif !params[:recipient_email].match(/^([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})$/i)
      flash[:error] = t('share.email_incorrecto')
      render :action => "email_item" and return
    end
    
    email = Notifier.create_email_document(params[:sender_name], params[:recipient_name], params[:recipient_email], @document)

    begin
      logger.info("Mandando documento")
      Notifier.deliver(email)
      flash[:notice] =  t('share.pagina_enviada')
    rescue Net::SMTPServerBusy, Net::SMTPSyntaxError => err_type
      logger.info("Error al mandar mail de pagina: " + err_type)
      flash[:error] = t('session.Error_servidor_correo')
    end
    
    redirect_to @document
  end
  
  def user_default
    redirect_to default_url_for_user
  end
  
  private
  def get_iphone_info  
    @news = News.published.listable.translated.find(:all, 
      :include => :organization, 
      :order => "featured, published_at DESC", :limit => 11)
    @fn = @news.delete_at(0)
    
    @events = Event.published.translated.future(Time.zone.now - 1.day).find(:all, :limit => 10)
  end
  
  def should_translate_url_slug(referer_params)
    !referer_params[:id].blank? && referer_params[:action].eql?("show") &&
    %w(documents news events pages pages videos proposals).include?(referer_params[:controller])
  end
  
end
