#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Controlador para la WebTV.
class VideosController < ApplicationController
  before_filter :get_context, :only => [:index]
  before_filter :get_criterio, :only => [:show]
  after_filter :track_clickthrough, :only => [:show]

  def index
    prepare_videos(@context, request.xhr?)
    
    respond_to do |format|
      format.html do
        if request.xhr?
          render :partial => '/shared/list_items', :locals => {:items => @videos, :type => 'video'}, :layout => false
        else
          render
        end
      end
    end
    
  end

  def summary
    @featured_video = Video.published.featured
    @featured_albums = Album.published.with_photos.featured.find(:all, :limit => 2)
    # @featured_photo1 = featured_albums[0].cover_photo
    # @featured_photo2 = featured_albums[1].cover_photo
    render :layout => !request.xhr?
  end
  
  def cat
    @categories = Video.categories
    @category = Category.find(params[:id])
    @title = @category.name
    @videos = @category.videos.paginate(:per_page => 12, :order => "published_at DESC, matching_tag_counter DESC", :page  => params[:page] || 1)
  end                        
  
  # Página de un video
  def show
    begin
      @video = Video.published.find(params[:id])
    rescue ActiveRecord::RecordNotFound
      if can_edit?("videos")
        @video = Video.find(params[:id])
      else
        raise ActiveRecord::RecordNotFound
      end
    end

    @page_title_for_head = @video.title
    @title = t('videos.web_tv')
    
    # Comments
    @parent = @video
    @comments = @video.comments.approved.paginate :page => params[:page], :per_page => 25
    
    @related_videos = (Video.published.translated.find_tagged_with(@video.tag_list, :order => "matching_tag_counter DESC, published_at DESC ") - [@video])[0..10]
        
  end

  caches_page :podcast
  def podcast
    @videos = Video.published.translated.find :all, :limit => 100, :order => "published_at DESC"
    # respond_to do |format|
    #   format.any(:html, :xml) {logger.info "ZZZZZZZZZZZ";render :action => "podcast.xml", :layout => false, :content_type => 'application/xml'}
    # end
    
    render :action => "podcast.xml", :layout => false, :content_type => 'application/xml'
  end


  private 
  
  # Construye los breadcrumbs de cada acción de la WebTV
  def make_breadcrumbs
    if @context
      @breadcrumbs_info << [t('videos.title'), send("#{@context.class.to_s.downcase}_videos_path", @context)]
    else
      @breadcrumbs_info = [[t('videos.web_tv'), videos_path]]    
      if @category
        @category.ancestors.reverse.each {|a| @breadcrumbs_info << [a.name, cat_videos_url(:id => a)]}
        @breadcrumbs_info << [@category.name, cat_videos_url(@category)]
      end
      if @video
        @breadcrumbs_info << [@video.title, video_path(@video)]
      end
    end
    @breadcrumbs_info
  end
  
  def prepare_videos(context, is_xhr)
    @title = t('videos.web_tv')
    if context
      @title << " #{t('shared.from_context', :name => context.public_name)}"
      if params[:page].eql?(1) || params[:page].nil?
        @videos = @context.videos.published.paginate(:page => params[:page], :per_page => 17, :order => 'published_at DESC')      
        @featured_video = @videos.shift
      else
        @videos = @context.videos.published.paginate(:page => params[:page], :per_page => 16, :order => 'published_at DESC')      
      end
    else
      @featured_video = Video.published.featured
    end
  end

end
