#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Clase para los álbums de la fototeca
class Album < ActiveRecord::Base
  include Sluggable  
  has_many :album_photos, :dependent => :destroy
  # I cannot put :order clause here because nested order clauses are not working with named_scope
  # and I'm most interested in using this as <tt>album.photos.ordered_by_title</tt>
  has_many :photos, :through => :album_photos 
  belongs_to :document
    
  validates_presence_of :title_es
  validates_length_of :title_es, :maximum => 255
  validates_length_of :title_eu, :title_en, :maximum => 255, :allow_blank => true
  
  translatable_columns :title
  acts_as_taggable
  
  include Tools::WithAreaTag
  include Tools::WithPoliticiansTags
  
  named_scope :published, :conditions => "draft='f'"
  named_scope :with_photos, :conditions => "album_photos_count>0", :order => "created_at DESC"
  named_scope :featured, :conditions => "featured='t'", :order => "created_at DESC"
  # moved to acts_as_taggable
  # named_scope :tagged_with, lambda {|tag_list| Album.find_options_for_find_tagged_with(tag_list) }
      
  # # Versión corta del título para que quepa en la portada del álbum
  # def title_for_cover
  #   title.length > 50 ? title[0..48].sub(/\s[^\s]+$/, ' ...') : title
  # end
  
  # Los vídeos son contenido principal y salen en la lista de acciones de la parte pública de la web.  
  include Tools::Content
  
  # Foto de portada para este album
  def cover_aphoto
    album_photos.find_by_cover_photo(true) || album_photos.first
  end
  
  def cover_photo
    cover_aphoto.photo if cover_aphoto
  end
  
  def first_photo
    self.photos.find(:first, :order => "date_time_original")
  end
  
  def last_photo
    self.photos.find(:first, :order => "date_time_original DESC")
  end

  def published_at
    self.updated_at
  end
  
  include Elasticsearch::Base

  # Determina si el álbum se muestra en Irekia.
  # Sólo relevante para la busqueda
  def is_public?
    !draft?
    # true
  end
  
  def published?
    !draft?
  end
  alias_method :approved?, :published? 
  
  # para poder reusar /documents/_share
  def body
    ""
  end      
  
  # Obtener departamentos del album a partir de los tags
  def organization
    Department.find(:first, :conditions => {:tag_name => self.tags.private.map(&:name)})    
  end
  
  def self.categories
    Tree.find_albums_tree ? Tree.find_albums_tree.categories.roots : []
  end
end
