#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

class Argument < ActiveRecord::Base
  include Floki
  
  belongs_to :argumentable, :polymorphic => true, :counter_cache => true
  
  belongs_to :user
  belongs_to :author, :class_name => "User", :foreign_key => "user_id"
  
  named_scope :in_favor, :conditions => "value = 1"
  named_scope :against, :conditions => "value = -1"
  named_scope :published, lambda { |*args| {:conditions => ["arguments.published_at <= ?", (args.first || Time.zone.now)]}}
  named_scope :rejected, lambda { |*args| {:conditions => ["rejected_at <= ?", (args.first || Time.zone.now)]}}

  named_scope :for_debates,   :conditions => "argumentable_type='Debate'"
  named_scope :for_proposals, :conditions => "argumentable_type='Contribution'"
  
  validates_length_of :reason, :maximum => 255

  include Notifiable
  
  def approved?
    !published_at.blank?
  end
  
  def status
    published_at.nil? ? 'pendiente' : 'aprobado'
  end
  
  def approve!
    self.update_attribute(:published_at, Time.zone.now)
  end

  def in_favor
    value == 1
  end

  before_create :approve_if_admin
  
  private
  def approve_if_admin
    if self.user_id
      if User.find(self.user_id).is_staff?
        self.published_at = Time.zone.now if self.published_at.nil?
      end
    end
  end
end
