#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Banners de la pagina de inicio
class Banner < ActiveRecord::Base
  has_attached_file :logo_es, :url  => "/assets/banners/:id/es/:sanitized_basename.:extension",
                    :path => ":rails_root/public/assets/banners/:id/es/:sanitized_basename.:extension"
  has_attached_file :logo_eu, :url  => "/assets/banners/:id/eu/:sanitized_basename.:extension",
                    :path => ":rails_root/public/assets/banners/:id/eu/:sanitized_basename.:extension"
  has_attached_file :logo_en, :url  => "/assets/banners/:id/en/:sanitized_basename.:extension",
                    :path => ":rails_root/public/assets/banners/:id/en/:sanitized_basename.:extension"

  translatable_columns :alt, :url
  
  #Pagination
  cattr_reader :per_page
  
  named_scope :active, lambda {{:conditions => "active='t'", :order => "position ASC"}}
  
  before_create :set_position
  
  validates_presence_of :url_es, :alt_es
  validates_format_of :url_es, :url_eu, :url_en, :allow_blank => true, 
                      :with => /(^$)|(^(http|https):\/\/[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(([0-9]{1,5})?\/.*)?$)/ix , 
                      :message => 'no es correcta'
  
  validates_attachment_presence :logo_es, :message => 'no puede estar vacío'
  validates_attachment_size :logo_es, :less_than => 250.kilobytes, :message => "debe tener un tamaño máximo de 250kB"
  validates_attachment_size :logo_eu, :less_than => 250.kilobytes, :message => "debe tener un tamaño máximo de 250kB"
  validates_attachment_size :logo_en, :less_than => 250.kilobytes, :message => "debe tener un tamaño máximo de 250kB"
  validates_attachment_content_type :logo_es, :message => "no es un tipo de fichero válido", 
                                    :content_type => ['image/jpeg', 'image/pjpeg', 'image/png', 'image/x-png', 'image/gif']
  validates_attachment_content_type :logo_eu, :message => "no es un tipo de fichero válido",
                                    :content_type => ['image/jpeg', 'image/pjpeg', 'image/png', 'image/x-png', 'image/gif']
  validates_attachment_content_type :logo_en, :message => "no es un tipo de fichero válido",
                                    :content_type => ['image/jpeg', 'image/pjpeg', 'image/png', 'image/x-png', 'image/gif']
  validate :banner_size_valid?
  
  def banner_size_valid?
    begin
      # Para cada uno de los idiomas disponibles, se comprueba si hay foto
      # y cual es su tamaño
      ['es', 'eu', 'en'].each do |lang|
        unless self.send("logo_#{lang}").tempfile.nil?
          size=Paperclip::Geometry.from_file(self.send("logo_#{lang}").tempfile.path)
          if size.width.to_i != 170 || size.height.to_i != 110
            self.errors.add("logo_#{lang}", 'debe tener un tamaño de 170x100px')
            false
          end
        end
      end
    rescue NotIdentifiedByImageMagickError, NoMethodError => err
      logger.error "Error checking banner size: #{err}"
    end
  end
  
  # Antes de guardar un nuevo banner, se le asigna la última posición
  def set_position
    self.position = Banner.all.map{|b| b.position}.max.to_i+1
  end
  
  def logo(locale=I18n.locale)
    self.send("logo_#{locale}")
  end  

end

  