#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Clase para los comentarios en cualquiera de los recursos disponibles:
# noticias, propuestas, videos...
class Comment < ActiveRecord::Base
  include Floki
  # belongs_to :document, :counter_cache => true
  belongs_to :commentable, :polymorphic => true, :counter_cache => true
  belongs_to :user
  belongs_to :author, :class_name => "User", :foreign_key => "user_id"
  has_many :ratings, :as => :rateable, :dependent => :destroy
  named_scope :approved, :conditions => "comments.status in ('aprobado', 'aprobado tras denuncia')", :order => 'comments.created_at'
  named_scope :pending, :conditions => "comments.status = 'pendiente'", :order => 'comments.created_at'
  named_scope :approved_or_pending_for, lambda { |*args| {:conditions => ["comments.status in ('aprobado', 'aprobado tras denuncia') OR (comments.status='pendiente' AND user_id=?)", args.first],
                :order => 'comments.created_at'}}

  named_scope :tagged_with, lambda {|tag_name_es| {:include => {:commentable => [:tags]}, 
                                                   :conditions => "tags.sanitized_name_es IN ('#{tag_name_es}')"}}

  named_scope :official, :conditions => ["is_official=?", true]
  validates_presence_of :user, :name, :body
  validates_presence_of :email, :unless => Proc.new {|c| c.user && c.user.is_outside_user?}
  validates_format_of :email, :with => /^([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})$/i, :unless => Proc.new {|c| c.user && c.user.is_outside_user?}
  validates_format_of :url, :with => /(^$)|(^(http|https):\/\/[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(([0-9]{1,5})?\/.*)?$)/ix , :message => I18n.t('comments.blog_url_incorrect'), :allow_blank => true
  # validates_format_of :url, :with => URI::regexp(%w(http https)), :allow_blank => true
  
  has_one :comment_response
  
  # Para poder asignarle tag de área
  acts_as_taggable
  # Para poder saber el área de un comentario
  include Tools::WithAreaTag
  
  # Indica si un comentario está aprobado
  def approved?
    status.eql?('aprobado') || status.eql?('aprobado tras denuncia')
  end
  
  # Indica si un comentario está rechazado
  def rejected?
    status.eql?('rechazado')
  end
  
  # Indica si un comentario es spam
  def spam?
    status.eql?('spam')
  end
  
  # Indica si un comentario está pendiente de aprobación
  def pending?
    status.eql?('pendiente')
  end
  
  # Indica si un comentario ha sido denunciado
  def abused?
    status.eql?('denunciado')
  end
  
  # Indica si el comentario es una respuesta oficial desde la aplicación RRCC
  def rrcc_response?
    self.rrcc_answered_at.present?
  end
  
  before_validation :set_user_name_and_email
  before_validation :nullify_url_if_necessary
  before_create :set_comment_locale
  # before_save :check_abuse_counter
  
  # Akismet
  # Based in: http://railscasts.com/episodes/65-stopping-spam-with-akismet
  before_create :check_for_spam
  before_create :assign_parents_area
  
  # preserve this order
  before_create :assign_is_official
  before_create :approve_if_official_commenter
  # / preserve this order
  
  include Notifiable
  
  def request=(request) # :nodoc:
    self.user_ip    = request.remote_ip
    self.user_agent = request.env['HTTP_USER_AGENT']
    self.referrer   = request.env['HTTP_REFERER']
  end

  
  # Atributos necesarios para que Akismet determine si es spam
  def akismet_attributes # :nodoc:
    {
      # :key                  => '41e2384dc1ef',
      :key                  => '1f15e546ea8b',
      :blog                 => 'http://www.irekia.euskadi.net',
      :user_ip              => user_ip,
      :user_agent           => user_agent,
      :comment_author       => name,
      :comment_author_email => email,
      :comment_author_url   => url,
      :comment_content      => body
    }
  end

  # Marca el comentario como spam, comunicándoselo a Akismet
  def mark_as_spam!
    update_attribute(:status, 'spam')
    Akismetor.submit_spam(akismet_attributes)
  end

  # Marca el comentario como no-spam, comunicándoselo a Akismet
  def mark_as_ham!
    update_attribute(:status, 'aprobado')
    Akismetor.submit_ham(akismet_attributes)
  end
  # / Akismet

  # Devuelve el nombre del autor del comentario
  def author_name
    self.name || self.user.public_name
  end

  # Indica si ha sido redirigido a Zuzenean
  def zforwarded?
    !zforwarded_at.nil?
  end
  
  def published_at
    created_at
  end
  
  def get_commentable
    # HQ
    begin 
      item = self.commentable
    rescue ActiveRecord::SubclassNotFound  => err
      if err.to_s.match(/The single-table inheritance mechanism failed to locate the subclass: 'Question'/)
        logger.info "Hiding comment on question #{self.commentable_id}"
      end
      item = nil
    end
    return item
  end

  protected
    # Copia el nombre, el email y la url del comentarista de los datos de usuario. Se llama en before_validation. 
    # Hay dos casos especiales:
    # * El comentario se ha importado automáticamente con Ubervu: en este caso el email es común para todos
    #   (sn@irekia.euskadi.net) y el nombre es el nick de twitter
    # * El comentarista se ha loggeado con las credenciales de Twitter o Facebook, y por lo tanto no tenemos su email
    #
    def set_user_name_and_email
      if user
        self.name = self.user.public_name if self.name.blank?
        self.email = self.user.email
        self.url = self.user.url if self.url.blank?
      end
    end
    
    # El valor por defecto del campo "url" en el formuario es "http://". No es una URL válida, por lo que si se deja asi se vacía. 
    # Se llama before_validation
    def nullify_url_if_necessary
      self.url = nil if self.url.eql?("http://")
    end
    
    # Especifica el idioma del comentario a partir del idioma en el que navega el comentarista, 
    # para poder mostrar en cada idioma sólo los comentarios en ese mismo idioma.
    def set_comment_locale
      self.locale = I18n.locale.to_s
    end
    
    # Aprueba automáticamente el comentario, si es que lo ha hecho un usuario miembro del "staff".
    # Se llama desde before_create
    def approve_if_official_commenter
      self.status = 'aprobado' if self.is_official?
    end
    
    # Marca el comentario como "denunciado", y por tanto desaparece de la web, cuando ha habido 5 denuncias o más.
    # Se llama desde before_save.
    def check_abuse_counter
      if self.abuse_counter >= 5 && !self.status.eql?('aprobado tras denuncia')
        self.status = 'denunciado'
      end
    end
    
    # Envía el comentario a Akismet, quien determina si es spam o no
    def check_for_spam
      if RAILS_ENV.eql?("production") && Akismetor.spam?(akismet_attributes)
        self.status = 'spam'
      else
        return true
      end
    end
    
    def assign_parents_area
      self.tag_list = self.commentable.area_tags if self.commentable.respond_to?(:area_tags)
    end
    
    def assign_is_official
      self.is_official = self.user_id && self.user.is_official_commenter?
      return true
    end
    
end
