#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Conexión con la aplicación RRCC.
# Documentación: https://intranet.efaber.net/projects/ogov/wiki/Integración_con_RRCC

require 'savon'
require 'base64'

require "rexml/document"

class CommentResponseRequest < ActiveRecord::Base
  belongs_to :comment_response
  
  attr_accessor :request_type
  attr_accessor :soap_action

  RESPONSE_RESULTS = {  
  "0"  => "Sin conexión",
  "1"  => "Correcto",
  "11" => "Error no existe departamento. Se ha asignado un departamento y no se puede saber cual es.",
  "12" => "Error no existe Solicitud de respuesta. Se ha recibido una acción de modificación y no se ha encontrado la solicitud de respuesta.",
  "13" => "Fecha de plazo errónea, solo cuando se rellena el campo en la solicitud.", 
  "14" => "Solicitud errónea.",
  "15" => "Solicitud respondida. No se puede modificar.",
  "16" => "Error al realizar anotación de entrada en el libro de registro",
  "17" => "Error ya existe solicitud de respuesta sin responder para ese comentario a una noticia en ese departamento",
  "31" => "Solicitud en proceso",
  "32" => "Solicitud respondida",  
  "99" => "Error genérico"}

  before_save :send_request
  after_save  :set_response_status

  def initialize(opts={})
    @request_type = opts[:request_type] || nil
    super    
  end

  def first_request?
    res = self.comment_response.status.eql?('new')
  end
  
  def request_type
    @request_type ||= first_request? ? 'envio' : 'consulta'
  end
  
  def pretty_result
    RESPONSE_RESULTS[self.result]
  end
    
  def send_request
    if self.comment_response.status.eql?('published')
      self.result = "-1"
      self.response_body = ""
      self.response_error = "Comentario respondido (respuesta ID: #{self.comment_response.answer_comment_id})"
      self.response_status = 'denegado' # para indicar que no hemos podidido conectar con al aplicación rrcc
    else
      self.request_data ||= prepare_request_data

      self.soap_action = "#{self.request_type}_noticia_propuesta_irekia".to_sym
      soap_namespace = :q77

    
      Savon.env_namespace = :soapenv
      client = Savon::Client.new do |wsdl|
        wsdl.endpoint = service_data[:endpoint]
        wsdl.namespace = service_data[:namespace]
      end
    
      response = nil 
      begin
        response = client.request soap_namespace, self.soap_action do
          service_data[:soap_namespaces].each do |key, val|
            soap.namespaces[key] = val
          end      
          soap.header = {}
          soap.body = prepare_request_body() 
        end # end of request data
        self.response_body = response.to_hash
        parse_response()
        self.response_status = "received" # para indicar que sí hemos recibido respuesta de la aplicación rrcc
      rescue => error
        self.result = "0"
        self.response_body = ""
        self.response_error = error
        self.response_status = 'error' # para indicar que no hemos podidido conectar con al aplicación rrcc
      end
    end    
    return true
  end


  #
  # Notas sobre los datos del request:
  # 1.- Aunque es un xml, hay que enviarlo como un textos sustituyendo los corchetes por los simbolos &lt; y &gt;
  # 2.- Para que los deprtamentos coincidan, enviamos sólo el primer texto del nombre hasta la primera coma 
  #     (hay unas pequeñas diferencias en el resto del nombre) 
  # 3.- El texto en el tag descripción tiene que ir encodeado en Base64.
  #     (si va como texto "normal" la respuesta es "Error valudaciones",
  #     ref: http://www.java-samples.com/showtutorial.php?tutorialid=1090).   
  #
  def prepare_request_data
    comment = self.comment_response.comment
    commentable = comment.commentable
    
    data_params = {}
    data_params = if self.request_type.eql?('envio')
      # enviar solicitud
      {"accion"     => 'A',
      "referencia"       => commentable.id,
      "tipo"             => commentable.is_a?(News) ? 'N' : 'P',
      "tituloCastellano" => commentable.title_es,
      "tituloEuskera"    => commentable.title_eu,
      "tituloIngles"     => commentable.title_en,
      "identificadorComentario" => comment.id,      
      "descripcion"             => Base64.b64encode(comment.body.to_s.strip),
      "departamento"            => self.comment_response.organization.name.split(",").first,
      "departamentoAnterior"    => "",
      "fechaPlazo"              => "",
      "listaTags"               => commentable.tag_list.map {|tag| "<tag>#{tag}</tag>"}.join(" "),
      "fechaRespuesta"          => "",
      "identificadorCiudadano"  => "",
      "requiereAnotacion"       => 'S'
      }
    else
      # comprobar el estado de la solicitud
      {"accion"     => 'S',
       "referencia" => commentable.id,
       "tipo"             => commentable.is_a?(News) ? 'N' : 'P',        
       "identificadorComentario" => comment.id,      
       "departamento"            => self.comment_response.organization.name.split(",").first,
       "identificadorCiudadano"  => self.comment_response.citizen_id
        }
    end
    
    txt = ""  
    data_params.each do |key, val|
      txt += "<#{key}>#{val}</#{key}>"
    end
    txt.gsub(/</,'&lt;').gsub(/>/,'&gt;')
  end

    
  private
  
  def service_data
    sd = {:namespace => "http://www.ejie.es/webServiceClase/q77bWebServicesWar",
          :soap_namespaces => {"xmlns:soapenv" => "http://schemas.xmlsoap.org/soap/envelope/",
                               "xmlns:q77" => "http://www.ejie.es/webServiceClase/q77bWebServicesWar"}}
    if RAILS_ENV.eql?("production")
      sd[:endpoint] = "https://svc.integracion.euskadi.net/ctxweb/Q77BIrekia"
    else
      sd[:endpoint] = "http://svc.integracion.test.euskadi.net/ctxweb/Q77BIrekia"
    end
    
    sd
  end
  
  
  def prepare_request_body
    r_start = "<q77:paramNoticiaPropuesta>"
    r_end = "</q77:paramNoticiaPropuesta>"
    
 "#{r_start}&lt;datos#{self.request_type.titleize}Propuesta&gt;#{self.request_data}&lt;/datos#{self.request_type.titleize}Propuesta&gt;#{r_end}"
  end
  
  
  def parse_response()
    body = self.response_body["#{self.soap_action}_response".to_sym]["respuesta_#{self.request_type}_propuesta".to_sym]
    
    response_items = {}
    data = REXML::Document.new(body)
    data.elements.each do |item|
      item.elements.each do |val|
        response_items[val.name] = val.get_text.to_s
      end
    end

    self.result = response_items["resultadoEjecucion"]
    self.response_error = "#{response_items["descripcionErrorCastellano"]} / #{response_items["descripcionErrorEuskera"]}"
    self.registered_at = response_items["fechaAnotacionLibroRegistro"]
    self.registration_id = response_items["numeroAnotacionLibroRegistro"]
    self.citizen_id = response_items["identificadorCiudadano"]


    # Parece que el texto de la respuesta está codificado y el texto sin codificar está en latin1.
    self.answer_text = Iconv.iconv("utf-8", "ISO-8859-1//TRANSLIT", Base64.decode64(response_items["textoRespuesta"].to_s))
    
    self.answer_type = response_items["tipoRespuesta"]
    
    true
  end
  
  def set_response_status
    self.comment_response.set_status_and_citizen_id!
  end
end
