#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

class Contribution < ActiveRecord::Base
  include Sluggable

  attr_accessor :notify_proposer
  attr_accessor :notify_department
  
  STATUS = ['pendiente', 'aprobado', 'rechazado', 'aprobado tras denuncia']
  MODERATORS = ["jon-olaizola@ej-gv.es", "s-publicidad@ej-gv.es", "prensa2@ej-gv.es", "a-urbina@ej-gv.es", "lpetri@ej-gv.es", "n-zeberio@ej-gv.es"]
  
  belongs_to :user
  belongs_to :department, :class_name => "Organization", :foreign_key => "organization_id"
  has_many :comments, :as => :commentable, :order => "comments.created_at DESC", :dependent => :destroy
  has_many :notifications, :as => :notifiable, :order => "notifications.created_at", :dependent => :destroy
  
  named_scope :approved, :conditions => "contributions.status='aprobado'", :order => 'contributions.created_at'
  named_scope :published, lambda { |*args| {:conditions => ["published_at IS NOT NULL AND contributions.published_at <= ?", (args.first || Time.zone.now)]}}
  named_scope :translated, lambda {{:conditions => "coalesce(title_#{I18n.locale}, '') <> '' AND coalesce(body_#{I18n.locale}, '') <> '' "}}
  named_scope :active, :joins => :department, :conditions => "active='t'"
  
  validates_associated :user
  validates_presence_of :user  
  validates_length_of :title_es, :title_eu, :title_en, :maximum => 255, :allow_blank => true
  
  validate :validates_presence_of_any_language
  validate :cannot_approve_if_empty_department
  
  # OBSOLETE: los admin ya no pueden crear propuestas ciudadanas
  # before_create :approve_if_admin

  before_save :set_notify_proposer
  
  translatable_columns :title, :body

  # preserve this order: it is necessary for WithAreaTag#sync_comments_area_tags to work
  # together with ActsAsTaggable#add_tags and ActsAsTaggable#add_tags and 
  # WithAreaTag#save_tag_list_and_area_tag_list (adding tags through tag_list instead of taggings)
  acts_as_taggable
  include Tools::WithAreaTag
  # / preserve this order
  
  include Tools::WithPoliticiansTags

  
  
  # Preserve this order
  include DraftUtils::InstanceMethods
  before_save :sync_draft_and_published_at # definido en draft_utils.rb
  # / Preserve this order
  
  include Tools::Content
  
  include Elasticsearch::Base

  # Valida que el título y el cuerpo no está vacío al menos en un idioma
  def validates_presence_of_any_language
    titles_empty = Proposal::LANGUAGES.collect {|l| l if self.send("title_#{l}").blank?}.compact
    if titles_empty.length == Proposal::LANGUAGES.length
      Proposal::LANGUAGES.each do |l|
        self.errors.add "title_#{l}", "El título no puede estar vacío" if self.send("title_#{l}").blank?
      end
      
    end
  end

  def cannot_approve_if_empty_department
    if self.status_changed? && self.status.eql?("aprobado") && !self.organization
      errors.add_to_base "No puedes aprobar la propuesta sin asignarle departamento antes"
    end
  end

  def organization
    department
  end
  
  # Indica si la propuesta está publicada
  def published?
    !published_at.nil? &&  published_at <= Time.now
  end
  
  # Indica si la propuesta está traducida a <em>lang_code</em>, idiomas definidos en Proposal::LANGUAGES
  def translated_to?(lang_code)
    self.send("title_#{lang_code}").present? && self.send("body_#{lang_code}").present?
  end
  
  # Indica si la propuesta está aprobada
  def approved?
    self.status.eql?('aprobado') || self.status.eql?('aprobado tras denuncia')
  end

  # Indica si la propuesta está rechazada
  def rejected?
    status.eql?('rechazado')
  end

  # Indica si la propuesta es spam
  def spam?
    status.eql?('spam')
  end

  # Indica si la propuesta está pendiente de aprobación
  def pending?
    status.eql?('pendiente')
  end

  # Nombre de la persona que hace la propuesta
  def author_name
    self.user ? self.user.public_name : self.name
  end
  
  def is_public?
    !self.published_at.nil?
  end
  
  def notify_proposer
    @notify_proposer
  end
  
  private
  
  # OBSOLETE: los admin ya no pueden crear propuestas ciudadanas
  # # Las propuestas de los administradores se aprueban automáticamente
  # def approve_if_admin
  #   if self.user_id
  #     if User.find(self.user_id).is_staff?
  #       self.status = 'aprobado'
  #       # self.governmental = true
  #     end
  #   end
  #   self.published_at = Time.now if self.published_at.nil?
  # end
  
  def set_notify_proposer
    if self.status_changed? && self.approved?
      @notify_proposer = true
    end 
  end
  
end
