#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Clase para los departamentos en los que se pueden clasificar los News y Events.
# Es subclase de Organization, por lo que su tabla es <tt>organizations</tt>
class Department < Organization
  has_many :organizations, :class_name => "Organization", :foreign_key => "parent_id"
  has_one :schedule
  
  has_many :members, :class_name => 'User', :foreign_key => "department_id"
  
  has_many :sorganizations, :order => "lower(name_#{I18n.locale})"
  
  has_many :subscriptions
  
  validates_presence_of :tag_name
  validates_format_of :tag_name, :with => /^_[a-z0-9_]+$/
  validates_uniqueness_of :name_es, :tag_name

  # Los departamentos sin GV (corresponde a internal_id=0)
  named_scope :without_gv, :conditions => "(internal_id IS NULL) OR (internal_id > 0)"
  named_scope :for_term, lambda { |*args| {:conditions => ["term = ?", args.first], :order => 'position'}}

  # Legislaturas: se usan para poder separar los departamentos por legislatura
  # Las entidades no tienen asignada ninguna legislatura
  TERMS = ["X", "IX"]

  # Cada departamento tiene asociado un tag oculto que se añade automáticamente
  def self.tag_names
    self.find(:all, :select => "tag_name").map {|d| d.tag_name}
  end

  # La etiqueta del departamento es su tag oculto sin el '_' al principio.
  # Se usa en las URL de acciones que muestran contentido sólo para un departamento.
  def label
    self.tag_name.gsub(/^_+/,'')
  end  
  
  # Devuelve el departamento con la etiqueta indicada.
  def self.find_by_label(dept_label)
    tag_name = "_"+dept_label.gsub(/[^\w_]/,'')
    dept = self.tag_names.include?(tag_name) ? self.find_by_tag_name(tag_name) : nil
    dept
  end
  
  # Array de departamentos en formato óptimo para hacer un select en un view.
  # El formato del output es 
  # <tt>[[[dep.nombre, dep.id], [[subdep1.nombre, subdep1.id], [subdep2.nombre, subdep2.id]...]], [[dep2.nombre, dep2.id], [...]]]</tt>
  def self.grouped_options
    opts = []
    self.find(:all, :order => "term, position").each do |dept|
      opts.push [[dept.name, dept.id], dept.organizations.map {|d| [d.name, d.id]}]
    end
    opts
  end
  
  def official_commenters
    candidates = self.members.approved
    list = candidates.select {|u| u.is_official_commenter?}
    list
  end
  
  def department_editors
    self.members.approved.find(:all, :conditions => "type='DepartmentEditor'")
  end
  
  def department_members_official_commenters
    self.members.approved.find(:all, :conditions => "type='DepartmentMember'").select {|u| u.is_official_commenter?}
  end
  
end
