#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

class Time
  def strftime_search
    self.strftime("%Y-%m-%dT%H:%M:%S")    
  end  
end   

module Elasticsearch::ElasticsearchParams
  I18n.locale ||= :es

  # Exclusions = ['y', 'de']
  ITEMS_PER_PAGE = 18
  
  MONTH_NAMES = {1 => 'Enero', 2 => 'Febrero', 3 => 'Marzo', 4 => 'Abril', 5 => 'Mayo', 6 => 'Junio', 7 => 'Julio', 8 => 'Agosto', 9 => 'Septiembre', 10 => 'Octubre', 11 => 'Noviembre', 12 => 'Diciembre'}
  
  DATE_RANGES = {
    '24h' => { 'from' => 24.hours.ago.strftime_search, 'to' => Time.now.strftime_search},
    '1w' => { 'from' => 1.week.ago.strftime_search, 'to' => Time.now.strftime_search},
    '1m' => { 'from' => 1.month.ago.strftime_search, 'to' => Time.now.strftime_search},
    '1y' => { 'from' => 1.year.ago.strftime_search, 'to' => Time.now.strftime_search},
    '4y' => { 'from' => 4.year.ago.strftime_search, 'to' => Time.now.strftime_search} 
  }
  # '2y' => { 'from' => I18n.localize(2.year.ago, :format => 'search'), 'to' => I18n.localize(Time.now, :format => 'search')}
    
  DATE_IN_HOURS = { 24 => '24h', 168 => '1w', 672 => '1m', 696 => '1m', 720 => '1m', 744 => '1m', 8760 => '1y', 8784 => '1y', 17520 => '2y', 17544 => '2y' }

  FACETS = {
    'type' => {
      :type => 'terms', :field => '_type', :size => 100, :show => {:no_facets => 5}
    },
    'areas' => {
      :type => 'terms', :field => 'areas_an', :size => 15, :show => {:no_facets => 5}
    },
    'tags' => {
      :type => 'terms', :field => 'tags_an', :size => 15, :show => {:no_facets => 5}
    },
    'politicians' => {
      :type => 'terms', :field => 'politicians_an', :size => 15, :show => {:no_facets => 5}
    },
    # 'organization' => {
    #   :type => 'terms', :field => 'organization_an', :size => Organization.count, :show => {:no_facets => 5}
    # },
    'date' => {
      :type => 'range', :field => 'published_at', :other => {'ranges' => [ DATE_RANGES['24h'], DATE_RANGES['1w'], DATE_RANGES['1m'], DATE_RANGES['1y'] ]}, :show => {:no_facets => 4}
    },
    'year' => {
      :type => 'terms', :field => 'year', :size => 100, :other => {'order' => 'reverse_term'}, :show => {:no_facets => 5}
    },
    'month' => {
      :type => 'terms', :field => 'month', :size => 12, :show => {:no_facets => 12}
    },
    'term' => {
      :type => 'terms', :field => 'term', :size => 5, :show => {:no_facets => 5}
    },
    'rango' => {
      :type => 'terms', :field => 'rango_an', :size => 15, :show => {:no_facets => 5}
    },
    'seccion' => {
      :type => 'terms', :field => 'seccion_an', :size => 15, :show => {:no_facets => 5}
    },
    'organo' => {
      :type => 'terms', :field => 'organo_an', :size => 15, :show => {:no_facets => 5}
    },
    'materias' => {
      :type => 'terms', :field => 'materias_an', :size => 15, :show => {:no_facets => 5}
    }
  }
  
  # An array is needed in order to preserve order
  # 'organization', 
  FACET_FIELDS = ['type', 'areas', 'tags', 'politicians', 'term', 'date', 'year', 'month', 'materias', 'dept', 'seccion', 'rango']
  
  # these are indexed in es|eu|en
  TRANSLATABLE_FACETS = ['tags', 'organization', 'month', 'areas', 'seccion', 'rango', 'organo', 'materias', 'term']
  
  BOPV_FACETS = ['seccion', 'rango', 'organo', 'materias']
  
end
