#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

module Elasticsearch::Search
  module ClassMethods
    def full_query(q, filters, from=0, sort=nil, only_title=false, size = Elasticsearch::Base::ITEMS_PER_PAGE)
      # , 'indices_boost' => {'irekia4' => 2.5, 'bopv' => 1}
      fq = { 'explain' => true, 'size' => size, 
             'from' => "#{from}", 'query' => query(q, filters, only_title), 'facets' => facets }
      fq.merge!(sort_string(sort))
      fq
    end
      
    def query(q, filters, only_title)
      if filters.present?
        { 'filtered' => { 'query' => query_string(q, only_title) }.merge(filters(filters)) }
      else
        # query_string(q.gsub(' AND ', ' ').gsub(/\s+/, ' AND '))
        query_string(q.gsub(/\s+/, ' '), only_title)
      end
    end
    
    def query_string(q, only_title)
      if only_title                           
        fields = ['title_es^7', 'title_eu^7', 'title_en^7']                           
      else
        fields = ['title_es^7', 'title_eu^7', 'title_en^7', 'body_es', 'body_eu', 'body_en', 'tags^6', 'year', 'month', 
          'organization^5', 'areas^5', 'politicians^5', 'speaker^4', 'public_name^7', 'role^7', 'subtitles_es', 'subtitles_eu', 'subtitles_en']  
        fields_bopv = ['materias', 'seccion', 'rango', 'organo', 'no_bol', 'no_disp', 'no_orden']
        fields = fields + fields_bopv
      end    
      { 'query_string' => { 'fields' => fields, 'query' => "#{q}", 'default_operator' => 'AND', 'analyzer' => 'default'} }
    end
    
    def filters(filters)
      f=[]
      filters.each do |term|
        case term[0]
        when 'date'
          f << {'range' => {'published_at' => Elasticsearch::Base::DATE_RANGES[term[1]]}}
        else  
          f << {'term' => {term[0] => term[1] } }
        end  
      end  
      {'filter' => { 'and' => f }}
    end
    
    def facets
      fh = Hash.new
      Elasticsearch::Base::FACETS.each_pair do |facet, att|
        value = {'field' => att[:field], 'size' => att[:size]}
        value.merge!(att[:other]) if att[:other].present?
        fh[facet] = { att[:type] => value }
      end  
      fh  
    end
    
    def sort_string(sort)
      if sort.present? && sort.eql?('date')
        {'sort' => {'published_at' => {'order' => 'desc'} }}        
      else                                                      
        {'sort' => ['_score', {'published_at' => {'order' => 'desc'} }]}
      end    
    end
    
    def do_elasticsearch_with_facets(q, filters, from, sort=nil, only_title=false)
      qs=full_query(q, filters, from, sort, only_title)
      # Elasticsearch::Base::log "QUERY STRING: #{qs.inspect}"
      es_uri = ELASTICSEARCH_URI
      if defined?(ELASTICSEARCH_BOPV_URI)
        es_uri << ",#{ELASTICSEARCH_BOPV_URI.split('/').last}"
      end
      begin
        uri=(URI.parse("#{es_uri}/_search?pretty=true"))
        Net::HTTP.start(uri.host, uri.port) do |http|
          headers = { 'Content-Type' => 'capplication/json'}
          data = qs.to_json
          response = http.send_request("GET", uri.request_uri, data, headers)
           # Elasticsearch::Base::log "ElasticSearch#search response: #{response.code} #{response.message} #{response.body}"
          Elasticsearch::Base::log "ElasticSearch#search response: #{response.code} #{response.message}"
          @code=response.code
          @body=response.body
        end  
      rescue => e
        Elasticsearch::Base::log "ElasticSearch#search ERROR #{e}"
        return []
      end    
      if @code.eql?('200')
        results=Hash.new
        results['hits']=[]
        
        results['total_hits'] = JSON.parse(@body)['hits']['total']
        JSON.parse(@body)["hits"]["hits"].each do |result| 
          # Elasticsearch::Base::log "### #{result['_type']}, #{result['_id']}, score #{result['_score']}"
          item=result['_type'].classify.constantize.find_by_id(result["_id"])
          if item.present?
            item.score = result['_score']
            item.explanation = result['_explanation']
            results['hits'].push(item)
          end  
        end
        
        results['facets']=Hash.new
        facets_body=JSON.parse(@body)['facets']
        # Facets
        Elasticsearch::Base::FACETS.keys.each do |att|
          if att.eql?('date')
            terms='ranges'
          else
            terms='terms'
          end
          results['facets'][att]=[]
          facets_body[att][terms].each do |facet|
            unless facet['count'].zero?
              case att
              when 'date'
                value=Elasticsearch::Base::DATE_IN_HOURS[((facet['from'] - facet['to']).to_i.abs/3600000)]
              else
                value=facet['term']  
              end    
              results['facets'][att].push([value, facet['count']])
            end  
          end  
        end
      else
        return []
      end
      results
    end
    
    def self.do_elasticsearch_for_ios(q, type, from)
      qs=full_query(q, nil, from, nil, 15)
      # Elasticsearch::Base::log "QUERY STRING: #{qs.inspect}"
      begin
        uri=(URI.parse("#{ELASTICSEARCH_URI}/#{type}/_search?pretty=true"))
        Net::HTTP.start(uri.host, uri.port) do |http|
          headers = { 'Content-Type' => 'application/json'}
          data = qs.to_json
          response = http.send_request("GET", uri.request_uri, data, headers)
          # Elasticsearch::Base::log "ElasticSearch#search response: #{response.code} #{response.message} #{response.body}"
          # Elasticsearch::Base::log "ElasticSearch#search response: #{response.code} #{response.message}"
          @code=response.code
          @body=response.body
        end  
      rescue => e
        Elasticsearch::Base::log "ElasticSearch#search ERROR #{e}"
        return []
      end    

      if @code.eql?('200')
        results=Hash.new
        results['hits']=[]
        
        results['total_hits'] = JSON.parse(@body)['hits']['total']
        JSON.parse(@body)["hits"]["hits"].each do |result|
          # Elasticsearch::Base::log "### #{result['_type']}, #{result['_id']}, score #{result['_score']} "
          item=result['_type'].classify.constantize.find_by_id(result["_id"])
          results['hits'].push(item) if item.present?
        end
      else
        return []
      end
      results
    end
  end

  module ViewHelpers
    ActionView::Base.send :include, ViewHelpers
    
    def will_paginate_search(collection, total)
      output=[]
      current=@page.to_i
      per_page=Elasticsearch::Base::ITEMS_PER_PAGE
      last_page=lastpage(total, per_page)
      min=minmaxpage(current, last_page, 'min')
      max=minmaxpage(current, last_page, 'max')

      # Previous page
      aux = []
      if min < current
        aux << link_to_page(t('search.anterior'), current-1)
      else
        aux << content_tag(:span, t('search.anterior'))
      end  
      output << content_tag(:li, aux, :class => 'prev_page')

      # Page numbers
      aux=[]
      if min > 1 
        aux << link_to_page(1, 1)
        aux << content_tag(:span, " ... ", :class => 'gap')
      end  
      x=min
      while x <= max 
        if x.to_i == current.to_i
          aux << content_tag(:span, x, :class => 'current')
        else
          aux << link_to_page(x.to_s, x)
        end  
        x=x+1
      end  
      x=x-1

      if x == 1 || total <= Elasticsearch::Base::ITEMS_PER_PAGE
        ""
      else  
        if max < last_page
          aux << content_tag(:span, " ... ", :class => 'gap')
          aux << link_to_page(last_page.to_s, last_page)
        end  
        output << content_tag(:li, aux.join('   '), :class => 'page_numbers')
        
        # Next page
        aux=[]
        if max > current
          aux << link_to_page(t('search.siguiente'), current + 1)
        else
          aux << content_tag(:span, t('search.siguiente'))
        end  
        output << content_tag(:li, aux, :class => 'next_page')
        content_tag(:div, content_tag(:ul, output), :class => 'pagination')
      end  
    end
    
    def link_to_page(text, page)
      # link_to_remote(text, :url => search_index_path(:criterio_id => @criterio.id, :page => page), :method => :post)
      link_params={:page => page}
      if @criterio.present? && !@criterio.new_record? 
        link_params.merge!(:id => @criterio.id) 
        link_uri='search_path'
      else
        link_uri = 'new_search_path'
      end    
      link_params.merge!(:sort => @sort) if @sort.present?
      link_to(text, send(link_uri, link_params))  
    end  
    
    def lastpage(total, per_page)
      last_page = total/per_page + 1
      if total%per_page == 0
        last_page = last_page - 1
      end  
      return last_page
    end

    def minmaxpage(current, total_pages, option)
      min = current - 2
      if min < 1
        min = 1
        max = 1 + 4
        if max > total_pages then max = total_pages end
      end
      max = min + 4
      if max > total_pages
        max = total_pages
        min = max - 4
        while min < 1
          min = min + 1
        end
      end
      if option.eql?('min') then return min else return max end
    end
  end  
end
