#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Clase para las Noticias. Es subclase de Document, por lo que usa la tabla <tt>documents</tt>
class News < Document
  # Las noticias, igual que los debates tienen directorio multimedia
  validates_format_of :multimedia_dir, :with => /^(\d{4}\/\d{2}\/\d{2}\/)[a-z0-9_]+$/i, :allow_blank => true
  
  validates_presence_of :organization_id
  
  has_many :ratings, :as => :rateable, :dependent => :destroy
  has_many :comments, :as => :commentable, :order => "created_at DESC", :dependent => :destroy
  has_many :tweets, :class_name => "DocumentTweet", :foreign_key => "document_id", :dependent => :destroy
  has_many :sent_tweets, :class_name => "DocumentTweet", :foreign_key => "document_id", :conditions => "tweeted_at IS NOT NULL", :dependent => :destroy

  has_many :related_events, :as => :eventable, :dependent => :destroy
  has_many :events, :through => :related_events  
  
  has_one :debate, :dependent => :nullify

  has_attached_file :cover_photo, :styles => PHOTOS_SIZES,
                    :url  => "/assets/cover_photos/:id/:style/:sanitized_basename.:extension",
                    :path => ":rails_root/public/assets/cover_photos/:id/:style/:sanitized_basename.:extension"
                    
  validates_attachment_size :cover_photo, :less_than => 5.megabytes
  validates_attachment_content_type :cover_photo, :content_type => ['image/jpeg', 'image/pjpeg', 'image/png', 'image/x-png', 'image/gif']  
  validates_length_of :cover_photo_alt_es, :cover_photo_alt_eu, :cover_photo_alt_en,
                      :maximum => 255 , :allow_blank => true
  
  translatable_columns :cover_photo_alt


  named_scope :listable, :conditions => ["consejo_news_id IS NULL", true]
  named_scope :exported_to_enet, :conditions => "exported_to_enet_at IS NOT NULL"
  
  before_save :set_and_create_multimedia_path
  before_save :disable_unnecessary_fields
  before_save :assign_department_tag # definido en document.rb
  before_save :schedule_tweets
  before_update :check_only_one_a_featured
  before_update :check_only_four_b_featured
  before_update :nullify_empty_featured

  # See http://thewebfellas.com/blog/2008/11/2/goodbye-attachment_fu-hello-paperclip#comment-2415
  def attachment_for name
    @_paperclip_attachments ||= {}
    @_paperclip_attachments[name] ||= Attachment.new(name, self, self.class.attachment_definitions[name])
  end
  
  include CachedMethods
  
  include Floki
  
  # Para euskadi.net
  def subtitle(lang_code=I18n.locale)
    if self.send("body_#{lang_code}").present? && m = self.send("body_#{lang_code}").match(/<.+class=\"r01Subtitular\".*?>(.+?)</) 
      return m[1]
    else
      return ""
    end
  end
  
  def entradilla(lang_code=I18n.locale)
    if self.send("body_#{lang_code}").present? && m = self.send("body_#{lang_code}").match(/<.+class=\"r01Entradilla\".*?>(.+?)</) 
      return m[1]
    else
      return ""
    end
  end
  
  def cuerpo(lang_code=I18n.locale)
    c = self.pretty_body(lang_code).dup
    if self.subtitle.present?
      c = c.gsub(/<[^<]+class="r01Subtitular".*?>#{Regexp.escape(self.subtitle(lang_code))}<\/.+?>/, '')
    end
    if self.entradilla.present?
      c = c.gsub(/<[^<]+class="r01Entradilla".*?>#{Regexp.escape(self.entradilla(lang_code))}<\/.+?>/, '')
    end
    
    return c
  end
  # / Para euskadi.net
  
  def self.ley_vivienda
    begin
      self.find(4597)
    rescue
      nil
    end
  end
  
  def self.most_recent
    self.published.translated.listable.find :all, 
      :include => :organization, 
      :order => "published_at DESC", 
      :limit => 29
  end
  
  def self.most_recent_with_image
    self.most_recent.select {|n| n.has_video? || n.has_cover_photo? || n.has_photos?}
  end
  
  def self.featured_a
    featured_a = self.published.translated.listable.find :first, 
      :conditions => "featured='1A'",
      :order => "published_at DESC"
    
    # Si no hay ninguna, cogemos la más reciene con foto
    if featured_a.nil?
      featured_a = self.find(self.most_recent_with_image.shift.id) if self.most_recent_with_image.length > 0
    end
    return featured_a
  end
  
  def self.featured_4b
    featured_b = self.published.translated.listable.find :all, 
      :conditions => "featured='4B'",
      :order => "published_at DESC"
    most_recent_without_featured_1a = self.most_recent_with_image.delete_if {|n| n.id == self.featured_a.id}
    featured_b = featured_b + most_recent_without_featured_1a[0..(4-featured_b.length-1)] if featured_b.length < 4
    return featured_b
  end                             
  
  def self.related_coefficient
    0.1
  end
  
  def is_consejo_news
    !consejo_news_id.nil? && consejo_news_id != 19841984
  end
  
  def is_programa_de_gobierno_news
    consejo_news_id == 19841984
  end  

  # Noticias que corresponden a Debate
  def debate_id
    self.debate.present? ? self.debate.id : nil
  end
  
  def debate_id=(d_id)
    if debate = Debate.find(d_id)
      self.debate = debate
    end
    true
  end

  protected
    # No todas las columnas de la tabla documents se utilizan en las noticias,
    # por lo que nos aseguramos de que están vacías.
    # Se llama desde before_save
    def disable_unnecessary_fields
      self.starts_at = nil
      self.ends_at = nil
      self.place = nil
      self.lat = nil
      self.lng = nil
      self.location_for_gmaps = nil
      self.stream_flow_id = false
      self.journalist_alert_version = 0
      self.staff_alert_version = 0
    end
    
    # Programa el auto-tweeteo de las noticias publicadas en irekia, con fecha igual a la fecha de publicacion en la web.
    # Se llama desde before_save
    def schedule_tweets
      # Si ha pasado más de un mes desde su publicación, no lo twitteamos
      if self.published_at && self.published_at >= 1.month.ago
        # Primero borramos los tweets pendientes de este evento, por si ha cambiado la fecha
        DocumentTweet.delete_all(["document_id= ? AND tweeted_at IS NULL", self.id])

        Event::LANGUAGES.each do |l|
          if self.published? && self.translated_to?(l.to_s) && !DocumentTweet.exists?(:document_id => self.id, :tweet_locale => l.to_s)
            self.tweets.build(:tweet_account => "irekia_news", :tweet_at => self.published_at, :tweet_locale => l.to_s)
          end
        end    
      end
    end

    # Garantiza que solo habrá una noticia destacada 'A' en Irekia.
    # Se llama before_update
    def check_only_one_a_featured
      Document.update_all("featured=null", "featured='1A'") if featured.eql?('1A') && featured_changed?
    end
    
    # Garantiza que solo habrá cuatro noticias destacadas '4B' en Irekia.
    # Si hay más, se quitan las más antiguas
    # Se llama before_update
    def check_only_four_b_featured
      if featured.eql?('4B') && featured_changed?
        if News.featured_4b.length == 4
          News.featured_4b.last.update_attribute(:featured, nil)
        end
      end
    end
    
    def nullify_empty_featured
      self.featured = nil if self.featured == ''
    end
    
  # private
  # def files_of_type(type, extension, path)
  #   files = {}
  #   if self.send("#{type}_path").present?
  #     files_in_dir = (Dir.glob(path + self.send("#{type}_path") + "_e[sun].#{extension}") + Dir.glob(path + self.send("#{type}_path") + ".#{extension}")).collect {|p| p.sub(/^#{path}/,'')}
  #     Document::LANGUAGES.each do |l|
  #       files[l.to_sym] = files_in_dir.select {|c| c.match(/_#{l}\.#{extension}$/)}
  #     end
  #     files[:common] = files_in_dir - (files[:es] + files[:eu] + files[:en])
  #   end
  #   return files
  # end

end
