#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Clase para organismos a los que pertenecen los Event
class Organization < ActiveRecord::Base
  has_many :events
  belongs_to :parent, :class_name => "Organization", :foreign_key => "parent_id"

  translatable_columns :name
  
  validates_presence_of :name_es, :enet_workarea
  validates_uniqueness_of :name_es
  # validates_format_of :enet_workarea, :with => /^[a-z0-9_]+\|[a-z0-9_]+$/

  acts_as_tree :order => :position
  acts_as_list :scope => :parent

  before_save :set_type                 
  after_update :reindex_related_documents

  named_scope :active, :conditions => {:active => true}
  
  # Si es departamento, devuelve el mismo, si es organizacion, devuelve su parent
  def department
    if self.is_a?(Department)
      self
    elsif self.is_a?(Entity)
      self.parent
    end
  end

  # Enlace a la página de la organización en la guía de comunicación
  def gc_link
    self.gc_id.present? ? "http://www2.irekia.euskadi.net/#{I18n.locale}/entities/#{self.gc_id}" : nil
  end  
  
  protected 
    
    # Clasifica el organismo como Departament o Entity en funcion de si es
    # de primer nivel o de segundo
    def set_type
      if self.parent_id == nil
        self.type = "Department"
      else
        self.type = "Entity"
      end
    end
    
    def reindex_related_documents
      if name_es_changed? || name_eu_changed? || name_en_changed?  
        i=0     
        all_doc = Document.find(:all, :conditions => {:organization_id => self.id})
        while i < all_doc.length do
          Document.find(:all, :conditions => {:organization_id => self.id}, :limit => 100, :offset => i).each do |doc|
            doc.update_elasticsearch_server if doc.respond_to?(:update_elasticsearch_server)            
          end                                                                                                
          i += 100
        end  
      end  
    end
  
end
