#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Clase para las páginas de información 
class Page < Document
  validates_presence_of :multimedia_dir
  validates_format_of :multimedia_dir, :with => /^[a-z0-9_]+$/i
  validates_presence_of :organization_id
  
  before_save :set_and_create_multimedia_path  
  before_save :disable_unnecessary_fields
  
  has_one :debate, :dependent => :nullify
  
  ABOUT = [{:label => "about", :title => I18n.t('site.irekia')},
           {:label => "tos", :title => I18n.t('site.condiciones_uso')}, 
           {:label => "best_practices", :title => I18n.t('site.guia_buenas_practicas')},
           {:label => "privacy", :title => I18n.t('site.politica_privacidad')},
           {:label => "how_to_participate", :title => I18n.t('site.como_participar')},
           {:label => "faq", :title => I18n.t('site.faq')}, 
           {:label => "colaborations", :title => I18n.t('site.aplicaciones_interes')}]
  
  PREDEFINED = [{:label => "legal_iphone", :title => "Aviso Legal iPhone"},
                {:label => "prop_int_iphone", :title => "Propiedad intelectual iPhone"},
                {:label => "openirekia", :title => "OpenIrekia"},
                {:label => "help", :title => "Ayuda"}]
  
  class << self
    (ABOUT + PREDEFINED).each do |page|
      define_method page[:label] do
        self.tagged_with("_#{page[:label]}")
      end
    end
  end
    
  def comments
    []
  end
  
  # See http://thewebfellas.com/blog/2008/11/2/goodbye-attachment_fu-hello-paperclip#comment-2415
  def attachment_for name
    @_paperclip_attachments ||= {}
    @_paperclip_attachments[name] ||= Attachment.new(name, self, self.class.attachment_definitions[name])
  end
  
  # Páginas que corresponden a Debate
  def debate_id
    self.debate.present? ? self.debate.id : nil
  end
  
  def debate_id=(d_id)
    if debate = Debate.find(d_id)
      self.debate = debate
    end
    true
  end
  
  private

    def self.tagged_with(tag)
      pages = Page.find_tagged_with(tag, :scope => :private, :limit => 1)
      if pages.length > 0
        return pages.first
      else
        raise ActiveRecord::RecordNotFound
      end
    end

    # Se usa el método definido en Tools::Multimedia
    #
    # # Crea el directorio donde irán los contenidos multimedia de esta página.
    # # Se llama desde before_create
    # def set_and_create_multimedia_path
    #   self.multimedia_path = "paginas/#{self.multimedia_dir}/"
    #   FileUtils.mkdir_p(Document::MULTIMEDIA_PATH + self.multimedia_path)
    # end
    
    # Las páginas comparten tabla con News y Event y algunas de las columnas
    # no son necesarias para las páginas. Aquí se vacían. Se llama desde before_save
    def disable_unnecessary_fields
      self.has_comments = false
      self.comments_closed = true
      self.has_comments_with_photos = false
      self.has_ratings = false
      # self.video_path = nil
      # self.audio_path = nil
      # self.photos_path = nil
      # self.files_path = nil
      self.comments_count = 0
      self.starts_at = nil
      self.ends_at = nil
      self.place = nil
      self.speaker_es = nil
      self.speaker_eu = nil
      self.speaker_en = nil
      self.lat = nil
      self.lng = nil
      self.location_for_gmaps = nil

      self.cover_photo_file_name = nil
      self.cover_photo_content_type = nil
      self.cover_photo_file_size = nil
      self.cover_photo_updated_at = nil

      self.stream_flow_id = false
      self.journalist_alert_version = 0
      self.staff_alert_version = 0
    end
    
    
end
