#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Clase para las fotos de la fototeca
class Photo < ActiveRecord::Base
  has_many :album_photos, :dependent => :destroy
  has_many :albums, :through => :album_photos
  belongs_to :document
  
  validates_presence_of :title_es, :title_eu, :title_en, :file_path, :on => :create
  validates_uniqueness_of :file_path
  validates_length_of :title_es, :title_eu, :title_en, :city, :province_state, :country, :maximum => 255, :allow_blank => true
  validates_format_of :dir_path, :with => /^[a-z0-9_\/]+$/i, :message => 'El directorio sólo puede tener letras sin tildes, números, "_" y "/".<br/> Ni espacios, ni tildes, ni ñ.', :on => :create
  
  # Para poder validar que al importar las fotos ponen un directorio sin espacios ni tildes
  attr_accessor :dir_path
  
  named_scope :ordered_by_title, lambda {{:order => "title_#{I18n.locale} DESC"}}
  named_scope :published, :joins => :albums, :conditions => ["albums.draft=?", false]
    
  # Multimedia directories defined in RAILS_ROOT/config/multimedia_directories-*.rb
  include PhotoPaths
  include Floki
    
  acts_as_taggable

  include Tools::WithAreaTag
    
  translatable_columns :title
  
  def date
    date_time_original || created_at
  end
  
  # Devuelve la versión de tamaño <em>size</em> de la foto actual.
  # ==== Parámetros
  # * <tt>size:</tt> tamaño deseado. Los valores permitidos son los keys de Tools::Multimedia::PHOTOS_SIZES, definido en el módulo #PhotoPaths
  # * <tt>path:</tt> no se usa
  def version(size=:n70, path=:absolute)
    if Tools::Multimedia::PHOTOS_SIZES.keys.include?(size)
      version_path = "#{File.dirname(self.file_path)}/#{size}/#{File.basename(self.file_path)}"
    else
      version_path = self.file_path
    end
    if path.eql?(:absolute)
      return Photo::PHOTOS_URL + version_path
    else
      return version_path
    end
  end
  
  def is_public?
    true
  end

  # Indica si la foto está publicada. Lo está si está en algún album publicado
  def published?
    albums = self.albums
    albums.length > 0 && albums.collect(&:draft).include?(false)
  end
  
  def draft?
    !published?
  end
  alias_method :draft, :draft?
  
  # Para poder reusar /documents/_share
  def body
    ""
  end
  
  # Should try to use Paperclip::Geometry instead
  # returns [width, height], if possible
  def geometry_from_file
    file = Photo::PHOTOS_PATH + version(:original, :relative)
    geometry = begin
                 Paperclip.run("identify", %Q[-format "%wx%h" "#{file}"])
               rescue => err
                 logger.info "No he podido coger las dimensiones de #{self.id}: #{err}"
                 ""
               end
    if match = (geometry.match(/\b(\d*)x?(\d*)\b([\>\<\#\@\%^!])?/))
      match.to_a[1,2]
    else
      logger.info "No he podido coger las dimensiones de #{self.id}: #{geometry}"
      ""
    end
    # parse(geometry) ||
    #   raise(NotIdentifiedByImageMagickError.new("#{file} is not recognized by the 'identify' command."))
  end

  # True if the dimensions represent a square
  def square?
    height == width  if !(width.blank? && height.blank?)
  end

  # True if the dimensions represent a horizontal rectangle
  def horizontal?
    height < width if !(width.blank? && height.blank?)
  end

  # True if the dimensions represent a vertical rectangle
  def vertical?
    height > width if !(width.blank? && height.blank?)
  end

  def orientation
    self.vertical? ? 'portrait' : 'landscape'
  end

  # The aspect ratio of the dimensions.
  def aspect
    width.to_f / height.to_f if !(width.blank? && height.blank?)
  end
  
  before_save :set_geometry
  def set_geometry
    self.width, self.height = self.geometry_from_file
  end
  
end
