#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Clase para las propuestas ciudadanas
class Proposal < Contribution
  include Floki
  alias_method :author, :user

  has_many :votes, :as => :votable, :dependent => :destroy
  
  has_many :arguments, :as => :argumentable, :dependent => :destroy
  
  has_one :proposal_data, :dependent => :destroy
  
  has_many :attachments, :as => :attachable, :class_name => '::Attachment', :dependent => :destroy, :foreign_key => 'attachable_id'
  
  named_scope :featured, :conditions => ["featured=?", true]
  
  validates_length_of :url, :name, :email, :maximum => 255, :allow_blank => true
  validates_format_of :url, :with => /(^$)|(^(http|https):\/\/[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(([0-9]{1,5})?\/.*)?$)/ix, 
                            :message => I18n.t('comments.blog_url_incorrect'), :allow_blank => true
  # validates_format_of :url, :with => URI::regexp(%w(http https)), :allow_blank => true
  
  validates_presence_of :name #, :email
  validates_format_of :email, :with => /^([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})$/i, :allow_blank => true

  validates_presence_of :area_id
  
  before_create :check_user_is_citizen
  
  # Las propuestas se pueden votar
  include Tools::Votable

  # See http://thewebfellas.com/blog/2008/11/2/goodbye-attachment_fu-hello-paperclip#comment-2415
  def attachment_for name
    @_paperclip_attachments ||= {}
    @_paperclip_attachments[name] ||= Attachment.new(name, self, self.class.attachment_definitions[name])
  end

  # Para compatibilidad con los documentos.
  # Nos permite compartir el partial de attachments entre Document y Proposal
  def class_name
    self.class.to_s
  end  


  %w(image image_cache remove_image).each do |method|
    define_method method do
      self.proposal_data.send(method) if self.proposal_data
    end
  end
  
  def image_cache=(img)
    self.build_proposal_data unless self.proposal_data
    self.proposal_data.image_cache = img
  end
  
  def image=(img)
    self.build_proposal_data unless self.proposal_data    
    self.proposal_data.image = img
  end
  
  def remove_image=(val)
    if val.to_i == 1
      self.proposal_data.remove_image = true
    end
  end
  
  # Cuerpo de la propuesta, quitando el separador de la entradilla
  def pretty_body
    body ? body.sub(/<p.*>###<.*\/p>/, '') : ""
  end

  # Indica si la propuesta tiene foto
  def has_image?
    image.present?
  end
  alias_method :has_photo?, :has_image?
  
  # Indica si este documento tiene documentos adjuntos 
  def has_files?
    attachments.count > 0
  end
  
  def attached_files(lang=I18n.locale)
    self.attachments
  end
  
  before_validation :set_name_and_email
  before_validation :nullify_url_if_necessary  
  before_save :set_notify_department

  after_update :save_proposal_data
  # Copia el nombre e email del usuario en la propuesta. Ya no es necesario. Se llama desde before_validation
  def set_name_and_email
    if self.user
      self.name = self.user.public_name
      self.email = self.user.email
      self.url = self.user.url
    end
  end
  
  # Si el usuario deja la URL con el valor por defecto, se quita porque no es 
  # una URL válida
  def nullify_url_if_necessary
    self.url = nil if self.url.eql?("http://")
    return true
  end

  def notify_department
    @notify_department
  end
    
  def to_yaml( opts = {} )
    if self.image.present?
      FileUtils.cp(self.image.path, File.join(RAILS_ROOT, 'data', 'fotos_propuestas_gov'))
    end
    YAML.quick_emit( self.id, opts ) { |out|
      out.map( taguri, to_yaml_style ) { |map|
        atr = @attributes.dup
        atr["area_tag_name"] = self.area.present? ? self.area.tag_name_es : ''
        atr["filename4photo"] = self.image.present? ? File.basename(self.image.path) : ''
        map.add("attributes",  atr)
      }
    }
  end
    
  private
  def set_notify_department
    if (self.organization_id_changed? || self.status_changed?) && self.approved?
      @notify_department = true
    end 
  end
  
  def save_proposal_data
    self.proposal_data.save if self.proposal_data
  end
  

  # Sólo los usuarios que son de tipo ciudadano pueden crear propuestas ciudadanas.
  def check_user_is_citizen
    unless self.user.is_citizen?
      self.errors.add(:base, 'No puede crear propuestas ciudadanas')
      return false
    end
    
    true
  end    
  
end
