#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

# Eventos de las agendas privadas.
class ScheduleEvent < ActiveRecord::Base  
  include Tools::Event
  include Tools::Calendar::InstanceMethods
  
  extend Tools::Calendar::ClassMethods

  belongs_to :schedule
  belongs_to :creator, :class_name => "User", :foreign_key => "created_by"
  belongs_to :last_editor, :class_name => "User", :foreign_key => "updated_by"
  
  validates_presence_of :title
  validates_presence_of :starts_at
  validates_presence_of :ends_at
  validates_presence_of :schedule_id  
  validate :ends_later_than_it_starts, :if => Proc.new {|event| event.ends_at && event.starts_at}
  validates_length_of   :location_for_gmaps, :maximum => 500, :allow_blank => true
  validates_length_of   :title, :place, :city, :speaker, :maximum => 255, :allow_blank => true
  
  before_save :fill_lat_lng_data
  before_save :set_l_attends_if_necessary

  named_scope :l_attends, :conditions => {:l_attends => true}

  # Atributos y métodos para compatibilidad con el modelo Event
  attr_accessor :organization_id
  attr_accessor :area_id
  attr_accessor :politicians_tag_list

  attr_accessor :has_photographers, :has_journalists, :all_journalists, :only_photographers

  ['is_public?', 'published?', 'irekia_coverage', 'irekia_coverage?', 'deleted?', 'has_photographers?', 'all_journalists?', 
    'only_photographers?', 'has_journalists?', 'irekia_coverage_photo?', 'irekia_coverage_video?', 'irekia_coverage_audio?', 
    'irekia_coverage_article?', 'streaming_live?', 'alert_this_change', 'alertable'].each do |mn|
    define_method mn do
      return false
    end
  end
  
  ['is_destroyable?', 'is_private?', 'private_event'].each do |mn|
    define_method mn do
      return true
    end
  end
  
  
  # Los eventos de las agendas privadas nunca tienen periodistas ni fotógrafos.
  ['show_in','organization_id', 'is_private', 'alertable', 'alert_this_change', 'published_at'].each do |m|
    define_method "#{m}=" do |val|
      nil
    end
  end
  
  ['irekia_tag', 'show_in', 'published_at'].each do |m|
    define_method m do
      return nil
    end
  end
  
  # Para compatibilidad con Event, cuando salen eventos de los tipos en una lista.
  def politicians
    []
  end

  # Devuelve la organización que corresponde al evento. Si no existe devuleve un objeto de la clasе <tt>Organization</tt>.  
  def organization
    self.schedule.department || Organization.new(:name => '')
  end

  # Devuelve el departamento que corresponde al evento. Si no existe devuleve nil.  
  def department
    self.schedule.department || nil
  end

  # Devuelve el id de la organización que corresponde аl evento о <tt>nil</tt>
  # si la agenda del evento no pertenece a ningún departamento.
  def organization_id
    self.schedule.department_id.blank? ? nil : self.schedule.department_id
  end
  
  
  # Devuelve siempre <tt>true</tt> aunque se intente asignar otro valor a este campo.
  def private_event=(val)
    true
  end
  
  # Los campos title y body están traducidos para los eventos compartidos pero no lo son para los privados.
  # Por esto hay que definir a mano los métodos <tt>title_[lang]</tt> y <tt>body_[lang]</tt>
  Event::LANGUAGES.each do |l|
    ['body', 'title'].each do |m|
      define_method "#{m}_#{l}" do
        return self.send(m)
      end

      define_method "#{m}_#{l}=" do |val|
        self.send("#{m}=", val)
      end
    end
  end


  # Devuelve el evento en formato _ical_.
  def to_ics(cal, &block)
      if self.deleted?
        cal.add_x_property('METHOD', 'CANCEL')
      else
        cal.add_x_property('METHOD', 'REQUEST')
      end
      cal.event do |event|
        event.uid = "uid_" + self.id.to_s + "schedule_event_irekia_euskadi_net"
        event.sequence = self.staff_alert_version
        event.summary = self.title
        event.description = self.body.strip_html #.gsub(/\n/, '\n')
        event.dtstart =  self.starts_at.getutc.strftime("%Y%m%dT%H%M%SZ")
        event.dtend = self.ends_at.getutc.strftime("%Y%m%dT%H%M%SZ")
        event.location = self.pretty_place
        event.url = yield[:url] if block.present?
        event.created = self.created_at.getutc.strftime("%Y%m%dT%H%M%SZ")
        event.last_modified = self.updated_at.getutc.strftime("%Y%m%dT%H%M%SZ")
        event.dtstamp = Time.now.getutc.strftime("%Y%m%dT%H%M%SZ")
        if self.deleted?
          event.status = "CANCELLED"
        else
          event.status = "CONFIRMED"
        end
      end
  end

  # Devuelve el nombre de la agenda a la que pertenece el evento.
  def schedule_name
    self.schedule.name
  end


  # Copia los datos del evento <tt>evt</tt> a <tt>self</tt>.
  def copy_from(evt)
    evt_attr = evt.attributes
    evt_attr.delete('id')
    evt_attr.each do |at, value|
      if self.respond_to?("#{at}=")
        self.send("#{at}=", value)
      end
    end
    
    if evt.is_a?(Event)
      ['title', 'speaker'].each do |fld|
        self.send("#{fld}=", [evt.send("#{fld}_es"), evt.send("#{fld}_eu"), evt.send("#{fld}_en")].uniq.compact.join(" | "))
      end
      
      self.body ||= ''
      # if evt.organization.is_a?(Department)
      #   self.body += "\n\nDepartamento: #{evt.organization.name}"
      # else
      #   self.body += "\n\nOrganismo: #{evt.organization.name} (#{evt.organization.department.name})"
      # end      
    end
    
    self    
  end
 
  # Indica si el evento pertenece a la agenda del Lehendakari.
  def belongs_to_l_schedule?
    self.schedule_id.eql?(Schedule.lehendakari.id)
  end
 
  # Indica si el evento se puede pasar a otra agenda.
  # 
  # Los eventos de las agendas que no tienen departamento, no pueden cambiar de agenda 
  # ni pueden transformarse en eventos compartidos.
  #
  def visibility_can_change?
    !self.schedule.department.nil?
  end
 
  # Para compatibilidad con Event
  def attendee_names
    ""
  end


 
  private

  # Asigna el valor del campo <tt>l_attends</tt> dependiendo de la agenda a la que pertenece el evento.
  def set_l_attends_if_necessary
    self.l_attends ||= false
  
     if self.belongs_to_l_schedule?
       self.l_attends = true
     end
     
    true
  end
  
end
