#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

class Sorganization <ActiveRecord::Base
  has_many :snetworks, :dependent => :destroy, :order => :position
  belongs_to :department
  
  has_attached_file :icon, :styles => {:tiny => "39x39#"},
                    :url  => "/assets/sorganizations/:id/:style/:sanitized_basename.:extension",
                    :path => ":rails_root/public/assets/sorganizations/:id/:style/:sanitized_basename.:extension"
  
  translatable_columns :name
  
  validates_presence_of :name
  validate :icon_size_valid?
  validates_associated :snetworks
  
  after_update :save_snetworks
  
  def icon_size_valid?
    begin
      # El tamaño del icono debe ser 39x37
      unless self.icon.tempfile.nil?
        size=Paperclip::Geometry.from_file(self.icon.tempfile.path)
        if size.width.to_i != 39 || size.height.to_i != 39
          self.errors.add(:icon, 'debe tener un tamaño de 39x39px')
          false
        end
      end
    rescue NotIdentifiedByImageMagickError, NoMethodError => err
      logger.error "Error checking icon size: #{err}"
    end
  end

  def new_snetworks_attributes=(values)
    values.each_pair do |key, snet|
      self.snetworks.build(snet)
    end  
  end
  
  def existing_snetworks_attributes=(values)
    self.snetworks.reject(&:new_record?).each do |snet|      
      attributes=values[snet.id.to_s]
      if attributes['deleted'].to_i == 1
        snetworks.delete(snet)
      else
        snet.attributes=attributes  
      end  
    end  
  end
  
  def save_snetworks
    snetworks.each do |snetwork|
      snetwork.save
    end
  end
  
end
