#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

ActionController::Routing::Routes.draw do |map|
  map.irekia_iframe 'iframe/:action/:id', :controller => 'irekia_iframes', :format => 'html'
  map.irekia_iframe_with_locale ':locale/iframe/:action/:id', :controller => 'irekia_iframes', :format => 'html',  :requirements => {:locale => /(es|eu|en)/}

  map.resources :oauth_consumers,:member=>{:callback=>:get}

  map.admin '/admin/', :controller => 'sadmin/news', :action => 'home'
  
  map.s3_config 's3_config', :controller => "admin/misc", :action => "s3_config", :locale => "es"
  
  map.namespace :sadmin, :path_prefix => ':locale/sadmin', :requirements => {:locale => /(es|eu|en)/} do |sadmin|
    sadmin.resources  :news, :collection => {:published => :get, :export_for_enet => :post}, :has_many => [:subtitles]
    sadmin.resources  :attachments
    sadmin.resources  :events, 
                      :collection => {:myfeed => :get, :calendar => :any, :list => [:get, :post], :week => [:get, :post]},
                      :member     => {:mark_for_deletion => :get, :delete => :put, :unrelate => :post} 
    sadmin.resources  :schedule_events, 
                      :only   => [:new, :show, :edit, :create, :update, :destroy]
    sadmin.resource :account, :controller => 'account', :only => [:edit, :show, :update]
  end  
  
  
  map.namespace :admin, :path_prefix => ':locale/admin', :requirements => {:locale => /(es|eu|en)/} do |admin|
    admin.resources :room_managers, :only => [:index]

    admin.resources :videos, :collection => {:find_video => :post, :create_with_subtitles => :post}, 
                             :member => {:update_subtitles => :post, :delete_subtitles => :post, :publish => :put}

    admin.resources :trees do |trees|
      trees.resources :categories, :collection => {:sort => :post}, :member => {:edit_tags => :get}
    end

    admin.resources :documents,
      :member => {:comments => :get, :update_comments_status => :put, :edit_tags => :get, :edit2 => :get, :publish => :put}

    admin.resources :proposals,
      :member => {:comments => :get, :edit_common => :get, :publish => :put, :update_status => :put,
                  :reject => :get, :do_reject => :put},
      :collection => {:arguments => :get}
    
    admin.resources :arguments, :member => {:approve => :put}

    # HQ
    # admin.resources :questions, :except => [:new, :create],
    #   :member => {:comments => :get, :edit_common => :get, :publish => :put, :update_status => :put}

    admin.resources :debates,
      :member => {:comments => :get, :publish => :put, :update_status => :put, 
                  :common => :get, :edit_common => :get, :translations => :get},
      :collection => {:arguments => :get} do |debate|
        debate.resources :entities, :controller => "debate_entities", :collection => {:sort => :post}
    end
    
    admin.resources :outside_organizations

    admin.resources :images

    admin.resources :materials

    map.pending_admin_comments 'pending', :controller => "admin/comments", :action => 'pending', :format => "iphone"

    admin.resources :comments, 
                    :collection => {:update_comments_status => :put}, 
                    :member => {:mark_as_spam => :put, :mark_as_ham => :put, :update_status => :put, 
                                :comments_on_item => :get, :zforward_form => :post, :zforward => :post,
                                :reject => :get, :do_reject => :put},
                    :has_one => [:comment_response]
    
    
    
    admin.resources :tags, :only => [:index, :update], 
                           :collection => {:find_duplicates => :get, :merge => :post},
                           :member => {:set_tag_name => :get, :set_tag_kind => :any}

    admin.resources :users, :member => {:pwd_edit => :get, :pwd_update => :put, :make_admin => :get}, 
      :collection => {:search => :get, :permisos => :get, :export => :get} do |user|
        user.resource :permissions
      end

    admin.resources :people, :member => {:change_status => :put}

    admin.resource :comment_response, :member => [:check]

    admin.resources :stream_flows, :except => :show, :member => {:update_status => :post}, :collection => {:list => :get, :order => :get, :sort => :post}

    admin.resources :photos, :collection => {:batch_edit => :get, :batch_update => :put, :find_photos => :post, :orphane => :get},
                             :member => {:add_to_album => :post}

    admin.resources :albums, :member => {:choose_cover => :post, :publish => :put} do |album|
      album.resources :photos
    end

    admin.resources :organizations
    
    admin.resources :areas do |area|
      area.resources :users, :controller => "area_users", :collection => {:sort => :post}
    end

    admin.resources :stats, :only => :index, :collection => {:midterm => :get, :contents => :get}

    admin.resources :event_locations
    
    admin.resource  :misc, :except => :all, :controller => "misc", :member => {:s3_config => :get}
    
    admin.resources :banners, :collection => {:sort => :post}
    
    admin.resources :sorganizations
    
    admin.resources :headlines, :collection => {:delete_from_entzumena => :delete}, :member => {:update_area => :put}
  end
  
  
  map.with_options :requirements => {:locale => /(es|eu|en)/} do |m|
    m.cat_videos '/:locale/web_tv/cat/:id', :controller => 'videos', :action => "cat"
    # Agenda by departaments
    m.dept_events ':locale/dept/:dept_label/events', :controller => 'events', :action => 'index'
    m.dept_events_archive ':locale/dept/:dept_label/events/archive', :controller => 'events', :action => 'index', :archive => 't'
    m.dept_events4tag ':locale/dept/:dept_label/events/:tag_label', :controller => 'events', :action => 'index'
  end
  
  
  map.with_options :path_prefix => ':locale', :requirements => {:locale => /(es|eu|en|_LNG_)/} do |m|
    m.signup '/signup', :controller => 'people', :action => 'new'
    m.login  '/login', :controller => 'sessions', :action => 'new'
    m.mlogin  '/mlogin', :controller => 'sessions', :action => 'mobile'
    m.logout '/logout', :controller => 'sessions', :action => 'destroy'
    m.password_reminder '/password_reminder', :controller => 'sessions', :action => 'password_reminder'
    m.nav_user_info '/nav_user_info', :controller => 'sessions', :action => 'nav_user_info'
    m.podcast '/podcast', :controller => 'videos', :action => 'podcast', :format => :xml
    m.podcast '/podcast.:format', :controller => 'videos', :action => 'podcast'
    
    m.resources :areas, :only => [:index, :show], 
      :collection => {:search => :get}, 
      :member => {:activity => :get, :what => :get} do |area|
        area.resources :politicians, :only => [:index]
        area.resources :actions, :only => [:index]
        area.news 'news', :controller => 'news', :action => 'index' 
        area.resources :events, :only => [:index], :collection => {:list => :get}
        # HQ
        # area.resources :questions, :only => [:index]
        area.resources :proposals, :only => [:index]
        area.resources :debates, :only => [:index]
        area.resources :videos, :only => [:index]
        area.resources :albums, :only => [:index]
        area.resources :answers, :only => :index
    end
    
    m.resources :politicians, :only => [:index, :show] do |politician|
      politician.resources :actions, :only => [:index]
      politician.news 'news', :controller => 'news', :action => 'index'
      politician.resources :events, :only => [:index], :collection => {:list => :get}
      politician.resources :proposals, :only => [:index, :new]
      # HQ
      # politician.resources :questions, :only => [:index, :new]
      politician.resources :videos, :only => [:index]
      politician.resources :albums, :only => [:index]
      politician.resources :attachments, :only => [:new, :create] 
    end
    
    m.resources :actions, :only => [:index, :show]
    
    m.resource :account,  :controller => 'account', :except => [:new, :create],
      :member => {:pwd_edit => :get, :pwd_update => :put, :confirm_delete => :get, :activate => :get, :settings => :get,
                  :image => :post, :photo => :post, :activity => :get, :proposals => :get, :actions => :get, 
                  :followings => :get, :notifications => :get}
                  # HQ :questions => :get
  
    m.resources :people, :only => [:new, :edit, :create, :update], 
      :collection => {:validate_field => :post, :intro => :get}
    
    m.resources :videos, :as => :web_tv, 
                         :only => [:new, :index, :show], 
                         :has_many => :comments,
                         :collection => {:closed_captions => :get, :summary => :get}
    
    m.resources :proposals, :except => [:edit, :update, :destroy],
      :has_many => :comments, 
      :collection => {:image => :post, :summary => :get, :department => :get} do |proposal|
        proposal.resources :votes, :only => :create
        proposal.resources :arguments, :only => :create
    end

    m.resources :debates, :only => [:index, :show], :member => {:compress => :get}, :collection => {:department => :get},
      :has_many => :comments do |debate|
        debate.resources :votes, :only => :create
        debate.resources :arguments, :only => :create
    end

    # HQ
    # m.resources :questions, :only => [:index, :show, :create], :has_many => :comments do |question|
    #   question.resources :answer_requests, :only => :create
    #   question.resources :answers, :only => :create
    # end

    # HQ
    # m.namespace :api, :path_prefix => ':locale/api', :requirements => {:locale => /(es|eu|en)/} do |api|
    #   api.resources :questions, :only => [:create, :show]
    # end

    m.resources :pages, :only => [:show]

    # Mantain old URLs
    m.gallery '/gallery', :controller => "albums", :action => "index"
    m.cat_gallery '/gallery/:cat', :controller => "albums", :action => "index"
    
    m.resources :photos, :only => [:show]
    m.cat_albums '/:locale/albums/cat/:id', :controller => 'albums', :action => "cat"
    m.resources :albums, :only => [:index, :show]
    m.album_photo '/albums/:id/photos/:photo_id', :controller => "albums", :action => "show", :requirements => {:photo_id => /\d+/}
 
    m.resources :news, :only => [:index, :show],
                       :collection => {:image => :get, :summary => :get}, 
                       :member => {:show2 => :get, :compress => :get, :department => :get},
                       :has_many => "comments"
    m.new_news_comment 'news/:news_id/comment', :controller => 'comments', :action => 'new' 
    m.hemeroteca 'hemeroteca', :controller => "news", :action => "index"
    
    m.resources :events, :only => [:index, :show], :collection => {:myfeed => :get, :list => :get, :summary => :get}
    
    # To enable posibility of reporting abuse of comments, include: :report_abuse => :post
    m.resources :comments, :member => {:photo => :get}, :collection => {:list => :get, :department => :get}
    
    m.resources :tags, :only => [:index, :show], :member => {:search => :post}# , :collection => {:inicio => :get}
    
    m.resource :session, :only => [:new, :create, :destroy],
      :collection => {:password_reminder => :get, :email_password => :post, :email_activation => :get, :mobile => :get, :auth_info => :get}
    
    m.resource :twitter_session, :only => [:create, :destroy], :member => {:finalize => :get}
    m.resource :fb_session, :only => [:create], :collection => {:finalize => :get}
    
    m.resources :search, :controller => :search, :except => [:index], :collection => {:get_create => :get}
    # m.resources :search, :controller => :search, :requirements => {:create => {:method => :get}}
    
    m.gallery '/gallery', :controller => "albums", :action => "index"
    
    m.page_site '/site/page/:label', :controller => "site", :action => 'page', :requirements => {:label => /\w+/}
    
    # old routes
    m.connect '/site/tos', :controller => "site", :action => "page", :label => "tos"
    m.connect '/site/about', :controller => "site", :action => "page", :label => "about"
    m.connect '/site/privacy', :controller => "site", :action => "page", :label => "privacy"
    # /old routes
    
    m.resource :site, :controller => :site, :only => [:show], 
      :member => {:legal => :get, :search => :get, :change_locale => :get, :contact => :get, :send_contact => :post, 
                  :splash => :get, :snetworking => :get, :sitemap => :get, 
                  :feeds => :get, :email_item => :get, :send_email => :post, :home => :get, :user_default => :get}

    m.resource :transparency, :controller => "transparency"
    
    m.resources :streamings, :only => [:index, :show], :member => {:live => :get}
    m.resources :recommendation_ratings, :only => [:create]
    m.resources :banners, :only => [:index]

    m.cached_news '/cached/news/:id', :controller => 'cached', :action => 'show', :requirements => {:type => 'Document'}
    
    m.resources :followings, :collection => {:state => :get}

    m.resources :users, :only => [:show], 
        :member  => {:connect => :get, :actions => :get, 
                     :followings => :get, :agenda => :get, :settings => :get} do |user|
      # HQ
      # user.resources :questions, :only => [:index]
      user.resources :proposals, :only => [:index]
    end
    m.resource :mob_app, :controller => "mob_app",
       :except => [:new, :create, :edit, :update, :destroy], 
       :member => {:news => :get, :events => :get, :photos => :get, :videos => :get, :appdata => :get, :about => :get, :search => :get, 
                   :tags => :get, :areas => :get, :area => :get, :v3 => :get, :team => :get, :argazki => :get,
                   :root => :get}, :format => "json"

    m.resources :stats, :only => [:index]                                 
    m.resources :orders, :only => [], :collection => {:search => :post}    
    m.order 'orders/:no_orden', :controller => 'orders', :action => 'show'
    m.resources :journalists, :only => [:new, :create]
    m.resources :answers, :only => [:index]
  end

  map.namespace :mob_app, :path_prefix => ':locale/mob_app', :requirements => {:locale => /(es|eu|en|_LNG_)/} do |mob|
    mob.resources :iui, :only => [:new, :create], :collection => {:step => :post}
  end
  
  map.mov_version '/mob_app/:v', :controller => "mob_app", :action => "show", :requirements => {:v => /\d+/}
  map.mov_v3_version '/mob_app/v3/:v', :controller => "mob_app", :action => "v3", :requirements => {:v => /\d+/}
  map.mov_news_version '/mob_app/:action:locale', :controller => "mob_app", :requirements => {:locale => /(es|eu|en|_LNG_)/}
  
  # The priority is based upon order of creation: first created -> highest priority.


  # You can have the root of your site routed with map.root -- just remember to delete public/index.html.
  map.root_with_locale '/', :controller => "site", :action => 'show', 
                            :path_prefix => ':locale', :requirements => {:locale => /(es|eu|en)/}
  map.root :controller => "site", :action => 'show'
  map.lang '/lang', :controller => "site", :action => 'splash'
  map.iphone '/iphone', :controller => "site", :action => 'show', :format => "iphone", 
                        :path_prefix => ':locale', :requirements => {:locale => /(es|eu|en)/}
  map.connect '/iphone', :controller => "site", :action => 'show', :format => "iphone", :locale => "es"

  map.android '/android', :controller => "site", :action => 'show', :format => "android", 
                          :path_prefix => ':locale', :requirements => {:locale => /(es|eu|en)/}
  map.connect '/android', :controller => "site", :action => 'show', :format => "android", :locale => "es"

  # See how all your routes lay out with "rake routes"

  # Install the default routes as the lowest priority.
  # Note: These default routes make all actions in every controller accessible via GET requests. You should
  # consider removing the them or commenting them out if you're using named routes and resources.
  map.connect ':controller/:action/:id'
  map.connect ':controller/:action/:id.:format'
  
  map.connect "*path", :controller => "site", :action => "notfound" unless ActionController::Base.consider_all_requests_local
  
end
