require File.join(RAILS_ROOT, 'config', 'environment')
require 'csv'

namespace 'export' do
  desc "Export users data"
  task :export_users do
    p "Exporting users data ..."
    
    filename = "#{Document::EXPORT_PATH}/datos_usuarios.csv"
    @users = User.approved.find(:all, :order => "id", :conditions => "email not like '%efaber%'")
    CSV.open(filename, "w", ";") do |writer|
      writer << ['id', 'Email', 'Contraseña', 'Salt', \
                 'Twitter screen_name', 'Twitter atoken', 'Twitter asecret', 'Facebook ID', \
                 'Nombre', 'Apellidos', 'Tipo', 'Departamento', 'Lugar', 'Código postal', 'Provincia', 'Ciudad'] 
      
      @users.each do |user|
        department = user.respond_to?("department") ? user.department.name : ""
        writer << [user.id, "#{user.email}", "#{user.crypted_password}", "#{user.salt}",\
                   "#{user.screen_name}", "#{user.atoken}", "#{user.asecret}", "#{user.fb_id}", \
                   "#{user.name}", "#{user.last_names}", "#{User::TYPES[user.type]}", \
                   "#{department}", "#{user.raw_location}", "#{user.zip}", "#{user.state}", "#{user.city}"]
      end
    end
    
    p "Export saved to #{filename}"
  end
  
  desc "Export areas" 
  task :irekia3_areas do
    filename = "#{Document::EXPORT_PATH}/datos_areas.yml"
    f = File.open(filename, 'w')
    f.write Area.find(:all).to_yaml
    f.close
    p "Export saved to #{filename}"
  end
  
  desc "Expor políticos"
  task :irekia3_politicos do
    filename = "#{Document::EXPORT_PATH}/datos_politicos.yml"
    f = File.open(filename, 'w')
    f.write Politician.approved.find(:all).to_yaml
    f.close
    p "Export saved to #{filename}"
  end
  
  desc "Export propuestas del gobierno"
  task :irekia3_propsals_gov do
    filename = "#{Document::EXPORT_PATH}/datos_propuestas_gov.yml"
    f = File.open(filename, 'w')
    f.write Proposal.from_politicians.find(:all).to_yaml
    f.close
    p "Export saved to #{filename}"    
  end
  
  desc "Export accesos de políticos"
  task :irekia3_politicians_last_login do
    filename = "#{Document::EXPORT_PATH}/ultimos_accesos_politicos.csv"
    politicians = Politician.approved.find(:all)
    CSV.open(filename, "w", ";") do |writer|
      writer << ['URL', 'Email', 'Nombre', 'Áreas', 'Último acceso'] 
      
      politicians.each do |user|
        last_login = user.session_logs.find :first, :order => "action_at DESC"
        writer << ["https://www.irekia.euskadi.net/es/admin/users/#{user.id}", "#{user.email}", "#{user.public_name}", "#{user.areas.map {|a| a.name}.join(", ")}", "#{last_login.present? ? last_login.updated_at.to_date : ''}"]
      end
    end
    
    p "Export saved to #{filename}"    
  end
  
  desc "Export questions and answers"
  task :irekia3_questions do
    filename = "#{Document::EXPORT_PATH}/questions.csv"
    questions = Question.find(:all, :include => :answer, :order => "published_at DESC")
    CSV.open(filename, "w", ";") do |writer|
      writer << ["id", "Pregunta", "Respuesta", "Fecha pregunta", "Fecha respuesta", "Tiempo de respuesta (días)", "Quién", "Para quién", "Departamento"]
      questions.each do |question|
        for_whom = question.for_whom.present? ? question.for_whom.public_name : question.area.name
        department = question.department.present? ? question.department.name : nil
        if question.answer.present?
          days = ((question.answer.answered_at - question.published_at).to_i/(60.00*60*24)).round
          writer << [question.id, question.title, question.answer.body, question.published_at, question.answer.answered_at, days, question.user.public_name, for_whom, department]
        else
          writer << [question.id, question.title, nil, question.published_at, nil, nil, question.user.public_name, for_whom, department]
        end
      end
    end
    p "Export saved to #{filename}"
  end

  desc "Export de propuestas ciudadanas"
  task :irekia3_citizens_proposals do
    filename = "#{Document::EXPORT_PATH}/citizens_proposals.csv"
    proposals = Proposal.from_citizens.find(:all, :order => "published_at DESC")
    CSV.open(filename, "w", ";") do |writer|
      writer << ["id", "Propuesta", "Fecha propuesta", "Nº comentarios oficiales", "Quién", "Para quién", "Departamento"]
      proposals.each do |proposal|
        from_who = proposal.user.present? ? proposal.user.public_name : nil
        for_whom = proposal.area.present? ? proposal.area.name : nil
        department = proposal.department.present? ? proposal.department.name : nil
        official_comments = proposal.comments.select {|c| c.user && c.user.is_official_commenter?}.length
        writer << [proposal.id, proposal.title, proposal.published_at, official_comments, from_who, for_whom, department]
      end
    end
    p "Export saved to #{filename}"    
  end
  
  desc "Export de eventos"
  task :irekia3_events do
    filename = "#{Document::EXPORT_PATH}/events.csv"
    events = Event.published.find(:all, :order => "starts_at DESC")
    CSV.open(filename, "w", ";") do |writer|
      writer << ["id", "Título", "Fecha inicio", "Fecha fin", "Departamento", "Lugar"]
      events.each do |event|
        department = event.department.present? ? event.department.name : nil
        writer << [event.id, event.title, event.starts_at, event.ends_at, department, event.place]
      end
    end
    p "Export saved to #{filename}"
  end
end
