require 'yaml'
require File.join(RAILS_ROOT, 'config', 'environment')

namespace :openirekia do
  desc "Load sample data"
  task :load_sample_data => [:load_organizations, :load_areas, :load_users, :load_video_and_photo_categories, :load_news, :load_events, :load_proposals, :load_banners, :load_debates, :load_pages] do
    p "Los datos han sido guardados en la base de datos."
  end
  
  desc "Create departaments"
  task :load_organizations do
    p "Crear departamentos ..."
    #Departamentos
    dept = Department.create!(:name_es => "Proyecto OpenIrekia", 
      :name_eu => "OpenIrekia", 
      :name_en => "OpenIrekia",
      :tag_name => "_openirekia",
      :position => 0,
      :internal_id => 0,
      :enet_workarea => "dummy")
      
    Organization.create!(:name_es => "Organización", 
        :name_eu => "Organización", 
        :name_en => "Organización",
        :tag_name => "_organizacion",
        :parent => dept,
        :position => 1,
        :enet_workarea => "dummy")
        
    Department.create!(:name_es => "Departamento 1", 
      :name_eu => "Departamento 1", 
      :name_en => "Departamento 1",
      :tag_name => "_dept1",
      :position => 1,
      :internal_id => 1,
      :enet_workarea => "dummy")
      
    Department.create!(:name_es => "Departamento 2", 
      :name_eu => "Departamento 2", 
      :name_en => "Departamento 2",
      :tag_name => "_dept2",
      :position => 2,
      :internal_id => 2,
      :enet_workarea => "dummy")
      
    p "OK."
  end
  
  task :load_areas => [:environment] do
    Area.create!(:name_es => "Proyecto OpenIrekia", 
      :name_eu => "OpenIrekia", 
      :name_en => "OpenIrekia",
      :area_tag_name => "_a_openirekia",
      :position => 0)
    
    Area.create!(:name_es => "Area 2", 
      :name_eu => "Atala 2", 
      :name_en => "Area 2",
      :area_tag_name => "_a_area2",
      :position => 1)
  end
  
  desc "Create users of different types"
  task :load_users => [:environment] do
    p "Create users ..."
    unless admin = User.find_by_email("admin@example.com")  
      admin = Admin.create(:name => 'Administrador', :email=> "admin@example.com", :password => 'openirekia', :password_confirmation => 'openirekia')
      admin.permissions.build(:module => 'permissions', :action => 'administer')
      admin.save!      
    end
    
    DepartmentEditor.create(:name => 'Jefe de departamento', :email=> "jefe_dept@example.com", :password => 'openirekia', :password_confirmation => 'openirekia', :department_id => Department.find(:first).id) unless User.find_by_email("jefe_dept@example.com")
    
    unless j = Journalist.find_by_email("periodista@example.com")
      j = Journalist.new(:name => 'Periodista', :last_names => 'de la Radio', :media => 'Radio', :email=> "periodista@example.com", :password => 'openirekia', :password_confirmation => 'openirekia')
      j.subscriptions.build(:department => Department.find(:first))
      j.save!
    end
    
    StreamingOperator.create(:name => 'Operador de streaming', :email=> "operador@example.com", :password => 'openirekia', :password_confirmation => 'openirekia') unless StreamingOperator.find_by_email("operador@example.com")
    
    unless person = Person.find_by_email('usuario@example.com')
      person = Person.create(:name => 'Usuario registrado', :email => 'usuario@example.com', :password => 'openirekia', :password_confirmation => 'openirekia') 
    end
    person.update_attribute(:status, 'aprobado')
    
    politician = Politician.create!(:name => "Político", :email => "politician@example.com", :password => 'openirekia', :password_confirmation => 'openirekia')
    AreaUser.create!(:user => politician, :area => Area.find_by_name_es("Proyecto OpenIrekia"))
    
    p "OK."
  end  
  
  
  desc "Video and photo categories"
  task :load_video_and_photo_categories => [:environment] do
    webtv_menu = Tree.create!(:name_es => 'Canales WebTV', :name_en => 'WebTV channels', :name_eu => 'WebTB kanalak', :label => "videos")
    webtv_menu.categories.build(:name_es => 'Canal 1', :name_eu => 'Kanal 1', :name_en => 'Channel 1', :tag_list => ['_webtv'])
    
    photo_menu = Tree.create!(:name_es => 'Canales Fototeca', :name_en => 'Photo channels', :name_eu => 'Fototeka kanalak', :label => "albums")
    photo_menu.categories.build(:name_es => 'Canal 1', :name_eu => 'Kanal 1', :name_en => 'Channel 1', :tag_list => ['_fototeca'])
    
    ['menu', 'navbar_left', 'navbar_right'].each do |tree_label|
      Tree.create!(:name_es => tree_label, :name_en => tree_label, :name_eu => tree_label, :label => tree_label)
    end  
    
  end
  
  desc "Load news, videos and photos"
  task :load_news => [:environment] do
    p "Load news, videos and photos..."
    n = News.new()
    organizations = Organization.find(:all)
    areas = Area.find(:all)
    File.open(File.join(RAILS_ROOT, 'data', 'openirekia_news.yml'))  { |yf| YAML::load( yf ) }.each do |doc|
      puts "Importing news with id=#{doc.attributes['id']}"
      news = News.find_or_initialize_by_id(doc.attributes['id'])
      doc.attributes.each do |a, value|
        news.send("#{a}=", value)
      end
      news.organization = organizations[rand(organizations.size)]
      news.area_id = areas[rand(areas.size)].id
      unless news.cover_photo_file_name.blank?
        file_path = File.join(RAILS_ROOT, 'data/images', news.cover_photo_file_name)
        if File.exists?(file_path)
          news.cover_photo = File.new(file_path) 
        else
          news.cover_photo = nil
        end
      end
            
      news.save!
      
      if doc.attributes['id'].to_i == 2
        attachment = news.attachments.create!(:file => File.new(File.join(RAILS_ROOT, 'data/files', 'Discurso_LHDK.pdf')), :show_in_es => true)
      end
      
    end
    
    update_sequence('Document')
     
    Rake::Task["ogov:include_new_videos_in_webtv"].invoke
    Rake::Task["ogov:include_new_photos_in_gallery"].invoke
    
    album = Album.first
    album.update_attribute(:featured, true) if album
    
    p "OK."
  end
  
  
  desc "Crear ubicaciones"
  task :load_event_locations  do
    l = EventLocation.find_or_create_by_place('Lehendakaritza')
    l.update_attributes(:city => 'Vitoria-Gasteiz', :address => 'C/ Navarra 2', :lat => '42.839135', :lng => '-2.678053')
    
    l = EventLocation.find_or_create_by_place('Lakua')
    l.update_attributes(:city => 'Vitoria-Gasteiz', :address => 'C/ Donostia-San Sebastián, 1', :lat => '42.859128', :lng => '-2.686994')
  end
  
  desc "Crear flujos de streaming"
  task :load_streaming_flows do
    s = StreamFlow.find_or_create_by_code('SIT1.SDP')
    s.update_attributes(:title_es => 'Lehendakaritza', :title_eu => 'Lehendakaritza', :title_en => 'Lehendakaritza', :send_alerts => true, :photo => File.new(File.join(RAILS_ROOT, 'data/images', 'streaming.jpg')) )
    
    s = StreamFlow.find_or_create_by_code('SIT2.SDP')
    s.update_attributes(:title_es => 'Lakua', :title_eu => 'Lakua', :title_en => 'Lakua', :send_alerts => true)
  end

  desc "Crear eventos" 
  task :load_events => [:load_event_locations, :load_streaming_flows] do
    p "Añadir eventos  ..."
    UserActionObserver.current_user = User.find_by_email('admin@example.com')
    areas = Area.all
    File.open(File.join(RAILS_ROOT, 'data', 'openirekia_events.yml'))  { |yf| YAML::load( yf ) }.each do |doc|
      puts "Importing event with id=#{doc.attributes['id']}"
      evt = Event.find_or_initialize_by_id(doc.attributes['id'])
      doc.attributes.each do |a, value|
        evt.send("#{a}=", value)
      end
      evt.organization = Department.find_by_tag_name('_openirekia')
      evt.area_id = areas[rand(areas.size)].id
      evt.stream_flow = StreamFlow.find_by_code('SIT1.SDP') unless evt.stream_flow_id.blank?
      evt.save!
    end
    
    File.open(File.join(RAILS_ROOT, 'data', 'openirekia_related_events.yml'))  { |yf| YAML::load( yf ) }.each do |related|
      re = RelatedEvent.new
      related.attributes.each do |a, value|
        re.send("#{a}=", value)
      end
      re.save
    end
    
    # Eventos con streaming hoy
    loc1 = EventLocation.find_by_place('Lakua')    
    loc2 = EventLocation.find_by_place('Lehendakaritza')
    evt_live = Event.create(:starts_at => Time.now.at_beginning_of_day + 10.hours,
                        :ends_at => Time.now.at_beginning_of_day + 12.hours,    
                        :title_es => 'Evento con streaming en directo de 10:00h a 12:00h',
                        :body_es => 'Un evento de prueba con streaming en directo el día de hoy.',
                        :organization_id => Department.find_by_tag_name('_openirekia').id,
                        :speaker => 'Persona que participa en el evento',
                        :place => loc1.place, 
                        :city => loc1.city, 
                        :location_for_gmaps => loc1.address,
                        :published_at => Time.now.at_beginning_of_day - 1.day,
                        :irekia_coverage => true,  
                        :streaming_live => true,
                        :stream_flow_id => StreamFlow.find_by_code('SIT2.SDP').id,
                        :streaming_for => 'irekia',
                        :area_id => areas[rand(areas.size)].id
                        )
                        
    # Indicar que el evento evt_live se está emitiendo en OpenIrekia y Agencia Multimedia.
    sf = StreamFlow.find_by_code('SIT2.SDP')
    sf.event_id = evt_live.id
    sf.update_attribute(:announced_in_irekia, false)
    sf.update_attribute(:show_in_irekia, true)
    
    evt_today = Event.create(:starts_at => Time.now.at_beginning_of_day + 22.hours,
                        :ends_at => Time.now.at_beginning_of_day + 23.hours,    
                        :title_es => 'Evento con streaming en directo de 22:00h a 23:00h',
                        :body_es => 'Segundo evento de prueba con streaming en directo desde Lehendakaritza el día de hoy.',
                        :organization_id => Organization.find_by_tag_name('_organizacion').id,
                        :speaker => 'Persona que participa en el evento',
                        :place => loc1.place, 
                        :city => loc1.city, 
                        :location_for_gmaps => loc1.address,
                        :published_at => Time.now.at_beginning_of_day - 1.day,
                        :irekia_coverage => true,
                        :streaming_live => true,
                        :stream_flow_id => StreamFlow.find_by_code('SIT2.SDP').id,
                        :streaming_for => 'irekia',
                        :area_id => areas[rand(areas.size)].id
                        )
                        
    evt_announced = Event.create(:starts_at => Time.now.at_beginning_of_day + 13.hours,
                                 :ends_at => Time.now.at_beginning_of_day + 14.hours,    
                                 :title_es => 'Evento con streaming en directo de 13:00h a 14:00h',
                                 :body_es => 'Evento de prueba con streaming en directo desde Lakua el día de hoy.',
                                 :organization_id => Department.find_by_tag_name('_dept1').id,
                                 :speaker => 'Persona que participa en el evento',
                                 :place => loc2.place, 
                                 :city => loc2.city, 
                                 :location_for_gmaps => loc2.address,
                                 :published_at => Time.now.at_beginning_of_day - 1.day,
                                 :irekia_coverage => true,                                 
                                 :streaming_live => true,
                                 :stream_flow_id => StreamFlow.find_by_code('SIT1.SDP').id,
                                 :streaming_for => 'irekia',
                                 :area_id => areas[rand(areas.size)].id
                                 )
      # Indicar que el evento evt_announced se está anunciado en OpenIrekia.
      sf = StreamFlow.find_by_code('SIT1.SDP')
      sf.event_id = evt_announced.id
      sf.update_attribute(:show_in_irekia, false)
      sf.update_attribute(:announced_in_irekia, true)
    
    update_sequence('Document')
    p "OK."
  end
  
  desc "Añadir propuesta de ejemplo"
  task :load_proposals do
    p "Añadir propuesta de ejemplo ...."
    user = Person.find(:first)
    prop = Proposal.new(:name => user.name,
                        :email => user.email, 
                        :user => user,
                        :title_es => 'Propuesta de ejemplo',
                        :title_eu => '', 
                        :title_en => 'Example Proposal',
                        :body_es => 'Una propuesta de ejemplo creada por un usuario registrado.',
                        :body_eu => '',
                        :body_en => 'Example proposal created by a registered user.',
                        :organization_id => Department.find(:first).id,
                        :draft => false,
                        :status => 'aprobado',
                        :user_ip => '127.0.0.1',
                        :featured => true,
                        :area_id => Area.find_by_name_es("Proyecto OpenIrekia").area_tag.id)
    prop.save!
    p "OK."
  end
  
  task :load_banners do
    banner_dir = "#{RAILS_ROOT}/data/banners/"
    
    Banner.create! :url_es => "http://opendata.euskadi.net/w79-home/es", :alt_es => "Open Data Euskadi",
                   :logo_es => File.open("#{banner_dir}/open_data_euskadi_es.png"),
                   :url_eu => "http://opendata.euskadi.net/w79-home/eu", :alt_eu => "Open Data Euskadi",
                   :logo_eu => File.open("#{banner_dir}/open_data_euskadi_eu.png"),
                   :active => true
                   
    Banner.create! :url_es => "http://www.irekia.euskadi.net/es/pages/3359", :alt_es => "Open Irekia",
                   :logo_es => File.open("#{banner_dir}/open_irekia.png"),
                   :active => true
    
  end
  
  desc "Load debates"
  task :load_debates => [:environment] do
    p "Loading debates..."
    organizations = Organization.find(:all)
    areas = Area.find(:all)
    File.open(File.join(RAILS_ROOT, 'data', 'openirekia_debates.yml'))  { |yf| YAML::load( yf ) }.each do |debate|
      puts "Importing debate with id=#{debate.attributes['id']}"
      d = Debate.find_or_initialize_by_id(debate.attributes['id'])
      debate.attributes.each do |a, value|
        if a.eql?('stages')
          value.each do |stage|
            d.stages.build(stage.attributes)
          end
        else
          d.send("#{a}=", value)
        end
      end
      d.save!
    end
    
    update_sequence('Debate')
    update_sequence('DebateStage')
  end
  
  
  desc "Load pages linked from layout footer"
  task :load_pages => [:environment] do
    p "Loading pages..."
    x = Page.new()
    File.open(File.join(RAILS_ROOT, 'data', 'openirekia_pages.yml'))  { |yf| YAML::load( yf ) }.each do |xx|
      puts "Importing page with id=#{xx.attributes['id']}"
      p = Page.find_or_initialize_by_id(xx.attributes['id'])
      xx.attributes.each do |a, value|
          p.send("#{a}=", value)
      end
      p.save!
    end
    update_sequence('Document')
  end
  
  def update_sequence(model)
    table = model.tableize
    count = model.constantize.count_by_sql("SELECT MAX(id) FROM #{table}")
    seq = "#{table}_id_seq"
    ActiveRecord::Base.connection.execute("select setval('#{seq}', #{count})")
  end
end
