require File.join(RAILS_ROOT, 'config', 'environment')

namespace :update do
  
  desc "Update admin and staff avatar"
  task :staff_avatar => [:environment] do
    count = 0
    User.find(:all, :conditions => {:type => User::STAFF}).each do |user|
      unless user.photo.present?
        user.photo = File.new(File.join(RAILS_ROOT, 'public/images/icons/ejgv.png'))
        if user.save
          count += 1
        else
          puts "ERROR saving avatar for user #{user.id}\n"  
        end  
      end  
    end  
    puts "#{count} users successfully updated!"
  end           
  
  desc "Create nav bar menu categories"
  task :nav_bar_menus => [:environment] do
    navbar_left_tree = Tree.find_navbar_left_tree
    if navbar_left_tree.present?   
      navbar_left_tree.categories.destroy_all
      Category.create(:name_es => '"Noticias":/es/transparency', :name_eu => '"Albisteak":/eu/transparency', :name_en => '"News":/en/transparency', :tree_id => navbar_left_tree.id, :position => 1)
      Category.create(:name_es => '"Agenda":/es/events', :name_eu => '"Agenda":/eu/events', :name_en => '"Agenda":/en/events', :tree_id => navbar_left_tree.id, :position => 2)    
      Category.create(:name_es => '"Redes y Blogs":/es/site/snetworking', :name_eu => '"Blog eta sareak":/eu/site/snetworking', :name_en => '"Blogs and networks":/en/site/snetworking', :tree_id => navbar_left_tree.id, :position => 3)        
      videoteca = Category.create(:name_es => '"Videoteca":/es/web_tv', :name_eu => '"Bideoteka":/eu/web_tv', :name_en => '"Web TV":/en/web_tv', :tree_id => navbar_left_tree.id, :position => 4)    
      Category.create(:name_es => '"Subtitulados":/es/web_tv/closed_captions', :name_eu => '"Azpititulatutakoak":/eu/web_tv/closed_captions', :name_en => '"Subtitled":/en/web_tv/closed_captions', :tree_id => navbar_left_tree.id, :position => 1, :parent_id => videoteca.id)        
      Category.create(:name_es => '"Fototeca":/es/albums', :name_eu => '"Fototeka":/eu/albums', :name_en => '"Photo Gallery":/en/albums', :tree_id => navbar_left_tree.id, :position => 5)            
      Category.create(:name_es => '"RSS todo el contenido":/es/site/feeds', :name_eu => '"RSS eduki osoa":/eu/site/feeds', :name_en => '"RSS all contents":/en/site/feeds', :tree_id => navbar_left_tree.id, :position => 6)
    end  
    navbar_right_tree = Tree.find_navbar_right_tree
    if navbar_right_tree.present?        
      navbar_right_tree.categories.destroy_all      
      Category.create(:name_es => '"Irekia: el Gobierno Abierto":/es/pages/2902', :name_eu => '"Gobernu irekia":/eu/pages/2902', :name_en => '"Irekia: the Open Government":/en/pages/2902', :tree_id => navbar_right_tree.id, :position => 1)                                                                                                                                              
      Category.create(:name_es => '"Open Irekia: Software Libre":/es/pages/3359', :name_eu => '"Open Irekia: Software Askea":/eu/pages/3359', :name_en => '"Open Irekia: Open Source":/en/pages/3359', :tree_id => navbar_right_tree.id, :position => 2)                                                                                                                                              
      Category.create(:name_es => '"Ayuda":/es/pages/781', :name_eu => '"Laguntza":/eu/pages/781', :name_en => '"Help":/en/pages/781', :tree_id => navbar_right_tree.id, :position => 3)                                                                                                                                                    
      Category.create(:name_es => '"Guía de Buenas prácticas":/es/pages/10049', :name_eu => '"Jardunbide egokien gida":/eu/pages/10049', :name_en => '"Best practices":/en/pages/10049', :tree_id => navbar_right_tree.id, :position => 4)                                                                                                                                                          
      Category.create(:name_es => '+', :name_eu => '+', :name_en => '+', :tree_id => navbar_right_tree.id, :position => 5)                                                                                                                                                          
      Category.create(:name_es => '"Argazki":http://argazki.irekia.euskadi.net/es', :name_eu => '"Argazki":http://argazki.irekia.euskadi.net/eu', :name_en => '"Argazki":http://argazki.irekia.euskadi.net/es', :tree_id => navbar_right_tree.id, :position => 6)                                                                                                                                                                 
      Category.create(:name_es => '"Escucha activa":http://entzumena.irekia.euskadi.net/es', :name_eu => '"Entzute aktiboa":http://entzumena.irekia.euskadi.net/eu', :name_en => '"Active listening":http://entzumena.irekia.euskadi.net/es', :tree_id => navbar_right_tree.id, :position => 7)                                                                                                                                                                
    end  
    
    puts "Finished creating categories for nav_bar menus!"
  end  
  
  desc "Create all tags associated criterios"
  task :create_tag_criterio => [:environment] do
    total_tags = Tag.public.count
    i = 0
    while i < total_tags
      tags = Tag.find(:all, :order => 'id ASC', :offset => i, :limit => 100)
      tags.each do |tag|
        tag.save
      end                                                                      
      i += 100
    end       
    puts "Finished creating tag criterios!!"    
  end  
  
  desc "Update subscriptions of journalists to new legislature"
  task :update_subscriptions_to_new_legislature => [:environment] do
    puts "Journalist count: #{Journalist.count}"
    new_departments = Department.active.find(:all, :order => 'id')
    # Only update subscribers who are not already subscribed to any new department
    subscribers_old_departments = Journalist.find(:all, :conditions =>  "id not in (select user_id from subscriptions where department_id in (#{new_departments.collect(&:id).join(',')}))")
    subscribers_old_departments.each do |old_subscriber|
      puts "Processing subscriber #{old_subscriber.id} #{old_subscriber.email}"
      puts "Unsubscribe from departments #{old_subscriber.subscriptions.collect {|s| s.department.name}.join(',')}"
      old_subscriber.subscriptions.clear
      new_departments.each do |department|
        new_subscription = Subscription.new
        new_subscription.user_id = old_subscriber.id
        new_subscription.department_id = department.id
        new_subscription.save
      end
      puts "--------"
    end
    puts "Finished updating subscriptions !"
    puts "Journalist count: #{Journalist.count}"
  end
                                           
end  
