#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

#
# Implementation of RAKE (Rapid Automatic Keyword Extraction) Algoritm in Ruby
#
# Ref: S Rose, D Engel, N Cramer, W Cowley, Automatic keyword extraction from individual documents,
#      in "Text Mining: Applications and Theory", John Wiley & Sons, 2010
#
# (c) svet@efaber.net, Jan 2011
#

require "pp"
require "matrix"

class Matrix
  def []=(i, j, x)
    @rows[i][j] = x
  end
end


class RAKE

  def initialize(t)
    @t = t.downcase
    # add space around phrase separators "." and ","
    @t = @t.gsub(/([\,\.\:\;])/, ' \1 ')
    # reduce 2+ spaces to just one
    @t = @t.gsub(/(\ ){2,}/, " ")
  end
  
  def find_candidates()
    w = @t.split(" ")

     #stopwords = ["of", "the", "a", "in", "the", "all", "over", "and", "are", 
     #         "for", "can", "be", "given", "used", "these", "mixed", "types", 
     #         "considered"]

    stopwords = ["el", "de", "del", "y", "estan", "que", "las", "para", "con", 
                  "una", "su", "por", "ha", "en", "un", "a", "la", "estos", "al", 
                  "esta", "son", "los", "o", "se", "ya", "lo", "entre", "sus", "es",
                  "alguna", "haber", "cuando", "este", "algunos" ]

    phsep = [".", ",", ":", ";"]
 
    # puts "Candidate keywords ..."
    # candidate keywords
    @cw=[]
    cwc = ''
    w.each do |ow| 
      isstop = stopwords.any? {|e| e==ow}
      isphsep = phsep.any? {|e| e==ow}
      if isstop || isphsep
         @cw << " "+cwc.strip+" " if !cwc.empty?
         cwc = ""
      else
        cwc += " "+ow
      end
    end
    @cw << " "+cwc.strip+" " if !cwc.empty?

    # as sep words:
    @sepwords = @cw.join(" ").split(" ").sort.uniq
    @gc = Matrix.zero(@sepwords.size)

    # puts "Calc graph of word co-ocurrences ..."
    # calc the graph of word co-ocurrences
    @sepwords.each_with_index do |row, i|
      # puts "  ... "+row
      rr = (@cw.find_all {|c| c.match(/[^\w]#{row}[^\w]/) })
      @sepwords.each_with_index do |col, j|
        # @gc[i,j] = ((@cw.find_all {|c| c.match(/[^\w]#{row}[^\w]/) }).find_all {|c| c.match(/[^\w]#{col}[^\w]/) }).size  
        if j>=i 
          @gc[i,j] = (rr.find_all {|c| c.match(/[^\w]#{col}[^\w]/) }).size  
        else
          @gc[i,j] = @gc[j,i] 
        end
      end
    end
    # puts "Done ..."
  end


  def freq(word)
    (@cw.find_all {|c| c.match(/[^\w]#{word}[^\w]/) }).size
  end


  def deg(word)
    k = @sepwords.find_index(word)
    @gc[ k, 0..-1].inject(0){|sum,item| sum + item}
  end


  def keywords()
    self.find_candidates()
    wordscores = Hash.new()
    @sepwords.each do |word|
      degv = deg(word)
      freqv = freq(word)
      dfr = 1.0 * degv / freqv
      wordscores[word] = [degv, freqv, dfr]
    end

    kwscores = Hash.new()
    @cw.uniq.each do |kw|
      score = 0.0
      kw.split.each { |onew| score += wordscores[onew][2] }
      kwscores[kw.strip] = score
    end
    kwsorted = kwscores.sort {|a,b| b[1]<=>a[1]}
    kwsorted
  end
end



# ------------------------------------------------------------------------------------------
# extract as: 
#
# tit = News.find(4604).title.tildes.strip_html.gsub(/<\/?[^>]*>/, "").gsub(/[\r|\n\(\)\-\'\"]/, " ").gsub(/([0-9]+)\.([0-9]+)/,'\1\2')
# bod = News.find(4604).body.tildes.strip_html.gsub(/<\/?[^>]*>/, "").gsub(/[\r|\n\(\)\-\'\"]/, " ").gsub(/([0-9]+)\.([0-9]+)/,'\1\2')
#

tit=IO.readlines("prueba_t.txt").join(".").gsub(/[\r|\n\(\)\-\'\"]/, " ")
bod=IO.readlines("prueba_b.txt").join(" ")

rc = RAKE.new(bod)
r = rc.keywords()

rakeq = tit
inir = r[0][1]
r.each do |sk|
  if (sk[1] >= inir/3.5) && (rakeq.length < 250)
     rakeq = rakeq + " " + sk[0]
     # pp sk[1]
  end
end

pp r
puts rakeq
# pp inir


