/**
 * flowplayer.playlist 3.0.8. Flowplayer JavaScript plugin.
 * 
 * This file is part of Flowplayer, http://flowplayer.org
 *
 * Author: Tero Piirainen, <info@flowplayer.org>
 * Copyright (c) 2008-2010 Flowplayer Ltd
 *
 * Dual licensed under MIT and GPL 2+ licenses
 * SEE: http://www.opensource.org/licenses
 * 
 * Date: 2010-05-04 05:33:23 +0000 (Tue, 04 May 2010)
 * Revision: 3405 
 */ 

	flowplayer.addPlugin("playlist", function(wrap, options) {

		// self points to current Player instance
		var self = this;	
		
		var opts = {
			playingClass: 'playing',
			pausedClass: 'paused',
			progressClass:'progress',
			template: '<a href="${url}">${title}</a>',
			loop: false,
			playOnClick: true,
			manual: false
		};		
		
		Object.extend(opts, options);
		wrap = $(wrap);		

		var manual = self.getPlaylist().length <= 1 || opts.manual; 
		var els = null;

		
//{{{ "private" functions
				
		function toString(clip) {
			var el = template;
			
			Object.each(clip, function(key, val) {	
				if (!Object.isFunction(val)) {
					el = el.replace("$\{" +key+ "\}", val).replace("$%7B" +key+ "%7D", val);			
				}
			}); 
			return el;
		}
		
		// assign onClick event for each clip
    // function bindClicks() {      
    //  els = getEls().unbind("click.playlist").bind("click.playlist", function() {
    //    return play($(this), els.index(this));            
    //  });   
    // }
		
		function buildPlaylist() {
			wrap.empty();
			
			Object.each(self.getPlaylist(), function() {  
				wrap.append(toString(this)); 
			});				
			
			bindClicks();
		} 

		
		function play(el, clip)  {
			if (el.hasClassName(opts.playingClass) || el.hasClassName(opts.pausedClass)) {
				self.toggle();
			} else {
				el.addClassName(opts.progressClass);
				self.play(clip); 							
			}			
			
			return false;
		}	
		
		
		function clearCSS() {
			if (manual) { els = getEls(); }
			els.each(function(el) {
			  el.removeClassName(opts.playingClass);
			  el.removeClassName(opts.pausedClass);
			  el.removeClassName(opts.progressClass);			
			});
		}
		
		function getEl(clip) {
		  var current_url = clip.originalUrl || clip.url;
			return wrap.select("[href=" + current_url + "]").first();
		}
		
		function getEls() {
			var els = wrap.select("a");
			return els.length ? els : wrap.children();
		}
//}}}  
		 
		/* setup playlists with onClick handlers */ 
		
		// internal playlist
		if (!manual) {
			var template = wrap.is(":empty") ? opts.template : wrap.html(); 
			buildPlaylist();			
			
			
		// manual playlist
		} else {
			els = getEls();			

			els.each(function(elem){
  			elem.observe('click',function(evt) {
  				var el = $(this);
  				play(el, el.href);
  				$('video_title').down('p.video_duration').innerHTML = el.select('span.video_duration').first().innerHTML;
  				$('video_title').down('p.video_title_text').innerHTML = el.select('span.video_title').first().innerHTML;  				
          // $('video_embed').innerHTML = el.select('span.embed_code').first().innerHTML;          
  				evt.stop();
  				return false;
  			});									  
			});
			
						 
					
			// setup player to play first clip
			var clip = self.getClip(0);
			if (!clip.url && opts.playOnClick) {
				clip.update({url: els[0].href});		
			}   
		}
		
		// onBegin
		self.onBegin(function(clip) {
			clearCSS();
			var el = getEl(clip);
			el.addClassName(opts.playingClass);
		});	
		
		// onPause	
		self.onPause(function(clip) {
			getEl(clip).removeClassName(opts.playingClass).addClassName(opts.pausedClass);		
		});	
		
		// onResume
		self.onResume(function(clip) {
			getEl(clip).removeClassName(opts.pausedClass).addClassName(opts.playingClass);		
		});		
		
		// what happens when clip ends ?
		if (!opts.loop && !manual) {
			
			// stop the playback exept on the last clip, which is stopped by default
			self.onBeforeFinish(function(clip) {
				if (!clip.isInStream && clip.index < els.length -1) {
					return false;
				}
			}); 
		}
		
		// on manual setups perform looping here
		if (manual && opts.loop) {
			self.onBeforeFinish(function(clip) {
				var el = getEl(clip);
				if (el.next().length) {
					el.next().click();	 		
				} else {
					els.eq(0).click();	
				} 
				return false;				
			}); 
		}
		
		// onUnload
		self.onUnload(function() {
			clearCSS();		
		});
		
		// onPlaylistReplace
		if (!manual) {
			self.onPlaylistReplace(function() {
				buildPlaylist();		
			});
		}
		
		// onClipAdd
		self.onClipAdd = function(clip, index) {	
			els.eq(index).before(toString(clip));			
			bindClicks(); 
		};		
		
		return self;
		
	});
