#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

ELASTICSEARCH_URI="http://localhost:9200/irekia4_test"
ELASTICSEARCH_RELATED_URI="http://localhost:9200/irekia4_news_test"
ELASTICSEARCH_BOPV_URI="http://localhost:9200/bopv_test"
ELASTICSEARCH_ALBISTE_URI="http://localhost:9200/albiste_test"

module ElasticsearchTestHelper
  
  def create_test_index
    Elasticsearch::Base::create_index
  end       

  def index_test_items(type)
    type.classify.constantize.all.each do |item|
      item.update_elasticsearch_server
      item.update_elasticsearch_related_server # only for news
    end  
    sleep 2
  end
  
  def delete_test_index
    Elasticsearch::Base::delete_index    
  end

  def prepare_elasticsearch_test(type=nil)
    delete_test_index
    create_test_index
    if type.present?
      if type.eql?('related')
        Elasticsearch::Base::delete_related_index    
        Elasticsearch::Base::create_related_index    
      else
        index_test_items(type) 
      end
    end
  end

  def prepare_bopv_elasticsearch_test
    Elasticsearch::Base::delete_bopv_index    
    Elasticsearch::Base::create_bopv_index    
  end

  # ActiveSupport::TestCase
  def assert_indexed_in_elasticsearch(item, index=ELASTICSEARCH_URI)
    response_body = get_item_from_elasticsearch(item, true, index)      
    assert response_body['exists'].eql?(true), "Expected #{item} to be indexed in elasticsearch, but it isn't"    
  end

  def assert_deleted_from_elasticsearch(item, index=ELASTICSEARCH_URI)
    response_body = get_item_from_elasticsearch(item, false, index)      
    assert response_body['exists'].nil? || response_body['exists'].eql?(false), "Expected #{item} not to be indexed in elasticsearch, but it is"        
  end
  
  def get_item_from_elasticsearch(actual, expected_return, index=ELASTICSEARCH_URI)
    uri=(URI.parse("#{index}" + "/#{actual.class.to_s.tableize}/#{actual.id}"))
    begin
      Net::HTTP.start(uri.host, uri.port) do |http|
        headers = { 'Content-Type' => 'application/json'}
        response = http.send_request("GET", uri.request_uri, '', headers)
        return JSON.parse(response.body)
      end                                        
    rescue => e
      return {"exists" => expected_return}
    end 
  end

  def elasticsearch_available?
    uri=URI.parse("#{ELASTICSEARCH_URI}")
    begin
      Net::HTTP.start(uri.host, uri.port) do |http|
        headers = { 'Content-Type' => 'application/json'}
        response = http.send_request("GET", uri.request_uri, '', headers) 
        return true
      end                                        
    rescue   => e
      return false
    end    
  end
  
end  

