#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class AccountControllerTest < ActionController::TestCase
  
  def setup
    http_login()
  end

  # ["politician_interior", "admin", "jefe_de_prensa", "miembro_que_modifica_noticias", "jefe_de_gabinete", "operador_de_streaming", "colaborador", "comentador_oficial", "secretaria_interior", "room_manager"].each do |role|
  ["politician_interior", "politician_one", "politician_lehendakaritza"].each do |role|
    test "#{role}'s account should redirect to politician page" do
      login_as(role)
      get :show
      assert_redirected_to politician_path(users(role.to_sym))
    end
  end

  
  ["jefe_de_prensa", "miembro_que_modifica_noticias", "jefe_de_gabinete", "operador_de_streaming", "colaborador", "comentador_oficial", "secretaria_interior", "room_manager"].each do |role|
    test "#{role}'s account should redirect to administration" do
      login_as(role)
      get :show
      assert_redirected_to sadmin_account_path
    end
  end
  
  ["admin", "twitter_user", "facebook_user", "periodista", "visitante"].each do |role|
    test "#{role}'s account should show his profile" do
      login_as(role)
      get :show
      assert_response :success
      assert_template "show"
    end
  end
  
  
  test "account edit for secretaria de interior should redirect to admin" do
    login_as(:secretaria_interior)
    get :edit
    assert_redirected_to admin_path
  end

  ["twitter_user", "facebook_user"].each do |role|
    # TODO
    test "#{role} cannot view password edit link" do
      # login_as("twitter_user")
      # get :show
      # assert_response :success
      # assert_select "ul.edit_links li", :count => 2 do
      #   assert_select "a", :text => "Modificar mis datos"
      #   assert_select "a", :text => "Eliminar mi cuenta"
      #   assert_select "a", :text => "Modificar mi contraseña", :count => 0
      # end
    end
  
    test "#{role} can edit info" do
      login_as(role)
      get :edit
      assert_response :success
      assert_template "edit"
      assert_select "form[action=?]", '/es/account' do
        assert_select "input[name=?]", 'user[email]'
        assert_select "input[name=?]", 'user[name]'
        assert_select "input[name=?]", 'user[last_names]'
        assert_select "input[name=?]", 'user[password]', :count => 0
      end
    end
  
    test "#{role} cannot edit password" do
      login_as(role)
      get :pwd_edit
      assert_not_authorized
    end
  
  
    test "#{role} cannot update password" do
      login_as(role)
      put :pwd_update
      assert_not_authorized
    end
  end
  
  context "edit personal account" do 
    ["periodista", "visitante", "politician_lehendakaritza"].each do |role|
      should "should edit account for #{role}" do
        login_as(role)
        get :edit
        assert_response :success
        assert_template "edit"
      end  
    end  
    
    ["colaborador", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
     "jefe_de_gabinete", "jefe_de_prensa", "miembro_que_modifica_noticias", "room_manager", "admin", "politician_interior"].each do |role|
      should "should redirect to admin for #{role}" do
        login_as(role)
        get :edit
        assert_response :redirect
        assert_redirected_to admin_url
      end
    end
  end  

  context "show personal account" do 
    ["twitter_user", "facebook_user", "periodista", "visitante", "admin"].each do |role|
      should "should edit account for #{role}" do
        login_as(role)
        get :show
        assert_response :success
      end  
    end  
    
    ["politician_lehendakaritza", "politician_interior"].each do |role|
      should "redirect to politician show for #{role}" do
        login_as(role)
        get :show
        assert_response :redirect
        assert_redirected_to politician_path(assigns(:current_user))
      end
    end
    
    ["colaborador", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
     "jefe_de_gabinete", "jefe_de_prensa", "miembro_que_modifica_noticias", "room_manager"].each do |role|
      should "should redirect to sadmin/account for #{role}" do
        login_as(role)
        get :show
        assert_response :redirect
        assert_redirected_to sadmin_account_url
      end
    end
  end  

  
  ["colaborador", "periodista", "visitante", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
     "jefe_de_gabinete", "jefe_de_prensa", "miembro_que_modifica_noticias", "room_manager", "twitter_user", "facebook_user"].each do |role|
    test "#{role} can delete his account" do
      login_as(role)
      assert_no_difference 'User.count' do
        delete :destroy
      end
      assert_equal I18n.t('account.cuenta_eliminada'), flash[:notice]
      assert_redirected_to root_path
    end
  end
  
  context "periodista con alertas" do
    setup do
      login_as(:periodista_con_alertas)
    end
    
    should "view department subscription info" do
      get :edit
      assert_select 'input.depts'
    end
        
    should "be able to unsubscribe from all departments" do
      put :update, :user => {:department_ids => []}
      assert_response :redirect
    end
  
    should "should delete pending alerts when unsubscribing from a department" do
      # Una alerta es de lehendakaritza y la otra de gobierno vasco
      assert_equal 2, EventAlert.unsent.count(:conditions => ["spammable_id=? AND spammable_type='Journalist'", users("periodista_con_alertas").id])
      # era de lehendakaritza y lo pasamos a gobierno vasco
      put :update, :user => {:department_ids => [organizations(:gobierno_vasco).id]}
      assert_equal 1, EventAlert.unsent.count(:conditions => ["spammable_id=? AND spammable_type='Journalist'", users("periodista_con_alertas").id])
    end
  end
  
  test "activate user account" do
    user = users(:visitante_sin_activar)
    get :activate, :u => user.id, :p => user.crypted_password
    assert_response :success
    assert_template 'activate.html.erb'
    assert_select "h1", :text => I18n.t('account.cuenta_activada')
  end  
    
  test "do not activate user account invalid password" do
    user = users(:visitante_sin_activar)
    get :activate, :u => user.id, :p => "00742970dc9e6319f8019fd54864d3ea740f04b2"
    assert_response :success
    assert_template 'activate.html.erb'
    assert_select "h1", :text => I18n.t('account.informacion_incorrecta')
  end

  # HQ 
  # context "my questions" do
  #   setup do
  #     login_as('visitante')
  #     get :questions
  #   end
  #   
  #   should "list approval pending questions" do
  #     assert assigns(:questions).include?(contributions(:unapproved_question_for_politician))
  #     assert assigns(:questions).include?(contributions(:unapproved_question))
  #     assert assigns(:questions).include?(contributions(:question_for_politician))
  #     assert_select 'li.not_moderated', :count => 3
  #   end
  # end
  
  context "my proposals" do
    setup do
      login_as('visitante')
      get :proposals
    end
    
    should "list approval pending proposals" do
      assert assigns(:proposals).include?(contributions(:unapproved_proposal))
      assert_select 'li.not_moderated', :count => 2
    end
  end
  
  context "floki activity" do
    setup do
      login_as('visitante')
      get :activity, :format => "floki"
    end
    
    should "respond successfully" do
      assert_response :success
      assert_template 'activity.json'
      @output = JSON.parse(@response.body)
    end

    context "fetched_content" do
      setup do
        @output = JSON.parse(@response.body)
        
        @response_titles = @output["items"].collect {|item| item['title']}
      end
      
      # HQ
      # should "show response to my question" do
      #   assert assigns(:my_content).include?(answers(:answer))
      #   @response_titles.include?(I18n.t('floki.activity.answer_to_your_question', :question => answers(:answer).question.title))
      # end
      
      should "show votes to my proposals" do
        assert assigns(:my_content).include?(votes(:one))
        value = votes(:one).value
        @response_titles.include?(I18n.t('floki.activity.vote_for', :value => (value == -1 ? I18n.t('proposals.against') : I18n.t('proposals.in_favor')), :title => votes(:one).votable.title))
      end
      
      # HQ
      # should "show comment on my question" do
      #   assert assigns(:my_content).include?(comments(:comentario_de_pregunta))
      #   item =comments(:comentario_de_pregunta)
      #   @response_titles.include?(I18n.t('floki.activity.comment_in', :content_type => item.commentable.class.human_name, :title => item.commentable.title))
      # end
      
    end
  end

  # HQ
  # context "floki my questions" do
  #   setup do
  #     login_as('visitante')
  #     get :questions, :format => "floki"
  #   end
  #   
  #   should "list approval pending questions" do
  #     assert assigns(:questions).include?(contributions(:unapproved_question_for_politician))
  #     assert assigns(:questions).include?(contributions(:unapproved_question))
  #     assert assigns(:questions).include?(contributions(:question_for_politician))
  #     output = JSON.parse(@response.body)
  #     bodies = output['items'].collect {|item| item['body']}
  #     assert_equal 3, bodies.select {|b| b.match(/#{I18n.t('questions.create.moderation_pending')}/)}.length
  #   end
  # end
  # 
  # context "floki my proposals" do
  #   setup do
  #     login_as('visitante')
  #     get :proposals, :format => "floki"
  #   end
  #   
  #   should "list approval pending proposals" do
  #     assert assigns(:proposals).include?(contributions(:unapproved_proposal))
  #     output = JSON.parse(@response.body)
  #     bodies = output['items'].collect {|item| item['body']}
  #     assert_equal 1, bodies.select {|b| b.match(/#{I18n.t('proposals.create.moderation_pending')}/)}.length
  #   end
  # end
end
