#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Admin::CommentsControllerTest < ActionController::TestCase
  def setup
    http_login()
  end
  
  test "unlogged user should not be redirected" do
    get :index
    assert_not_authorized
  end
  
  ["admin", "jefe_de_gabinete", "jefe_de_prensa"].each do |role|
    test "show if logged as #{role}" do
      login_as(role)
      get :index
      assert_response :success
      assert_template "index"
    end
  end
  
  ["periodista", "visitante", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
   "miembro_que_modifica_noticias", "colaborador", "room_manager"].each do |role|
     test "redirect if logged as #{role}" do
       login_as(role)
       get :index
       assert_not_authorized     
     end     
  end
  
  ["jefe_de_gabinete", "jefe_de_prensa"].each do |role|
    test "#{role} cannot edit comments" do
      login_as(role)
      get :edit, :id => comments(:aprobado_castellano).id
      assert_not_authorized
    end
  end  
  
  ["jefe_de_gabinete", "jefe_de_prensa"].each do |role|
    test "#{role} cannot update comments" do
      login_as(role)
      put :update, :id => comments(:aprobado_castellano).id, :comment => {:body => "changed"}
      assert_not_authorized
    end
  end
  
  test "admin can edit comment" do
    login_as("admin")
    get :edit, :id => comments(:aprobado_castellano).id
    assert_response :success
    assert_template "edit"
  end
  
  test "admin can update comment" do
    login_as("admin")
    put :update, :id => comments(:aprobado_castellano).id, :comment => {:body => "changed"}
    assert_redirected_to admin_comments_path
  end
  
  test "should not list other departments comments when department is selected" do
    login_as(:admin)
    get :index, :dep_id => organizations(:interior).id
    assert_response :success
    assert_select 'h1.title', "Comentarios del departamento Interior"
    assert !assigns(:comments).include?(comments(:aprobado_castellano))
  end
  
  test "should list comments belonging to my department if i am logged in as a department editor" do
    login_as(:jefe_de_prensa)
    get :index
    assert_response :success
    assert_select 'h1.title', "Comentarios del departamento Presidencia"
    # All comments of resources tagged with '_lehendakaritza' tag should be listed
    documents_with_lehendakaritza_tag = Document.find_tagged_with(organizations(:lehendakaritza).tag_name)
    comments_on_documents_with_lehendakaritza_tag = Comment.find(:all, :conditions => "commentable_type='Document' AND commentable_id in (#{documents_with_lehendakaritza_tag.collect(&:id).join(', ')})")
    comments_on_documents_with_lehendakaritza_tag.each do |doc_comment|
      assert assigns(:comments).include?(doc_comment), "Comment ##{doc_comment.id} on document #{doc_comment.commentable_id} should be listed but it is not"
    end
    
    videos_with_lehendakaritza_tag = Video.find_tagged_with(organizations(:lehendakaritza).tag_name)
    comments_on_videos_with_lehendakaritza_tag = Comment.find(:all, :conditions => "commentable_type='Video' AND commentable_id in (#{videos_with_lehendakaritza_tag.collect(&:id).join(', ')})")
    comments_on_videos_with_lehendakaritza_tag.each do |video_comment|
      assert assigns(:comments).include?(video_comment), "Comment ##{video_comment.id} on video #{video_comment.commentable_id} should be listed but it is not"
    end
    
    assert !assigns(:comments).include?(comments(:aprobado_de_interior)), "El comentario de la noticia de interior no debería estar ahí"
  end
  
  test "should show forward links and rrcc links for comments that are not offical" do
    login_as("admin")
    get :index
    assert_response :success
  end

end
