#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Admin::EventLocationsControllerTest < ActionController::TestCase
  def setup
    http_login()
  end
  
  test "unlogged user should not be redirected" do
    get :index
    assert_not_authorized
  end
  
  ["admin"].each do |role|
    test "get index if logged as #{role}" do
      login_as(role)  
      get :index
      assert_response :success
      assert_template "index"
    end
  end
  
  ["periodista", "visitante", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
   "jefe_de_gabinete", "jefe_de_prensa", "miembro_que_modifica_noticias", "colaborador", "room_manager"].each do |role|
     test "redirect if logged as #{role}" do
       login_as(role)
       get :index
       assert_not_authorized     
     end     
  end
  
  
  test "show location" do
    login_as(:admin)
    get :show, :id => event_locations(:el_lehendakaritza).id
    assert_response :success
  end

  test "edit location" do
    login_as(:admin)
    get :edit, :id => event_locations(:el_lehendakaritza).id
    assert_response :success
    
    assert_select 'input#event_location_address'
  end

  test "new location" do
    login_as(:admin)
    get :new
    assert_response :success
    
    assert_select 'input#event_location_address'
  end
  
  test "create location" do
    login_as(:admin)
    
    assert_difference "EventLocation.count" do
      post :create, :event_location => location_params
    end
      
    assert_response :redirect
  end


  test "update location" do
    login_as(:admin)
    
    assert_no_difference "EventLocation.count" do
      put :update, :id => event_locations(:el_lehendakaritza).id, :event_location => location_params
    end
      
    assert_response :redirect
    assert assigns(:location)
    assert_equal "Pamplona", assigns(:location).city
  end

  private 
  def location_params
    {:place => "Sede Pamplona", :city => "Pamplona", :address => "Gran Vía", :lat => "42.820680", :lng => "-1.644545"}
  end  
end
