#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Admin::HeadlinesControllerTest < ActionController::TestCase
  def setup
    http_login()
  end
  
  test "should redirect if not logged in" do
    get :index
    assert_not_authorized
  end
  
  test "should redirect if not admin" do
    login_as('jefe_de_gabinete')
    get :index
    assert_not_authorized
  end

  test "should get #index if admin" do
    login_as("admin")
    get :index
    assert_response :success    
  end             
  
  test "should #update headline status" do
    login_as(:admin)    
    hl = headlines(:headline_media)
    put :update, :id => hl.id, :headline => {:draft => true}, :format => 'js'
    assert_response :success
  end                       
  
  test "should #update headline area" do
    login_as(:admin)    
    hl = headlines(:headline_media)
    area = areas(:a_lehendakaritza)
    assert_equal nil, hl.area
    put :update_area, :id => hl.id, :area_id => area.id, :format => 'js'
    assert_response :success                                         
    hl.reload
    assert_equal area, hl.area
  end

  test "should #update headline tag_list" do
    login_as(:admin)    
    hl = headlines(:headline_media)
    assert_equal [], hl.tag_list_without_areas
    assert_difference 'Tagging.count', +1 do
      assert_difference 'Tag.count', +1 do
        put :update, :id => hl.id, :build_params => 'true', :tag_list => 'new_tag', :format => 'js'
      end
    end
    assert_response :success                                         
    hl.reload
    assert_equal ['new_tag'], hl.tag_list
  end
  
  test "should #destroy" do
    login_as(:admin)    
    hl = headlines(:headline_media)
    assert_difference 'Headline.count', -1 do
      delete :destroy, :id => hl.id, :format => 'js'
    end                                            
    assert_response :success
  end                       
  
  test "should #delete from entzumena" do    
    hl = headlines(:headline_media)                                                                               
    assert_difference 'Headline.count', -1 do
      delete :delete_from_entzumena, :source_item_id => hl.source_item_id, :source_item_type => hl.source_item_type
    end  
    assert_response :success
  end  
      
end
