#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Admin::OrganizationsControllerTest < ActionController::TestCase
  def setup
    http_login()
  end
  
  test "unlogged user should not be redirected" do
    get :index
    assert_not_authorized
  end
  
  ["admin"].each do |role|
    test "show if logged as #{role}" do
      login_as(role)
      get :index
      assert_response :success
      assert_template "index"
    end
  end
  
  ["periodista", "visitante", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
   "jefe_de_gabinete", "jefe_de_prensa", "miembro_que_modifica_noticias", "colaborador", "room_manager"].each do |role|
     test "redirect if logged as #{role}" do
       login_as(role)
       get :index
       assert_not_authorized     
     end     
  end
  
  # 2DO
  
  test "should get index" do
    login_as('admin')
    get :index
    assert_response :success
    assert_not_nil assigns(:organizations)
  end
  
  test "should get new organization" do
    login_as('admin')
    get :new
    assert_response :success
    assert assigns(:organization).is_a?(Organization)
  end
  
  test "should get new department" do
    login_as('admin')
    get :new, :d => 1
    assert_response :success
    assert assigns(:organization).is_a?(Department)
  end
  
  test "should create organization" do
    login_as('admin')
    assert_difference('Organization.count') do
      post :create, :organization => {:name_es => "New organization", :enet_workarea => "area_actuacion|workarea" }
    end
  
    assert_redirected_to admin_organization_path(assigns(:organization))
  end

  test "should create department" do
    login_as('admin')
    assert_difference('Department.count') do
      post :create, :organization => {:name_es => "New department", :tag_name=> "_new", :enet_workarea => "area_actuacion|workarea" }
    end
    assert_redirected_to admin_organization_path(assigns(:organization))
  end
  
  test "should show organization" do
    login_as('admin')
    get :show, :id => organizations(:lehendakaritza).id
    assert_response :success
  end
  
  test "should get edit" do
    login_as('admin')
    get :edit, :id => organizations(:emakunde).id
    assert_response :success
  end
  
  test "should update organization" do
    login_as('admin')
    put :update, :id => organizations(:emakunde).id, :organization => {:name_es => "Emakunde 2" }
    assert_redirected_to admin_organization_path(assigns(:organization))
  end
  
  # test "should destroy organization" do
  #   assert_difference('Organization.count', -1) do
  #     delete :destroy, :id => admin_organizations(:lehendakaritza).id
  #   end
  # 
  #   assert_redirected_to admin_organizations_path
  # end
end
