#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Admin::OutsideOrganizationsControllerTest < ActionController::TestCase
  def setup 
    http_login()
    login_as('admin')
  end

  context "list" do
    setup do
      get :index
    end
    should_respond_with :success    
    
    should "show create debate link" do
      assert_select "ul.edit_links li a", :text => "Añadir entidad"
    end

    should "show submenu with Entidades tab" do
      assert_select "div.submenu" do
        assert_select "ul" do
          assert_select "li", :count => 3
          assert_select "li a", :text => "Propuestas"        
          assert_select "li a", :text => "Argumentos"
          assert_select "li", :text => "Entidades"          
        end
      end
    end
  end
  
  context "new organization" do
    should "get new" do
      get :new
      assert_response :success
    end
    
    should "create organization" do
      assert_difference "OutsideOrganization.count", 1 do
        post :create, :organization => {"name_es"=>"Nueva entidad relacionada sin logo"}
      end
      assert_response :redirect
      assert_redirected_to admin_outside_organizations_path
    end
    
    should "not create organization without name_es" do
      assert_no_difference "OutsideOrganization.count" do
        post :create, :organization => {"name_eu"=>"Nueva entidad relacionada sin logo"}
      end
      assert_response :success
    end    
  end
  
  context "existing organization" do
    setup do
      @org = outside_organizations(:organization_for_debate)
    end
    
    should "get edit" do
      get :edit, :id => @org.id
      assert_response :success
    end
    
    should "update organization" do
      put :update, :id => @org.id, :organization => {"name_es"=>"Entidad", "name_eu"=>"Entidad EU", "name_en"=>"Entity",      
                                                    "logo"=> File.new(File.join(RAILS_ROOT, 'test/data/photos', 'test70x70.png'))}
      assert_response :redirect
    end
    
    should "destroy organization" do
      assert_difference "OutsideOrganization.count", -1 do
        delete :destroy, :id => @org.id
      end
      assert_response :redirect
    end
  end
end
