#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Admin::PermissionsControllerTest < ActionController::TestCase
  def setup
    http_login()
  end

  test "redirect for unlogged" do
    get :show
    assert_not_authorized
  end
  
  ["periodista", "visitante", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
   "jefe_de_gabinete", "jefe_de_prensa", "colaborador", "admin", "room_manager"].each do |role|
    test "#{role} cannot administer permissions" do
      login_as(role)
      get :show, :user_id => users(:periodista).id
      assert_not_authorized
      
      get :edit, :user_id => users(:periodista).id
      assert_not_authorized
      
      put :update, :user_id => users(:miembro_que_crea_noticias).id, :perm => {:news => {:create => 1, :complete => 1}, :events => {:create_private => 1}, :schedule_events => {:can_edit => 1}}
      assert_not_authorized
    end
  end  
  
  test "superadmin can view permissions" do
    login_as("superadmin")
    get :show, :user_id => users(:periodista).id
    assert_response :success
    assert_template "show"
    assert_select "ul.edit_links li a.edit" do
      assert_select "[href=?]", /.+permissions\/edit/
    end
  end
  
  test "miembro_que_crea_noticias has correct permissions" do
    login_as("superadmin")
    get :edit, :user_id => users(:miembro_que_crea_noticias).id
    assert_response :success
    assert_template "edit"
    assert_select "input[type=checkbox]", :count => 10
    assert_select "input[type=checkbox][checked=checked]", :checked => true, :count => 1
    assert_select "input#perm_news_create", :checked => true
  end
  
  test "i can change permissions for miembro_que_crea_noticias" do
    login_as("superadmin")
    user = users(:miembro_que_crea_noticias)
    assert_equal 1, user.permissions.count
    assert_equal 0, user.schedules_permissions.count
    
    # Es del departamento de interior, asi que esta es su agenda
    agenda_interior = schedules(:agenda_interior)
    assert !user.can_edit_schedule?(agenda_interior.id)
    assert !user.can_change_schedule?(agenda_interior.id)
    
    assert user.can?("create", "news")
    assert !user.can?("complete", "news")
    assert !user.can?("create_private", "events")
    
    put :update, :user_id => user.id, :perm => {:news => {:create => 1, :complete => 1}, :events => {:create_private => 1}, :schedule_events => {:can_edit => 1}}
    assert_redirected_to admin_user_path(user)
    
    assert_equal 3, user.permissions.count
    assert_equal 1, user.schedules_permissions.count

    assert user.can_edit_schedule?(agenda_interior.id)
    assert !user.can_change_schedule?(agenda_interior.id)
    
    assert user.can?("create", "news")
    assert user.can?("complete", "news")
    assert user.can?("create_private", "events")
  end
  
  test "should delete all permissions of one user" do
    login_as("superadmin")
    user = users(:miembro_con_permisos_de_todo_tipo)
    assert_equal 3, user.permissions.count
    assert_equal 1, user.schedules_permissions.count
    get :edit, :user_id => user.id
    assert_select "input[type=checkbox][checked=checked]", :count => 5
    assert_select "input#perm_news_create[checked=checked]"
    assert_select "input#perm_comments_official[checked=checked]"
    assert_select "input#perm_events_create_irekia[checked=checked]"
    assert_select "input#perm_schedule_events_can_edit[checked=checked]"
    assert_select "input#perm_schedule_events_can_change_schedule[checked=checked]"
    
    put :update, :user_id => user.id # quitamos todos los permisos
    assert_redirected_to admin_user_path(assigns(:user))
    assert_equal 0, assigns(:user).permissions.count
    assert_equal 0, assigns(:user).schedules_permissions.count
  end
  

end
