#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Admin::ProposalsControllerTest < ActionController::TestCase
  def setup 
    http_login()
    login_as('admin')
    ActionMailer::Base.deliveries = []
  end
  
  test "only citizen proposals are shown" do
    get :index
    
    assert_select "div.submenu" do
      assert_select 'li', :text => "Propuestas"
      assert_select 'li', :text => "Argumentos"      
      assert_select 'li', :text => "Gubernamentales", :count => 0            
    end
  end

  test "show proposals arguments" do
    get :arguments
    
    assert_response :success
    assert assigns(:arguments)
    assert_nil assigns(:arguments).detect {|a| a.argumentable_type.eql?('Debate')}
  end
  
  test "pending proposal without organization cannot be approved" do
    proposal = contributions(:unapproved_proposal)
    get :index
    assert_select "tr#proposal_#{proposal.id} td.status" do
      assert_select 'a', :count => 1
      assert_select 'a', 'Eliminar' #no hay link de aprobar
    end
  end
  
  context "with lehendakaritza proposal" do
    setup do
      @proposal = contributions(:unapproved_proposal)
      @proposal.update_attribute(:organization_id, organizations(:lehendakaritza).id)
      @proposal.reload
    end
    
    should "show proposal" do
      get :show, :id => @proposal.id
      assert_response :success
    end
    
    should "pending proposal with organization can be approved" do
      get :index
      assert_select "tr#proposal_#{@proposal.id} td.status" do
        assert_select 'a', 'Aprobar'
      end
    end
    
    should "send email to author and department staff when approving proposal" do
      xhr :put, :update, :id => @proposal.id, :proposal => {:status => 'aprobado'}
      # son 2, uno al departament_editor y al department member con permiso para responder oficialmente y otro al autor
      assert_equal 2, ActionMailer::Base.deliveries.size 
      email = ActionMailer::Base.deliveries.detect {|e| e.to.eql?([@proposal.user.email])}
      # email = ActionMailer::Base.deliveries.last
      # assert_equal [@proposal.user.email], email.to
      assert email.subject.match(I18n.t('notifier.proposal_approval.subject'))
    end
    
    should "changing pending proposal's department does not send email to department responders" do
      put :update, :id => @proposal.id, :proposal => {:organization_id => organizations(:emakunde).id}
      assert_equal 0, ActionMailer::Base.deliveries.size
    end

    context "approve proposal" do
      setup do
        @proposal = contributions(:featured_proposal)
      end
      
      should "changing department sends email to department responders" do
        put :update, :id => @proposal.id, :proposal => {:organization_id => organizations(:interior).id}
        # uno para el department editor y dos para los department members official commenters del departamento nuevo
        assert_equal 1, ActionMailer::Base.deliveries.size
        email = ActionMailer::Base.deliveries.last
        @proposal.reload
        assert_equal (@proposal.department.department_editors.collect(&:email)+@proposal.department.department_members_official_commenters.collect(&:email)).sort, email.to.sort
        assert email.subject.match("Nueva propuesta en Irekia")
      end
    end
  end
end
