#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Admin::UsersControllerTest < ActionController::TestCase
  def setup
    http_login()
  end

  test "redirect for unlogged" do
    get :index
    assert_not_authorized
  end
  
  ["periodista", "visitante", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
   "jefe_de_gabinete", "jefe_de_prensa", "colaborador", "room_manager", "admin"].each do |role|
    test "redirect if logged as #{role}" do
      login_as(role)
      get :index
      assert_not_authorized
    end    
  end  
  
  test "get index for superadmins" do
     login_as(:superadmin)
     get :index
     assert_response :success
     assert_template "admin/users/index.rhtml"
  end
  
  User::TYPES.each do |t, pretty_name|
    test "should show users of type #{t} for admin" do
      login_as(:superadmin)
      get :index, :t => t
      assert_response :success      
    end
  end  

  
  
  test "bann user" do
    login_as(:superadmin)
    u = users(:jefe_de_gabinete)
    post :update, :id => u.id, :user => {:status => "vetado"}
    assert_response :redirect
    
    updated = User.find(u.id)
    assert_equal "vetado", updated.status
    assert_equal u.department_id, updated.department_id
  end
  
  test "update user data" do
    login_as(:superadmin)
    u = users(:jefe_de_gabinete)
    
    new_params = {:name => "JG", :last_names => "Xxx", :telephone => "555-555-555", :email => 'jg@xxx.com', :department_id => organizations(:gobierno_vasco).id, :status => "aprobado"}
    post :update, :id => u.id, :user => new_params
    assert_response :redirect
    
    updated = User.find(u.id)    
    new_params.each do |key, value|
      assert_equal value, updated.send(key.to_s)
    end    
  end
  
  test "should send access info to journalist after approval" do
    login_as(:superadmin)
    user = users(:periodista_sin_aprobar)
    assert_difference 'ActionMailer::Base.deliveries.size', + 1 do  
      put :update, :id => user.id, :locale => "eu",
          :user => {:type => "Journalist", :status => "aprobado" } 
    end 

    m = ActionMailer::Base.deliveries.last
    assert_equal m.subject, "[#{Notifier::SERVICENAME}] Ongi etorri Irekia(e)ra", m.subject
    assert_equal m.to[0], 'periodista_sin_aprobar@efaber.net'
    assert_match 'Ésta es tu información de acceso en', m.body
  end
  
  # Deprecated porque ahora los permisos de editar usuario y editar permiso van juntos
  # test "admin should not see permissions edit link" do
  #   login_as("admin")
  #   get :show, :id => users(:jefe_de_gabinete).id
  #   assert_response :success
  #   assert_template "show"
  #   assert_select "ul.edit_links li a.edit" do
  #     assert_select "[href=?]", /.+permissions\/edit/, :count => 0
  #   end
  # end
  
  test "superadmin should see permissions edit link" do
    login_as("superadmin")
    get :show, :id => users(:jefe_de_gabinete).id
    assert_response :success
    assert_template "show"
    assert_select "ul.edit_links li a.edit" do
      assert_select "[href=?]", /.+permissions\/edit/
    end
  end
  
  # test "admin does not administer users" do
  #   login_as("admin")
  #   !helper.can_access?("users")
  # end
  # 
  # test "superadmin administers users" do
  #   login_as("superadmin")
  #   helper.can_access?("users")
  # end

  test "should show media for journalists list" do
    login_as(:superadmin)
    get :index, :t => 'Journalist'
    assert_response :success
    
    assert_select "table.users_list" do
      assert_select 'th', 'Nombre'
    end
    
    assert_select 'table.last_logins' do
      assert_select 'th', 'Cuándo'
    end
  end

  test "should find twitter user" do
    login_as(:superadmin)
    twitter_user = users(:twitter_user)
    
    
    get :index, :t => 'Person', :q => twitter_user.name
    assert_response :success
    
    assert assigns(:users)
    assert assigns(:users).detect {|u| u.eql?(twitter_user)}
  end  
end


