#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Admin::VideosControllerTest < ActionController::TestCase
  def setup
    http_login()
  end
  
  test "unlogged user should not be redirected" do
    get :index
    assert_not_authorized
  end
  
  ["admin", "colaborador"].each do |role|
    test "show if logged as #{role}" do
      login_as(role)
      get :index
      assert_response :success
      assert_template "index"
    end
  end
  
  ["periodista", "visitante", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
   "jefe_de_gabinete", "jefe_de_prensa", "miembro_que_modifica_noticias", "room_manager"].each do |role|
     test "redirect if logged as #{role}" do
       login_as(role)
       get :index
       assert_not_authorized     
     end     
  end
  
  test "should get index" do
    login_as(:admin)
    get :index
    assert_response :success
    assert_template "index"
  end
  
  # Subtitles
  
  test "should not upload subtitles if no document is present" do
    login_as(:admin)
    video = videos(:only_es)
    file = File.new(File.join(RAILS_ROOT, "test", "data", "test.srt"))
    assert_nil video.document_id
    
    post :update_subtitles, :id => video.id, :locale => 'es', :subtitles_es => file
    assert_response :redirect
    assert_redirected_to admin_video_path(video)
  end

  test "should upload subtitles if video belongs to document" do
    login_as(:admin)
    video = videos(:only_es)
    file = File.new(File.join(RAILS_ROOT, "test", "data", "test.srt"))
    document = documents(:news_with_multimedia_dir_and_related_webtv_video_and_gallery_photo)
    
    video.update_attribute(:document_id, document.id)
    assert_not_nil video.document_id
    
    assert !video.subtitles_es.exists?
    
    post :update_subtitles, :id => video.id, :locale => 'es', :video => {:subtitles_es => file}
    assert_response :redirect
    assert_redirected_to sadmin_news_subtitles_path(:news_id => document.id)
    
    video.reload
    assert video.subtitles_es.exists?
  end

  test "should delete subtitles for a specific language" do
    login_as(:admin)
    video = videos(:only_es)
    file = File.new(File.join(RAILS_ROOT, "test", "data", "test.srt"))
    document = documents(:news_with_multimedia_dir_and_related_webtv_video_and_gallery_photo)
    
    video.update_attributes(:document_id => document.id, :subtitles_es => file)
    assert video.subtitles_es.exists?
    
    post :delete_subtitles, :id => video.id, :locale => 'es', :lang => "es"
    assert_response :redirect
    assert_redirected_to sadmin_news_subtitles_path(:news_id => document.id)
    
    video.reload
    assert !video.subtitles_es.exists?
  end

  test "should upload subtitles and create video if document and flv are given" do
    login_as(:admin)
    srt_file = File.new(File.join(RAILS_ROOT, "test", "data", "test.srt"))
    document = documents(:news_with_multimedia_dir_and_related_webtv_video_and_gallery_photo)
    prepare_content_for_news_with_multimedia(document)
    
    flv_file_name = "test.flv"
    video_path = "#{document.multimedia_path}#{flv_file_name.gsub(/\.flv/,'')}"
    
    assert_equal "#{document.multimedia_path}#{flv_file_name}", document.videos[:featured][:es]
    assert !Video.find_by_video_path(video_path)
    assert_nil document.webtv_videos.detect {|v| v.video_path.eql?(video_path)}
    

    assert_difference("Video.count", 1) do
      post :create_with_subtitles, :locale => 'es', :video => {:document_id => document.id, :flv_url => document.videos[:featured][:es], :subtitles_es => srt_file}
    end
    assert_response :redirect
    assert_redirected_to sadmin_news_subtitles_path(:news_id => document.id)
    
    document.reload
    assert document.webtv_videos.detect {|v| v.video_path.eql?(video_path)}
    assert document.webtv_videos.detect {|v| v.video_path.eql?(video_path)}.subtitles_es.exists?
    
    clear_multimedia_dir(document)
  end
  
end
