#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class CommentsControllerTest < ActionController::TestCase
  def setup
    http_login()
  end
  
  test "should get comments RSS feed" do
    get :index, :format => "rss"
    assert_response :success
    assert_not_nil assigns(:comments)
    %w(aprobado_castellano aprobado_euskera aprobado_de_interior comentario_aprobado_en_video comentario_aprobado_en_pagina).each do |c|
      assert assigns(:comments).include?(comments(c.to_sym))
    end
    
    %w(rechazado_castellano pendiente_castellano spam_castellano rechazado_euskera pendiente_euskera spam_euskera).each do |c|
      assert !assigns(:comments).include?(comments(c.to_sym))
    end
    
    %w(aprobado_castellano aprobado_euskera aprobado_de_interior comentario_aprobado_en_video comentario_aprobado_en_pagina).each do |c|
      parent = comments(c.to_sym).commentable
      parent_url = eval("#{parent.class.to_s.downcase}_url(parent)")
      assert_tag 'link', :parent => {:tag => 'item'}, :content => /#{parent_url}/
    end
  end

  
  test "should get index" do
    get :index
    assert_response :success
    assert_not_nil assigns(:comments)
  end

  
  test "should create comment" do
    login_as(:admin)
    assert_difference('Comment.count') do
      post :create, :news_id => documents(:one_news).id, :comment => {:body => "comment" }
    end
  
    assert_redirected_to news_path(assigns(:parent))
  end
  
  test "should destroy comment" do
    login_as(:admin)
    assert_difference('Comment.count', -1) do
      delete :destroy, :news_id => documents(:commentable_news).id, :id => comments(:aprobado_castellano).id
    end

    assert_redirected_to comments_path
  end
  
  def assert_comments_not_authorized
    assert_equal I18n.t('session.tienes_que_registrarte2', :what => Comment.human_name.pluralize), flash[:notice]
    assert_template ""
  end
  
  %w(superadmin admin periodista visitante colaborador jefe_de_prensa jefe_de_gabinete secretaria comentador_oficial twitter_user facebook_user).each do |role|
    test "#{role} should be able to create comments" do
      login_as(role)
      assert_difference 'Comment.count', +1 do
        post :create, :news_id => documents(:commentable_news).id, :comment => {:body => "comentario"}
      end
      assert_redirected_to documents(:commentable_news)
      assert_equal (users(role).is_official_commenter? ? I18n.t('comments.comentario_guardado') : I18n.t('comments.comment_pending')), flash[:notice]
    end
  end
  
  
  %w(operador_de_streaming miembro_que_crea_noticias room_manager).each do |role|
    test "#{role} should not be able to create comments" do
      login_as(role)
      assert_no_difference 'Comment.count' do
        post :create, :news_id => documents(:commentable_news).id, :comment => {:body => "comentario"}
      end
      assert_redirected_to new_session_path
      assert_not_authorized
    end
  end
  
  test "comentador_oficial can change the name he signs with" do
    login_as(:comentador_oficial)
    assert_difference 'Comment.count', +1 do
      post :create, :news_id => documents(:commentable_news).id, :comment => {:body => "comentario", :name => "nombre cambiado"}
    end
    assert_equal "nombre cambiado", assigns(:comment).name
  end

end
