#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require "test_helper"

class IrekiaAuthControllerTest < ActionController::TestCase

  def setup
    @salt = IrekiaAuthController::SALT
    @token = IrekiaAuthController::AUTH_TOKEN
  end
  
  test "should authenticate approved user with post" do
    user = users(:visitante)
    
    post :authenticate, :data => "{\"token\": \"#{@token}\", \"email\": \"#{user.email}\", \"password\": \"#{User.encrypt(user.plain_password, @salt)}\"}"
    assert_response :success
    
    assert assigns(:auth_response)
    ar = JSON.parse(assigns(:auth_response))
    
    assert_equal IrekiaAuthController::AUTH_STATUS['aprobado'], ar["status"]
    assert_equal user.id, ar["user_id"]    
    
    ['role', 'name', 'lastname', 'postal_code', 'province', 'city'].each do |label|
      assert_equal ar[label], user.send("#{label}4api")
    end
    
  end

  test "should not authenticate approved user with wrong password" do
    user = users(:visitante)
    # salt = IrekiaAuthController::SALT
    post :authenticate, :data => "{\"token\": \"#{@token}\", \"email\": \"#{user.email}\", \"password\": \"#{User.encrypt(user.plain_password.to_s+'xxx', @salt)}\"}"
    assert_response :success
    
    assert assigns(:auth_response)
    ar = JSON.parse(assigns(:auth_response))
    
    assert_equal IrekiaAuthController::AUTH_STATUS['not_found'], ar["status"]
    assert_nil ar["user_id"]    
    
    ['role', 'name', 'lastname', 'postal_code', 'province', 'city'].each do |label|
      assert_nil ar[label]
    end
    
  end


  test "should authenticate approved user with get" do
    user = users(:visitante)
    # salt = IrekiaAuthController::SALT
    get :authenticate, :data => "{\"token\": \"#{@token}\", \"email\": \"#{user.email}\", \"password\": \"#{User.encrypt(user.plain_password, @salt)}\"}"
    assert_response :success
    
    assert assigns(:auth_response)
    ar = JSON.parse(assigns(:auth_response))
    
    assert_equal IrekiaAuthController::AUTH_STATUS['aprobado'], ar["status"]
    assert_equal user.id, ar["user_id"]    
    
    ['role', 'name', 'lastname', 'postal_code', 'province', 'city'].each do |label|
      assert_equal ar[label], user.send("#{label}4api")
    end
    
  end


  test "should not authenticate not approved user" do
    user = users(:visitante_sin_activar)
    # salt = IrekiaAuthController::SALT
    post :authenticate, :data => "{\"token\": \"#{@token}\", \"email\": \"#{user.email}\", \"password\": \"#{User.encrypt(user.plain_password, @salt)}\"}"
    assert_response :success
    
    assert assigns(:auth_response)
    ar = JSON.parse(assigns(:auth_response))    
    assert_equal IrekiaAuthController::AUTH_STATUS['pendiente'], ar["status"]
    assert_equal user.id, ar["user_id"]    
  end
  
  
  test "should register new user and send activation email" do
    data = {:token => @token,
            :email => 'dos@efaber.net', 
            :password => 'xxxxx', 
            :name => 'Uno', 
            :lastname => 'Apellidos', 
            :postal_code => '48600'}
    assert_difference('Person.count', +1) do
      assert_difference 'ActionMailer::Base.deliveries.size', +1 do  
        post :register, :data => JSON.generate(data)
        assert_response :success 
      end
    end
    
    welcome_email = ActionMailer::Base.deliveries.last
    assert_equal "[#{Notifier::SERVICENAME}] Bienvenida", welcome_email.subject
    assert_equal 'dos@efaber.net', welcome_email.to[0]
    assert_match 'Para asegurarnos de que realmente has solicitado el alta en', welcome_email.body    
  end
  
end
