#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Sadmin::AttachmentsControllerTest < ActionController::TestCase
  def setup
    http_login()
  end
  
  test "unlogged user should be redirected" do
    get :new, :attachable_id => documents(:one_news), :attachable_type => 'Document'
    assert_redirected_to new_session_path
  end
  
  ["periodista", "visitante", "comentador_oficial", "operador_de_streaming", "secretaria_interior"].each do |role|
    test "redirect if logged as #{role}" do
      login_as(role)
      get :new, :attachable_id => documents(:one_news).id, :attachable_type => 'Document'
      assert_not_authorized
    end
  end
  
  test "redirect if logged as colaborador and tries to add attachment to event" do
    login_as("colaborador")
    get :new, :attachable_id => documents(:passed_event).id, :attachable_type => 'Document'
    assert_not_authorized
  end
  
  ["periodista", "visitante", "comentador_oficial", "secretaria_interior", "operador_de_streaming", \
   "jefe_de_gabinete", "jefe_de_prensa", "colaborador", "miembro_que_modifica_noticias", "room_manager"].each do |role|
     test "redirect if logged as #{role} and tries to add attachment to page" do
       login_as(role)
       get :new, :attachable_id => documents(:page_in_menu).id, :attachable_type => 'Document'
       assert_not_authorized     
     end     
  end
    
  test 'should get new document' do
    login_as(:jefe_de_gabinete)
  
    get :new, :attachable_id => documents(:current_event).id, :attachable_type => 'Document'
    assert_response :success
    
    assert_select "div.menu_admin ul li.active", :text => "Agenda"
  end
  
  test 'should get new proposal' do
    login_as(:admin)
  
    get :new, :attachable_id => contributions(:governmental_proposal).id, :attachable_type => 'Proposal'
    assert_response :success
    
    assert_select "div.menu_admin ul li.active", :text => I18n.t('admin.menu.propuestas')
  end

  test 'should get new debate' do
    login_as(:admin)
  
    get :new, :attachable_id => debates(:debate_completo).id, :attachable_type => 'Debate'
    assert_response :success
    
    assert_select "div.menu_admin ul li.active", :text => I18n.t('admin.menu.debates')
  end
  
  test "should create debate attachment and redirect" do
    login_as(:admin)
    
    debate = debates(:debate_completo)
    put :create, :attachable_id => debate.id, :attachable_type => 'Debate', :attachment => {:file => File.open(File.join(RAILS_ROOT, "test", "data", "test.txt"))}
    assert_redirected_to admin_debate_path(debate)
    assert 1, debate.attachments.count
  end
  
  test 'should get edit' do
    login_as(:jefe_de_gabinete)
  
    e = documents(:current_event)
    f = File.new(File.join(RAILS_ROOT, "test", "data", "test.txt"))
    e.attachments.create(:file => f, :attachable_type => 'Document', :attachable_id => e.id)
  
    get :edit, :id => e.attachments.first.id 
    assert_response :success
    
    assert_select "div.menu_admin ul li.active", :text => "Agenda"
  end

  test "miembro_que_crea_noticias can also add attachments" do
    login_as(:miembro_que_crea_noticias)
    get :new, :attachable_id => documents(:one_news).id, :attachable_type => 'Document'
    assert_response :success
    assert_template 'new'

    put :create, :attachable_id => documents(:one_news).id, :attachable_type => 'Document', :attachment => {:file => File.open(File.join(RAILS_ROOT, "test", "data", "test.txt"))}
    assert_redirected_to sadmin_news_path(documents(:one_news))
    assert 1, documents(:one_news).attachments.count
    
    # Now we modify visibility
    get :edit, :id => documents(:one_news).attachments.first.id
    assert_response :success
    assert_template 'edit'
    assert_select "div.menu_admin ul li.active", :text => "Noticias"
    
    put :update, :id => documents(:one_news).attachments.first.id, :attachment => {:show_in_eu => "1"}
    assert_redirected_to sadmin_news_path(documents(:one_news))
    assert_equal "El documento adjunto se ha guardado correctamente.", flash[:notice]
    assert documents(:one_news).attachments.first.show_in_eu?
    
    # Now, we can delete the recently created attachment
    delete :destroy, :id => documents(:one_news).attachments.first.id
    assert_redirected_to sadmin_news_path(documents(:one_news))
    assert_equal "El documento adjunto se ha eliminado correctamente.", flash[:notice]
  end  

end
