#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Sadmin::EventsControllerTest < ActionController::TestCase

  def setup
    http_login()
  end
      
  test "redirect if not logged" do
    get :index
    assert_not_authorized
  end

  ["admin", "jefe_de_prensa", "jefe_de_gabinete", "miembro_que_modifica_noticias", "comentador_oficial"].each do |role|
    test "show index if logged as #{role}" do
      login_as(role)
      get :index
      assert_response :success
      assert_template "index"
      
      assert_select "div.calendar_legend" do
        assert_select "ul" do
          assert_select "li:nth-child(1)", "Leyenda:"          
          assert_select "li:nth-child(2)", "Uso interno del Gobierno"
          assert_select "li:nth-child(3)", "Publicado en irekia"
          assert_select "li:nth-child(4)", "Cubre Irekia"
        end
      end      
      
      assert_select "ul.edit_links li a", "Suscripción"
    end
  end

  test "show index if logged as secretaria_interior" do
    login_as("secretaria_interior")
    get :index
    assert_response :success
    assert_template "index"
    
    assert_select "div.calendar_legend" do
      assert_select "ul" do
        assert_select "li:nth-child(1)", "Leyenda:"          
        assert_select "li:nth-child(2)", "Uso interno del dept. Interior"        
        assert_select "li:nth-child(3)", "Uso interno del Gobierno"
        assert_select "li:nth-child(4)", "Publicado en irekia"
        assert_select "li:nth-child(5)", "Cubre Irekia"
      end
    end      
    
    assert_select "ul.edit_links li a", "Suscripción"
  end


  ["periodista", "visitante", "colaborador", "operador_de_streaming", "room_manager"].each do |role|
    test "redirect if logged as #{role}" do
      login_as(role)
      get :index
      assert_not_authorized        
    end
  end
  
  test "index for secretaria interior" do
    login_as(:secretaria_interior)
    get :index
    assert_response :success
    
    # Department members have only the 'Agenda' option in the menu bar.
    assert_select "div.menu_admin" do
      assert_select "ul" do
        ["Agenda"].each_with_index do |text, n|
         assert_select "li:nth-child(#{n+1})" do
           assert_select 'a', text
         end
       end
       assert_select "ul li", :count => 1
      end
    end
    

    month = Time.now.month
    today = Time.now.day
    last_day= Time.now.at_end_of_month.day
    assert assigns(:events)

    # Department members can see all the events of the shared agenda.
    assert assigns(:events)[month][today].detect {|e| e.is_a?(Event) && !e.organization_id.eql?(organizations(:interior).id)}
    
    # Department member do see the public events that correspond to their department.
    assert assigns(:events)[month][last_day].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:interior_public_event).id)}
    
  end
  
  test "weekly view should show event starting in one week and ending in another" do
    login_as("admin")
    date = Date.today
    get :week, :day => date.day, :month => date.month, :year => date.year
    assert assigns(:events).detect {|e| e.id.eql?(documents(:event_with_duration_of_more_than_one_week).id)}, "Event event_with_duration_of_more_than_one_week should be listed"
    
    assert_select "div.check_schedule", :count => 0
  end

  test "weekly view should start with the first day of week" do
    login_as("admin")
    date = Time.zone.now.to_date
    first_day = Time.zone.now.at_beginning_of_week.to_date
    
    get :week, :day => date.day, :month => date.month, :year => date.year
    assert_response :success
    assert assigns(:events)
    
    week_events_dates = assigns(:events).map {|e| e.starts_at.to_date if e.starts_at.to_date >= first_day}.compact.uniq.sort
    
    days = css_select('div.one_day')
    assert_select days[0], 'h3', :text => I18n.localize(week_events_dates.first, :format => :long).strip
    assert_select 'div.long_events' do
      assert_select 'div.one_day h3', :text => I18n.localize(documents(:event_that_starts_the_week_before_now_with_duration_of_more_than_one_week).starts_at.to_date, :format => :long).strip
    end
   
  end

  test "weekly view should show event starting the last day of the week" do
    login_as("admin")
    date = Date.today
    get :week, :day => date.day, :month => date.month, :year => date.year
    
    assert assigns(:events).detect {|e| e.id.eql?(documents(:event_with_duration_of_more_than_one_week).id)}, "Event event_with_duration_of_more_than_one_week should be listed"
    assert assigns(:events).detect {|e| e.id.eql?(documents(:sunday_event).id)}, "Sunday event should be listed"
    
    assert_select "div.check_schedule", :count => 0
  end


  test "calendar action renders index.html" do
    login_as(:jefe_de_gabinete)
    post :calendar
    assert_response :success
    assert_template 'index.html.erb'
  end

  test "jefe_de_gabinete does not see schedules links" do
    login_as(:jefe_de_gabinete)
    get :index
    assert_response :success
    
    assert_select "div.create_links ul.edit_links", :count => 1
    assert_select "div.create_links ul.edit_links li", :count => 1
    assert_select "div.check_schedule", :count => 0
  end  

  test "secreatria sees create event link and schedules links" do
    login_as(:secretaria)
    get :index
    assert_response :success
    
    assert_select "div.create_links ul.edit_links", :count => 1
    
    # Secretaria can create evens only for the schedules with can_edit='t'
    assert_select "div.create_links ul.edit_links li", :count => users(:secretaria).editable_schedules_with_create_option.size + 1
    
    # Secretaria can see the evens of all schedules she has permission for.
    assert_select "div.check_schedule", :count => 1 do
      assert_select 'ul', :count => 1 do
        assert_select 'li', :count => users(:secretaria).schedules_permissions.count(:all) + 3
      end
    end
    
    assert_select "div.calendar_legend ul" do
      assert_select "li", "Agenda Lehendakari"
      assert_select "li", "Agenda especial Nº 2"
    end
    
  end

  test "jefe_de_gabinite can see only the agenda" do
    login_as(:jefe_de_gabinete)
    month = Time.now.month
    get :calendar, :month => month, :year => Time.now.year, :show_agenda => ['agenda', 'a_especial']
    assert_response :success
    assert assigns(:events)
    
    today = Time.now.day
    
    assert assigns(:events)[month][today].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud be in the events list.'    
    assert_nil assigns(:events)[month][today].detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should not be in the events list.'
    
    assert_select "h1.title", "Agenda Gobierno compartida"
  end

  test "secretaria can see the calendar with all schedules" do
    login_as(:secretaria)
    
    month = Time.now.month
    today = Time.now.day
    
    get :calendar, :month => month, :year => Time.now.year, :show_agenda => ['agenda', 'a_especial']
    assert_response :success
    assert assigns(:events)

    assert assigns(:schedule_events)
    assert assigns(:schedule_events)[month][today].detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'
        
    assert assigns(:events)[month][today].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud be in the events list.'
    assert assigns(:events)[month][today].detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'
    assert assigns(:events)[month][today].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:private_event_with_l_attends).id)}, 'Private event with l_attends should be in the events list.'
    
    assert_select "h1.title", "Agenda Gobierno compartida y #{schedules(:agenda_especial).name}"
    
  end

  test "secretaria can see the calendar with evens marked as l_attends" do
    login_as(:secretaria)
    
    month = Time.now.month
    today = Time.now.day
    
    session[:l_attends] = true
    get :calendar, :month => month, :year => Time.now.year, :show_agenda => ['agenda', 'a_especial']
    assert_response :success
    assert assigns(:events)
    assert assigns(:schedule_events)

    assert_nil assigns(:events)[month][today].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud not be in the events list.'    
    assert_nil assigns(:events)[month][today].detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should not be in the events list.'
    assert assigns(:events)[month][today].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:private_event_with_l_attends).id)}, 'Private event with l_attends should be in the events list.'
    
    assert_select "h1.title", "Agenda Gobierno compartida y #{schedules(:agenda_especial).name}"
    assert_select 'div.l_attends_info', :count => 1  
    assert_select "h2.l_info", :text => I18n.translate('sadmin.events.Asiste_el_lehendakari')
  end

  test "department member can see the create links for the schedules" do
    login_as(:secretaria_interior)
    s = schedules(:agenda_interior)
    
    month = Time.now.month
    today = Time.now.day
    

    get :calendar, :month => month, :year => Time.now.year, :show_agenda => ['agenda', s.short_name]
    assert_response :success
    
    assert_select 'div.create_links ul.edit_links' do
      assert_select 'li a.new', :count => 1
    end
    
    assert_select 'div.calendar_legend ul' do
      assert_select "li.#{s.short_name}", :text => "Uso interno del dept. #{s.department_name}"
    end
  end


  test "secrearia with no l_agenda permission can not see the calendar with evens marked as l_attends" do
    login_as(:secretaria_interior)
    
    month = Time.now.month
    today = Time.now.day
    

    post :set_l_attends
    assert_nil session[:l_attends]

    get :calendar, :month => month, :year => Time.now.year, :show_agenda => ['agenda', 'agenda_interior']
    assert_response :success
    assert assigns(:events)
    assert assigns(:schedule_events)
    
    assert_equal ['agenda', 'agenda_interior'], session[:show_agenda]
    
    assert assigns(:events)[month][today].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud be in the events list. All shared events events are shown.'    
    assert assigns(:events)[month][today].detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_interior).id)}, 'Special event "evento_interior" should be in the events list.'
    assert_nil assigns(:events)[month][today].detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should not be in the events list.'
    
    assert_select "h1.title", "Agenda Gobierno compartida y #{schedules(:agenda_interior).name}"
    assert_select 'div.l_attends_info', :count => 0 
    assert_select "h2.l_info", :count => 0
  end

  #
  # If user has no permission on lehendakari agenda, she can not select the l_attends option.
  # If somehow this option is added to the quiery, it has no effect. The user sees all the event she
  # has permission to see without the l_attends info
  #
  test "user without permission can not see the calendar with events marked as l_attends" do
    login_as(:jefe_de_gabinete)
    
    month = Time.now.month
    today = Time.now.day
    
    post :set_l_attends
    assert_nil session[:l_attends]
    
    get :calendar, :month => month, :year => Time.now.year, :show_agenda => ['agenda', 'a_especial']
    assert_response :success
    assert assigns(:events)

    assert assigns(:events)[month][today].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event should not be in the events list.'
    assert assigns(:events)[month][today].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:private_event_with_l_attends).id)}, 'Private event with l_attends should be in the events list.'
    
    assert_select "h1.title", "Agenda Gobierno compartida"
    assert_select 'li.l_attends', :count => 0
    
  end



  test "should show the next month calendar" do
    login_as(:secretaria)
    date = (Time.now.at_end_of_month + 1.day)
    post :calendar, :year => date.year, :month => date.month, :show_agenda => ['agenda', 'a_especial']
    assert_response :success
    assert_template 'index.html.erb'    
    
    last_day_evt = schedule_events(:ee_last_day_of_month)
    
    assert assigns(:events)
    if last_day_evt.starts_at.at_end_of_week.month.eql?(date.month)
      assert assigns(:events)[last_day_evt.month]
      assert_not_nil assigns(:events)[last_day_evt.month][last_day_evt.day].detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(last_day_evt.id)}
    else
      assert_nil assigns(:events)[last_day_evt.month]
    end
    #
    # Si el més no empieza el lunes, hay que mostrar los eventos de los últimos días del mes anterior.
    #
    unless date.to_date.at_beginning_of_week.eql?(date.to_date)
      assert_select 'table.calendar tbody tr' do |trs|
        assert_select trs.first, "td.otherMonth" do |tds|
            assert_select tds.last, 'div.day_number' do
              assert_select 'a.day_number', :text => last_day_evt.day
            end
            
            # 3 = Sadmin::EventsHelper.num_events_in_cell
            # Si hay más de tres eventos, el last_day_evt no sale en el calendario.
            # Esto pasa cuando se ejecuta el test a final del mes y entonces el último día 
            # coinciden varios de los eventos largos que se solapan con el last_day_evt.
            if assigns(:events)[last_day_evt.month][last_day_evt.day].size <= 3
              assert_select tds.last, "div.schedule"
              assert_select tds.last, "div.#{last_day_evt.schedule.dom_element_class}"
            else
              assert_select tds.last, "div.more_events_link"
            end
        end
      end
    end
  end

  
  test "jefe_de_gabinete see agenda events list" do
    login_as(:jefe_de_gabinete)
    now = Time.now
    month = now.month
    get :list, :month => month, :year => now.year, :day => now.day
    assert_response :success
    assert assigns(:documents)

    assert assigns(:documents).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud be in the events list.'    
    assert_nil assigns(:documents).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should not be in the events list.'

    assert_select "div.check_schedule", :count => 0
  end

  test "secretaria sees agenda events and special events in the same list" do
    login_as(:secretaria)
    now = Time.now
    month = now.month
    get :list, :month => month, :year => now.year, :day => now.day, :show_agenda => ['agenda', 'a_especial', 'Lehendakari']
    assert_response :success
    assert assigns(:documents)
    
    assert assigns(:documents).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud be in the events list.'    
    assert assigns(:documents).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'
    assert assigns(:documents).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:l_event).id)}, 'Special "l_event" event should be in the events list.'
    assert assigns(:documents).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:ee_with_l_attends).id)}, 'Special "ee_with_l_attends" event should be in the events list.'
    
    assert_select "div.check_schedule", :count => 1
  end

  test "secretaria sees only l_attends events in the day events list" do
    login_as(:secretaria)
    
    session[:l_attends] = true
    now = Time.now
    month = now.month
    get :list, :month => month, :year => now.year, :day => now.day, :show_agenda => ['agenda', 'a_especial', 'Lehendakari']
    assert_response :success
    assert assigns(:documents)
    
    assert_nil assigns(:documents).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud not be in the events list.'    
    assert_nil assigns(:documents).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should not be in the events list.'
    assert assigns(:documents).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:l_event).id)}, 'Special "l_event" event should be in the events list.'
    assert assigns(:documents).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:ee_with_l_attends).id)}, 'Special "ee_with_l_attends" event should be in the events list.'
    
    assert_select "div.check_schedule", :count => 1
    assert_select "h2.l_info", :text => I18n.translate('sadmin.events.Asiste_el_lehendakari')
  end

  test "set show_agenda values" do
    login_as(:secretaria)
    now = Time.now
    month = now.month

    post :list, :month => month, :year => now.year, :day => now.day, :show_agenda => ['agenda', 'a_especial']
    assert_equal ['agenda', 'a_especial'], assigns(:show_agenda)

    post :list, :month => month, :year => now.year, :day => now.day
    assert_equal ['agenda', 'a_especial'], assigns(:show_agenda)

    post :list, :month => month, :year => now.year, :day => now.day, :show_agenda => ['agenda']
    assert_equal ['agenda'], assigns(:show_agenda)

    post :list, :month => month, :year => now.year, :day => now.day, :show_agenda => ['a_especial']
    assert_equal ['a_especial'], assigns(:show_agenda)
    
  end

  test "secretaria sees agenda events and special events in the same week view" do
    login_as(:secretaria)
    now = Time.now
    month = now.month
    get :week, :month => month, :year => now.year, :day => now.day, :show_agenda => ['agenda', 'a_especial']
    assert_response :success
    assert assigns(:events)
    
    assert assigns(:events).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud be in the events list.'    
    assert assigns(:events).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'
    
    assert_select "div.check_schedule", :count => 1
  end

  test "secretaria sees only special events in the week view unless agenda is checked" do
    login_as(:secretaria)
    now = Time.now
    month = now.month
    get :week, :month => month, :year => now.year, :day => now.day, :show_agenda => ['a_especial']
    assert_response :success
    assert assigns(:events)
    
    assert_nil assigns(:events).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud not be in the events list.'    
    assert assigns(:events).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'
    
    assert_select "div.check_schedule", :count => 1
  end

  test "secretaria sees l_attends events in the week view" do
    login_as(:secretaria)
    session[:l_attends] = true
    now = Time.now
    month = now.month
    get :week, :month => month, :year => now.year, :day => now.day, :show_agenda => ['agenda', 'a_especial']
    assert_response :success
    assert assigns(:events)
    
    assert_nil assigns(:events).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud be in the events list.'    
    assert_nil assigns(:events).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'
    assert assigns(:events).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:private_event_with_l_attends).id)}, 'Private event "private_event_with_l_attends" should be in the events list.'
    assert assigns(:events).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:ee_with_l_attends).id)}, 'Special event "ee_with_l_attends" should be in the events list.'
    
    assert_select "div.check_schedule", :count => 1
    assert_select 'h2.l_info'
  end

  test "secretaria interior sees all shared events in the week view" do
    login_as(:secretaria_interior)
    now = documents(:interior_public_event).starts_at
    month = now.month
    get :week, :month => month, :year => now.year, :day => now.day, :show_agenda => ['agenda', 'agenda_interior']
    assert_response :success
    assert assigns(:events)
    
    assert assigns(:events).detect {|e| e.is_a?(Event) && !e.department.id.eql?(organizations(:interior).id)}, 'Events for other depts shoud not be in the events list.'    
    assert assigns(:events).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:interior_public_event).id)}, 'Special event should be in the events list.'
    
    assert_select "div.check_schedule", :count => 1
  end

  

  test "selected schedules are saved in the session" do
    login_as(:secretaria)
    now = Time.now
    month = now.month
    get :calendar, :month => month, :year => now.year, :show_agenda => ['a_especial']
    assert_response :success
    assert assigns(:events)
    assert_equal ['a_especial'], session[:show_agenda]

    assert_nil assigns(:events)[month][now.day].detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud not be in the events list.'    
    assert assigns(:events)[month][now.day].detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'    
    
    # assert_nil assigns(:events).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud not be in the events list.'    
    # assert assigns(:events).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'
    
  end
  
  test "list should contain only special events" do
    login_as(:secretaria)
    session[:show_agenda] = ['a_especial']
    
    now = Time.now
    month = now.month
    get :list, :month => month, :year => now.year, :day => now.day
    assert_response :success
    assert assigns(:documents)
    
    assert_nil assigns(:documents).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud not be in the events list.'    
    assert assigns(:documents).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'    
  end

  test "list should contain only interior events and shared events" do
    login_as(:secretaria_interior)
    session[:show_agenda] = ['agenda', 'agenda_interior']
    
    now = documents(:interior_public_event).starts_at
    month = now.month
    get :list, :month => month, :year => now.year, :day => now.day
    assert_response :success
    assert assigns(:documents)

    assigns(:documents).map {|e| e unless e.department.id.eql?(organizations(:interior).id)}.compact do |evt|
      assert evt.is_a?(Event)
    end

    # assert_equal [organizations(:interior).id], assigns(:documents).map {|e| e.department.id}.uniq, 'Special events that not belong to interior department should not be in the events list.'    
  end

  test "list for a day without events should be empty if user can not edit events" do
    login_as(:secretaria_interior)
    session[:show_agenda] = ['agenda']
    
    now = Time.now.at_beginning_of_month
    month = now.month
    get :list, :month => month, :year => now.year, :day => now.day
    assert_response :success
    assert assigns(:documents)
    
  end


  test "list for a day without events should redirect is user can_edit events" do
    login_as(:secretaria)
    session[:show_agenda] = ['agenda']
    
    now = Time.now + 3.months
    month = now.month
    get :list, :month => month, :year => now.year, :day => now.day
    assert_response :redirect
  end

  ["admin", "jefe_de_prensa", "jefe_de_gabinete", "secretaria_interior", "miembro_que_modifica_noticias", "comentador_oficial"].each do |role|
    test "myfeed is available for #{role}" do
      user = users(role.to_sym)
      get :myfeed, :u => user.id, :p => user.crypted_password, :format => 'ics'      
      assert_response :success      
      assert assigns(:events)
    end
  end

  test "myfeed for jefe_de_gabinete includes only agenda events" do
    user = users(:jefe_de_gabinete)
    get :myfeed, :u => user.id, :p => user.crypted_password, :format => 'ics'
    assert_response :success
    assert assigns(:events)
    
    assert assigns(:events).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud be in the events list.'    
    assert_nil assigns(:events).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should not be in the events list.'    
  end  

  test "myfeed for secretaria includes agenda events and special events" do
    user = users(:secretaria)
    get :myfeed, :u => user.id, :p => user.crypted_password, :format => 'ics'
    assert_response :success
    assert assigns(:events)
    
    assert assigns(:events).detect {|e| e.is_a?(Event) && e.id.eql?(documents(:current_event).id)}, 'Current event shoud be in the events list.'    
    assert assigns(:events).detect {|e| e.is_a?(ScheduleEvent) && e.id.eql?(schedule_events(:evento_especial).id)}, 'Special event should be in the events list.'    
  end  

  test "get new event form" do
    login_as(:jefe_de_gabinete)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours

    get :new
    assert assigns(:event)
    assert assigns(:event).is_private?
    assert !assigns(:event).has_journalists
    assert !assigns(:event).has_photographers
    
    # NUEVO: comprobamos todos los campos del formulario
    ["politicians_tag_list", "speaker", "title_es", "place", "city", "location_for_gmaps", "all_journalists", "only_photographers"].each do |name|
      assert_select "input#event_#{name}"
    end
    
    assert_select 'div.schedule_radio_options', :count => 1 # siempre está porque dentro están las opciones de uso privado y agenda compartida además de las agendas privadas
    assert_select "input#event_schedule_id__1" # uso privado
    assert_select "input#event_schedule_id_0"  # evento público 
    assert_select "input.schedule_radio", :count => 0 # no hay agenda privada
    assert_select "input#event_alertable" # si se quiere alertar sobre este evento o no
  end

  test "should create new private event" do
    login_as(:jefe_de_gabinete)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    
    assert_difference('Event.count') do
      post :create, :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"   => 'El tema del evento',
                               "organization_id" => organizations(:lehendakaritza).id,
                               "schedule_id" => "-1", # así indicamos que el evento es privado
                               "alertable" => false
                               }
    end
    assert assigns(:event).is_private?
    assert !assigns(:event).published?
    assert_redirected_to sadmin_event_url(:id => assigns(:event).id, :fresh => 1)
  end   

  test "should create new public event" do
    login_as(:jefe_de_gabinete)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    
    assert_difference('Event.count') do
      post :create, :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"   => 'El tema del evento',
                               "organization_id" => organizations(:lehendakaritza).id,
                               "schedule_id" => '0', # así indicamos que el evento es público
                               "alertable" => true
                              }
    end
    assert assigns(:event).is_public?
    assert assigns(:event).published?
    
    assert !assigns(:event).all_journalists?
    assert !assigns(:event).only_photographers?
    assert !assigns(:event).has_photographers?
    assert !assigns(:event).has_journalists

    assert_redirected_to sadmin_event_url(:id => assigns(:event).id, :fresh => 1)
  end   



  test "should create new event for an entity which is not department" do
    login_as(:jefe_de_gabinete)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    
    assert_difference('Event.count') do
      post :create, :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"   => 'El tema del evento',
                               "organization_id" => organizations(:emakunde).id,
                               "alertable" => false}
                               
    end
    assert assigns(:event).is_private?
    assert_redirected_to sadmin_event_url(:id => assigns(:event).id, :fresh => 1)
  end   


  test "should not create new event if user can not edit events" do
    login_as(:secretaria_interior)
    assert !users(:secretaria_interior).can_edit?('events')

    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    
    assert_no_difference('Event.count') do
      post :create, :event => {"starts_at(1i)"   => starts_at.year.to_s,
                               "starts_at(2i)"   => starts_at.month.to_s,
                               "starts_at(3i)"   => starts_at.day.to_s,
                               "ends_at(1i)"     => ends_at.year.to_s,
                               "ends_at(2i)"     => ends_at.month.to_s,
                               "ends_at(3i)"     => ends_at.day.to_s,
                               "title_es"        => 'El tema del evento',
                               "body_es"         => 'El texto del evento',
                               "organization_id" => organizations(:lehendakaritza).id,
                               "only_photographers" => 0,
                               "alertable" => true}
    end
    assert_not_authorized
  end   

  test "should create new schedule event is user can create schedule events and a schedule is selected" do
    login_as(:secretaria_interior)
    assert !users(:secretaria_interior).can_create?('events')
    assert users(:secretaria_interior).can_create?('schedule_events')

    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    
    assert_difference('ScheduleEvent.count') do
      post :create, :event => {"starts_at(1i)"   => starts_at.year.to_s,
                               "starts_at(2i)"   => starts_at.month.to_s,
                               "starts_at(3i)"   => starts_at.day.to_s,
                               "ends_at(1i)"     => ends_at.year.to_s,
                               "ends_at(2i)"     => ends_at.month.to_s,
                               "ends_at(3i)"     => ends_at.day.to_s,
                               "title_es"        => 'El tema del evento',
                               "body_es"         => 'El texto del evento',
                               "schedule_id"     => schedules(:agenda_interior).id
                               }
    end
  end   

  test "should create new event if user can create schedule events and events and schedule_id is null" do
    login_as(:secretaria)
    assert users(:secretaria).can_create?('events')
    assert users(:secretaria).can_create?('schedule_events')

    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    
    assert_difference('Event.count') do
      post :create, :event => {"starts_at(1i)"   => starts_at.year.to_s,
                               "starts_at(2i)"   => starts_at.month.to_s,
                               "starts_at(3i)"   => starts_at.day.to_s,
                               "ends_at(1i)"     => ends_at.year.to_s,
                               "ends_at(2i)"     => ends_at.month.to_s,
                               "ends_at(3i)"     => ends_at.day.to_s,
                               "title_es"        => 'El tema del evento',
                               "body_es"         => 'El texto del evento',
                               "organization_id" => organizations(:gobierno_vasco).id,
                               "schedule_id"     => nil}
      assert assigns(:event).errors.blank?
    end
    assert assigns(:event)
    assert assigns(:event).is_a?(Event)
    assert assigns(:event).is_private?
  end   


  test "should create new public event if user can create schedule events and events and schedule_id is 0" do
    login_as(:secretaria)
    assert users(:secretaria).can_create?('events')
    assert users(:secretaria).can_create?('schedule_events')

    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    
    assert_difference('Event.count') do
      post :create, :event => {"starts_at(1i)"   => starts_at.year.to_s,
                               "starts_at(2i)"   => starts_at.month.to_s,
                               "starts_at(3i)"   => starts_at.day.to_s,
                               "ends_at(1i)"     => ends_at.year.to_s,
                               "ends_at(2i)"     => ends_at.month.to_s,
                               "ends_at(3i)"     => ends_at.day.to_s,
                               "title_es"        => 'El tema del evento',
                               "body_es"         => 'El texto del evento',
                               "organization_id" => organizations(:gobierno_vasco).id,
                               "schedule_id"     => "0",
                               "alertable" => true
                               }
      assert assigns(:event).errors.blank?
    end
    assert assigns(:event)
    assert assigns(:event).is_a?(Event)
    assert assigns(:event).is_public?
  end   

  
  test "should show event" do
    login_as(:jefe_de_gabinete)
 
    get :show, :id => documents(:current_event)
    assert_response :success
    assert assigns(:event)    
    
    assert_select "div.create_links", :count => 0
    
    # Evento de Irekia -> hay tres opciones: exportatr a ical, preview
    assert_select "div.event_edit_links" do
      assert_select "ul.edit_links" do |elements|
        assert_select elements.first, "li", 2
        # Evento de Irekia -> hay dos opciones: exportatr a ical, preview 
        assert_select elements.first, "li", 2
        # y aparte eliminar
        assert_select elements.last, "li", 1
      end
    end
    
    assert_select 'div.create_update_info'    
    
    assert_select 'table.admin' do
      assert_select 'tr:nth-child(1)' do
        assert_select 'th', /Visibilidad/
        assert_select 'td', /Sí/
        # assert_select 'td', /No, no se muestra en Agencia/
      end
    end
    
  end

  test "should show event with streamingxx" do
    login_as(:jefe_de_gabinete)
 
    get :show, :id => documents(:event_with_streaming)
    assert_response :success
    assert assigns(:event)    
    
    assert_select "div.create_links", :count => 0
    
    assert_select "div.event_edit_links" do
      assert_select "ul.edit_links" do |elements|
        # Evento de Irekia -> hay tres opciones: exportatr a ical, preview 
        assert_select elements.first, "li", 2
        # y eliminar aparte
        assert_select elements.last, "li", 1
      end
      assert assigns(:event).streaming_for_irekia?      
      assert_select 'div.coverage', :text => 'IREKIA CUBRIRÁ ESTE EVENTO CON CONEXIÓN EN DIRECTO.'
    end
    
    assert_select 'div.create_update_info'    
    
    assert_select 'table.admin' do
      assert_select 'tr:nth-child(1)' do
        assert_select 'th', /Visibilidad/
        assert_select 'td', /Sí/
        # assert_select 'td', /Sí, se muestra en Agencia/
      end
    end
    
  end

  test "should show fresh event" do
    login_as(:jefe_de_gabinete)
 
    get :show, :id => documents(:current_event), :fresh => 1
    assert_response :success
    assert assigns(:event)    
    
    assert_select "div.create_links", :count => 1
    assert_select 'div.create_update_info'
    
    assert_select 'div.related_news' do
      assert_select 'ul.edit_links' do
        assert_select 'li a', 'Crear borrador de noticia'
        assert_select 'li span', 'Buscar noticia'        
      end
    end
    
  end

  test "should show private event" do
    login_as(:jefe_de_gabinete)
 
    get :show, :id => documents(:private_event)
    assert_response :success
    assert assigns(:event)    
    
    assert_select "div.create_links", :count => 0

    # Evento privado -> hay una opción: exportatr a ical
    assert_select "div.event_edit_links" do
      assert_select "ul.edit_links" do |elements|
        # Evento de Irekia -> hay dos opciones: exportatr a ical
        assert_select elements.first, "li", 1
        # y aparte eliminar
        assert_select elements.last, "li", 1
      end
    end
    
    assert_select 'div.create_update_info'
    
    assert_select 'div.related_news' do
      assert_select 'ul.edit_links', :count => 1
    end
    
  end

  test "should show private event for user with schedules permission" do
    login_as(:secretaria)
   
    get :show, :id => documents(:private_event)
    assert_response :success
    assert assigns(:event)    
    
    assert_select "div.create_links", :count => 0
    
    # Evento privado -> hay dos opciones: exportatr a ical y eliminar
    assert_select "div.event_edit_links" do    
      assert_select "ul.edit_links" do
        assert_select "li", :count => 2
      end
    end
  end

  test "should show event with related news" do
    login_as(:jefe_de_prensa)

    get :show, :id => documents(:emakunde_passed_event)
    assert_response :success
    assert assigns(:event)    
    
    assert_select 'div.related_news' do
      assert_select 'div.news_title', /#{assigns(:event).news.first.title}/
      assert_select 'div.news_title a', :text => 'Desvincular'
    end
    
  end

  test "should show event without create related news links for creador_eventos_irekia" do
    login_as(:creador_eventos_irekia)

    get :show, :id => documents(:current_event)
    assert_response :success
    assert assigns(:event)    
    
    assert_select 'div.related_news' do
      assert_select 'ul.edit_links', :count => 0
    end
    
  end
  
  # Test event edit and update actions
  test "should get edit event form" do
    login_as(:jefe_de_gabinete)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours

    get :edit, :id => documents(:current_event)
    assert assigns(:event)

    # comprobamos todos los campos del formulario
    ["politicians_tag_list", "speaker", "title_es", "place", "city", "location_for_gmaps", "all_journalists", "only_photographers"].each do |name|
      assert_select "input#event_#{name}"
    end
    
    assert_select 'div.schedule_radio_options'
    assert_select "input#event_schedule_id__1" # uso privado
    assert_select "input#event_schedule_id_0[checked=checked]"  # evento público 
    assert_select "input.schedule_radio", :count => 0 # no hay agenda privada
    
    assert_select "input#event_all_journalists"
    assert_select "input#event_only_photographers"    
    
  end
  
  
  test "should get edit public event form if user can create schedule and public events" do
    login_as(:secretaria)
    get :edit, :id => documents(:current_event)
    assert_response :success    
    assert_select 'form', :action => /schedule_events/

    assert_select 'div.schedule_radio_options'
    assert_select "input#event_schedule_id__1" # uso privado
    assert_select "input#event_schedule_id_0[checked=checked]"  # evento público 
    assert_select 'input#event_all_journalists'
    assert_select 'input#event_only_photographers'


    assert !users(:secretaria).editable_schedules_with_radio.blank?    
    users(:secretaria).editable_schedules_with_radio.each do |s|
      assert_select "input#event_schedule_id_#{s.id}"      
    end
  end

  test "should get edit event form for private event" do
    login_as(:jefe_de_gabinete)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours

    get :edit, :id => documents(:private_event)
    assert_response :success

    assert_select 'form', :action => /events/

    # Comprobamos todos los campos del formulario
    ["politicians_tag_list", "speaker", "title_es", "place", "city", "location_for_gmaps", "all_journalists", "only_photographers"].each do |name|
      assert_select "input#event_#{name}"
    end
    
    assert_select 'div.schedule_radio_options'
    assert_select "input#event_schedule_id__1[checked=checked]" # uso privado
    assert_select "input#event_schedule_id_0"  # evento público 
    
    assert_select 'input#event_all_journalists[checked]', :count => 0
    assert_select 'input#event_only_photographers[checked]', :count => 0
        
  end


  test "should get edit private event form if user can create schedule and public events" do
    login_as(:secretaria)
    get :edit, :id => documents(:private_event)
    assert_response :success    
    assert_select 'form', :action => /schedule_events/

    # comprobamos todos los campos del formulario
    ["politicians_tag_list", "speaker", "title_es", "place", "city", "location_for_gmaps", "all_journalists", "only_photographers"].each do |name|
      assert_select "input#event_#{name}"
    end

    assert_select 'div.schedule_radio_options'
    assert_select "input#event_schedule_id__1[checked=checked]" # uso privado
    assert_select "input#event_schedule_id_0"  # evento público 
    
    assert_select 'input#event_all_journalists[checked]', :count => 0
    assert_select 'input#event_only_photographers[checked]', :count => 0

    assert !users(:secretaria).editable_schedules_with_radio.blank?    
    users(:secretaria).editable_schedules_with_radio.each do |s|
      assert_select "input#event_schedule_id_#{s.id}"      
    end
  end


  test "should not get edit event form" do
    login_as(:secretaria_interior)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours

    get :edit, :id => documents(:current_event)
    assert_response :redirect
  end


  test "should update shared event" do
    login_as(:jefe_de_gabinete)
    e = documents(:current_event)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    assert_no_difference('Event.count') do
      post :update, :id => e.id,
                    :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"   => 'El nuevo tema del evento',
                               "schedule_id" => '0',
                               "alertable" => true}
    end
    assert_equal 'El nuevo tema del evento', assigns(:event).title
    assert_response :redirect
    assert_redirected_to sadmin_event_path(:id => assigns(:event).id)
    
  end

  test "should change shared event to private event" do
    login_as(:secretaria)
    e = documents(:current_event)
    s = schedules(:agenda_educacion)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    assert_difference('Event.count', -1) do
      post :update, :id => e.id,
                    :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"       => 'El nuevo tema del evento',
                               "schedule_id" => s.id}
    end
    assert assigns(:new_event).is_a?(ScheduleEvent), "Schedule event is expected but #{assigns(:event).class.to_s} is found"
    assert_equal s.id, assigns(:new_event).schedule_id, "Schedule should be #{s.name}"
    assert_equal 'El nuevo tema del evento', assigns(:new_event).title
    assert_response :redirect
    assert_redirected_to sadmin_schedule_event_path(:id => assigns(:new_event).id, :schedule_id => s.id)
    
  end

  test "should change public shared event to private shared event" do
    login_as(:secretaria)
    e = documents(:current_event)
    s = schedules(:agenda_educacion)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    assert_no_difference('Event.count') do
      post :update, :id => e.id,
                    :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"       => 'El nuevo tema del evento',
                               "schedule_id" => '-1'}
    end
    assert assigns(:event).is_private?, 'Updated event should be private'
    assert_equal "-1", assigns(:event).schedule_id, "Schedule should be -1"
    assert_equal 'El nuevo tema del evento', assigns(:event).title
    assert_response :redirect
    assert_redirected_to sadmin_event_path(:id => assigns(:event).id)
    
  end


  # End edit and update tests
  
  # test "user with schedules permission should select another schedule for a private event" do
  #   login_as(:secretaria)
  #  
  #   get :select_schedule, :id => documents(:private_event)
  #   assert_response :success
  #   assert assigns(:event)    
  #   
  #   assert_select 'div.select_agenda' do
  #     assert_select 'form' do |elements|
  #       assert_equal 1, elements.size
  #       assert_match /change_schedule/, elements.first.attributes['action']
  #     end
  #     assert_select '#new_schedule_id' do 
  #       assert_select "option", schedules(:lehendakari).name
  #       assert_select "option", schedules(:agenda2).name
  #       assert_select "option", :text => schedules(:agenda_interior).name, :count => 0 
  #     end
  #   end
  #   
  # end

  test "should redirect if no schedule is specified for the event" do
    login_as(:secretaria)

    assert_no_difference('Event.count') do
      post :change_schedule, :id => documents(:private_event)
    end
    assert_redirected_to sadmin_event_path(assigns(:event)) 
  end

  test "user with schedules permission should change event schedule for a private event" do
    login_as(:secretaria)
    private_event = documents(:private_event)
    assert !private_event.deleted?
   
    assert_difference('ScheduleEvent.count', 1) do
      post :change_schedule, :id => documents(:private_event).id, :new_schedule_id => schedules(:lehendakari).id
    end
    
    assert assigns(:new_event)
    assert assigns(:event)
    
    assert_redirected_to sadmin_schedule_event_path(:id => assigns(:new_event).id, :schedule_id => schedules(:lehendakari).id) 
        
  end

  test "user with schedules permission should not change event schedule for a private event without permission" do
    login_as(:secretaria)
    private_event = documents(:private_event)
    assert !private_event.deleted?
   
    assert_no_difference('ScheduleEvent.count') do
      post :change_schedule, :id => documents(:private_event).id, :new_schedule_id => schedules(:agenda_especial).id
    end
    
    assert_nil assigns(:new_event)
    assert assigns(:event)
    
    assert_redirected_to sadmin_event_path(private_event) 
  end


  test "user with no schedules permission should not see the select_agenda page" do
    login_as(:jefe_de_gabinete)
   
    get :select_schedule, :id => documents(:private_event)
    
    assert_response :redirect
    assert_equal I18n.translate('no_tienes_permiso'), flash[:notice]
  end

  test "user with permissions see l_attends info" do
    login_as(:secretaria)
    private_event = documents(:private_event)
    
    get :show, :id => private_event.id
    assert_response :success
    assert_select 'div.l_attends_info'
  end  

  test "user without permissions can not see l_attends info" do
    login_as(:jefe_de_gabinete)
    private_event = documents(:private_event)
    
    get :show, :id => private_event.id
    assert_response :success
    assert_select 'div.l_attends_info', :count => 0
  end  

  test "do not toggle l_attends if user has no permission" do
    login_as(:jefe_de_gabinete)
    private_event = documents(:private_event)

    get :toggle_l_attends, :id => private_event.id, :format => 'js'
    assert_response :redirect
  end

  test "toggle l_attends if user has permission" do
    login_as(:secretaria)
    private_event = documents(:private_event)
    assert !private_event.l_attends?

    get :toggle_l_attends, :id => private_event.id, :format => 'js'
    assert assigns(:event)
    assert assigns(:event).l_attends?
    assert_select_rjs  :replace, 'l_attends_info_div'
  end

  test "secreataria should see three create event buttons" do
    login_as(:secretaria)
    
    get :index
    assert_response :success
    
    assert_select 'div.create_links ul.edit_links' do
      assert_select 'li', :count => 3
      assert_select 'li a', 'Crear Evento'
      assert_select 'li a', "Crear Evento(#{schedules(:lehendakari).name})"
      assert_select 'li a', "Crear Evento(#{schedules(:agenda2).name})"
    end    
  end


  test "secreataria interior should see one create event button" do
    login_as(:secretaria_interior)
    
    get :index
    assert_response :success
    
    assert_select 'div.create_links ul.edit_links' do
      assert_select 'li', :count => 1
      assert_select 'li a', 'Crear Evento'
    end    
    
    assert_select "div.calendar_legend ul" do
      assert_select "li", "Uso interno del dept. Interior"
    end
  end

  test "jefe de gabinete de interior should see one create event button" do
    login_as(:jefe_de_gabinete_de_interior)
    
    get :index
    assert_response :success
    
    assert_select 'div.create_links ul.edit_links' do
      assert_select 'li', :count => 1
      assert_select 'li a', 'Crear Evento'
    end    
  end

  test "should get new event form if user can create schedule and public events" do
    login_as(:secretaria)
    get :new
    assert_response :success    
    assert_select 'form', :action => /schedule_events/
    
    # Comprobamos todos los campos del formulario
    ["politicians_tag_list", "speaker", "title_es", "place", "city", "location_for_gmaps", "all_journalists", "only_photographers"].each do |name|
      assert_select "input#event_#{name}"
    end

    assert_select 'div.schedule_radio_options', :count => 1 # siempre está porque dentro están las opciones de uso privado y agenda compartida además de las agendas privadas
    assert_select "input#event_schedule_id__1" # uso privado
    assert_select "input#event_schedule_id_0"  # evento público 
    # agendas privadas 
    assert !users(:secretaria).editable_schedules_with_radio.blank?    
    users(:secretaria).editable_schedules_with_radio.each do |s|
      assert_select "input#event_schedule_id_#{s.id}"      
    end
    
  end
  
  test "should get new event form if user has only create schedule event permission" do
    login_as(:secretaria_interior)
    get :new
    assert_response :success    
    assert_select 'form', :action => /schedule_events/
    
    # Comprobamos todos los campos del formulario
    ["speaker", "title_es", "place", "city", "location_for_gmaps"].each do |name|
      assert_select "input#event_#{name}"
    end
    ["politicians_tag_list", "all_journalists", "only_photographers"].each do |name|
      assert_select "input#event_#{name}", :count => 0 # los eventos privados no tienen políticos
    end

    assert_select 'div.schedule_radio_options'
    
    assert_select "input#event_schedule_id__1", :count => 0 # no puede crear eventos compartidos de uso privado
    assert_select "input#event_schedule_id_0", :count => 0  # no puede crear eventos públicos 
    
    users(:secretaria_interior).editable_schedules.each do |s|
      assert_select "input#event_schedule_id_#{s.id}"      
    end
    
  end
  
  test "miembro de departamento cannot create events" do
    login_as(:miembro_que_modifica_noticias)
    get :index
    assert_response :success
    # No tiene opcion de añadir evento
    assert_select 'div.create_links ul.edit_links', 0
    
    get :new
    assert_not_authorized
  end
  
  test "creador_eventos_privadas can sees no visibility option" do
    login_as(:creador_eventos_privadas)
    get :index
    assert_response :success
    assert_select 'div.create_links ul.edit_links a', "Crear Evento"
    
    get :new

    # Comprobamos todos los campos del formulario
    ["politicians_tag_list", "speaker", "title_es", "place", "city", "location_for_gmaps"].each do |name|
      assert_select "input#event_#{name}"
    end
    ["all_journalists", "only_photographers"].each do |name|
      assert_select "input#event_#{name}", :count => 0
    end

    assert_select 'div.schedule_radio_options', :count => 1 
    assert_select "input[type=hidden][value=-1][name*=schedule_id]" # eventos privados
    assert_select "input#event_schedule_id_0", :count => 0  # no tiene permiso
    assert_select "input.schedule_radio", :count => 0 # no hay agenda privada    
  end

  test "creador_eventos_privadas can not create public events" do
    login_as(:creador_eventos_privadas)
    
    # Si intentamos publicar en irekia, no nos deja
    post :create, :event => {:title_es => "Nuevo evento", :starts_at => Time.zone.now, :ends_at => Time.zone.now + 3.hours, :organization_id => organizations(:interior).id, :schedule_id => "0"}
    assert_template 'new'
    assert_equal "No puedes crear eventos de este tipo", assigns(:event).errors.on(:base)
  end

  
  test "creador_eventos_irekia can add events" do
    login_as(:creador_eventos_irekia)
    get :index
    assert_response :success
    assert_select 'div.create_links ul.edit_links a', "Crear Evento"
  end
  
  test "check new event form fields" do
    login_as(:creador_eventos_irekia)
    get :new
    
    # Comprobamos todos los campos del formulario
    ["politicians_tag_list", "speaker", "title_es", "place", "city", "location_for_gmaps", "all_journalists", "only_photographers"].each do |name|
      assert_select "input#event_#{name}"
    end
    
    assert_select 'div.schedule_radio_options'
    assert_select "input#event_schedule_id__1", :count => 0 # no puede crear eventos compartidos de uso privado
    assert_select "input#event_schedule_id_0", :count => 1  # eventos públicos 
    assert_select "input.schedule_radio", :count => 0 # no hay agenda privada    
  end
    
  test "creador_eventos_irekia can create only irekia events" do
    login_as(:creador_eventos_irekia)
    # Si intentamos publicar en privado, no nos deja
    post :create, :event => {:title_es => "Nuevo evento", :starts_at => Time.zone.now, :ends_at => Time.zone.now + 3.hours, :organization_id => organizations(:interior).id, :schedule_id => "-1"}
    assert_template 'new'
    assert_equal "No puedes crear eventos de este tipo", assigns(:event).errors.on(:base)    
  end

  test "creador_eventos_irekia cannot edit private news" do
    login_as(:creador_eventos_irekia)
    get :show, :id => documents(:private_event_with_l_attends)
    assert_response :success
    # No hay boton de modificar
    assert_select 'ul#edit_links a', :text => "modificar", :count => 0
    # Si intentamos modificarlo, no nos deja
    get :edit, :id => documents(:private_event_with_l_attends)
    assert_redirected_to new_session_path
    assert_not_authorized
    
    put :update, :id => documents(:private_event_with_l_attends), :event => {:title => "cambio titulo"}
    assert_redirected_to new_session_path
    assert_not_authorized
  end  

  test "should create new public event with location taken from event_locations" do
    login_as(:jefe_de_gabinete)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    
    assert_difference('Event.count') do
      post :create, :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title_es"   => 'El tema del evento',
                               "organization_id" => organizations(:lehendakaritza).id,
                               "place" => event_locations(:el_lehendakaritza).place,
                               "city" => event_locations(:el_lehendakaritza).city,
                               "location_for_gmaps" => event_locations(:el_lehendakaritza).address,
                               "schedule_id" => "0"}
    end
    assert !assigns(:event).is_private?
    assert_redirected_to sadmin_event_url(:id => assigns(:event).id, :fresh => 1)
    assert_equal event_locations(:el_lehendakaritza).lat, assigns(:event).lat
    assert_equal event_locations(:el_lehendakaritza).lng, assigns(:event).lng
  end   
  
  test "staff_chief can see event type checkboxes" do
    login_as(:jefe_de_gabinete)
    get :new
    assert_response :success
    
    assert_select "input#event_schedule_id__1" # puede crear eventos privados
    assert_select "input#event_schedule_id_0"  # puede crear eventos publicos
    
  end


  # Acceso para políticos con diferentes permisos
  test "politician without permissions on agenda is redirected" do
    login_as(:politician_one)
    get :index
    assert_response :redirect
  end  
  
  test "politician with news permisson is redirected" do
    login_as(:politician_interior)
    get :index
    assert_response :redirect
  end

  test "politician with events permisson can see index" do
    login_as(:politician_lehendakaritza)
    get :index
    assert_response :success
  end
  
  
  test "autocomplete for event news title" do
    login_as(:jefe_de_prensa)
    post :auto_complete_for_event_related_news_title, :value => 'xxx', :id => documents(:passed_event).id
    assert_response :success
  end
  
  test "in_place edit for event news title" do
    login_as(:jefe_de_prensa)
    event = documents(:current_event)
    assert_equal [], event.news
    
    news = documents(:one_news)
    
    assert_difference("RelatedEvent.count", 1) do
      post :set_event_related_news_title, :value => news.title, :id => event.id
      assert_response :success
    end
    
    event.reload
    news.reload
    assert_equal [documents(:one_news)], event.news
    assert_equal [event], news.events
  end
  
  test "should delete pending unsent alerts and not schedule new ones if event is set as private" do
    login_as(:admin)
    event = documents(:event_with_unsent_alert)
    assert event.alerts.unsent.count != 0
    put :update, :id => event.id, :event => {:schedule_id => "-1", :starts_at => 3.hours.from_now, :ends_at => 4.hours.from_now}
    event.reload
    assert event.alerts.unsent.count == 0
  end
  
  test "setting alert_this_change to true in private_event should not schedule alerts" do
    login_as(:admin)
    event = documents(:private_event)
    assert event.department.subscriptions.count > 0
    # the javascript in the page should not allow you to do this anyway
    put :update, :id => event.id, :event => {:alert_this_change => "1", :starts_at => 3.hours.from_now, :ends_at => 4.hours.from_now}
    event.reload
    assert_equal 0, event.alerts.unsent.count
  end
  
end
