#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Sadmin::NewsControllerTest < ActionController::TestCase
  def setup
    http_login()
  end

  ["admin", "jefe_de_prensa", "jefe_de_gabinete", "colaborador", "miembro_que_modifica_noticias"].each do |role|
    test "show if logged as #{role}" do
      login_as(role)
      get :index
      assert_response :success
      assert_template "index"
    end
  end

  ["periodista", "visitante", "comentador_oficial", "operador_de_streaming", "secretaria_interior", "room_manager"].each do |role|
    test "redirect if logged as #{role}" do
      login_as(role)
      get :index
      assert_not_authorized        
    end
  end
  
    
  #
  # Main menu options.
  # Probably the following tests should be put into another test file.
  #
  test "admin menu options" do
    login_as(:admin)
    get :index 
    
    assert_select "div.menu_admin" do
      assert_select "ul" do
        ["Noticias", "Agenda", "Web TV", "Fototeca", "Páginas", "Prop. ciudadanas", "Prop. gobierno", "Streaming", "Comentarios", "Escucha activa"].each_with_index do |text, n|
         assert_select "li:nth-child(#{n+1})" do
           assert_select 'a', text
         end
       end
       assert_select "li:nth-child(11)", :count => 0
      end
    end
  end


  test "department editor menu options" do
    login_as(:jefe_de_prensa)
    get :index 
    
    assert_select "div.menu_admin" do
      assert_select "ul" do
        ["Noticias", "Agenda", "Comentarios"].each_with_index do |text, n|
         assert_select "li:nth-child(#{n+1})" do
           assert_select 'a', text
         end
       end
       assert_select "li:nth-child(4)", :count => 0
      end
    end
  end

  
  test "chief of staff menu options" do
    login_as(:jefe_de_gabinete)
    get :index 
    
    assert_select "div.menu_admin" do
      assert_select "ul" do
        ["Noticias", "Agenda"].each_with_index do |text, n|
         assert_select "li:nth-child(#{n+1})" do
           assert_select 'a', text
         end
       end
       assert_select "li:nth-child(4)", :count => 0
      end
    end
  end
  
  
  test "colaborator menu options" do
    login_as(:colaborador)
    get :index
    
    assert_select "div.menu_admin" do
      assert_select "ul" do
        ["Noticias", "Web TV", "Fototeca", "Streaming"].each_with_index do |text, n|
          assert_select "li:nth-child(#{n+1})" do
            assert_select 'a', text
          end
        end
        assert_select "li:nth-child(5)", :count => 0
      end
    end
  end

  test "department member menu options" do
    login_as(:secretaria_interior)

    get :home
    assert_response :redirect
    assert_redirected_to sadmin_events_path    
  end
  
  test "miembro_que_modifica_noticias menu options" do
    login_as(:miembro_que_modifica_noticias)
    get :index
    assert_response :success
    
    assert_select "div.menu_admin" do
      assert_select "ul" do
        ["Noticias", "Agenda"].each_with_index do |text, n|
         assert_select "li:nth-child(#{n+1})" do
           assert_select 'a', text
         end
       end
       assert_select "li:nth-child(3)", :count => 0
      end
    end
  end
  
  test "miembro_que_modifica_noticias can edit news" do
    login_as(:miembro_que_modifica_noticias)
    get :edit, :id => documents(:one_news)
    assert_response :success
  end
  
  test "miembro_que_modifica_noticias cannot create news" do
    login_as(:miembro_que_modifica_noticias)
    get :new
    assert_not_authorized
  end
  
  test "miembro_que_crea_noticias can also edit news" do
    login_as(:miembro_que_crea_noticias)
    get :show, :id => documents(:one_news).id
    assert_response :success
    # Hay boton de modificar
    assert_select 'ul.edit_links li a', :text => "Modificar", :count => 1
    # Hay boton de añadir documento
    assert_select 'ul.edit_links a#add_attachment[href*=?]', "attachable_type=Document", :text => "Crear documento adjunto", :count => 1
    
    assert_select 'div.create_update_info'
    
    # Si intentamos modificarlo, nos deja
    get :edit, :id => documents(:one_news).id
    assert_response :success
    assert_template 'edit'
    
    put :update, :id => documents(:one_news).id, :news => {:title => "cambio titulo"}
    assert_redirected_to sadmin_news_path(documents(:one_news).id)
    assert_equal "La noticia se ha guardado correctamente.", flash[:notice]
  end

  %w(jefe_de_prensa jefe_de_gabinete colaborador miembro_que_modifica_noticias periodista visitante comentador_oficial operador_de_streaming secretaria_interior room_manager).each do |role|
    test "#{role} should not be able to export news" do
      login_as(role)
      post :export_for_enet, :news_to_export => [documents(:commentable_news).id]
      assert_not_authorized
    end
  end
  
  %w(admin exportador_de_noticias).each do |role|
    test "#{role} should be able to export news" do
      login_as(role)
      post :export_for_enet, :news_to_export => [documents(:commentable_news).id]
      assert_response :success
      
      # Comprobamos que tiene los siguientes ejes: 
      #   * los que hay por defecto
      #   * los del tag "viajes oficiales" que tiene esta noticia. Y todos sus padres
      #   * los del tag "Agricultura, Ganadería y Forestal" (nivel 3). Y todos sus padres
      exported_ejes = assigns(:ejes)
      news_ejes = [enet_ejes(:perfiles_personas), enet_ejes(:hechos_vitales_personas), enet_ejes(:lehendakari_viajes_oficiales),
                   enet_ejes(:agricultura_ganaderia_forestal), enet_ejes(:actividades_economicas)].collect {|e| e.ancestors + [e]}.flatten
      assert_equal 0, (exported_ejes - news_ejes).length
      date = documents(:commentable_news).published_at.strftime("%Y_%m_%d")
      assert File.exists?("#{Document::EXPORT_PATH}#{date}_comment_on_me/#{date}_comment_on_me-content.xml")
      
    end
  end

  test "get new news form withouf default department" do
    login_as(:admin)
    get :new
    assert_response :success
    
    assert assigns(:news)
    assert_select "select#news_organization_id" do
      assert_select "option", /Elige/
    end
  end

  
  test "get new news with relted event" do
    event = documents(:current_event)
    login_as(:admin)
    get :new, :related_event_id => event.id
    assert_response :success
    
    assert assigns(:news)
    assert_equal [event.id], assigns(:news).event_ids
    assert_equal event.title, assigns(:news).title
    assert_equal event.speaker, assigns(:news).speaker
    assert_equal event.starts_at.to_date.to_s.gsub('-', '/') + '/', assigns(:news).multimedia_dir
    assert_equal event.area_id, assigns(:news).area_id
    assert_equal event.politicians_tag_list, assigns(:news).politicians_tag_list    
    assert assigns(:news).is_private?
    
    assert_select 'input#news_event_ids_0[type=hidden][value=?]', event.id
  end

  test "create new news with related event" do
    event = documents(:current_event)
    login_as(:admin)
    now = Time.zone.now
    if event.is_public? 
      time_params = {"published_at(1i)".to_sym => now.year.to_s, "published_at(2i)".to_sym => now.month.to_s, 
                     "published_at(3i)".to_sym => now.day.to_s, "published_at(4i)".to_sym => now.hour.to_s, 
                     "published_at(5i)".to_sym => now.min.to_s}
    else
       time_params = {}
    end
    
    assert_difference("RelatedEvent.count", 1) do
      post :create, :news => {:organization_id => event.organization_id, 
                              :speaker_es => event.speaker_es, 
                              :title_es => event.title_es,
                              :event_ids => [event.id]}.merge(time_params)
      assert_response :redirect
    end
    
    assert assigns(:news)
    assert_equal [event], assigns(:news).events
    assert_equal event.title_eu, assigns(:news).title_eu
    assert_equal event.speaker_eu, assigns(:news).speaker_eu
    
    assert_equal "La noticia se ha guardado correctamente.", flash[:notice]
  end


  test "create new news with related event should not duplicate politician and area tags" do
    politician_tag = tags(:tag_politician_lehendakaritza)
    lehendakaritza_area = areas(:a_lehendakaritza)
    event = documents(:current_event)
    event.tag_list.add politician_tag.name_es
    assert event.save
    event.reload
    login_as(:admin)
    
    post :create, :news => {:organization_id => event.organization_id, 
                            :politicians_tag_list => politician_tag.name_es,
                            :area_id => lehendakaritza_area.id, 
                            :speaker_es => event.speaker_es, 
                            :title_es => event.title_es,
                            :event_ids => [event.id]}
    assert assigns(:news)

    assert_equal 1, assigns(:news).taggings.count(:conditions => "tag_id = #{politician_tag.id}")
    assert_equal 1, assigns(:news).taggings.count(:conditions => "tag_id = #{lehendakaritza_area.area_tag.id}")
  end

  test "one news menu for admin" do
    login_as(:admin)
    news = documents(:news_with_multimedia_dir_and_related_webtv_video_and_gallery_photo)
    
    get :show, :id => news.id
    assert_response :success
    
    assert_select 'div.one_news_submenu ul' do
      assert_select 'li', 'Contenido principal'
      assert_select 'li', 'Contenido adicional'
      assert_select 'li', 'Traducciones'
      assert_select 'li', 'Subtítulos'            
    end
    
  end

  test "one news menu for colaborador" do
    login_as(:colaborador)
    news = documents(:news_with_multimedia_dir_and_related_webtv_video_and_gallery_photo)
    
    get :show, :id => news.id
    assert_response :success
    
    assert_select 'div.one_news_submenu ul' do
      assert_select 'li', 'Contenido principal'
      assert_select 'li', 'Contenido adicional'
      assert_select 'li', 'Traducciones'
      assert_select 'li', :text => 'Subtítulos', :count => 0
    end
    
  end


  ["jefe_de_prensa", "jefe_de_gabinete", "miembro_que_modifica_noticias"].each do |role|
    test "one news menu for #{role}" do
      login_as(role)
      assert !users(role).can?("manage_subtitles", "news")
      news = documents(:news_with_multimedia_dir_and_related_webtv_video_and_gallery_photo)
    
      get :show, :id => news.id
      assert_response :success
    
      assert_select 'div.one_news_submenu ul' do
        assert_select 'li', 'Contenido principal'
        assert_select 'li', :text => 'Contenido adicional', :count => 0
        assert_select 'li', 'Traducciones'
        assert_select 'li', :text => 'Subtítulos', :count => 0
      end    
    end
  end

  context "with debate" do
    setup do
      login_as(:admin)
      @debate = debates(:debate_nuevo)
    end
    
    should "get new news" do
      get :new, :debate_id => @debate.id
      assert_response :success

      assert assigns(:news)
      assert_equal @debate.title, assigns(:news).title
      assert_equal @debate.organization, assigns(:news).organization
      assert_equal @debate.tag_list_es, assigns(:news).tag_list_es
      assert_equal areas(:a_lehendakaritza).id, assigns(:news).area_id
      assert assigns(:news).is_private?

      assert_select "form input#news_debate_id"
      assert_select "span.debate_news_notice"
    end

    should "create new news" do
      assert_difference("News.count", 1) do
        post :create, :news => {:debate_id       => @debate.id,
                                :organization_id => @debate.organization_id, 
                                :title_es        => @debate.title_es,
                                :area_id         => @debate.area_id,
                                :tag_list_es     => @debate.tag_list_es
                                }
      end
      assert_response :redirect      
      assert_redirected_to sadmin_news_path(:id => assigns(:news).id)

      assert assigns(:news)
      assert_equal @debate, assigns(:news).debate

      assert_equal "La noticia se ha guardado correctamente.", flash[:notice]
    end

    should "show new form if news is not valid" do
      assert_no_difference("News.count") do
        post :create, :news => {:debate_id       => @debate.id,
                                :organization_id => @debate.organization_id, 
                                :title_es        => nil,
                                :area_id         => @debate.area_id,
                                :tag_list_es     => @debate.tag_list_es
                                }
      end
      assert_response :success      
      assert_template "new"

      assert assigns(:news)
      assert_equal @debate, assigns(:news).debate

      assert_select "div#errorExplanation"
      assert_select "form input#news_debate_id"      
    end
   
    should "show news with debate info" do
      get :show, :id => documents(:news_for_debate).id
      assert_response :success

      assert_select "span.debate_news_notice"
    end
    
  end  
       
end
