#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class Sadmin::ScheduleEventsControllerTest < ActionController::TestCase
  
  def setup
    http_login()
  end
  
  test "should redirect if not logged in" do
    get :new, :schedule_id => schedules(:agenda_especial).id
    assert_not_authorized
  end
  
  test "should redirect if user hasn't got permission" do
    login_as(:jefe_de_gabinete)
    get :new, :schedule_id => schedules(:agenda_especial).id
    assert_not_authorized
  end

  test "should get new if user has permission on the schedule" do
    login_as(:secretaria)
    s = schedules(:agenda2)
    
    get :new, :schedule_id => s.id
    assert_response :success
    assert assigns(:event)    
    
    assert_select "legend", s.name
    assert_select "div.#{s.dom_element_class}"
  end

  test "should not get new if user has no edit permission on the schedule" do
    login_as(:secretaria)
    s = schedules(:agenda_interior)
    
    get :new, :schedule_id => s.id
    assert_response :redirect
    assert_redirected_to new_session_path()

    assert_select 'select#event_area_id', :count => 0    
  end

  test "should get new for a specific date" do
    login_as(:secretaria)
    s = schedules(:agenda2)
    
    date = Time.now - 2.days
    
    get :new, :schedule_id => s.id, :day => date.day, :month => date.month, :year => date.year
    assert_response :success
    assert assigns(:event)    
    assert assigns(:selected_date)
    assert_equal date.day, assigns(:selected_date)[:day]
    assert_equal date.month, assigns(:selected_date)[:month]
    assert_equal date.year, assigns(:selected_date)[:year]
    
    assert_select "legend", s.name
    assert_select "div.#{s.dom_element_class}"
    
    assert_equal date.to_date, assigns(:event).starts_at.to_date
  end

  
  test "should create event" do
    login_as(:secretaria)
    s = schedules(:agenda_especial)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    assert_difference('ScheduleEvent.count') do
      post :create, :schedule_id => s.id, 
                    :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"   => 'El tema del evento'}
    end
    assert_response :redirect
    assert_redirected_to sadmin_schedule_event_path(:id => assigns(:event).id, :schedule_id => s.id, :fresh => 1)
    
    assert_equal users(:secretaria).id, assigns(:event).created_by
  end

  test "should create event if user is department editor" do
    login_as(:secretaria_interior)
    s = schedules(:agenda_interior)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    assert_difference('ScheduleEvent.count') do
      post :create, :schedule_id => s.id, 
                    :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"   => 'El tema del evento'}
    end
    assert_response :redirect
    assert_redirected_to sadmin_schedule_event_path(:id => assigns(:event).id, :schedule_id => s.id, :fresh => 1)
  end

  
  test "should show schedule event for changable schedule" do
    login_as(:secretaria)
    s = schedules(:lehendakari)

    get :show, :schedule_id => s.id, :id => schedule_events(:l_event)    
    assert_response :success
    
    # Evento especial -> hay dos opciones: exportatr a ical y eliminar
    assert_select "div.event_edit_links" do    
      assert_select "ul.edit_links" do
        assert_select "li", :count => 2
      end
    end
    
  end

  test "should show schedule event for not changable schedule" do
    login_as(:secretaria)
    s = schedules(:agenda_especial)

    get :show, :schedule_id => s.id, :id => schedule_events(:evento_especial)    
    assert_response :success
    
    # Evento especial -> hay dos opciones: exportatr a ical, eliminar
    assert_select "div.event_edit_links" do    
      assert_select "ul.edit_links" do
        assert_select "li", :count => 2
      end
    end
    
  end


  test "should edit schedule event" do
    login_as(:secretaria)
    s = schedules(:agenda_educacion)

    get :edit, :schedule_id => s.id, :id => schedule_events(:evento_educacion).id    
    assert_response :success    
    
    assert_select 'div.schedule_radio_options' do
      assert_select 'input#event_schedule_id__1'      
      assert_select 'input#event_schedule_id_0'
      users(:secretaria).editable_schedules_with_radio.each do |s|
        assert_select "input#event_schedule_id_#{s.id}"
      end
      
    end
    
    assert_select 'select#event_area_id', :count => 0    
  end
  
  test "should get edit l_scheule event form without visibility block" do
    login_as(:secretaria)
    get :edit, :id => schedule_events(:l_event), :schedule_id => schedules(:lehendakari).id
    assert_response :success    
    assert_select 'form', :action => /schedule_events/

    assert_select 'div.schedule_radio_options', :count => 0
    assert_select "input#event_schedule_id_", :count => 0
    users(:secretaria).editable_schedules_with_radio.each do |s|
      assert_select "input#event_schedule_id_#{s.id}", :count => 0
    end
  end
  
  
  
  test "should update event" do
    login_as(:secretaria)
    s = schedules(:agenda_especial)
    e = schedule_events(:evento_especial)
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    assert_no_difference('ScheduleEvent.count') do
      post :update, :schedule_id => s.id, 
                    :id => e.id,
                    :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"       => 'El nuevo tema del evento',
                               "schedule_id" => s.id}
    end
    assert_response :redirect
    assert_redirected_to sadmin_schedule_event_path(:id => assigns(:event).id, :schedule_id => s.id)
  end

  test "should change event from special to private" do
    login_as(:secretaria)
    s = schedules(:agenda_educacion)
    e = schedule_events(:evento_educacion)
    
    assert users(:secretaria).can_change_schedule?(s)
    assert_equal s.department_id, e.organization_id
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    assert_difference('ScheduleEvent.count', -1) do
      post :update, :schedule_id => s.id, 
                    :id => e.id,
                    :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"   => 'El nuevo tema del evento',
                               "schedule_id" => nil, 
                               "organization_id" => e.organization_id}
    end

    assert_response :redirect
    assert assigns(:new_event).is_a?(Event)
    
    assert_redirected_to sadmin_event_path(:id => assigns(:new_event).id)
  end

  test "should not change event from special to private if its visibility can not be changed" do
    login_as(:secretaria)
    s = schedules(:lehendakari)
    e = schedule_events(:l_event)
    
    assert users(:secretaria).can_change_schedule?(s)
    assert_equal s.department_id, e.organization_id
    
    starts_at = Time.now + 1.day
    ends_at = starts_at + 3.hours
    assert_no_difference('ScheduleEvent.count') do
      post :update, :schedule_id => s.id, 
                    :id => e.id,
                    :event => {"starts_at(1i)" => starts_at.year.to_s,
                               "starts_at(2i)" => starts_at.month.to_s,
                               "starts_at(3i)" => starts_at.day.to_s,
                               "ends_at(1i)" => ends_at.year.to_s,
                               "ends_at(2i)" => ends_at.month.to_s,
                               "ends_at(3i)" => ends_at.day.to_s,
                               "title"   => 'El nuevo tema del evento',
                               "schedule_id" => nil}
    end
    assert_response :redirect
    assert assigns(:event).is_a?(ScheduleEvent)
    
    assert_redirected_to sadmin_schedule_event_path(:id => assigns(:event).id, :schedule_id => assigns(:schedule).id)
  end


  test "should destroy event" do
    login_as(:secretaria)
    s = schedules(:agenda2)
    e = schedule_events(:evento_agenda2)

    assert_difference('ScheduleEvent.count', -1) do
      post :destroy, :schedule_id => s.id, :id => e.id
    end
    
    assert_response :redirect  
  end  
  
  test "should show fresh event" do
    login_as(:secretaria)
    s = schedules(:agenda_especial)
    e = schedule_events(:evento_especial)
 
    get :show, :id => e.id, :fresh => 1, :schedule_id => s.id
    assert_response :success
    assert assigns(:event)    
    
    assert_select "div.create_links", :count => 1
  end

  test "user without schedules can_change permission should not select another schedule for a private event" do
    login_as(:secretaria)
   
    get :select_schedule, :id => schedule_events(:evento_especial).id, :schedule_id => schedules(:agenda_especial)
    assert_response :redirect
    
    # access denied
    assert_redirected_to new_session_path
    
  end

  test "user with schedules can_change permission should select another schedule for a private event" do
    login_as(:secretaria)
   
    get :select_schedule, :id => schedule_events(:l_event).id, :schedule_id => schedules(:lehendakari)
    assert_response :success
    assert assigns(:event)    
    
    assert_select 'div.select_agenda' do
      assert_select 'form' do |elements|
        assert_equal 1, elements.size
        assert_match /change_schedule/, elements.first.attributes['action']
      end
      assert_select '#new_schedule_id' do 
        assert_select "option", :text => 'Agenda Gobierno compartida'
        assert_select "option", :text => "#{schedules(:agenda2).name}"
        assert_select "option", :text => "#{schedules(:agenda_interior).name}", :count => 0
      end
    end
    
  end

  test "user with schedules can_change permission should change schedule event to private event" do
    login_as(:secretaria)
    ev = schedule_events(:l_event)
    department_id = ev.schedule.department_id
    
    assert_difference('ScheduleEvent.count', -1) do
      post :change_schedule, :id => ev.id, :schedule_id => ev.schedule_id, :new_schedule_id => 'agenda'
    end
    
    assert assigns(:new_event)
    assert assigns(:event)
    
    assert_redirected_to sadmin_event_path(:id => assigns(:new_event).id) 
        
  end


  test "user with no schedules permission should not see the select_agenda page" do
    login_as(:jefe_de_gabinete)
   
    get :select_schedule, :id => schedule_events(:evento_especial).id, :schedule_id => schedules(:agenda_especial).id
    
    assert_response :redirect
    assert_equal I18n.translate('no_tienes_permiso'), flash[:notice]
  end

  test "user with schedules permission can_change should change a special event to private event" do
    login_as(:secretaria)
    special_event = schedule_events(:l_event)
    schedule = schedules(:lehendakari)

    assert_difference('ScheduleEvent.count', -1) do
      post :change_schedule, :id => special_event.id, :schedule_id => schedule.id, :new_schedule_id => 'agenda'
    end
    
    assert assigns(:new_event)
    assert assigns(:event)
    
    assert_redirected_to sadmin_event_path(:id => assigns(:new_event).id) 
  end

  test "user with schedules permission can_edit should not change a special event to private event" do
    login_as(:secretaria)
    special_event = schedule_events(:evento_especial)
    schedule = schedules(:agenda_especial)

    assert_no_difference('ScheduleEvent.count') do
      post :change_schedule, :id => special_event.id, :schedule_id => schedule.id, :new_schedule_id => 'agenda'
    end
    
    # access_denied
    assert_redirected_to new_session_path()
  end


  test "user without schedules can_change permission should not change event schedule for a special event to other schedule" do
    login_as(:secretaria)
    special_event = schedule_events(:evento_especial)
    schedule = schedules(:agenda_especial)
   
    assert_no_difference('ScheduleEvent.count') do
      post :change_schedule, :id => special_event.id, :schedule_id => schedule.id, :new_schedule_id => schedules(:agenda2).id
    end

    # access_denied
    assert_redirected_to new_session_url()
        
    # assert assigns(:event)
    # assert_equal schedules(:agenda2).id, assigns(:event).schedule_id
    # 
    # assert_redirected_to sadmin_schedule_event_path(:id => assigns(:event).id, :schedule_id => schedules(:agenda2).id) 
  end

  test "user with schedules permission should change event schedule for a special event to other schedule" do
    login_as(:secretaria)
    special_event = schedule_events(:l_event)
    schedule = schedules(:lehendakari)
   
    assert_no_difference('ScheduleEvent.count') do
      post :change_schedule, :id => special_event.id, :schedule_id => schedule.id, :new_schedule_id => schedules(:agenda2).id
    end
    
    assert assigns(:event)
    assert_equal schedules(:agenda2).id, assigns(:event).schedule_id
    
    assert_redirected_to sadmin_schedule_event_path(:id => assigns(:event).id, :schedule_id => schedules(:agenda2).id) 
  end

  test "on schedule event page a message on l_attends is shown" do
    login_as(:secretaria)
    e = schedule_events(:evento_especial)
    
    get :show, :id => e.id, :schedule_id => e.schedule_id
    assert_response :success
    
    assert_select 'div.create_update_info'
    
    assert_select 'div.l_attends_info'
  end  

  test "on schedule event page no message on l_attends is shown is user has no permission" do
    login_as(:secretaria_interior)
    e = schedule_events(:evento_interior)
    
    get :show, :id => e.id, :schedule_id => e.schedule_id
    assert_response :success
    assert_select 'div.l_attends_info', :count => 0
  end  


  test "on schedule event page for lehendakari schedule no message on l_attends is shown" do
    login_as(:secretaria)
    e = schedule_events(:l_event)
    
    get :show, :id => e.id, :schedule_id => e.schedule_id
    assert_response :success
    assert_select 'div.l_attends_info', :count => 0
  end  
     
  test "toggle l_attends" do
    login_as(:secretaria)
    e = schedule_events(:evento_especial)
    assert !e.l_attends?

    get :toggle_l_attends, :id => e.id, :format => 'js', :schedule_id => e.schedule_id
    assert assigns(:event)
    assert assigns(:event).l_attends?
    assert_select_rjs  :replace, 'l_attends_info_div'
  end
  
end
