#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class SessionsControllerTest < ActionController::TestCase
  def setup
    http_login()
    I18n.locale = :es
  end

  def teardown
    I18n.locale = :es
  end

  test "should email forgotten password to approved user" do
    assert_difference 'ActionMailer::Base.deliveries.size', + 1 do  
      post :email_password, :email => 'visitante@efaber.net'
    end 

    m = ActionMailer::Base.deliveries.last
    assert_equal m.subject, "[#{Notifier::SERVICENAME}] Información de acceso", m.subject
    assert_equal m.to[0], 'visitante@efaber.net'  
    assert_match 'Ésta es tu información de acceso a', m.body
  end
  
  test "should not email forgotten password to pending user" do
    assert_no_difference 'ActionMailer::Base.deliveries.size' do
      post :email_password, :email => 'visitante_sin_activar@efaber.net', :locale => "eu"
    end
    assert_redirected_to password_reminder_path
    assert_equal "Erabiltzaile hau aktibazioaren zai dago", flash[:error]
  end
  
  test "should not email forgotten password to unexistent user" do
    assert_no_difference 'ActionMailer::Base.deliveries.size' do
      post :email_password, :email => 'noexiste@efaber.net', :locale => "eu"
    end
    assert_redirected_to password_reminder_path
    assert_equal "Ez dago email hau duen erabiltzailerik", flash[:error]    
  end
  
  test "should not email forgotten password to facebook user" do
    assert_no_difference 'ActionMailer::Base.deliveries.size' do
      post :email_password, :email => 'facebookuser@example.com', :locale => "es"
    end
    assert_redirected_to login_path
    assert_equal I18n.translate('session.usuario_facebook_sin_password'), flash[:notice]    
  end
  
  
  test "should see login form" do
    return_to = new_news_comment_path(documents(:commentable_news))
    get :new, :return_to => return_to
    assert_response :success
    assert_template "new"
    assert_select 'form[action=?]', session_path do
      assert_select "input[type=text][name=email]"
      assert_select "input[type=password][name=password]"
    end
    assert_select "a[href=?]", password_reminder_path
    assert_select "a[href=?]", twitter_session_path(:return_to => return_to)
    assert_select "a[href=?]", fb_session_path(:return_to => return_to)
  end
  
  test "redirect to root path if logged in as admin" do
    user = users("admin")
    post :create, {:email => user.email, :password => user.plain_password}
    assert_redirected_to admin_path
    # assert_equal I18n.t('session.Has_entrado', :name => user.name), flash[:notice]
  end
  
  ["periodista", "visitante"].each do |role|
    test "redirect to account_path if logged as #{role}" do    
      user = users(role)
      post :create, {:email => user.email, :password => user.plain_password}
      assert_redirected_to account_path    
    end
  end
  
  ["comentador_oficial", "secretaria_interior", "operador_de_streaming", \
   "jefe_de_gabinete", "jefe_de_prensa", "colaborador"].each do |role|
    test "redirect to admin_path if logged as #{role}" do
      user = users(role)
      post :create, {:email => user.email, :password => user.plain_password}
      assert_redirected_to admin_path
    end    
  end  
  
  test "redirect to account path if logged in as room manager" do
    user = users("room_manager")
    post :create, {:email => user.email, :password => user.plain_password}
    # Los room_manager van a sadmin/account
    assert_redirected_to sadmin_account_path
  end
  
  
  test "should log out" do
    login_as("admin")
    delete :destroy
    assert_redirected_to root_path
    assert_equal I18n.t('session.Has_salido', :name => users("admin").name), flash[:notice]
  end
  
  test "should redirect to specified url once logged in" do
    user = users("admin")
    post :create, {:return_to => new_news_comment_path(documents(:commentable_news)), :email => user.email, :password => user.plain_password}
    assert_redirected_to new_news_comment_path(documents(:commentable_news))
    # assert_equal I18n.t('session.Has_entrado', :name => user.name), flash[:notice]
  end
  
  test "should get mobile login form" do
    get :new, :format => 'floki', :locale => 'es'
    assert_response :success
    assert_template 'new.floki.erb'
    assert_select 'form[action=?]', session_url(:format => 'floki') do
      assert_select "input[type=email][name=email]"
      assert_select "input[type=password][name=password]"
    end
  end  
  
  test "should login from mobile" do
    user = users("admin")
    post :create, :email => user.email, :password => user.plain_password, :format => 'floki'
    assert_response :success
    assert_template 'success.floki.erb'
  end  
  
  test "should get auth_info" do
    get :auth_info, :format => 'json'
    assert_response :success
    assert_template 'auth_info.json.erb'
  end
  
  test "should destroy session mobile" do
    login_as("admin")
    delete :destroy, :format => 'floki'
    assert_response :success
  end  
  
end
