#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class StreamingsControllerTest < ActionController::TestCase
  
  test "should show index for irekia" do
    sf = stream_flows(:sf_two)
    sf.announced_in_irekia = true
    assert sf.save
    
    get :index, :locale => 'es'
    assert assigns(:flows).detect {|f| f.eql?(sf)}
    assert_irekia_layout
  end

  test "should show streaming page for streaming for irekia without event in irekia" do
    sf = stream_flows(:sf_one)
    assert sf.update_attribute(:event_id, nil)
    assert_nil sf.event
    sf.announced_in_irekia = true
    assert sf.save
    
    get :show, :id => sf.id
    assert_response :success
    assert_irekia_layout
  end
  
  test "should redirect to event page for streaming with irekia event in irekia" do
    sf = stream_flows(:sf_one)
    event = documents(:event_with_streaming_and_sent_alert_and_show_in_irekia)
    assert sf.update_attribute(:event_id, event.id)
    
    assert_not_nil sf.event
    assert sf.event.is_public?
    
    get :show, :id => sf.id
    assert_response :redirect
    assert_redirected_to event_url(sf.event)
  end
  
  test "should redirect to not found for streaming with agencia event in irekia unless stream_flow is show_in_irekia" do
    sf = stream_flows(:sf_one)
    event = documents(:private_event)
    event.update_attribute(:irekia_coverage, true)
    assert sf.update_attributes(:event_id => event.id, :show_in_irekia => false)
    
    assert_not_nil sf.event
    assert sf.event.is_private?
    
    get :show, :id => sf.id
    assert_response :missing
    assert_template 'site/notfound.html'
    assert_layout 'layouts/application'
  end
  
  test "should show streaming with agencia event in irekia if stream_flow is show_in_irekiaxx" do
    sf = stream_flows(:sf_one)
    event = documents(:private_event)
    event.update_attribute(:irekia_coverage, true)
    assert sf.update_attributes(:event_id => event.id, :show_in_irekia => true)
    
    assert_not_nil sf.event
    assert sf.show_in_irekia?
    assert !sf.event.is_public?
    
    get :show, :id => sf.id
    assert_response :success
    assert_irekia_layout
  end

  
end
