#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class NewsCommentsFlowsTest < ActionController::IntegrationTest
  fixtures :all

  def setup
    @htauth = "Basic " + Base64.encode64("euskadi:cambio42")
  end
  
  test "anonymous user's comment" do
    get news_path(:id => documents(:commentable_news).id)
    assert_response :success
    
    init_comments_count = assigns(:document).comments.approved.count
    assert_select 'div.comments' do 
      assert_select 'ul li.clearfix', :count => init_comments_count + 1
      assert_select 'ul li.clearfix div.item_content', /aprobado/
    end

    assert_select 'div.comments li.form div.item_content' do
      assert_select 'form[action=?]', '/es/comments'
      assert_select 'div.textarea-comment' do
        assert_select 'span.holder', /algo que decir/i
        assert_select 'textarea[name=?]', 'comment[body]'
      end
    end
    
    # No puedo hacer comentarios anonimos
    get_via_redirect new_news_comment_path(documents(:commentable_news))
    assert_equal new_session_path, path
    
    return_to = new_news_comment_path(:news_id => documents(:commentable_news).id, :only_path => true)
    get new_session_path(:return_to => return_to)
    # Login incorrecto
    post_via_redirect session_path, :email => users(:visitante).email, :password => "wrong", :return_to => return_to # no deberia necesitar el return_to (?)
    assert_template "new"
    
    # Login correcto
    post_via_redirect session_path, :email => users(:visitante).email, :password => users(:visitante).plain_password, :return_to => return_to # no deberia necesitar el return_to (?)
    assert_equal return_to, path

    # Comentario sin texto, no entra
    post comments_path(:news_id => documents(:commentable_news).id, :comment => {:body => ''})
    assert assigns(:comment)
    assert_equal "no puede estar vacío", assigns(:comment).errors.on(:body)
    assert_template "new"
    
    # Comentario con texto
    post_via_redirect comments_path(:news_id => documents(:commentable_news).id, :comment => {:body => "comentario", :name => users(:visitante).name, :email => users(:visitante).email}) # no deberia necesitar el name y email (?)
    assert_equal news_path(documents(:commentable_news)), path
    assert_equal '¡Gracias por tu comentario! La persona moderadora lo publicará en breve.', flash[:notice] 
    assert_select 'div.comments' do 
      assert_select 'ul li.clearfix', init_comments_count + 1 # el comentario no sale antes de moderarlo
    end
    
  end
  
  test "admin's comment" do
    get news_path(:id => documents(:commentable_news).id)
    assert_response :success
    
    init_comments_count = assigns(:document).comments.approved.count
    assert_select 'div.comments' do 
      assert_select 'ul li.clearfix', :count => init_comments_count + 1
    end
    
    return_to = new_news_comment_path(:news_id => documents(:commentable_news).id, :only_path => true)
    get new_session_path(:return_to => return_to)
    
    # Login correcto
    post_via_redirect session_path, :email => users(:admin).email, :password => users(:admin).plain_password, :return_to => return_to # no deberia necesitar el return_to (?)
    assert_equal return_to, path
    
    # Comentario con texto
    post_via_redirect comments_path(:news_id => documents(:commentable_news).id, :comment => {:body => "comentario", :name => users(:visitante).name, :email => users(:visitante).email}) # no deberia necesitar el name y email (?)
    assert_equal news_path(documents(:commentable_news)), path
    assert_equal 'Tu comentario se ha publicado correctamente.', flash[:notice] 
    assert_select 'div.comments' do 
      assert_select 'ul li.clearfix', init_comments_count + 2
    end
  end

end
