#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

ENV["RAILS_ENV"] = "test"
require File.expand_path(File.dirname(__FILE__) + "/../config/environment")
require 'test_help'
require 'action_view/test_case'
require 'elasticsearch_test_helper'

begin
  require 'turn'
rescue LoadError
  puts 'Install the Turn gem for prettier test output.'
end

class ActiveSupport::TestCase
  # Transactional fixtures accelerate your tests by wrapping each test method
  # in a transaction that's rolled back on completion.  This ensures that the
  # test database remains unchanged so your fixtures don't have to be reloaded
  # between every test method.  Fewer database queries means faster tests.
  #
  # Read Mike Clark's excellent walkthrough at
  #   http://clarkware.com/cgi/blosxom/2005/10/24#Rails10FastTesting
  #
  # Every Active Record database supports transactions except MyISAM tables
  # in MySQL.  Turn off transactional fixtures in this case; however, if you
  # don't care one way or the other, switching from MyISAM to InnoDB tables
  # is recommended.
  #
  # The only drawback to using transactional fixtures is when you actually 
  # need to test transactions.  Since your test is bracketed by a transaction,
  # any transactions started in your code will be automatically rolled back.
  self.use_transactional_fixtures = true

  # Instantiated fixtures are slow, but give you @david where otherwise you
  # would need people(:david).  If you don't want to migrate your existing
  # test cases which use the @david style and don't mind the speed hit (each
  # instantiated fixtures translates to a database query per test method),
  # then set this back to true.
  self.use_instantiated_fixtures  = false

  # Setup all fixtures in test/fixtures/*.(yml|csv) for all tests in alphabetical order.
  #
  # Note: You'll currently still have to declare fixtures explicitly in integration tests
  # -- they do not yet inherit this setting
  fixtures :all
  
  include ElasticsearchTestHelper

  # Add more helper methods to be used by all tests here...
  # For the HTTP authentication
  # Source: http://railsforum.com/viewtopic.php?id=14315
  def http_login
    @request.env['HTTP_AUTHORIZATION'] = 'Basic ' + Base64::encode64("euskadi:cambio42")
  end
  
  
  # fixtures :users
  # Define login method to be used in functional tests.
  # Source: http://alexbrie.net/1526/functional-tests-with-login-in-rails/
  def login(email='foo@bar.com', password='fooblitzky')
    old_controller = @controller
    @controller = SessionsController.new
    @request.env['REMOTE_ADDR'] = "http://127.0.0.1"
    post :create, :email => email, :password => password
    assert_redirected_to @controller.send('default_url_for_user')
    assert_not_nil(session[:user_id])
    @controller = old_controller
  end
  
  def twitter_login(user)
    # @controller.current_user = users(user)
    @controller.session = {}
    @controller.send("current_user=", users(user))
    # session[:user_id] = users(user).id
  end

  def login_as(user)
    if user.eql?("twitter_user") || user.eql?("facebook_user")
      twitter_login(user)
    else
      login(users(user).email, users(user).plain_password)
    end
  end

  def logout
    old_controller = @controller
    @controller = SessionsController.new
    delete :destroy
    assert_nil session[:user]
    assert_response :redirect
  end
  
  def assert_not_authorized
    # The response of access_denied method is :success, although it makes a redirect inside the respond_to block.
    # assert_response :redirect
    # So, here we check that no template is rendered.
    
    assert_equal I18n.t('no_tienes_permiso'), flash[:notice]
    assert_template ""
  end

  def assert_layout(layout)
    assert_equal layout, @response.layout
  end
  
  def assert_irekia_layout
    assert_layout('layouts/application')
  end
  
  def assert_agencia_layout
    assert_layout('layouts/ma')
  end

  def should_not_be_empty(user, attr_name)
    assert !user.valid?
    assert user.errors.on(attr_name).include?("no puede estar vacío")
  end
  
  def prepare_content_for_news_with_multimedia(news)
    # We copy one file in this new created directory
    FileUtils.mkdir_p("#{Document::MULTIMEDIA_PATH}#{news.multimedia_path}")
    FileUtils.mkdir_p("#{Document::MULTIMEDIA_PATH}#{news.multimedia_path}solo_agencia")
    FileUtils.cp("#{Document::MULTIMEDIA_PATH}/photos/test.jpg", "#{Document::MULTIMEDIA_PATH}#{news.multimedia_path}")
    FileUtils.cp("#{Document::MULTIMEDIA_PATH}/test.txt", "#{Document::MULTIMEDIA_PATH}#{news.multimedia_path}")
    FileUtils.cp("#{Document::MULTIMEDIA_PATH}/test.txt", "#{Document::MULTIMEDIA_PATH}#{news.multimedia_path}/photo_test.jpg")
    FileUtils.cp("#{Document::MULTIMEDIA_PATH}/test.txt", "#{Document::MULTIMEDIA_PATH}#{news.multimedia_path}solo_agencia/")
    FileUtils.cp("#{Document::MULTIMEDIA_PATH}/test.txt", "#{Document::MULTIMEDIA_PATH}#{news.multimedia_path}/test.flv")
    FileUtils.cp("#{Document::MULTIMEDIA_PATH}/test.txt", "#{Document::MULTIMEDIA_PATH}#{news.multimedia_path}solo_agencia/test.flv")
    FileUtils.cp("#{Document::MULTIMEDIA_PATH}/test.txt", "#{Document::MULTIMEDIA_PATH}#{news.multimedia_path}solo_agencia/solo_agencia.mp3")
  end
  
  def clear_multimedia_dir(news)
    FileUtils.rm_rf(Dir.glob("#{Document::MULTIMEDIA_PATH}2010"))
  end
  
  def assert_content_is_tracked(target_content)
    source = documents(:commentable_news)
    @request.env["HTTP_REFERER"] = news_url(source)
    assert_difference 'Clickthrough.count', +1 do
      get :show, :id => target_content.id, :t => 1
      assert_response :success
    end

    last_clickthrough = Clickthrough.find :first, :order => "id DESC"
    assert_equal 'Document', last_clickthrough.source_type
    assert_equal source.id, last_clickthrough.source_id
    
    # Eli: Ahora los news tiene su propio controller así que este if no es necesario.
    # if target_content.class.name.eql?('News')
    #   assert_equal 'Document', last_clickthrough.target_type
    # else  
    # assert_equal target_content.class.name, last_clickthrough.target_type
    # end  
    
    assert_equal target_content.class.name, last_clickthrough.target_type    
    assert_equal target_content.id, last_clickthrough.target_id
  end
  
  def create_news_with_attached_audio
    news = News.new :title_es => "News with attached audio", 
                    :organization => organizations(:gobierno_vasco), 
                    :body_es => "News with  attached audio", 
                    :published_at => Date.parse("2009-12-02")
    news.valid?
    assert news.save
    
    audio_at = news.attachments.new
    audio_at.file = File.new(File.join(Document::MULTIMEDIA_PATH, 'audios', 'sword.mp3'))
    assert audio_at.save
    assert news.attachments.include?(audio_at)

    return [news, audio_at]    
  end
  
  def assert_tag_in(*opts)
    target = HTML::Document.new(opts.shift, false, false)
    opts = opts.size > 1 ? opts.last.merge({ :tag => opts.first.to_s }) : opts.first
    assert !target.find(opts).nil?, "expected tag, but no tag found matching #{opts.inspect} in:\n#{target.inspect}"
  end
  
end

TEST_SOAP = false

Stats::CouchDB::DRB = 'http://localhost:4000/ilog4'

CarrierWave.configure do |config|
  config.storage = :file
end
