#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class BannerTest < ActiveSupport::TestCase

  test "should not save banner with blank url" do
    ban=Banner.new(:alt_es => 'Hola mundo', :logo_es => File.new("#{DocumentPaths::MULTIMEDIA_PATH}/photos/test-170x100.jpg"))
    assert !ban.valid?
    assert_equal "no puede estar vacío", ban.errors.on(:url_es)
  end  
  
  test "should not save banner with invalid url" do
    ban = Banner.new :alt_es => 'Hola mundo', :url_es => 'google.com',
                     :logo_es => File.new("#{DocumentPaths::MULTIMEDIA_PATH}/photos/test-170x100.jpg")
    assert !ban.valid?
    assert_equal "no es correcta", ban.errors.on(:url_es)
  end
  
  test "should not save banner with empty alt" do
    ban = Banner.new :url_es => 'http://www.google.es',
                   :logo_es => File.new("#{DocumentPaths::MULTIMEDIA_PATH}/photos/test-170x100.jpg")
    assert !ban.valid?
    assert_equal "no puede estar vacío", ban.errors.on(:alt_es)
  end
    
  test "should not save banner with emtpy logo file" do
    ban=Banner.new(:alt_es => 'Hola mundo', :url_es => 'http://www.google.es')
    assert !ban.valid?
    assert_equal "no puede estar vacío", ban.errors.on(:logo_es)
  end

  test "should not save banner with incorrect content_type" do
    ban = Banner.new :alt_es => 'Hola mundo', :url_es => 'http://www.google.es',
                     :logo_es => File.new("#{DocumentPaths::MULTIMEDIA_PATH}/test.txt")
    assert !ban.valid?
    assert_equal "no es un tipo de fichero válido", ban.errors.on(:logo_es)
  end

  test "should not save banner with incorrect size" do
    ban = Banner.new :alt_es => 'Hola mundo', :url_es => 'http://www.google.es',
                     :logo_es => File.new("#{DocumentPaths::MULTIMEDIA_PATH}/photos/test.jpg")
    assert_equal false, ban.valid?
    assert_equal "debe tener un tamaño de 170x100px", ban.errors.on(:logo_es)
  end
  
  test "should create banner and set its position accordingly" do
    ban=Banner.new :alt_es => 'Hola mundo', :url_es => 'http://www.google.es',
                   :logo_es => File.new("#{DocumentPaths::MULTIMEDIA_PATH}/photos/test-170x100.jpg")
    last_position = Banner.find(:first, :order => "position DESC").position
    assert ban.valid?
    assert ban.save
    assert_equal last_position + 1, ban.position
  end
    
end  