#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class CommentResponseTest < ActiveSupport::TestCase
  test "answered comment request" do
    if TEST_SOAP
      comment = comments(:comentario_aprobado_desde_rrcc)
      assert_equal 4, comment.id
    
      commentable = documents(:news_with_rrcc_response)
      assert_equal 8364, commentable.id
    
      comment_response = comment_responses(:comentario_aprobado_con_rrcc_response)
      assert_not_nil comment_response.comment
      assert_not_nil comment_response.comment.commentable
    
      comments_num = Comment.count 
      crr = comment_response.response_requests.create()
      if crr.result.to_i > 0
        assert_equal comments_num + 1, Comment.count
        assert_not_nil crr.answer_text
      else
        error = "SOAP connection time out"
        p "SOAP connection time out"
      end
    
      if error.nil?
        comment_response.reload
        p comment_response.answer_text
        assert_not_nil comment_response.answer_text
        assert_not_nil comment_response.answer_comment_id
        assert_equal "published", comment_response.status

        # Si hay respuesta oficial, publicarla directamente como comentario oficial:
        # comment.name = nombre del departamento
        # comment.rrcc_answered_at = fecha de la respuesta    
        assert_equal comment_response.organization.name, comment_response.answer.name
        assert_equal comment_response.answer_text, comment_response.answer.body
      end    
    else
      p "Testing SOAP disabled"
    end
  end
  
  test "waiting comment request" do
    if TEST_SOAP
      comment = comments(:comentario_aprobado_con_solicitud_rrcc)
      assert_equal 0, comment.id
    
      commentable = documents(:new_with_rrcc_request)
      assert_equal 8381, commentable.id
    
      comment_response = comment_responses(:comentario_aprobado_con_solicitud_rrcc_response)
      assert_not_nil comment_response.comment
      assert_not_nil comment_response.comment.commentable
      assert_equal "send", comment_response.status

      crr = comment_response.response_requests.create()
      if crr.result.to_i > 0
        assert_equal 31, crr.result.to_i
    
        comment_response.reload
        assert_equal "waiting", comment_response.status   
      else
        p "SOAP connection time out"
      end
    else
      p "Testing SOAP disabled"
    end
  end
  
  test "official comment with the received answer is created when the comment response is saved" do
    if TEST_SOAP
      comment_response = comment_responses(:comment_aprobado_castellano_response)
    
      assert_equal 2, comment_response.response_requests.count
      assert_equal 32,  comment_response.response_requests.first.result.to_i, "Response status expected to be 32"
      assert_equal 'send', comment_response.status
        
      assert_difference('Comment.count', +1) do
        comment_response.set_status_and_citizen_id!
      end
    
      assert_equal 'published', comment_response.status
      assert_not_nil comment_response.answer_comment_id
    else
      p "Testing SOAP disabled"
    end
  end
  
  
end
