#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class CommentTest < ActiveSupport::TestCase
  
  def default_fields
    {:commentable => documents(:commentable_news), :user => users(:visitante), :body => "Este artículo es muy interesante"}
  end
  
  [:user, :body].each do |field|
    test "should not save without #{field}" do
      comment = Comment.new(default_fields.merge(field => nil))
      assert !comment.save
      assert comment.errors.on(field).include?("no puede estar vacío")
    end
  end
  
  %w(superadmin admin periodista visitante colaborador jefe_de_prensa jefe_de_gabinete secretaria comentador_oficial operador_de_streaming miembro_que_crea_noticias).each do |user|
    test "comments made by #{user} should not have empty email" do
      comment = Comment.new({:commentable => documents(:commentable_news), :user => users(user), :body => "Este artículo es muy interesante", :email => nil})
      # Gets saved because there is a before_validation that fixes the empty email
      assert comment.save
      assert_equal users(user).email, comment.email
    end
  end
  
  test "twitter_user should be able to make a comment without email" do
    comment = Comment.new :user => users(:twitter_user), :commentable => documents(:commentable_news), :body => "Este artículo es muy interesante"
    assert comment.save
    assert_nil comment.email
  end
  
  test "facebook_user should be able to make a comment without email" do
    comment = Comment.new :user => users(:facebook_user), :commentable => documents(:commentable_news), :body => "Este artículo es muy interesante"
    assert comment.save
    assert_nil comment.email
  end
  
  test "assign language to comment" do
    comment = Comment.new(default_fields)
    assert comment.save
    assert_not_nil comment.locale
  end
  
  [:visitante, :periodista, :colaborador].each do |user|
    test "#{user} comments have pending status" do
      comment = Comment.new(default_fields.merge(:user => users(user)))
      assert comment.save
      assert comment.status.eql?("pendiente")
    end
  end
  
  [:admin, :jefe_de_prensa, :jefe_de_gabinete].each do |user|
    test "#{user} comments have approved status" do
      comment = Comment.new(default_fields.merge(:user => users(user)))
      assert comment.save
      assert comment.status.eql?("aprobado")    
    end
  end
  
  test "parent area is assigned when creating comment" do
    comment = Comment.create(default_fields)
    assert_equal comment.tag_list, documents(:commentable_news).area_tags
  end
  
  context "notifications" do
    should "not create notification for unapproved comment in news" do
      assert_no_difference 'Notification.count' do
        comment = Comment.create(default_fields)
      end
    end
  
    should "not create notification approved comment in news" do
      assert_no_difference 'Notification.count' do
        comment = Comment.create(default_fields.merge({:status => "aprobado"}))
      end
    end
    
    should "not create notification for unapproved comment in unapproved proposal" do
      assert_no_difference 'Notification.count' do
        comment = contributions(:unapproved_proposal).comments.create(:user => users(:visitante), :body => "Este artículo es muy interesante")
      end
    end

    should "not create notification for approved comment in unapproved proposal" do
      assert_no_difference 'Notification.count' do
        comment = contributions(:unapproved_proposal).comments.create(:user => users(:visitante), :body => "Este artículo es muy interesante", :status => "aprobado")
      end
    end
  
    should "not create notification for unapproved comment in approved proposal" do
      assert_no_difference 'Notification.count' do
        comment = contributions(:approved_and_published_proposal).comments.create(:user => users(:visitante), :body => "Este artículo es muy interesante")
      end
    end
    
    should "create notification for approved comment in approved proposalxx" do
      comment = contributions(:approved_and_published_proposal).comments.create(:user => users(:visitante), :body => "Este artículo es muy interesante")
      assert_difference 'Notification.count', +1 do
        comment.update_attributes(:status => "aprobado")
      end
    end
  end
  
end
