#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class DebateTest < ActiveSupport::TestCase
  def setup
    @debate_params = {:title_es => "Título en castellano del debate", 
                      :body_es => "Introducción en castellano", 
                      :description_es => "Descripción en castellano",
                      :organization_id => organizations(:lehendakaritza).id,
                      :hashtag => "#nuevodebate",
                      :multimedia_dir => "nuevodebate"}
  end

  context "create debate" do
    setup do
      @debate = Debate.new(@debate_params)
    end
    
    should "not validate if hashtag is empty" do
      @debate.hashtag = nil
      assert !@debate.valid?
      assert @debate.errors.on("hashtag")
      assert_match "no puede estar vacío", @debate.errors.on("hashtag").join(", ")
    end        

    should "not validate if hashtag is not unique" do
      @debate.hashtag = debates(:debate_completo).hashtag
      assert !@debate.valid?
      assert @debate.errors.on("hashtag")
      assert_match @debate.errors.on("hashtag"), "ya está cogido"
    end        
    
    should "not validate if multimedia_dir contains invalid characters" do
      @debate.multimedia_dir = " á "
      assert !@debate.valid?
      assert @debate.errors.on("multimedia_dir")
      assert_match @debate.errors.on("multimedia_dir"), "no es válido"
    end

    should "create a tag on create" do
      assert @debate.valid?

      assert_difference "Tagging.count", 1 do
        assert_difference "Tag.count", 1 do
          assert @debate.save
        end
      end
      
      debate_tag = @debate.tags.first
      assert_equal "hashtag_#{@debate.hashtag.gsub(/#/,'')}", debate_tag.sanitized_name_es
      assert_equal "hashtag_#{@debate.hashtag.gsub(/#/,'')}", debate_tag.sanitized_name_eu
      assert_equal "hashtag_#{@debate.hashtag.gsub(/#/,'')}", debate_tag.sanitized_name_en            
      
      assert_equal @debate.tag_name, @debate.tags.first.name_es
    end
    
    should "add # to the hashtag unless present" do
      @debate.hashtag = "nuevodebate"
      assert @debate.save
      assert_equal "#nuevodebate", @debate.hashtag
    end

    should "add debate as prefix for multimedia_dir" do
      assert @debate.valid?
      assert @debate.save
      assert_not_nil @debate.multimedia_path      
    end
 
    should "set_multimedia_path" do
      assert @debate.valid?
      assert @debate.save
      assert_not_nil @debate.multimedia_path      
    end
 
    should "create multimedia dir" do
      assert @debate.valid?
      assert @debate.save
      assert_not_nil @debate.multimedia_path
      
      assert_match /^debates\//, @debate.multimedia_path

      assert File.exists?("#{Document::MULTIMEDIA_PATH}#{@debate.multimedia_path}")
      assert File.directory?("#{Document::MULTIMEDIA_PATH}#{@debate.multimedia_path}")
      FileUtils.rm_rf(Dir.glob("#{Document::MULTIMEDIA_PATH}#{@debate.multimedia_path}"))
    end

    context "set default image" do
      should "if empty" do
        assert_equal false, @debate.cover_image.present?
        assert @debate.save
        assert_equal true, @debate.cover_image.present?
        assert_equal true, @debate.header_image.present?
        assert_equal 'debate_default_00.jpg', @debate.cover_image.url.split('/').last
      end  

      should "increment default image index" do
        assert @debate.save
        assert_equal 'debate_default_00.jpg', @debate.cover_image.url.split('/').last
        @debate2 = Debate.new(@debate_params.merge(:hashtag => "#nuevodebate2"))
        assert @debate2.save
        assert_equal 'debate_default_01.jpg', @debate2.cover_image.url.split('/').last
        @debate3 = Debate.new(@debate_params.merge(:hashtag => "#nuevodebate3"))
        assert @debate3.save
        assert_equal 'debate_default_02.jpg', @debate3.cover_image.url.split('/').last
        @debate4 = Debate.new(@debate_params.merge(:hashtag => "#nuevodebate4"))
        assert @debate4.save
        assert_equal 'debate_default_03.jpg', @debate4.cover_image.url.split('/').last
        @debate5 = Debate.new(@debate_params.merge(:hashtag => "#nuevodebate5"))
        assert @debate5.save
        assert_equal 'debate_default_00.jpg', @debate5.cover_image.url.split('/').last
      end
    end
    
    should "get stages" do
      @debate.init_stages
      assert_equal @debate.stages.detect {|s| s.label.eql?('presentation')}, @debate.presentation_stage
      assert_equal @debate.stages.detect {|s| s.label.eql?('discussion')}, @debate.discussion_stage
      assert_equal @debate.stages.detect {|s| s.label.eql?('contribution')}, @debate.contribution_stage      
      assert_equal @debate.stages.detect {|s| s.label.eql?('conclusions')}, @debate.conclusions_stage            
    end

    context "stages dates" do
      setup do
        @debate.init_stages
        assert @debate.presentation_stage
        assert @debate.discussion_stage
        assert @debate.contribution_stage 
        assert @debate.conclusions_stage 
      
        @stage_presentation = @debate.stages.detect {|s| s.label.eql?('presentation')}
        @stage_discussion = @debate.stages.detect {|s| s.label.eql?('discussion')}
        @stage_contribution = @debate.stages.detect {|s| s.label.eql?('contribution')}      
        @stage_conclusions = @debate.stages.detect {|s| s.label.eql?('conclusions')}
        
        @stage_presentation.starts_on = Date.today
        @stage_discussion.starts_on = 2.days.from_now.to_date
        @stage_contribution.starts_on = 3.days.from_now.to_date
        @stage_conclusions.starts_on = 4.days.from_now.to_date
        @stage_conclusions.ends_on = 5.days.from_now.to_date
      end
      
      should "set pulished_at to nil for draft debate" do      
        assert @debate.draft.eql?("1")
                
        assert @debate.save
        assert_nil @debate.published_at
      end

      should "assign presentation stage starts_on as published_at for published debate" do      
        assert @debate.draft = 0
                
        assert @debate.save
        assert_equal @stage_presentation.starts_on, @debate.published_at.to_date
      end 
      
      should "save debate even if stage dates are not ordered" do
        @debate.presentation_stage.starts_on = 2.days.from_now.to_date
        @debate.discussion_stage.starts_on = 1.day.from_now.to_date
        @debate.contribution_stage.starts_on = 4.days.from_now.to_date
        @debate.conclusions_stage.starts_on = 3.days.from_now.to_date
        @debate.conclusions_stage.ends_on = 5.days.from_now.to_date
        assert @debate.save
        assert @debate.errors.empty?
      end

      should "save debate if stage dates are ordered" do
        @debate.presentation_stage.starts_on = 1.days.from_now.to_date
        @debate.discussion_stage.starts_on = 2.day.from_now.to_date
        @debate.contribution_stage.starts_on = 3.days.from_now.to_date
        @debate.conclusions_stage.starts_on = 4.days.from_now.to_date
        @debate.conclusions_stage.ends_on = 5.days.from_now.to_date
        assert @debate.save
        assert @debate.errors.empty?
      end      
      
      should "set stage positions" do
        assert @debate.save
        assert_equal 1, @debate.presentation_stage.position
        assert_equal 2, @debate.discussion_stage.position
        assert_equal 3, @debate.contribution_stage.position        
        assert_equal 4, @debate.conclusions_stage.position                
      end      
      
    end
  end
  
  context "update debate" do
    setup do 
      @debate = Debate.create(@debate_params)
      @debate.init_stages
      @debate.save
      DebateStage::STAGES.each do |stage_label|
        assert_not_nil @debate.send("#{stage_label}_stage")
      end
    end
    
    should "update debate tag if hashtag is changed" do
      old_hashtag = @debate.hashtag
      debate_tag = @debate.tags.find_by_name_es(@debate.tag_name)
      
      new_hashtag = "#nuevohashtag"
      @debate.hashtag = new_hashtag
      assert @debate.valid?

      assert_no_difference "Tagging.count" do
        assert_no_difference "Tag.count" do
          assert @debate.save
        end
      end

      debate_tag.reload
      assert_equal "hashtag_nuevohashtag", debate_tag.sanitized_name_es
      assert_equal "hashtag_nuevohashtag", debate_tag.sanitized_name_eu      
      assert_equal "hashtag_nuevohashtag", debate_tag.sanitized_name_en      
      assert_equal @debate.tag_name, debate_tag.name_es
    end
    
    should "add # to the hashtag unless present" do
      @debate.hashtag = "nuevodebate"
      assert @debate.save
      assert_equal "#nuevodebate", @debate.hashtag
    end
    
    should "not change multimedia_dir on update" do
      old_multimedia_dir = @debate.multimedia_dir
      
      @debate.title_es = "Cambio de título"      
      assert @debate.save
      
      @debate.reload
      assert_equal old_multimedia_dir, @debate.multimedia_dir
    end
    
    should "save debate even if stage dates are not in order" do
      @debate.presentation_stage.starts_on = 2.days.from_now.to_date
      @debate.discussion_stage.starts_on = 1.day.from_now.to_date
      @debate.contribution_stage.starts_on = 4.days.from_now.to_date
      @debate.conclusions_stage.starts_on = 3.days.from_now.to_date
      @debate.conclusions_stage.ends_on = 5.days.from_now.to_date
      assert @debate.save
      assert @debate.errors.empty?
    end
    
    should "save debate if stage dates are in order" do
      @debate.presentation_stage.starts_on = 1.days.from_now.to_date
      @debate.presentation_stage.ends_on = 1.days.from_now.to_date
      @debate.discussion_stage.starts_on = 2.day.from_now.to_date
      @debate.contribution_stage.starts_on = 3.days.from_now.to_date
      @debate.conclusions_stage.starts_on = 4.days.from_now.to_date
      @debate.conclusions_stage.ends_on = 5.days.from_now.to_date
      assert @debate.save
      assert @debate.errors.empty?
    end
    
    should "destroy non active stages" do
      stages_attributes = @debate.stages.map {|s| {:id => s.id, :label => s.label, :starts_on => s.starts_on, :ends_on => s.ends_on, :position => s.position}}
      stages_attributes.last["_destroy"] = 1
      assert_difference "DebateStage.count", -1 do
        @debate.stages_attributes = stages_attributes
        @debate.save
      end
    end
  end

  context "debate completo" do
    setup do 
      @debate = debates(:debate_completo)
      assert @debate.stages.present?
    end 

    should "respond to class_name" do
      assert_equal "Debate", @debate.class_name
    end
 
    should "get stages ordered by position" do
      assert_equal DebateStage::STAGES, @debate.stages.map {|s| s.label.to_sym}
      
      assert @debate.contribution_stage.update_attribute(:position, 10)
      @debate.reload
      assert_equal [:presentation, :discussion, :conclusions, :contribution].to_s, @debate.stages.map {|s| s.label.to_sym}.to_s
    end
    
    should "get current stage for finished debate" do
      assert @debate.stages.last.ends_on < Date.today
      assert @debate.finished?
      
      assert_equal "conclusions", @debate.current_stage.label
    end
    
    should "get current stage for future debate" do
      @debate.stages.each_with_index do |s, i|
        s.update_attributes(:starts_on => Date.today + (i+1).months, :ends_on => Date.today + (i+2).months)
      end
      assert @debate.future?
      assert_equal "presentation", @debate.current_stage.label
    end
    
    should "get current stage for debate in presentation" do
      @debate.stages.each_with_index do |s, i|
        s.update_attributes(:starts_on => Date.today + (i+1).months, :ends_on => Date.today + (i+2).months)
      end      
      @debate.stages.first.update_attributes(:starts_on => Date.today, :ends_on => Date.today + 1.months)
      assert !@debate.finished?
      assert !@debate.future?
      assert @debate.current_stage.eql?(@debate.stages.first)
    end

    should "get current stage for debate in discussion" do
      @debate.stages.each_with_index do |s, i|
        s.update_attributes(:starts_on => Date.today + (i+1).months, :ends_on => Date.today + (i+2).months)
      end      
      @debate.stages.first.update_attributes(:starts_on => Date.today - 1.month, :ends_on => Date.today - 1.day)      
      @debate.stages[1].update_attributes(:starts_on => Date.today, :ends_on => Date.today + 1.months)
      
      assert !@debate.finished?
      assert !@debate.future?
      assert @debate.current_stage.eql?(@debate.stages[1])
    end

    should "get current stage for debate in contribution" do
      @debate.stages.each_with_index do |s, i|
        s.update_attributes(:starts_on => Date.today + (i+1).months, :ends_on => Date.today + (i+2).months)
      end      
      @debate.stages.first.update_attributes(:starts_on => Date.today - 2.month, :ends_on => Date.today - 1.month - 1.day)      
      @debate.stages[1].update_attributes(:starts_on => Date.today - 1.month, :ends_on => Date.today - 1.day)
      @debate.stages[2].update_attributes(:starts_on => Date.today, :ends_on => Date.today + 1.months)      
      
      assert !@debate.finished?
      assert !@debate.future?
      assert @debate.current_stage.eql?(@debate.stages[2])
    end

    should "get current stage for debate in conclusions" do
      @debate.stages.each_with_index do |s, i|
        s.update_attributes(:starts_on => Date.today + (i+1).months, :ends_on => Date.today + (i+2).months)
      end      
      @debate.stages[0].update_attributes(:starts_on => Date.today - 3.month, :ends_on => Date.today - 2.month - 1.day)      
      @debate.stages[1].update_attributes(:starts_on => Date.today - 2.month, :ends_on => Date.today - 1.month - 1.day)
      @debate.stages[2].update_attributes(:starts_on => Date.today - 1.month, :ends_on => Date.today - 1.day)
      @debate.stages[3].update_attributes(:starts_on => Date.today, :ends_on => Date.today + 1.months)            
      
      assert !@debate.finished?
      assert !@debate.future?
      assert @debate.current_stage.eql?(@debate.stages[3])
    end
  end
  
  context "syncronization between debate areas and it's comments areas" do
    setup do
      @debate = debates(:debate_nuevo)
      @debate.comments.create(:body => "thoughtful comment", :user => users(:comentador_oficial))
    end
    
    should "have lehendakaritza area tag" do
      assert_equal [areas(:a_lehendakaritza)], @debate.areas
      assert @debate.comments.count > 0
      @debate.comments.each do |comment|
        assert_equal [areas(:a_lehendakaritza).area_tag], comment.tags
      end
    end
    
    context "via taggings" do
      should "add new area to comment" do
        @debate.taggings.build(:tag => areas(:a_interior).area_tag)
        @debate.save
        @debate.reload
        assert @debate.areas.include?(areas(:a_interior))
        @debate.comments.each do |comment|
          assert comment.tags.include?(areas(:a_interior).area_tag)
        end
      end
      
      should "remove area from comment" do
        # in proposals area_id cannot be blank
        @debate.taggings.create(:tag => areas(:a_interior).area_tag)
        @debate.taggings.delete(@debate.taggings.find_by_tag_id(areas(:a_lehendakaritza).area_tag.id))
        @debate.save
        @debate.reload
        assert !@debate.areas.include?(areas(:a_lehendakaritza))
        @debate.comments.each do |comment|
          assert !comment.tags.include?(areas(:a_lehendakaritza).area_tag)
        end
      end
      
      should "not sync tags that are not area tags" do
        new_tag = tags(:tag_politician_lehendakaritza)
        @debate.taggings.build(:tag => new_tag)
        @debate.save
        @debate.reload
        assert @debate.tags.include?(new_tag)
        @debate.comments.each do |comment|
          assert !comment.tags.include?(new_tag)
        end
      end
    end

    context "via area_tags=" do
      # This is what the form in admin/documents/edit_tags uses
      should "add new area to comment" do
        @debate.area_tags= "#{areas(:a_lehendakaritza).area_tag.name_es}, #{areas(:a_interior).area_tag.name_es}"
        assert @debate.save
        @debate.reload
        assert @debate.areas.include?(areas(:a_interior))
        @debate.comments.each do |comment|
          assert comment.tags.include?(areas(:a_interior).area_tag)
        end
      end
      
      should "remove area from comment" do
        @debate.area_tags = "#{areas(:a_interior).area_tag.name_es}"
        @debate.save
        @debate.reload
        assert !@debate.areas.include?(areas(:a_lehendakaritza))
        @debate.comments.each do |comment|
          assert !comment.tags.include?(areas(:a_lehendakaritza).area_tag)
        end
      end
      
      should "not sync tags that are not area tags" do
        new_tag = tags(:tag_politician_lehendakaritza)
        @debate.area_tags = "#{new_tag.name_es}"
        @debate.save
        @debate.reload
        assert @debate.tags.include?(new_tag)
        @debate.comments.each do |comment|
          assert !comment.tags.include?(new_tag)
        end
      end
      
    end

    context "via tag_list" do
      should "add new area to comment" do
        @debate.tag_list.add areas(:a_interior).area_tag.name_es
        @debate.save
        @debate.reload
        assert @debate.areas.include?(areas(:a_interior))
        @debate.comments.each do |comment|
          assert comment.tags.include?(areas(:a_interior).area_tag)
        end
      end
    
      should "remove area from comment" do
        @debate.tag_list.remove areas(:a_lehendakaritza).area_tag.name_es
        @debate.save
        @debate.reload
        assert !@debate.areas.include?(areas(:a_lehendakaritza))
        @debate.comments.each do |comment|
          assert !comment.tags.include?(areas(:a_lehendakaritza).area_tag)
        end
      end
      
      should "not sync tags that are not area tags" do
        new_tag = tags(:tag_politician_lehendakaritza)
        @debate.taggings.build(:tag => new_tag)
        @debate.save
        @debate.reload
        assert @debate.tags.include?(new_tag)
        @debate.comments.each do |comment|
          assert !comment.tags.include?(new_tag)
        end
      end
    end
  end

  test "should index to elasticsearch after save" do
    prepare_elasticsearch_test
    debate = debates(:debate_completo)
    assert_deleted_from_elasticsearch debate
    assert debate.save
    assert_indexed_in_elasticsearch debate
  end

  test "should delete from elasticsearch after destroy" do
    prepare_elasticsearch_test
    debate = debates(:debate_completo)
    assert_deleted_from_elasticsearch debate
    assert debate.save
    assert_indexed_in_elasticsearch debate
    assert debate.destroy
    assert_deleted_from_elasticsearch debate
  end

end
