#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class DepartmentTest < ActiveSupport::TestCase
  
  test "label" do
    assert_equal 'interior', organizations(:interior).label
    assert_equal organizations(:interior), Department.find_by_label(organizations(:interior).label)
  end

  test "should not save department with duplicated tag_name" do
    org = Department.new(organizations(:lehendakaritza).attributes)
    assert !org.valid?
    assert org.errors.on(:tag_name).include?("ya está cogido")
  end
  
  test "should not save department with empty tag_name" do
    org = Department.new(:name_es => "nuevo departamento", :tag_name => nil)
    assert !org.valid?
    assert org.errors.on(:tag_name).include?("no puede estar vacío")
  end
  
  test "should not save department with invalid tag_name" do
    org = Department.new(:name_es => "nuevo departamento", :tag_name => "this is inválid")
    assert !org.valid?
    assert org.errors.on(:tag_name).include?("no es válido")
  end  
  
  test "department editors" do
    lehendakaritza = organizations(:interior)
    assert_equal ['DepartmentEditor'], lehendakaritza.department_editors.collect(&:type).collect(&:to_s).uniq
  end
  
  test "department members official commenters" do
    lehendakaritza = organizations(:interior)
     
    lehendakaritza.department_members_official_commenters.each do |miembro|
      assert miembro.is_a?(DepartmentMember) && miembro.is_official_commenter?
    end
    
  end
  
  test "should set active to true for new department" do
    org = Department.new(:name_es => "nuevo departamento", :tag_name => "_new_department", :enet_workarea => 'xxx')
    assert org.save
    org.reload
    assert org.active?
  end
  
  test "should get only active departments" do
    active_depts = Department.active.find(:all)
    assert active_depts.length > 0 # nos aseguramos que hay departamentos activos
    assert active_depts.length < Department.count # nos aseguramos que el total de departamentos es más que los activos
    assert_nil active_depts.detect {|d| !d.active?}
  end
  
  test "should get only departemts for IX term" do
    depts_ix = Department.find(:all, :conditions => "term = 'IX'").map {|d| d.id}.sort
    assert_equal depts_ix, Department.for_term("IX").find(:all).map {|d| d.id}.sort
  end
  
end
