#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class NotifierTest < ActionMailer::TestCase

  def setup
    I18n.locale = :es
  end
  
  def teardown
    I18n.locale = :es
  end
  
  test "should send email in spanish to journalist about event published" do
    # Send the email, then test that it got queued
    email = Notifier.deliver_journalist_event_alert(event_alerts(:unsent_alert))
    assert !ActionMailer::Base.deliveries.empty?
    
    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in spanish
    assert_equal ['periodista2@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] #{event_alerts(:unsent_alert).event.title_es.tildes}", email.subject
    assert_match 'Alerta sobre evento en', email.body    
    #assert_match 'Por la presente te informamos que respecto al evento', email.body
    assert_match "Fecha: #{event_alerts(:unsent_alert).event.pretty_dates('es')}", email.body
  end

  test "email in spanish to journalist about event published should contain speakers text in spanish" do
    UserActionObserver.current_user = users(:admin)
    evt = event_alerts(:unsent_alert).event
    politician_tag = tags(:tag_politician_lehendakaritza)
    politician = Politician.find(politician_tag.kind_info)
    speaker = 'Otro invitado'
    evt.update_attributes(:tag_list_es => politician_tag.name, :speaker_es => speaker)
    assert_equal "#{politician.public_name_and_role}, #{speaker}", evt.attendee_names
    UserActionObserver.current_user = nil
    
    # Send the email, then test that it got queued
    email = Notifier.deliver_journalist_event_alert(event_alerts(:unsent_alert))
    assert !ActionMailer::Base.deliveries.empty?
    
    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in spanish
    assert_equal ['periodista2@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] #{event_alerts(:unsent_alert).event.title_es.tildes}", email.subject
    assert_match "#{I18n.t('events.speaker', :locale => 'es')}: #{evt.attendee_names}", email.body    
  end

  
  test "should send email in basque to journalist about event published" do
    event_alert = event_alerts(:unsent_alert_eu)
    # Send the email, then test that it got queued
    email = Notifier.deliver_journalist_event_alert(event_alert)
    assert !ActionMailer::Base.deliveries.empty?
    
    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in spanish, but the event title is not available but in spanish
    assert_equal ['periodista_eu@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] #{event_alert.event.title_es.tildes}", email.subject
    assert_match 'Ekitaldi baten inguruko alerta', email.body
    assert_match "Data: #{event_alerts(:unsent_alert).event.pretty_dates('eu')}", email.body
  end

  test "email in basque to journalist about event published should contain speakers text in basque" do
    event_alert = event_alerts(:unsent_alert_eu)
    UserActionObserver.current_user = users(:admin)
    evt = event_alert.event
    politician_tag = tags(:tag_politician_lehendakaritza)
    politician = Politician.find(politician_tag.kind_info)
    speaker_es = 'Otro invitado'
    speaker_eu = 'Bestea'
    evt.update_attributes(:tag_list_es => politician_tag.name, :speaker_es => speaker_es, :speaker_eu => speaker_eu)
    I18n.locale = :eu
    assert_equal "#{politician.public_name} (#{politician.public_role_eu}), Bestea", evt.attendee_names
    I18n.locale = :es    
    UserActionObserver.current_user = nil
    
    email_locale = event_alert.spammable.alerts_locale
    assert_equal 'eu', email_locale
    
    # Send the email, then test that it got queued
    email = Notifier.deliver_journalist_event_alert(event_alert)
    assert !ActionMailer::Base.deliveries.empty?
    
    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in basque
    assert_match 'Ekitaldi baten inguruko alerta', email.body    
    assert_match "#{I18n.t('events.speaker', :locale => 'eu')}: #{evt.attendee_names('eu')}", email.body    
  end



  test "should send email in spanish to journalist on event only for photographersxx" do
    ea = EventAlert.create(:event => documents(:event_with_tag_one), :spammable_id => users(:periodista_con_alertas).id, :spammable_type => 'Journalist', :version => 1, :send_at => Time.zone.now - 30.minutes)
    
    email = Notifier.deliver_journalist_event_alert(ea)
    assert !ActionMailer::Base.deliveries.empty?

    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in spanish, but the event title is not available but in spanish
    assert_equal [users(:periodista_con_alertas).email], email.to
    assert_equal "[#{Notifier::SERVICENAME}] #{documents(:event_with_tag_one).title_es.tildes}", email.subject.tildes
    assert_match 'Alerta sobre evento en', email.body
    #assert_match 'Por la presente te informamos que respecto al evento', email.body
    assert_match "Fecha: #{ea.event.pretty_dates('es')}", email.body
    
    assert_match "Sólo para medios gráficos", email.body
  end

  
  test "should send email in spanish to creator of event with streaming" do
    ea = event_alerts(:unsent_alert_for_creator_of_event_with_streaming)
    email = Notifier.deliver_staff_event_alert(ea)
    assert !ActionMailer::Base.deliveries.empty?

    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in spanish, but the event title is not available but in spanish
    assert_equal ['admin@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Notificacion sobre streaming a Administrador", email.subject
    assert_match 'desde Lehendakaritza le informamos', email.body
    assert_match "Fecha: #{ea.event.pretty_dates('es')}", email.body
    
    assert_match "Irekia cubrirá este evento con", email.body
    
    assert_match "Este evento en la sala #{ea.event.stream_flow.title_es} tendrá streaming que se emitirá en #{ea.event.streaming_for_pretty}", email.body
  end

  test "should send email in spanish to department of event with no coverage" do
    ea = EventAlert.create(:event => documents(:future_event), :spammable_id => users(:jefe_de_prensa).id, :spammable_type => 'DepartmentEditor', :version => 1, :send_at => Time.zone.now - 30.minutes, :notify_about => 'coverage')
    email = Notifier.deliver_staff_event_alert(ea)
    assert !ActionMailer::Base.deliveries.empty?

    # Test the body of the sent email contains what we expect it to
    assert_equal [users(:jefe_de_prensa).email], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Notificacion sobre cobertura a Jefe de prensa", email.subject
    assert_match 'desde Lehendakaritza le informamos', email.body
    assert_match "Fecha: #{ea.event.pretty_dates('es')}", email.body
    
    assert_no_match /Irekia cubrirá este evento con/, email.body    
    assert_match I18n.t('sadmin.events.irekia_sin_determinar'), email.body
  end

  test "should send email in spanish to creator of event with related_news and article coverage" do
    ea = EventAlert.create(:event => documents(:emakunde_passed_event), :spammable_id => users(:jefe_de_prensa).id, :spammable_type => 'DepartmentEditor', :version => 1, :send_at => Time.zone.now - 30.minutes, :notify_about => 'coverage')
    
    email = Notifier.deliver_staff_event_alert(ea)
    assert !ActionMailer::Base.deliveries.empty?

    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in spanish, but the event title is not available but in spanish
    assert_equal [users(:jefe_de_prensa).email], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Notificacion sobre cobertura a Jefe de prensa", email.subject
    assert_match 'desde Lehendakaritza le informamos', email.body
    assert_match "Fecha: #{ea.event.pretty_dates('es')}", email.body
    
    assert_match "Irekia cubrirá este evento con crónica", email.body
    assert_match "Noticia relacionada", email.body
  end


  test "should send email in spanish to creator of event with draft realed_news" do
    ea = EventAlert.create(:event => documents(:emakunde_future_event), :spammable_id => users(:jefe_de_prensa).id, :spammable_type => 'DepartmentEditor', :version => 1, :send_at => Time.zone.now - 30.minutes, :notify_about => 'coverage')
    
    email = Notifier.deliver_staff_event_alert(ea)
    assert !ActionMailer::Base.deliveries.empty?

    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in spanish, but the event title is not available but in spanish
    assert_equal [users(:jefe_de_prensa).email], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Notificacion sobre cobertura a Jefe de prensa", email.subject
    assert_match 'desde Lehendakaritza le informamos', email.body
    assert_match "Fecha: #{ea.event.pretty_dates('es')}", email.body
    
    assert_match I18n.t('sadmin.events.irekia_sin_determinar'), email.body
    assert_match "BORRADOR inicial", email.body
  end



  test "should send email in spanish to streaming operator of event with streaming" do
    ea = event_alerts(:unsent_alert_for_streaming_operator_of_event_with_streaming)
    email = Notifier.deliver_staff_event_alert(ea)
    assert !ActionMailer::Base.deliveries.empty?

    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in spanish, but the event title is not available but in spanish
    assert_equal ['operador@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Notificacion sobre streaming a Operador de streaming", email.subject
    assert_match 'desde Lehendakaritza le informamos', email.body
    assert_match "Fecha: #{ea.event.pretty_dates('es')}", email.body
    assert_match "Este evento en la sala #{ea.event.stream_flow.title} tendrá streaming que se emitirá en #{ea.event.streaming_for_pretty}", email.body
    
    assert_no_match /Irekia cubrirá este evento con/, email.body
  end

  test "should send email in spanish to room_manager of event with streaming" do
    ea = event_alerts(:unsent_alert_for_room_manager_of_event_with_streaming)
    email = Notifier.deliver_staff_event_alert(ea)
    assert !ActionMailer::Base.deliveries.empty?

    # Test the body of the sent email contains what we expect it to
    # This journalist has requested alerts in spanish, but the event title is not available but in spanish
    assert_equal ['manager@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Notificacion sobre streaming a Responsable de sala", email.subject
    assert_match 'desde Lehendakaritza le informamos', email.body
    assert_match "Fecha: #{ea.event.pretty_dates('es')}", email.body
    assert_match "Este evento en la sala #{ea.event.stream_flow.title} tendrá streaming que se emitirá en #{ea.event.streaming_for_pretty}", email.body
  end
  
  test "alert about event with streaming should contain info about room" do
    ea = event_alerts(:unsent_alert_for_streaming_operator_of_event_with_streaming)
    email = Notifier.deliver_staff_event_alert(ea)
    assert !ActionMailer::Base.deliveries.empty?
    assert_match "Este evento en la sala #{ea.event.stream_flow.title} tendrá streaming que se emitirá en #{ea.event.streaming_for_pretty}", email.body
  end

  test " staff alert should contain info about attendee" do
    ea = event_alerts(:unsent_alert_for_streaming_operator_of_event_with_streaming)
    
    UserActionObserver.current_user = users(:admin)
    evt = ea.event
    politician_tag = tags(:tag_politician_lehendakaritza)
    politician = Politician.find(politician_tag.kind_info)
    speaker = 'Otro invitado'
    evt.update_attributes(:tag_list_es => politician_tag.name, :speaker_es => speaker)
    assert_equal "#{politician.public_name_and_role}, #{speaker}", evt.attendee_names
    UserActionObserver.current_user = nil
    
    email = Notifier.deliver_staff_event_alert(ea)
    assert !ActionMailer::Base.deliveries.empty?
    assert_match "#{I18n.t('events.speaker', :locale => 'es')}: #{evt.attendee_names}", email.body        
  end
  
  
  test "should send account activation email in spanish" do
    email = Notifier.deliver_activate_person_account(users(:visitante_sin_activar))
    assert !ActionMailer::Base.deliveries.empty?
    
    assert_equal ['visitante_sin_activar@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Bienvenida", email.subject
    assert_match 'Para asegurarnos de que realmente has solicitado el alta en', email.body    
  end
  
  test "should send account activation email in basque" do
    I18n.locale = :eu

    email = Notifier.deliver_activate_person_account(users(:visitante_sin_activar))
    assert !ActionMailer::Base.deliveries.empty?
    
    assert_equal ['visitante_sin_activar@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Ongi etorria", email.subject
    assert_match 'alta eskatu duzula, zure kontua aktiba dezazun behar dugu.', email.body    
    I18n.locale = :es
  end
  
  test "should email document to a friend" do
    email = Notifier.deliver_email_document("Remitente", "Receptor", "receptor@efaber.net", documents(:one_news))
    assert !ActionMailer::Base.deliveries.empty?
    
    assert_equal ['receptor@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Remitente te recomienda esta página", email.subject
    assert_match 'Remitente ha pensado que te interesará leer esta página', email.body    
  end
  
  test "should email forgotten password" do
    I18n.locale = :eu
    email = Notifier.deliver_password_reminder(users(:visitante))
    assert !ActionMailer::Base.deliveries.empty?
    
    assert_equal ['visitante@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Sartze informazioa", email.subject
    assert_match 'Hau da zure sartze informazioa', email.body        
    I18n.locale = :es
  end
  
  test "should welcome journalist" do
    I18n.locale = :eu
    email = Notifier.deliver_welcome_journalist(users(:periodista_sin_aprobar))
    assert !ActionMailer::Base.deliveries.empty?
    
    assert_equal ['periodista_sin_aprobar@efaber.net'], email.to
    assert_equal "[#{Notifier::SERVICENAME}] Ongi etorri Irekia(e)ra", email.subject
    assert_match 'Tu registro ha sido activado.', email.body
    assert_match 'Hau da zure sartze informazioa', email.body
    I18n.locale = :es
  end
  
end
