#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class OrderTest < ActiveSupport::TestCase
  
  ['titulo', 'texto', 'rango', 'seccion'].each do |att|
    test "validates presence and translation of #{att}" do
      order = Order.new()
      assert_equal false, order.valid?
      assert_equal 'no puede estar vacío', order.errors.on(att.to_sym)
    end  
  end  
  
  ['fecha_bol', 'no_bol', 'no_orden'].each do |att|
    test "validates presence of #{att}" do
      order = Order.new()
      assert_equal false, order.valid?
      assert_equal 'no puede estar vacío', order.errors.on(att.to_sym)
    end  
  end                       
  
  test "validates uniqueness of no_orden" do
    order = Order.new(:no_orden => '20001001')
    assert_equal false, order.valid?
    assert_equal 'ya está cogido', order.errors.on(:no_orden)    
  end  
  
  test "create valid order" do
    order = Order.new(:titulo_es => 'Orden2', :texto_es => 'Texto Orden2', :seccion_es => 'Seccion2', :rango_es => 'Rango1', :fecha_bol => Date.today, :no_orden => '20001002', :no_bol => '1000')
    assert_equal true, order.save
  end  

  test "should index to elasticsearch related after save" do
    prepare_bopv_elasticsearch_test
    order = orders(:order_one)
    assert_deleted_from_elasticsearch order, ELASTICSEARCH_BOPV_URI
    assert order.save
    assert_indexed_in_elasticsearch order, ELASTICSEARCH_BOPV_URI
  end

  test "should delete from elasticsearch related after destroy" do
    prepare_bopv_elasticsearch_test
    order = orders(:order_one)
    assert_deleted_from_elasticsearch order, ELASTICSEARCH_BOPV_URI
    assert order.save
    assert_indexed_in_elasticsearch order, ELASTICSEARCH_BOPV_URI
    assert order.destroy
    assert_deleted_from_elasticsearch order, ELASTICSEARCH_BOPV_URI
  end

  
end  
