#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class OrganizationTest < ActiveSupport::TestCase
  
  test "organizations for dept" do
    dept = organizations(:lehendakaritza)
    
    assert dept.is_a?(Department)
    assert_equal 1, dept.organizations.size
    assert_equal organizations(:emakunde), dept.organizations.first
  end
  
  test "grouped_options" do
    dept = organizations(:lehendakaritza_ix_leg)
    org = organizations(:organismo_ix_leg)
    
    go = Department.grouped_options
    assert go[1].is_a?(Array)
    assert_equal [dept.name, dept.id], go[1][0]
    assert_equal 2, go.first.size
    assert_equal [[org.name, org.id]], go[1][1]
  end
  
  test "organization events" do
    o = organizations(:interior)
    
    assert !o.events.blank?
    assert [o.id], o.events.map {|m| m.organization_id}.uniq
  end
  
  test "should not save organization with duplicated name" do
    org = Organization.new(organizations(:lehendakaritza).attributes)
    assert !org.valid?
    assert org.errors.on(:name_es).include?("ya está cogido")
  end
  
  test "should not save organization with empty name" do
    org = Organization.new(organizations(:lehendakaritza).attributes.merge(:name_es => nil))
    assert !org.valid?
    assert org.errors.on(:name_es).include?("no puede estar vacío")
  end

  test "should not save organization with empty workarea" do
    org = Organization.new(organizations(:lehendakaritza).attributes.merge(:enet_workarea => nil))
    assert !org.valid?
    assert org.errors.on(:enet_workarea).include?("no puede estar vacío")
  end

  test "should have gc methods" do
    o = organizations(:interior)
    assert_nil o.gc_id
    assert !o.gc_link.present?
    
    assert o.update_attribute(:gc_id, 1)
    assert_equal 1, o.gc_id
    assert o.gc_link.present?    
  end
  
end
