#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require "test_helper"

class PageTest < ActiveSupport::TestCase
  
  test "class_name is Document" do
    page = Page.find(:first)
    assert_equal "Document", page.class_name
  end

  test "should index to elasticsearch after save" do
    prepare_elasticsearch_test
    page = documents(:page_in_menu)
    assert_deleted_from_elasticsearch page
    assert page.save
    assert_indexed_in_elasticsearch page
  end

  test "should delete from elasticsearch after destroy" do
    prepare_elasticsearch_test
    page = documents(:page_in_menu)
    assert_deleted_from_elasticsearch page
    assert page.save
    assert_indexed_in_elasticsearch page
    assert page.destroy
    assert_deleted_from_elasticsearch page
    clear_multimedia_dir(page)
  end

  context "with debate" do

    should "have debate" do
      page = documents(:debate_page)
      debate = debates(:debate_completo)

      assert_equal debate, page.debate
    end

    should "assign debate" do
      page = documents(:published_page)
      debate = debates(:debate_nuevo)
      
      assert_nil page.debate
      
      page.debate_id = debate.id
      page.multimedia_dir = "aportaciones_#{debate.multimedia_dir}"
      assert page.save
      
      assert_equal debate, page.debate
    end    
    
    should "nullify page when deleted" do
      page = documents(:debate_page)
      debate = debates(:debate_completo)
      
      assert_equal debate, page.debate
      
      assert page.destroy
      
      debate.reload
      
      assert_nil debate.page_id
    end
  end
end
