#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class PersonTest < ActiveSupport::TestCase
  test "should not have empty email" do
    user = Person.new(:name => "name")
    should_not_be_empty(user, :email)
  end
  
  test "should not have empty password" do
    user = Person.new(:name => "name")
    should_not_be_empty(user, :password)
  end
  
  test "should not have empty name" do
    user = Person.new(:email => "mail@example.com")
    should_not_be_empty(user, :name)
  end
  
  test "twitter user could have empty email" do
    user = Person.new(:screen_name => "screenname", :name => "name")
    user.save
    assert !user.errors.on(:email)
  end
  
  test "twitter user could have empty password" do
    user = Person.new(:screen_name => "screenname", :name => "name")
    user.save
    assert !user.errors.on(:password)
  end
  
  test "twitter user should not have empty name" do
    user = Person.new(:screen_name => "screen_name")
    should_not_be_empty(user, :name)
  end
  
  test "twitter user is approved by default" do
    user = Person.new(:screen_name => "screen_name", :name => "name")
    user.save
    assert user.errors
    assert_equal "aprobado", user.status
  end
  
  test "twitter_user cannot use another users email" do
    user = users(:twitter_user)
    user.email = users(:admin).email
    assert !user.save
    assert user.errors.on(:email).include?("ya existe un usuario con este email")
  end


  test "facebook user could have empty email" do
    user = Person.new(:name => "Nuevo usuario desde Facebook", :fb_id => '123')
    user.save
    assert !user.errors.on(:email)
  end
  
  test "facebook user could have empty password" do
    user = Person.new(:name => "Nuevo usuario desde Facebook", :fb_id => '123')
    user.save
    assert !user.errors.on(:password)
  end
  
  test "facebook user should not have empty name" do
    user = Person.new(:fb_id => '123')
    should_not_be_empty(user, :name)
  end
  
  test "facebook user is approved by default" do
    user = Person.new(:fb_id => "123", :name => "name")
    user.save
    assert user.errors
    assert_equal "aprobado", user.status
  end
  
  test "facebook_user cannot use another users email" do
    user = users(:facebook_user)
    user.email = users(:admin).email
    assert !user.save
    assert user.errors.on(:email).include?("ya existe un usuario con este email")
  end
  
  test "should has many followings" do
    user = users(:person_follows)
    area = areas(:a_lehendakaritza)
    politician = users(:politician_one)
    following_area = Following.new(:followed_id => area.id, :followed_type => 'Area', :user_id => user.id)
    assert_equal true, following_area.save
    following_politician = Following.new(:followed_id => politician.id, :followed_type => 'Politician', :user_id => user.id)
    assert_equal true, following_politician.save

    assert_equal 2, user.followings.size
    assert_equal [area], user.following_areas
    assert_equal [politician], user.following_politicians
  end  
  
end
