#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class PhotoTest < ActiveSupport::TestCase
    
  # Replace this with your real tests.
  test "should not have empty title" do
    photo = Photo.new(:file_path => "photos/fixture3.png")
    assert !photo.save
    [:title_es, :title_eu, :title_en].each do |field|
      assert_equal photo.errors.on(field), "no puede estar vacío"
    end
  end
  
  test "should not have empty file_path" do
    photo = Photo.new(:title_es => "Title", :title_eu => "Titulo", :title_en => "Titulo")
    assert !photo.save
    assert_equal "no puede estar vacío", photo.errors.on(:file_path)
  end
  
  test "should not duplicate paths" do
    photo = Photo.new(:title_es => "Title", :title_eu => "Titulo", :title_en => "Titulo", :file_path => "photos/fixture.png")
    assert !photo.save
    assert_equal "ya está cogido", photo.errors.on(:file_path)
  end
  
  test "should save tags" do
    photo = photos(:one)

    assert_difference 'photo.tags.count', +1 do
      photo.tag_list.add tags(:irekia_actos).name_es
      photo.save
    end
  end
    
  test "cannot create photo with path containing spaces" do
    photo = Photo.new(:dir_path => "photos/This has spaces")
    assert !photo.valid?, "expected photo to be invalid"
    assert photo.errors.on("dir_path"), "expected photo to have incorrect directory"
  end

  test "orphane photo is not published" do
    photo = photos(:orphane_photo)
    assert !photo.published?
  end
  
end
