#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class ScheduleEventTest < ActiveSupport::TestCase
  test "fixture event" do
    e = schedule_events(:evento_especial)
    
    assert_equal schedules(:agenda_especial).id, e.schedule_id
    assert_equal users(:admin).id, e.created_by
  end
  
  test "validations" do
    e = ScheduleEvent.new(:title => 'Nuevo evento', :starts_at => Time.now - 1.day, :ends_at => Time.now - 1.day + 2.hours, :schedule_id => schedules(:agenda_especial).id)
    assert e.valid?
    
    e = ScheduleEvent.new(:starts_at => Time.now - 1.day, :ends_at => Time.now - 1.day + 2.hours, :schedule_id => schedules(:agenda_especial).id)
    assert !e.valid?
    
    e = ScheduleEvent.new(:title => 'Nuevo evento', :ends_at => Time.now - 1.day + 2.hours, :schedule_id => schedules(:agenda_especial).id)
    assert !e.valid?

    e = ScheduleEvent.new(:title => 'Nuevo evento', :starts_at => Time.now - 1.day + 2.hours, :schedule_id => schedules(:agenda_especial).id)
    assert !e.valid?

    e = ScheduleEvent.new(:title => 'Nuevo evento', :starts_at => Time.now - 1.day, :ends_at => Time.now - 1.day + 2.hours)
    assert !e.valid?
    
  end

  test "validates ends_later_than_it_starts" do
    e = ScheduleEvent.new(:title => 'Nuevo evento', :starts_at => Time.now - 1.day, :ends_at => Time.now - 2.days + 2.hours, :schedule_id => schedules(:agenda_especial).id)
    assert !e.valid?
    assert_not_nil e.errors.on(:base)
  end

  test "fill_lat_lng_data ater save" do
    e = ScheduleEvent.new(:title => 'Nuevo evento', 
                          :starts_at => Time.now - 1.day, 
                          :ends_at => Time.now - 1.day + 2.hours, 
                          :schedule_id => schedules(:agenda_especial).id,
                          :location_for_gmaps => 'La Alhondiga, Bilbao')
    assert e.save
    assert_not_nil e.location_for_gmaps
    assert_match '43.2', e.lat.to_f.to_s
    assert_match '-2.9', e.lng.to_f.to_s
    
  end

  test "by default events are not public" do
    e = ScheduleEvent.new(:title => 'Nuevo evento', :starts_at => Time.now - 1.day, :ends_at => Time.now - 1.day + 2.hours, :schedule_id => schedules(:agenda_especial).id)
    assert e.save
    assert !e.is_public?
    assert !e.deleted?
  end  
  
  test "to_ics is defined" do
    assert RiCal.Calendar {|cal| schedule_events(:evento_especial).to_ics(cal)}
  end

  test "microformat_body" do
    assert schedule_events(:evento_especial).microformat_body
  end  
  
  test "included calendar methods" do
    e = schedule_events(:evento_especial)
    ["one_day?", "has_hour?", "year", "month", "day", "first_day", "days"].each do |m|
      assert e.respond_to?(m)
    end
    
    ["month_events", "month_events_by_day", "month_events_by_day4cal"].each do |cm|
      assert ScheduleEvent.respond_to?(cm)
    end
  end

  test "get events list for current month calendar" do
    s = schedules(:agenda_especial)
    
    events = s.events.month_events_by_day4cal(Time.now.month, Time.now.year)
    
    assert_equal 1, events.size
    assert !events[Time.now.month].empty?
    assert_not_nil events[Time.now.month][schedule_events(:evento_especial).day]
    assert_not_nil events[Time.now.month][schedule_events(:ee_last_day_of_month).day]
    assert_not_nil events[Time.now.month][schedule_events(:ee_first_day_of_month).day]
    
  end

  test "get events list for month calendar with previous month events" do
    s = schedules(:agenda_especial)
    
    date = Time.now.at_end_of_month + 1.day
    events = s.events.month_events_by_day4cal(date.month, date.year)
   
    assert_equal 1, events.size
    assert !events[Time.now.month].empty?
    assert events[date.month].blank?
    assert_not_nil events[Time.now.month][schedule_events(:ee_last_day_of_month).day]
  end

  test "get events list for month calendar with next month events" do
    s = schedules(:agenda_especial)
    
    date = Time.now.at_beginning_of_month - 1.day
    events = s.events.month_events_by_day4cal(date.month, date.year)
   
    assert_equal 1, events.size
    assert !events[Time.now.month].empty?
    assert events[date.month].blank?
    
    assert_not_nil events[Time.now.month][schedule_events(:ee_first_day_of_month).day]
  end
  
  test "is_destroyable?" do
    assert schedule_events(:evento_especial).is_destroyable?
  end
  
  test "sets l_attends to false before save" do
    e = ScheduleEvent.new(:title => 'Nuevo evento', :starts_at => Time.now - 1.day, :ends_at => Time.now - 1.day + 2.hours, :schedule_id => schedules(:agenda_especial).id)
    assert e.valid?
    assert !e.l_attends?    
  end

  test "sets l_attends to true if specified so" do
    e = ScheduleEvent.new(:title => 'Nuevo evento', :starts_at => Time.now - 1.day, :ends_at => Time.now - 1.day + 2.hours, :schedule_id => schedules(:agenda_especial).id)
    assert e.valid?
    assert !e.l_attends?    
    
    e.l_attends = true
    assert e.save
    assert e.l_attends?
  end
  
  test "events from the lehendakari agenda have l_attends set to true" do
    e = schedule_events(:l_event)
    assert e.l_attends?
    
    e = ScheduleEvent.new(:title => 'Nuevo evento', :starts_at => Time.now - 1.day, :ends_at => Time.now - 1.day + 2.hours, :schedule_id => schedules(:lehendakari).id)
    assert e.save
    assert e.l_attends?
    
  end
  
  test "organization" do
    # For the schedules without department_id, the event.organization is a new record with empty name.
    assert_nil schedule_events(:l_event).organization_id
    assert schedule_events(:l_event).organization.new_record?
    
    # For the schedules with department_id, the event organization os the department of the schedule.
    assert_equal organizations(:interior), schedule_events(:evento_interior).organization
  end

  test "body for different locales" do
    assert schedule_events(:l_event).body_es
    assert schedule_events(:l_event).body_eu
    assert schedule_events(:l_event).body_en

    assert schedule_events(:l_event).body_es=('xx')
    assert schedule_events(:l_event).body_eu=('xx')
    assert schedule_events(:l_event).body_en=('xx')

  end

  test "title for different locales" do
    assert schedule_events(:l_event).title_es
    assert schedule_events(:l_event).title_eu
    assert schedule_events(:l_event).title_en
                                    
    assert schedule_events(:l_event).title_es=('xx')
    assert schedule_events(:l_event).title_eu=('xx')
    assert schedule_events(:l_event).title_en=('xx')
  end
  
  test "compatibility methods" do
    assert schedule_events(:l_event).has_photographers=(0)
    assert schedule_events(:l_event).organization_id=(0)
    assert schedule_events(:l_event).has_journalists=(0)
    assert schedule_events(:l_event).show_in=('eventos')    
    
    assert !schedule_events(:l_event).has_photographers?
    assert !schedule_events(:l_event).has_journalists?
    assert_nil schedule_events(:l_event).organization_id
  end

  test "visibility_can_change?" do
    assert !schedule_events(:l_event).visibility_can_change?
    assert schedule_events(:evento_interior).visibility_can_change?
  end
  
end
