#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class SchedulesPermissionsTest < ActiveSupport::TestCase
  test "user permissions for a schedule" do
    u1 = users(:secretaria)
    u2 = users(:secretaria_interior_y_agenda_especial)
    s = schedules(:agenda_especial)
    sl = schedules(:lehendakari)
    so = schedules(:agenda_educacion)
    s2 = schedules(:agenda2)
    
    assert SchedulesPermission.find_by_schedule_id_and_user_id(s.id, u1.id)
    
    assert_equal [s.id, sl.id, so.id, s2.id].sort, u1.schedule_ids.sort
    assert_equal 0, ([u1.id, u2.id] - s.user_ids).length
    
    assert u1.can_see_schedule?(s.id)
    assert u2.can_see_schedule?(s.id)
  end
  
  test "users without permission can not see the special schedule" do
    u = users(:jefe_de_gabinete)
    assert_equal [], u.schedules
    
    assert !u.can_see_schedule?(schedules(:agenda_especial).id)
  end
  
  test "user has schedule permissions" do
    assert users(:secretaria).has_schedule_permissions?

    assert !users(:jefe_de_gabinete).has_schedule_permissions?
  end
  
  test 'fixtures schedules permissions' do
    u = users(:secretaria)
    s = schedules(:agenda_especial)
    
    assert u.can_see_schedule?(s.id)
    assert !u.can_edit_schedule?(s.id)
    assert !u.can_change_schedule?(s.id)
    
    s = schedules(:lehendakari)
    assert u.can_see_schedule?(s.id)
    assert u.can_edit_schedule?(s.id)
    assert u.can_change_schedule?(s.id)

    s = schedules(:agenda_interior)
    assert !u.can_see_schedule?(s.id)
    assert !u.can_edit_schedule?(s.id)
    assert !u.can_change_schedule?(s.id)

    u = users(:secretaria_interior)
    s = schedules(:agenda_especial)
    
    assert !u.can_see_schedule?(s.id)
    assert !u.can_edit_schedule?(s.id)
    assert !u.can_change_schedule?(s.id)
    
    s = schedules(:lehendakari)
    assert !u.can_see_schedule?(s.id)
    assert !u.can_edit_schedule?(s.id)
    assert !u.can_change_schedule?(s.id)

    s = schedules(:agenda_interior)
    assert u.can_see_schedule?(s.id)
    assert u.can_edit_schedule?(s.id)
    assert u.can_change_schedule?(s.id)


  end
  
end
