#
# OpenIrekia v4.0.0
#
# Copyright 2009-2013 eFaber, S.L.
# Copyright 2009-2013 Ejie, S.A.
# Copyrigth 2009-2013 Dirección de Gobierno Abierto y Comunicación en Internet; 
#    Gobernu Irekirako eta Interneteko Komunikaziorako Zuzendaritza; Lehendakaritza.
#    Gobierno Vasco – Eusko Jaurlaritza 
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas 
# por la Comisión Europea– versiones posteriores de la EUPL (la Licencia);
# Solo podrá usarse esta obra si se respeta la Licencia. Puede obtenerse una 
# copia de la Licencia en: http://ec.europa.eu/idabc/eupl 
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye TAL CUAL,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones 
# que establece la Licencia
#
#  http://open.irekia.net, openirekia@efaber.net
#

require 'test_helper'

class UserTest < ActiveSupport::TestCase
  
  test "dept and organizations" do
    dept = organizations(:interior)

    u = users(:periodista)
    old_deps = u.departments.dup
    old_org_ids = u.departments.map {|d| d.organization_ids}.flatten
    u.departments << dept
    u.reload
    
    assert_equal (old_deps.collect(&:id)+ [dept.id]).sort, u.department_ids.sort
    assert_equal (old_deps.collect(&:id) + old_org_ids + [dept.id] + dept.organization_ids).sort , u.organization_ids.sort
    
    assert_equal [organizations(:educacion)], users(:periodista_educacion).departments
  end


  test "user can see l_schedule" do
    u = users(:secretaria)
    assert u.can_see_l_schedule?
    
    u = users(:jefe_de_gabinete)
    assert !u.can_see_l_schedule?
  end  
  
  test 'editable schedules' do
    assert_equal 3, users(:secretaria).editable_schedules.count
  end
  
  test 'editable schedules with radio option' do
    assert_equal 1, users(:secretaria_interior).editable_schedules_with_radio().size
    assert_equal 1, users(:secretaria).editable_schedules_with_radio().size
  end
  
  test 'can create schedule events' do
    assert !users(:jefe_de_gabinete).can_create?('schedule_events')
    assert users(:secretaria).can_create?('schedule_events')
  end
  
  test 'department_events for department members' do
    u = users(:secretaria_interior)
    
    assert !u.department.events.blank?
  end
  
  test "can create permission" do
    assert users(:jefe_de_gabinete).can_create?('events'), 'Jefe de gabinete should have create events permissions'
    assert !users(:jefe_de_gabinete).can_create?('schedule_events'), 'Jefe de gabinete should not have create schedule events permissions'

    assert users(:jefe_de_prensa).can_create?('events'), 'Jefe de prensa should have create events permissions'
    assert !users(:jefe_de_prensa).can_create?('schedule_events'), 'Jefe de prensa should not have create schedule events permissions'


    assert users(:jefe_de_gabinete_de_interior).can_create?('events'), 'Jefe de gabinete de Interior should have create events permissions'
    assert users(:jefe_de_gabinete_de_interior).can_create?('schedule_events'), 'Jefe de gabinete de Interior should  have create schedule events permissions'

    assert users(:jefe_de_prensa_de_interior).can_create?('events'), 'Jefe de prensa de Interior should have create events permissions'
    assert users(:jefe_de_prensa_de_interior).can_create?('schedule_events'), 'Jefe de prensa de Interior should have create schedule events permissions'

    
    assert !users(:secretaria).editable_schedules.blank?
    assert users(:secretaria).can_create?('events'), 'Secretaria should have persmission to create events'
    assert users(:secretaria).can_create?('schedule_events'), 'Secretaria should have persmission to create schedule events'

    assert !users(:secretaria_interior).can_create?('events'), 'Secretaria interior should not have persmission to create events'
    assert users(:secretaria_interior).can_create?('schedule_events'), 'Secretaria interior should have persmission to create schedule events'
    
  end
  
  %w(superadmin admin periodista visitante colaborador jefe_de_prensa jefe_de_gabinete secretaria comentador_oficial twitter_user facebook_user).each do |user|
    test "#{user} should be able to create comments" do
      assert users(user).can_create?("comments"), "#{user} should be able to create comments and he's not"
    end
  end
  
  %w(operador_de_streaming miembro_que_crea_noticias).each do |user|
    test "#{user} should not be able to create comments" do
      assert !users(user).can_create?("comments"), "#{user} should NOT be able to create comments and he is"
    end    
  end
  
  test "should reset all explicit permissions on permissions table if user role is changed" do
    # Miembro de departamento con permiso explicito para crear eventos en agencia
    user = users(:creador_eventos_irekia)
    assert_equal 1, user.permissions.count
    # El role no le atribuye ningun permiso, asi que el unico que tiene es el de la tabla "permissions"
    assert_equal 0, (user.all_editable_permissions - user.permissions).length
    
    # Lo convertimos en jefe de gabinete
    user.type='StaffChief'
    assert user.save
    # tengo que volver a coger el usuario de la base de datos para que se entere de que le he cambiado el role
    user = User.find(user.id)
    assert user.is_a?(StaffChief)
    
    # Se resetean sus permisos en la tabla permissions
    assert_equal 0, user.permissions.count
  end
  
  test "should reset schedules_permissions if user department is changed" do
    user = users(:secretaria_interior_y_agenda_especial)
    # Tiene permisos en la agenda de su departamento y en la agenda especial
    assert_equal 2, user.schedules_permissions.count

    # Le cambiamos el departamento
    user.department=Department.find_by_name_es("Presidencia")
    assert user.department_id_changed?

    user.valid?
    assert user.save

    assert_equal 1, user.schedules_permissions.count
    # Conserva el permiso en la agenda especial pero pierde la de departamento
    assert user.schedules_permissions == [schedules_permissions(:schedule_permission_agenda_especial_secretaria_interior_y_agenda_especial)]
  end
  
  test "admin should not be able to administer users" do
    user = users(:admin)
    assert !user.can_access?("users")
  end
  
  test "superadmin should not be able to administer users" do
    user = users(:superadmin)
    assert user.can_access?("users")
  end
    
  ["miembro_que_modifica_noticias", "admin"].each do |user|
    test "#{user} can rate recommendations" do
      user = users(user.to_sym)
      assert user.can?('rate', 'recommendations')
    end
  end
  
  test "miembro_que_crea_noticias cannot rate recommendations" do
    user = users(:miembro_que_crea_noticias)
    assert !user.can?('rate', 'recommendations')
  end
  
  test "deactivate user account" do
    user = users(:visitante)
    assert_no_difference 'User.count' do
      user.deactivate_account
    end  
    user.reload
    assert_equal 'eliminado', user.status
    assert_equal I18n.t('users.eliminado'), user.public_name
    assert_equal "deleted_#{user.id}@email.com", user.email
  end
  
  test "new official commenter is included in official commenters listxx" do
    user = users(:miembro_que_crea_noticias)
    assert !User.official_commenters.include?(user)
    Permission.create(:user_id => user.id, :module => "comments", :action => "official")
    assert User.official_commenters.include?(user)
  end
  
  test "removing official comments permission removes user from official comments listxx" do
    user = users(:comentador_oficial)
    assert User.official_commenters.include?(user)
    user.permissions.find(:first, :conditions => "module='comments' AND action='official'").destroy
    assert !User.official_commenters.include?(user)
  end
  
  test "new DepartmentEditor is included in official commenters listxx" do
    user = users(:miembro_que_crea_noticias)
    assert !User.official_commenters.include?(user)
    user.type = 'DepartmentEditor'
    user.save
    # Can't do user.reload because we have changed its class
    user = User.find(user.id)
    assert user.is_official_commenter?
    assert User.official_commenters.include?(user)
  end
  
  test "removing department editor removes user from official_commenters listxx" do
    user = users(:jefe_de_prensa_de_interior)
    assert User.official_commenters.include?(user)
    user.destroy
    assert !User.official_commenters.include?(user)
  end
end